Package: FamAgg
Type: Package
Title: Pedigree Analysis and Familial Aggregation
Version: 1.40.0
Authors@R: c(person(given = "Johannes", family = "Rainer",
	   email = "johannes.rainer@eurac.edu",
	   role = c("aut", "cre"),
	   comment = c(ORCID = "0000-0002-6977-7147")),
	   person(given = "Daniel", family = "Taliun",
	   role = "ctb"),
	   person(given = "Christian", family = "Weichenberger",
	   email = "christian.weichenberger@eurac.edu",
	   role = "ctb",
	   comment = c(ORCID = "0000-0002-2176-0274"))
	  )
Author: J. Rainer, D. Taliun, C.X. Weichenberger
Maintainer: Johannes Rainer <johannes.rainer@eurac.edu>
URL: https://github.com/EuracBiomedicalResearch/FamAgg
BugReports: https://github.com/EuracBiomedicalResearch/FamAgg/issues
Imports: gap (>= 1.1-17), Matrix, BiocGenerics, utils, survey
Depends: methods, kinship2, igraph
Suggests: BiocStyle, knitr, RUnit, rmarkdown
VignetteBuilder: knitr
Description: Framework providing basic pedigree analysis and plotting
        utilities as well as a variety of methods to evaluate familial
        aggregation of traits in large pedigrees.
Collate: Classes.R Generics.R Deprecated.R Constructors.R
        Methods-FAData.R Methods-FAKinGroupResults.R
        Methods-FAKinSumResults.R Methods-FAGenIndexResults.R
        Methods-FAIncidenceRateResults.R
        Methods-FAStdIncidenceRateResults.R Methods-FABinTestResults.R
        utils.R matched-controls.R Methods.R plotting-functions.R
        import-export.R zzz.R
biocViews: Genetics
License: MIT + file LICENSE
Copyright: Haplopainter1.043.pl (http://haplopainter.sourceforge.net)
        Copyright (c) 2004-2008 Holger Thiele
NeedsCompilation: no
RoxygenNote: 7.1.1
Config/pak/sysreqs: cmake libglpk-dev make libicu-dev libuv1-dev
        libxml2-dev libssl-dev
Repository: https://bioc-release.r-universe.dev
Date/Publication: 2026-04-28 12:42:15 UTC
RemoteUrl: https://github.com/bioc/FamAgg
RemoteRef: RELEASE_3_23
RemoteSha: e5d4b24ca16b609b6186b63d13b01d1ad2590cae
Packaged: 2026-04-29 21:14:32 UTC; root
Built: R 4.6.0; ; 2026-04-29 22:12:21 UTC; unix

## Namespace
import(utils)
import(methods)
importFrom("stats", "density", "approxfun")
## import(BiocGenerics)
##import(igraph)
importFrom("stats", "binom.test", "dbinom", "pbinom")
importFrom(igraph, graph, distances, shortest_paths, V, induced_subgraph, clusters)
importClassesFrom(Matrix, "dsCMatrix")
importMethodsFrom(Matrix, as.matrix, colSums, colMeans, "[", which)
importFrom(BiocGenerics, "density")
importFrom(gap, pfc.sim)
import(kinship2)
importFrom(survey, stratsample)

importFrom("graphics", "abline", "hist", "legend", "par", "plot",
           "points", "strheight", "strwidth", "text")
importFrom("grDevices", "dev.off", "pdf", "png")
importFrom("stats", "aggregate", "p.adjust", "p.adjust.methods")
##importFrom(kinship2, kinship, pedigree, plot.pedigree)
##importClassesFrom(graph, graphNEL)
##importFrom(graph, graphNEL)
## importMethodsFrom(graph, leaves, isConnected, edges, nodes, isDirected,
##                   removeNode)
## importFrom(DBI, dbDriver)
## importClassesFrom(AnnotationHub, AnnotationHub)
## importMethodsFrom(AnnotationHub, query, mcols)

## exported functions
export("FAData", "connectedSubgraph", "doPlotPed", "estimateTimeAtRisk",
       "factor2matrix", "ped2graph", "sliceAge", "subPedigree", "switchPlotfun")
## exported classes
exportClasses("FAData", "FAProbResults", "FAKinGroupResults",
              "FAKinSumResults", "FAGenIndexResults", "FAIncidenceRateResults",
              "FAStdIncidenceRateResults", "FABinTestResults")
## exported methods
exportMethods("$", "[", "affectedIndividuals", "affectedKinshipGroups", "age",
              "age<-", "buildPed", "cliques", "cliques<-", "cliqueAndTrait",
              "export", "family", "kinship", "lambda", "pedigree", "pedigree<-",
              "pedigreeSize", "phenotypedIndividuals", "removeSingletons",
              "plotPed", "plotRes", "resultForId", "shareKinship", "result",
              "trait", "trait<-", "traitByClique", "kinship", "timeAtRisk",
              "timeAtRisk<-",
              "timeInStrata")
## basic pedigree utils
exportMethods("countGenerations", "estimateGenerations", "generationsFrom",
              "getAncestors", "getChildren", "getSiblings", "getMissingMate",
              "getCommonAncestor", "findFounders", "getFounders",
              "getSingletons")
## matched control methods
exportMethods("getAll", "getExternalMatched", "getGenerationMatched",
              "getGenerationSexMatched", "getSexMatched")

## analysis methods...
exportMethods("familialIncidenceRate", "familialIncidenceRateTest", "fsir",
              "fsirTest", "genealogicalIndexTest", "kinshipSumTest",
              "kinshipGroupTest", "probabilityTest", "runSimulation")
export("binomialTest")
export("kinshipPairs")

[![Years in Bioconductor](http://www.bioconductor.org/shields/years-in-bioc/FamAgg.svg)](http://www.bioconductor.org/packages/release/bioc/html/FamAgg.html)
[![Bioconductor release build status](http://www.bioconductor.org/shields/build/release/bioc/FamAgg.svg)](http://www.bioconductor.org/packages/release/bioc/html/FamAgg.html)
[![Bioconductor devel build status](http://www.bioconductor.org/shields/build/devel/bioc/FamAgg.svg)](http://www.bioconductor.org/checkResults/devel/bioc-LATEST/FamAgg.html)
[![build status](https://github.com/EuracBiomedicalResearch/FamAgg/workflows/R-CMD-check-bioc/badge.svg)](https://github.com/EuracBiomedicalResearch/FamAgg/actions?query=workflow%3AR-CMD-check-bioc)
[![codecov.io](https://codecov.io/github/EuracBiomedicalResearch/FamAgg/coverage.svg?branch=master)](https://codecov.io/github/EuracBiomedicalResearch/FamAgg?branch=master)

# FamAgg: Pedigree Analysis and Familial Aggregation

<img align = "right" src="https://raw.githubusercontent.com/Bioconductor/BiocStickers/master/FamAgg/FamAgg.png" height="200">

Framework providing basic pedigree analysis and plotting utilities as well as a
variety of methods to evaluate familial aggregation of traits in large
pedigrees.

## Results for the binomial test.

.validateFABinTestResults <- function(object) {
    msg <- character()
    ## Check that we've got all required columns
    cn <- colnames(object@result)
    exp_cn <- c("total_phenotyped", "total_affected", "family", "phenotyped",
                "affected", "pvalue", "prob")
    if (!all(exp_cn %in% cn))
        msg <- c(msg,
                 paste0("Expected columns: ", paste(exp_cn, collapse = ", "),
                 " but got: ", paste(cn, collapse = ", ")))
    if (length(msg))
        msg
    else
        TRUE
}
setValidity("FABinTestResults", .validateFABinTestResults)

setMethod("show", "FABinTestResults", function(object){
    callNextMethod()
    cat(paste0("Result info:\n"))
    cat(paste0(" * Number of rows of the result data.frame: ",
               nrow(object@result),".\n"))
})

#' @description Performs a binomial test on the provided data.
#'
#' @details `NA`s are removed prior to testing. If the pedigree does not provide
#'     a family id or if the whole pedigree consists of a single family, by
#'     default a *global* test is conducted. This requires parameter `prob` to
#'     be defined (i.e. the probability of being affected from the whole
#'     population).
#'
#' @param x `FAData` or `data.frame` with a column `"family"`.
#'
#' @param trait `logical` or `integer` defining the trait. We assume that these
#'     are in the correct format, i.e. 0,1,NA or TRUE,FALSE,NA.
#'
#' @param global `logical` defining whether a global or a per-family test should
#'     be conducted. If no families are available in 
#'
#' @param prob `numeric(1)` providing the probability of being affected in the
#'     whole population.
#'
#' @param alternative `character(1)` indicates the alternative hypothesis and
#'     must be one of `"greater"` (default), `"less"` or `"two.sided"`.
#' 
#' @return `data.frame` with the results of the binomial test. One row for each
#' family (or a single row if `global = TRUE`). Columns are:
#' - `"total_phenotyped"`
#' - `"total_affected"`
#' - `"family"`: the ID (number) of the family.
#' - `"phenotyped"`
#' - `"affected"`
#' - `"pvalue"`: the p-value.
#' - `"prob"`: the probability of being affected. Either a *local* probability
#'    calculated based on all affected and phenotyped in the whole pedigree or
#'    a *global* (population) probability that was provided with argument
#'    `prob`.
#' 
#' @md
#'
#' @author Johannes Rainer
#'
#' @noRd
.binomialTest <- function(x, trait = NULL, global = is.null(x$family),
                          prob = NULL, alternative = c("greater", "less",
                                                       "two.sided")) {
    alternative <- match.arg(alternative)
    if (is.null(x$family) | global)
        fams <- rep_len("full pedigree", nrow(x@pedigree))
    else
        fams <- x$family
    ## Check that length of trait matches length of fams.
    if (length(fams) != length(trait))
        stop("length of 'trait' has to match the number of individuals in ",
             "the pedigree")
    fam_ids <- as.character(unique(fams))
    is_na <- is.na(trait)
    ## Kick out NAs.
    trait <- trait[!is_na]
    if (length(trait)) {
        ## Only perform the tests if we have something to test!
        fams <- fams[!is_na]
        trait_by_fam <- split(trait, f = fams)
        ## Now, if we have a single family assume we do a global test.
        if (length(trait_by_fam) == 1 & !global) {
            warning("Have a single family with phenotyped individuals. Will ",
                    "switch to a global test.")
            global <- TRUE
        }
        ## global = TRUE requires definition of prob.
        if (global & is.null(prob))
            stop("Argument 'prob' has to be specified if a global test is ",
                 "performed")
        total_phenotyped <- length(trait)
        total_affected <- sum(trait)
        fam_phenotyped <- lengths(trait_by_fam)
        fam_affected <- unlist(lapply(trait_by_fam, sum))
        if (is.null(prob))
            prob <- total_affected / total_phenotyped
        pvals <- .binom.test(fam_affected, fam_phenotyped, prob = prob,
                             alternative = alternative)
        ## Compile results data.frame
        res <- data.frame(total_phenotyped = total_phenotyped,
                          total_affected = total_affected,
                          family = names(trait_by_fam),
                          phenotyped = fam_phenotyped,
                          affected = fam_affected,
                          pvalue = pvals,
                          prob = prob,
                          check.names = FALSE, stringsAsFactors = FALSE)
    } else {
        if (is.null(prob))
            prob <- NA
        trait_by_fam <- c()
        total_phenotyped <- 0
        total_affected <- 0
        res <- data.frame(total_phenotyped = integer(),
                          total_affected = integer(),
                          family = character(),
                          phenotyped = integer(),
                          affected = integer(),
                          pvalue = numeric(),
                          prob = numeric(),
                          check.names = FALSE, stringsAsFactors = FALSE)
    }
    ## Add also results for families without affected.
    if (any(!(fam_ids %in% names(trait_by_fam)))) {
        na_fams <- fam_ids[!(fam_ids %in% names(trait_by_fam))]
        na_data <- data.frame(total_phenotyped = total_phenotyped,
                              total_affected = total_affected,
                              family = na_fams,
                              phenotyped = 0,
                              affected = 0,
                              pvalue = NA,
                              prob = prob,
                              check.names = FALSE, stringsAsFactors = FALSE)
        res <- rbind(res, na_data)
        
    }
    ## Re-order them.
    rownames(res) <- as.character(res$family)
    res[fam_ids, ]
}

## binomial test. Since that's a function it can be run on any objects
## inheriting FAData.
binomialTest <- function(object, trait, traitName, global = FALSE,
                         prob = NULL, alternative = c("greater", "less",
                                                      "two.sided")) {
    alternative <- match.arg(alternative)
    if (!is(object, "FAData"))
        stop("'object' is expected to be a 'FAData' object but I got '",
             class(object), "'")
    if (!is(object, "FABinTestResults"))
        object <- as(object, "FABinTestResults")
    if (missing(trait))
        trait <- trait(object)
    if (!length(trait))
        stop("'trait' is missing")
    ## Setting the trait. This ensures also that we don't provide wrong format.
    trait(object) <- trait
    if (!missing(traitName))
        object@traitname <- traitName
    ## Run the test.
    object@result <- .binomialTest(object, trait = trait(object),
                                   global = global, prob = prob,
                                   alternative = alternative)
    if (validObject(object))
        object
}

setReplaceMethod("trait", "FABinTestResults", function(object, value){
    object <- callNextMethod()
    ## reset the result
    object@result <- data.frame(total_phenotyped = integer(),
                                total_affected = integer(),
                                family = character(),
                                phenotyped = integer(),
                                affected = integer(),
                                pvalue = numeric(),
                                prob = numeric(),
                                check.names = FALSE, stringsAsFactors = FALSE)
    object@traitname <- character()
    if (validObject(object))
        object
})

setMethod("result", "FABinTestResults", function(object, method = "BH") {
    method <- match.arg(method, p.adjust.methods)
    traitName <- object@traitname
    if (!length(traitName))
        traitName <- NA
    res <- object@result
    padj <- numeric()
    if (nrow(res))
        padj <- p.adjust(res$pvalue, method = method)
    res <- cbind(trait_name = rep(traitName, nrow(res)),
                 res,
                 padj = padj, stringsAsFactors = FALSE)
    if (nrow(res))
        res <- res[order(res$pvalue), ]
    res
})

#' @description `.binom.test` used the `binom.test` function to calculate the
#'     p-value and supports different alternatives.
#' 
#' @md
#' @noRd
.binom.test <- function(x, size, prob, alternative = c("greater", "less",
                                                           "two.sided")) {
    alternative <- match.arg(alternative)
    mapply(x, size, FUN = function(a, b)
        binom.test(a, b, p = prob, alternative = alternative)$p.value)
}
#' @description `.pbinom.test` uses `pbinom` and `dbinom` to calculate the
#'     p-value and does only support `alternative = "greater"`. It is however
#'     considerably faster than `.binom.test`.
#'
#' @md
#' @noRd
.pbinom.test <- function(x, size, prob, alternative = "greater") {
    if (alternative != "greater")
        stop("Currently only 'alternative' 'greater' supported")
    pbinom(x, size, prob, lower.tail = FALSE) + dbinom(x, size, prob)
}

setMethod("[", "FABinTestResults", function(x, i, j, ..., drop){
    stop("Subsetting of a FABinTestResults object is not supported!")
})

data(minnbreast)
mbsub <- minnbreast[minnbreast$famid %in% 4:19, ]
PedDf <- mbsub[, c("famid", "id", "fatherid", "motherid", "sex")]
colnames(PedDf) <- FamAgg:::.PEDCN
PedDf$sex <- FamAgg:::sanitizeSex(PedDf$sex)
## generate the FAData.
fad <- FAData(pedigree=PedDf)
## specify the trait.
tcancer <- mbsub$cancer
names(tcancer) <- mbsub$id

test_dot_binomialTest <- function() {
    ## Working:
    res <- FamAgg:::.binomialTest(fad, trait = tcancer)
    checkEquals(res$total_phenotyped[1], sum(!is.na(tcancer)))
    checkEquals(res$total_affected[1], sum(tcancer, na.rm = TRUE))
    checkEquals(as.character(unique(fad$family)), res$family)
    ## Check errors
    checkException(FamAgg:::.binomialTest(fad, trait = tcancer,
                                          alternative = "other"))
    checkException(FamAgg:::.binomialTest(fad))
    checkException(FamAgg:::.binomialTest(fad, trait = 1:10))
    ## no affected.
    tr <- rep(0, nrow(fad@pedigree))
    res <- FamAgg:::.binomialTest(fad, trait = tr)
    checkEquals(unique(res$pvalue), 1)
    checkEquals(unique(res$affected), 0)
    checkEquals(as.character(unique(fad$family)), res$family)
    ## One affected in one family.
    tr[13] <- 1
    res <- FamAgg:::.binomialTest(fad, trait = tr)
    ## NAs
    tr[] <- NA
    res <- FamAgg:::.binomialTest(fad, trait = tr)
    checkEquals(as.character(unique(fad$family)), res$family)
    checkTrue(all(is.na(res$pvalue)))
    checkTrue(all(res$phenotyped == 0))
    ## only one family.
    tmp <- fad
    tmp@pedigree$family <- 1
    checkException(FamAgg:::.binomialTest(tmp, trait = tcancer))
    suppressWarnings(
        res <- FamAgg:::.binomialTest(tmp, trait = tcancer, prob = 1/16)
    )
    checkEquals(nrow(res), 1)
    ## Different alternative.
    res_2 <- FamAgg:::.binomialTest(tmp, trait = tcancer, prob = 1/16,
                                    alternative = "less", global = TRUE)
    checkTrue(res_2$pvalue != res$pvalue)
    ## Set one family to NA and check that the pvalue is NA for them.
    tr <- tcancer
    tr[fad$family == 5] <- NA
    res <- FamAgg:::.binomialTest(fad, trait = tr)
    checkTrue(is.na(res$pvalue[res$family == "5"]))
    checkTrue(all(!is.na(res$pvalue[res$family != "5"])))
    
    ## Set all families except one to NA.
    tr <- tcancer
    tr[] <- NA
    tr[fad$family == 5] <- tcancer[fad$family == 5]
    res <- FamAgg:::.binomialTest(fad, trait = tr, global = TRUE, prob = 1/16)
    checkTrue(nrow(res) == 1)

    tr <- tcancer
    tr[] <- NA
    tr[fad$family %in% c(5, 7)] <- tcancer[fad$family %in% c(5, 7)]
    res <- FamAgg:::.binomialTest(fad, trait = tr)
    checkTrue(!is.na(res$pvalue[res$family == "5"]))
    checkTrue(!is.na(res$pvalue[res$family == "7"]))
    checkTrue(all(is.na(res$pvalue[!(res$family %in% c("5", "7"))])))
}

test_validateFABinTestResults <- function() {
    tmp <- new("FABinTestResults")
    checkTrue(validObject(tmp))
    checkTrue(FamAgg:::.validateFABinTestResults(tmp))
    ## Modify the object
    tmp@result <- tmp@result[, 1:3, drop = FALSE]
    checkException(validObject(tmp))
    checkTrue(is(FamAgg:::.validateFABinTestResults(tmp), "character"))
}

test_binomialTest <- function() {
    res <- binomialTest(fad, trait = tcancer, traitName = "Breast Cancer")
    checkTrue(validObject(res))
    checkEquals(as.character(unique(fad$family)), res@result$family)
    checkTrue(all(res@result$pvalue < 1))
    res_tab <- result(res)
    checkTrue(all(res_tab$trait_name == "Breast Cancer"))
    checkEquals(res_tab$family, res_tab[order(res_tab$pvalue), "family"])
    checkEquals(res_tab$pvalue, sort(res@result$pvalue))
    trait(res) <- tcancer
    checkEquals(nrow(res@result), 0)
    ## Can run the analysis on this?
    res <- binomialTest(res, trait = tcancer)
    res_2 <- binomialTest(fad, trait = tcancer)
    checkEquals(res, res_2)

    res_tab <- result(res, method = "BH")
    res_tab_2 <- result(res, method = "bonferroni")
    checkTrue(all(res_tab$padj < res_tab_2$padj))
    
    ## all NA trait.
    tr <- rep(NA, length(fad$id))
    names(tr) <- fad$id
    checkException(binomialTest(fad, trait = tr))

    ## All except one family NA
    tr[fad$family == 8] <- tcancer[fad$family == 8]
    checkException(binomialTest(fad, trait = tr))
    res <- binomialTest(fad, trait = tr, global = TRUE, prob = 1/16)
    checkEquals(nrow(res@result), 1)
    checkTrue(res@result$pvalue[1] < 0.05)
    res_tab <- result(res)
    checkTrue(nrow(res_tab) == 1)
    
    res_2 <- binomialTest(fad, trait = tcancer, global = TRUE, prob = 1/16)
    checkTrue(res_2@result$pvalue > res@result$pvalue)
    
    ## Errors
    checkException(binomialTest(fad))
    checkException(binomialTest(fad, trait = 1:4))
    checkException(binomialTest(fad, trait = rep(4, length(fad$id))))
}

## ----echo = FALSE, results = "hide"-------------------------------------------
isWindows <- .Platform$OS.type != "unix"
library(FamAgg)
library(BiocStyle)

## ----libraries, warning = FALSE, message = FALSE------------------------------
library(FamAgg)

data(minnbreast)
## Subsetting to only few families of the whole data set.
mbsub <- minnbreast[minnbreast$famid %in% 4:14, ]
mbped <- mbsub[, c("famid", "id", "fatherid", "motherid", "sex")]
## Renaming column names.
colnames(mbped) <- c("family", "id", "father", "mother", "sex")
## Defining the optional argument age.
endage <- mbsub$endage
names(endage) <- mbsub$id
## Create the object.
fad <- FAData(pedigree = mbped, age = endage)

## ----access-data--------------------------------------------------------------
## Use the pedigree method to access the full pedigree
## data.frame,
head(pedigree(fad))

## or access individual columns using $.
## The ID of the father (0 representing "founders"):
head(fad$father)
## Mother:
head(fad$mother)
## Sex:
head(fad$sex)

## We can also access the age of each individual, if
## provided.
head(age(fad))

## ----family-------------------------------------------------------------------
## Extract the pedigree information from family "4"...
nrow(family(fad, family = 4))

head(family(fad, family = 4))

## ...which is the same as extracting the family pedigree
## for an individual of this family.
head(family(fad, id = 3))

## Note that IDs are internally always converted to character,
## thus, using id=3 and id="3" return the same information.
head(family(fad, id = "3"))

## ----subsetting---------------------------------------------------------------
## Subset the object to a single family.
fam4 <- fad[fad$family == "4", ]
table(fam4$family)

## ----plotPed, message=FALSE, fig.align='center', warning = FALSE--------------
switchPlotfun("ks2paint")
## By supplying device="plot", we specify that we wish to visualize the
## pedigree in an R plot. This is the default for "ks2paint", anyway.
plotPed(fad, id = 3, device = "plot")

## ----buildPed, message = FALSE------------------------------------------------
## Build the pedigree for individual 3.
fullPed <- buildPed(fad, id = "3")
nrow(fullPed)

## ----buildPed-prune, message = FALSE------------------------------------------
## Find the subpedigree for individuals 21, 22 and 17.
buildPed(fad, id = c(21, 22, 17), prune = TRUE)

## ----plotPed-3ids, message=FALSE, fig.align='center', warning = FALSE---------
plotPed(fad, id = c(21, 22, 17), prune = TRUE)

## ----plotPed-family-14, message=FALSE, fig.align='center', warning = FALSE----
plotPed(fad, family = "14", cex = 0.4)

## ----buildPed-for-individual, message = FALSE---------------------------------
## Check if we have individual 26064 from the second branch in the pedigree
## of individual 440.
any(buildPed(fad, id = "440")$id == "26064")

## What for the pedigree of 447?
any(buildPed(fad, id = "447")$id == "26064")

## ----findFounders, message = FALSE--------------------------------------------
## Find founders for family 4.
findFounders(fad, "4")

## ----getCommonAncestors, message = FALSE--------------------------------------
## Find the closest common ancestor.
getCommonAncestor(fad, id = c(21, 22, 17))

## ----getChildren-and-others, message = FALSE----------------------------------
## Get the children of ID 4.
getChildren(fad, id = "4", max.generations = 1)

## Get the offsprings.
getChildren(fad, id = "4")

## Get all ancestors.
getAncestors(fad, id = "4")

## Get the siblings.
getSiblings(fad, id = c("4"))

## ----affected-founders, message = FALSE---------------------------------------
## Add the trait information to the FAData object.
cancer <- mbsub$cancer
names(cancer) <- as.character(mbsub$id)
trait(fad) <- cancer

## Identify the affected founders.
## First all affected individuals.
affIds <- affectedIndividuals(fad)
## Identify founders for each family.
founders <- lapply(unique(fad$family), function(z){
    return(findFounders(fad, family = z))
})
names(founders) <- unique(fad$family)

## Track the affected founder.
affFounders <- lapply(founders, function(z){
    return(z[z %in% affIds])
})
## Interestingly, not all founders are affected! It seems in some cases
## parents of the affected participants in the screening phase have also
## been included.
affFounders <- affFounders[unlist(lapply(affFounders, length)) > 0]

## The number of families analyzed.
length(founders)

## The number of families with affected founder.
length(affFounders)


## ----affected-for-founders, message = FALSE-----------------------------------
kin2affFounders <- shareKinship(fad, unlist(affFounders))

## How many of these are affected?
sum(kin2affFounders %in% affIds)

## How many affected are not related to an affected founder?
sum(!(affIds %in% kin2affFounders))


## ----shareKinship, message = FALSE--------------------------------------------
## Get all individuals sharing kinship with individual 4.
shareKinship(fad, id = "4")

## ----shareKinshipRm, message = FALSE------------------------------------------
## Get all individuals sharing kinship with individual 4, but only with kinship
## higher than 0.125 (exclude first cousins, grand children, great grand
## parents etc, i.e. everybody with kinship 0.125 or lower)
shareKinship(fad, id = "4", rmKinship = 0.125)

## ----estimageGenerations, message = FALSE-------------------------------------
## Estimate generation levels for all families.
estimateGenerations(fad)[1:3]

## ----generationsFrom, message = FALSE-----------------------------------------
gens <- generationsFrom(fad, id = "4")

## ----plotPed-with-generations, message=FALSE, fig.align='center', warning = FALSE----
plotPed(fad, family = 4, label2 = gens)

## ----set-trait, results='hide', message=FALSE---------------------------------
## Extract the cancer trait information.
tcancer <- mbsub$cancer
names(tcancer) <- mbsub$id
## Set the trait.
trait(fad) <- tcancer


## ----affectedIndividuals, message = FALSE-------------------------------------
## Extract the trait information.
head(trait(fad))

## We can also extract the IDs of the affected individuals.
head(affectedIndividuals(fad))

## Or the IDs of the phenotyped individuals.
head(phenotypedIndividuals(fad))

## ----plotPed-with-trait, message=FALSE, fig.align='center', warning = FALSE----
## Plotting the pedigree for family "9".
plotPed(fad, family = "9")

## ----plotPed-trait-highlight, message=FALSE, fig.align='center', warning = FALSE----
## Plotting the pedigree for family "9".
plotPed(fad, family = "9", highlight.ids = list(a = c("185", "201", "198"),
						b = c("193")))

## ----ped2graph, message = FALSE-----------------------------------------------
## Transform the full pedigree to a graph.
fullGraph <- ped2graph(pedigree(fad))

## In addition, build the graph for a single family.
singleFam <- ped2graph(family(fad, family=4))

## ----plot-igraph, fig.align='center', message = FALSE, fig.cap = "Pedigrees represented as graphs."----
## Build the layout.
plot(fullGraph)
lay <- layout_(singleFam, on_grid())
plot(singleFam, layout = lay)

## ----connectedSubgraph, message = FALSE---------------------------------------
subgr <- connectedSubgraph(singleFam, nodes = c("7", "8", "27", "17"))

## ----plot-subgraph, subgraph-plot, fig.align='center'-------------------------
## Plot the graph.
plot(subgr)
## Similar to buildPed/plotPed with prune=TRUE.
plotPed(fad, id=c("7", "8", "17", "27"), prune=TRUE)

## ----import, message=FALSE----------------------------------------------------
## Import a "ped" file.
pedFile <- system.file("txt/minnbreastsub.ped.gz", package = "FamAgg")
## Quick glance at the file.
readLines(pedFile, n = 1)
fad <- FAData(pedFile)

head(pedigree(fad))


## ----import-generic, message=FALSE--------------------------------------------
## Create the FAData by reading data from a txt file.
pedFile <- system.file("txt/minnbreastsub.txt", package = "FamAgg")
fad <- FAData(pedigree = pedFile, header = TRUE, id.col = "id",
	      family.col = "famid", father.col = "fatherid",
	      mother.col = "motherid")


## ----export-------------------------------------------------------------------
tmpF <- tempfile()

## Subset the pedigree to family 4
fam4 <- fad[fad$family == 4, ]

## Export data in ped format.
export(fam4, tmpF, format = "ped")

## ----famagg-setup, warning=TRUE, message=FALSE--------------------------------
library(FamAgg)
set.seed(18011977)
data(minnbreast)
## Subset the dataset to reduce processing time.
mbsub <- minnbreast[minnbreast$famid %in% c(4:100, 173, 432), ]
## Uncomment the line below to use the whole dataset instead.
## mbsub <- minnbreast

## Define the number of simulations we perform.
## nsim <- 10000
nsim <- 1000

mbped <- mbsub[, c("famid", "id", "fatherid", "motherid", "sex")]
## Renaming column names.
colnames(mbped) <- c("family", "id", "father", "mother", "sex")
## Create the FAData object.
fad <- FAData(pedigree = mbped)

## Define the trait.
tcancer <- mbsub$cancer
names(tcancer) <- as.character(mbsub$id)

## ----gif, warning=FALSE, message=FALSE----------------------------------------
## Calculate the genealogical index of familiality.
gi <- genealogicalIndexTest(fad, trait = tcancer,
			    traitName = "cancer", nsim = nsim)

## Display the result.
result(gi)

## ----gif-2, warning=FALSE, eval=FALSE-----------------------------------------
# ## Calculate the genealogical index of familiality using random sampling from
# ## a sex matched control set.
# giSexMatch <- genealogicalIndexTest(fad, trait = tcancer,
# 				    traitName = "cancer", nsim = nsim,
# 				    controlSetMethod = "getSexMatched")
# 
# ## Use an external vector to perform the matching.
# ## The results are essentially identical.
# giExtMatch <- genealogicalIndexTest(fad, trait = tcancer,
# 				    traitName = "cancer", nsim = nsim,
# 				    controlSetMethod = "getExternalMatched",
# 				    match.using = fad$sex)

## ----gif-3, message=FALSE-----------------------------------------------------
## Evaluate the proportion of male and femal cases.
table(gi$sex[affectedIndividuals(gi)])

## We can use the gender information to perform stratified sampling, i.e.
## in each permutation a random set of 3 male and 15 females will be selected.
giStrata <- genealogicalIndexTest(fad, trait = tcancer,
				  traitName = "cancer", nsim = nsim,
				  strata = fad$sex)

result(giStrata)

## ----gif-4-plot, mbreast-genealogical-index-result, message=FALSE, warning=FALSE, fig.align='center'----
## Plot the result.
plotRes(giStrata)

## ----gif-gap, message=FALSE---------------------------------------------------
library(gap)

## Adding the trait information, so the extracted pedigree data.frame will
## also contain a column "affected" with that information.
trait(fad) <- tcancer

## Extract the pedigree and re-format it for the gif function.
pedi <- pedigree(fad)
## Remove singletons.
pedi <- removeSingletons(pedi)
pedi[is.na(pedi$father), "father"] <- 0
pedi[is.na(pedi$mother), "mother"] <- 0

## Identify the affected individuals.
affIds <- as.numeric(pedi$id[which(pedi$affected == 1)])

## Execute the gif method contained in the gap package.
gifRes <- gif(pedi[, c("id", "father", "mother")], affIds)

## Calculate the GIF using FamAgg's genealogicalIndexTest.
gifT <- genealogicalIndexTest(fad, trait = tcancer, nsim = 100)

## Comparing the results:
all.equal( result(gifT)$genealogical_index,  gifRes[[1]] )

## ----gif-5, message=FALSE, warning=FALSE--------------------------------------
## Perform the analysis (no strata etc) separately for each family.
giFam <- genealogicalIndexTest(fad, trait = tcancer, nsim = nsim,
			       perFamilyTest = TRUE,
			       traitName = "Cancer")

## Display the result from the analysis.
head(result(giFam))

## ----fir-1, warning=FALSE, message = FALSE------------------------------------
## Estimate the risk for each individual using the familial incidence
## rate method. We use the "endage" provided in the Minnesota Breast Cancer
## Record as a measure for time at risk.
fr <- familialIncidenceRate(fad, trait = tcancer, timeAtRisk = mbsub$endage)


## ----fir-2, mbreast-mean-fr-per-family, message=FALSE, warning=FALSE, fig.align='center'----
## Split the FIR by family and average the values within each.
frFam <- split(fr, f = fad$family)
frFamAvg <- lapply(frFam, mean, na.rm = TRUE)

## Sort and plot the averages.
frFamAvg <- sort(unlist(frFamAvg), decreasing = TRUE)
plot(frFamAvg, type = "h", xaxt = "n", xlab = "", ylab = "mean FIR",
     main = "Per family averaged familial incidence rate")
axis(side = 1, las = 2, at = 1:length(frFamAvg), label = names(frFamAvg))


## ----fir-3, warning=FALSE, message=FALSE--------------------------------------
## Estimate the risk for each individual using the familial incidence
## rate method. We use the endage provided in the Minnesota Breast Cancer
## Record as a measure for time at risk.
frTest <- familialIncidenceRateTest(fad, trait = tcancer,
				    traitName = "cancer",
				    timeAtRisk = mbsub$endage,
				    nsim = nsim)


## ----fir-4--------------------------------------------------------------------
head(familialIncidenceRate(frTest))
head(frTest$fir)


## ----fir-5--------------------------------------------------------------------
head(result(frTest))


## ----fir-6--------------------------------------------------------------------
frRes <- result(frTest)
frSig <- frRes[which(frRes$padj < 0.05), ]

## Split by family.
frFam <- split(frSig, frSig$family)
frRes <- data.frame(family = names(frFam),
		    no_sign_fir = unlist(lapply(frFam, nrow)))
## Determine the number of phenotyped and affected individuals per family.
noPheNAff <- sapply(names(frFam), function(z){
    fam <- family(frTest, family = z)
    return(c(no_pheno = sum(!is.na(fam$affected)),
	     no_aff = length(which(fam$affected == 1))
	     ))
})
frRes <- cbind(frRes, t(noPheNAff))

## Display the number of phenotyped and affected individuals as well as
## the number of individuals within the families with a significant FIR.
frRes[order(frRes[, "no_sign_fir"], decreasing = TRUE), ]

## ----kinsum-1, message = FALSE------------------------------------------------
## Perform the kinship sum test.
kinSum <- kinshipSumTest(fad, trait = tcancer, traitName = "cancer",
			 nsim = nsim, strata = fad$sex)
head(result(kinSum))


## ----kinsum-2, message = FALSE------------------------------------------------
## Extract the IDs of the individuals with significant kinship. By default,
## the raw p-values are adjusted for multiple hypothesis testing using the
## method from Benjamini and Hochberg.
kinSumRes <- result(kinSum)
kinSumIds <- as.character(kinSumRes[kinSumRes$padj < 0.1, "affected_id"])

## From which families are these?
table(kinSumRes[kinSumIds, "family"])

## ----kinsum-3, mbreast-family-432-FIR-compared-to-others, message=FALSE, warning=FALSE, fig.align='center'----
## Get the familial ratio of the significant in this family, of all in
## this family, and of all others.
famId <- kinSumRes[1, "family"]

## Extract the family.
fam <- family(kinSum, family = famId)

## Stratify individuals in affected/unaffected.
strat <- rep("All, unaff.", length(kinSum$id))
strat[which(kinSum$affected > 0)] <- "All, aff."
strat[kinSum$id %in% fam$id] <- paste0("Fam ", famId, ", unaff.")
strat[kinSum$id %in% fam$id[which(fam$affected > 0)]] <-
    paste0("Fam ",famId,", aff.")

famData <- data.frame(fr = fr, group = strat)
boxplot(fr~group, data = famData, na.rm = TRUE, ylab = "FIR",
	col = rep(c("#FBB4AE", "#B3CDE3"), 2))


## ----kinsum-4, mbreast-family-432-affected, message=FALSE, warning=FALSE, fig.align='center'----
## Plot the pedigree for the family of the selected individual removing
## all individuals that were not phenotypes.
plotPed(kinSum, id = kinSumIds[1], cex = 0.3, only.phenotyped = TRUE)



## ----kinsum-5, mbreast-family-432-affecte-res, message=FALSE, warning=FALSE, fig.align='center'----
plotRes(kinSum, id = kinSumIds[1])

## ----kingroup-1, message=FALSE------------------------------------------------
## Calculate the kinship test.
kinGroup <- kinshipGroupTest(fad, trait = tcancer,
			     traitName = "cancer",
			     nsim = nsim, strata = fad$sex)
head(result(kinGroup))


## ----kingroup-2, message = FALSE----------------------------------------------
kinGroupRes <- result(kinGroup)
## Create a data.frame with the summarized results.
resTab <- data.frame(total_families = length(unique(kinGroup$family)),
		     ratio_sign = length(unique(
			 kinGroupRes[kinGroupRes$ratio_padj < 0.05, "family"]
		     )),
		     kinship_sign = length(unique(
			 kinGroupRes[kinGroupRes$kinship_padj < 0.05, "family"]
		     ))
		     )
resTab

## ----kingroup-3, mbreast-family-432-affecte-res-kinship, message=FALSE, warning=FALSE, fig.align='center'----
plotPed(kinGroup, id = kinGroupRes[kinGroupRes$family == "432",
				   "group_id"][1],
	prune = TRUE, label1 = fr)

## ----bintest-1, message = FALSE-----------------------------------------------
binRes <- binomialTest(fad, trait = tcancer, traitName = "Cancer")

binResTab <- result(binRes)
head(binResTab)

## ----bintest-2, message = FALSE-----------------------------------------------
## Set the trait status to NA for all male individuals.
tcancer[fad$sex == "M" | is.na(fad$sex)] <- NA

## Perform the test providing also the population probability
binRes <- binomialTest(fad, trait = tcancer, prob = 1/8)

binResTab <- result(binRes)
head(binResTab)

## ----bintest-3, message = FALSE, fig.align = "center", fig.pos = "h!"---------
plotPed(binRes, family = 173)

## Methods and functions related to the FSIR from Kerber.

##************************************************
##
## FSIR Kerber 1995
## the familial standardized incidence ratio.
## What do we need to calculate it:
## * affected vector: can get the cases from that.
## * the kinship matrix: setting diag(kin) <- 0, so we exclude self-self
##   kinship.
## * strata: some sort of stratification of cases.
## * time in strata: related to the strata: the time each individual spend in
##   strata.
## * the population incidence rate per stratum: that's really tricky! how do
##   we get the population incidence if we have only the cohort available...
##   sampling? that will not work, if, only with replacement... but still.
##
## Side note: what if we use the incidence ratio from the whole pedigree? the
##            FSIR is anyway just calculated based on individuals from the same
##            family (got a kinship value of 0 for other individuals). Such
##            lambda would represent the expected number of affected
##            individuals in the family given the data from the whole pedigree.
##************************************************
## Note: this function assumes that all elements are in the correct order!
## Arguments:
## * affected: 0, 1 coding, length corresponds to the phenotyped; length n.
## * kin: nxn kinship matrix. ordering has to match individuals in affected.
## * lambda: population incidence rate. numeric vector, length has to match
##   the number of strata.
## * timeInStrata: ncol=length(lambda), nrow=n. has to have the same unit
##   than lambda.
doFsir <- function(affected, kin, lambda, timeInStrata) {
    if(missing(affected))
        stop("affected missing!")
    if(missing(kin))
        stop("kinship matrix kin missing!")
    if(missing(lambda))
        stop("lambda missing!")
    if(missing(timeInStrata))
        stop("timeInStrata missing!")
    if(length(unique(c(length(affected), nrow(kin), ncol(kin),
                       nrow(timeInStrata)))) != 1)
        stop("Length of affected has to match the number of rows and cols of",
             " kin and the number of rows of timeInStrata!")

    ## match names of lambda with colnames of timeInStrata:
    if(is.null(names(lambda)))
        stop("lambda has to be a named numeric vector with the names ",
             "corresponding to the colnames of timeInStrata!")
    if(is.null(colnames(timeInStrata)))
        stop("timeInStrata has to be a matrix with column names corresponding ",
             "to the names of argument lambda!")
    if(length(lambda) != ncol(timeInStrata))
        stop("length of lambda does not match number of columns of ",
             "timeInStrata!")
    if(!all(names(lambda) %in% colnames(timeInStrata)))
        stop("names of lambda do not match with colnames of timeInStrata!")
    lambda <- lambda[colnames(timeInStrata)]

    ## setting the diagonal of kin to 0 to avoid self-self kinship
    diag(kin) <- 0
    ## calculate nominator of Kerber 1995 formula (4): the observed cases
    ## this results in a vector length affected, with 0 if the individual
    ## is not related to any affected.
    nomi <- as.numeric(affected %*% kin)
    ## funny thing is the denominator that lists the expected cases.
    ## lambda should be a vector of length k (k=number of strata), timeInStrat
    ## a matrix with k columns, n rows with n being the number of individuals
    ## and kin a nxn matrix.
    ## lambda and timeInStrat should be in the same unit,
    ## i.e. either proportion, or proportion per person-year.
    denomi <- lambda %*% t(timeInStrata) %*% kin
    if(nrow(denomi) > 1)
        stop("Did not get the expected single-row matrix!")
    denomi <- as.numeric(denomi)
    res <- nomi/denomi
    names(res) <- names(affected)
    ##return(list(nomi=nomi, denomi=denomi))
    res
}


##****************************************************************************
##
##  runSimulation
##
setMethod("runSimulation",
          "FAStdIncidenceRateResults", function(object, nsim=50000,
                                                lambda=NULL, timeInStrata=NULL,
                                                strata=NULL, ...){
              if(length(trait(object)) == 0)
                  stop("No trait information available!")
              ## Check input parameter:
              if(is.null(timeInStrata))
                  stop("Required argument 'timeInStrata' missing!")
              if(is.null(lambda))
                  stop("Required argument 'lambda' missing!")
              if(is.null(ncol(timeInStrata))){
                  ## Special case: numeric vector, no strata.
                  timeInStrata <- matrix(timeInStrata, ncol=1)
                  colnames(timeInStrata) <- "A"
                  names(lambda)[1] <- "A"
              }
              ## Start defining input arguments for the downstream function
              affected <- trait(object)
              kin <- kinship(object)
              ## Check correct dimensions etc.
              if(nrow(timeInStrata) != length(affected))
                  stop("Number of rows of 'timeInStrata' (",
                       nrow(timeInStrata), ") has to match the number ",
                       "of individuals in the pedigree (",
                       length(affected), ")!")
              if(ncol(timeInStrata) != length(lambda))
                  stop("Number of columns of 'timeInStrata' (",
                       ncol(timeInStrata), ") has to match the length ",
                       "of 'lambda' (", length(lambda), ")!")
              if(any(colnames(timeInStrata) != names(lambda)))
                  stop("Column names of 'timeInStrata' have to match ",
                       "names of lambda!")
              if(!is.null(strata)){
                  if(length(strata) != length(affected))
                      stop("Length of 'strata' (", length(strata), ") ",
                           "has to match the number of individuals in ",
                           "the pedigree (", length(affected), ")!")
              }
              object@timeInStrata <- timeInStrata
              object@lambda <- lambda
              ## Remove entries with NAs
              ## trait
              trait <- trait(object)   # that way we ensure that we have the
              ## same ordering.
              kin <- kinship(object)
              ## Just to be on the save side... ensure that the ordering of
              ## id/Trait matches the kin
              kin <- kin[names(trait), names(trait)]
              diag(kin) <- 0
              ## Now start subsetting the data:
              message("Cleaning data set (got in total ", nrow(kin),
                      " individuals):")
              message(" * not phenotyped individuals...", appendLF=FALSE)
              ## * NA in trait
              nas <- is.na(trait)
              if(any(nas)){
                  trait <- trait[!nas]
                  kin <- kin[!nas, !nas]
                  timeInStrata <- timeInStrata[!nas, , drop=FALSE]
                  if(!is.null(strata))
                      strata <- strata[!nas]
                  message(" ", sum(nas), " removed.")
              }else{
                  message(" none present.")
              }
              ## * NA in timeInStrata.
              nas <- apply(timeInStrata, MARGIN=1, function(z){
                  any(is.na(z))
              })
              message(" * individuals with missing time in strata...",
                      appendLF=FALSE)
              if(any(nas)){
                  trait <- trait[!nas]
                  kin <- kin[!nas, !nas]
                  timeInStrata <- timeInStrata[!nas, , drop=FALSE]
                  if(!is.null(strata))
                      strata <- strata[!nas]
                  message(" ", sum(nas), " removed.")
              }else{
                  message(" none present.")
              }
              ## * NA in strata
              if(!is.null(strata)){
                  nas <- is.na(strata)
                  message(" * individuals without valid strata values...",
                          appendLF=FALSE)
                  if(any(nas)){
                      trait <- trait[!nas]
                      kin <- kin[!nas, !nas]
                      timeInStrata <- timeInStrata[!nas, , drop=FALSE]
                      strata <- strata[!nas]
                      message(" ", sum(nas), " removed.")
                  }else{
                      message(" none present.")
                  }
              }
              ## Anyway removing singletons here, since they result in NA values
              message(" * singletons (also caused by previous subsetting)...",
                      appendLF=FALSE)
              ## * Not related, i.e. individuals with a kinship sum of 0
              nas <- colSums(kin) == 0
              if(any(nas)){
                  trait <- trait[!nas]
                  kin <- kin[!nas, !nas]
                  timeInStrata <- timeInStrata[!nas, , drop=FALSE]
                  if(!is.null(strata))
                      strata <- strata[!nas]
                  message(" ", sum(nas), " removed.")
              }else{
                  message(" none present.")
              }
              message("Done")

              ## OK, now run the test...
              Sim <- doFsirSimulation(affected=trait, kin=kin, lambda=lambda,
                                      timeInStrata=timeInStrata,
                                      nsim=nsim, strata=strata, ...)
              fsirs <- Sim$fsir
              allFsirs <- rep(NA, length(object$id))
              names(allFsirs) <- object$id
              idx <- match(names(fsirs), object$id)
              allFsirs[idx] <- fsirs
              allPvals <- rep(NA, length(object$id))
              names(allPvals) <- object$id
              allPvals[idx] <- Sim$pvalue
              if(length(Sim$expDensity) > 0){
                  allExps <- vector("list", length(object$id))
                  names(allExps) <- object$id
                  allExps[idx] <- Sim$expDensity
                  Res <- list(fsir=allFsirs, pvalue=allPvals, expDensity=allExps)
              }else{
                  Res <- list(fsir=allFsirs, pvalue=allPvals)
              }
              object@sim <- Res
              object@nsim <- nsim
              return(object)
          })



##****************************************************************************
##
##  FSIR along with Monte Carlo simulations to
##  estimate significances.
##  We are NOT checking for NAs here, so that has to be done upstream!
##  Also, we assume that everything is in the right order.
##
##  lowMem: if TRUE a less memory demanding code is run; density information
##          is then however not available.
##****************************************************************************
doFsirSimulation <- function(affected, kin, lambda, timeInStrata,
                             nsim=50000, strata=NULL, lowMem=FALSE){
    if(missing(affected))
        stop("affected missing!")
    if(missing(kin))
        stop("kinship matrix kin missing!")
    if(missing(lambda))
        stop("lambda missing!")
    if(missing(timeInStrata))
        stop("timeInStrata missing!")
    if(length(unique(c(length(affected), nrow(kin), ncol(kin),
                       nrow(timeInStrata)))) != 1)
        stop("Length of affected has to match the number of rows ",
             "and cols of kin and the number of rows of timeInStrata!")
    if(!is.null(strata)){
        if(length(strata) != length(affected))
            stop("Length of arguments 'strata' and 'affected' have to match!")
    }
    ## match names of lambda with colnames of timeInStrata:
    if(is.null(names(lambda)))
        stop("lambda has to be a named numeric vector with the names",
             " corresponding to the colnames of timeInStrata!")
    if(is.null(colnames(timeInStrata)))
        stop("timeInStrata has to be a matrix with column names",
             " corresponding to the names of argument lambda!")
    if(length(lambda) != ncol(timeInStrata))
        stop("length of lambda does not match number of columns of ",
             "timeInStrata!")
    if(!all(names(lambda) %in% colnames(timeInStrata)))
        stop("names of lambda do not match with colnames of timeInStrata!")
    lambda <- lambda[colnames(timeInStrata)]

    ## setting the diagonal of kin to 0 to avoid self-self kinship
    diag(kin) <- 0
    ## calculate nominator of Kerber 1995 formula (4): the observed cases
    ## this results in a vector length affected, with 0 if the individual is not
    ## related to any affected.
    nomi <- as.numeric(affected %*% kin)
    ## funny thing is the denominator that lists the expected cases.
    ## lambda should be a vector of length k (k=number of strata), timeInStrat a
    ## matrix with k columns, n rows with n being the number of individuals and
    ## kin a nxn matrix. lambda and timeInStrat should be in the same unit,
    ## i.e. either proportion, or proportion per person-year.
    denomi <- lambda %*% t(timeInStrata) %*% kin
    if(nrow(denomi) > 1)
        stop("Did not get the expected single-row matrix!")
    denomi <- as.numeric(denomi)
    obsFsir <- nomi/denomi
    names(obsFsir) <- names(affected)
    ##
    ## Performing the simulation.
    sampleFrom <- 1:length(affected)
    nAff <- sum(affected)   ## affected has to be encoded 1,0 or TRUE,FALSE
    nAll <- length(affected)
    if(lowMem){
        largerEqual <- rep(0, nAll)
        if(is.null(strata)){
            for(i in 1:nsim){
                simIdx <- sample(sampleFrom, nAff)
                simaffs <- rep(0, nAll)
                simaffs[simIdx] <- 1
                tmp <- as.vector((simaffs %*% kin) / denomi)
                tmp[is.na(tmp)] <- 0
                largerEqual <- largerEqual + (tmp >= obsFsir)
            }
        }else{
            affStrataCounts <- table(strata[affected > 0])
            affStrataCounts <- affStrataCounts[affStrataCounts > 0]
            for(i in 1:nsim){
                simIdx <- stratsample(strata, counts=affStrataCounts)
                simaffs <- rep(0, nAll)
                simaffs[simIdx] <- 1
                tmp <- as.vector((simaffs %*% kin) / denomi)
                tmp[is.na(tmp)] <- 0
                largerEqual <- largerEqual + (tmp >= obsFsir)
            }
        }
        PVals <- largerEqual/nsim
        denses <- NULL
    }else{
        ## memory demanding and "slower" version
        if(is.null(strata)){
            SimFsir <- lapply(1:nsim, function(z){
                simIdx <- sample(sampleFrom, nAff)
                simaffs <- rep(0, nAll)
                simaffs[simIdx] <- 1
                return(as.vector((simaffs %*% kin) / denomi))
            })
        }else{
            affStrataCounts <- table(strata[affected > 0])
            affStrataCounts <- affStrataCounts[affStrataCounts > 0]
            SimFsir <- lapply(1:nsim, function(z){
                simIdx <- stratsample(strata, counts=affStrataCounts)
                simaffs <- rep(0, nAll)
                simaffs[simIdx] <- 1
                return(as.vector((simaffs %*% kin) / denomi))
            })
        }
        SimFsir <- do.call(cbind, SimFsir)
        PVals <- rowSums(SimFsir >= obsFsir)/nsim
        denses <- apply(SimFsir, MARGIN=1, density)
    }
    return(list(fsir=obsFsir, pvalue=PVals, expDensity=denses))
}

## convert a factor to a matrix, levels
factor2matrix <- function(x){
    if(class(x)!="factor")
        x <- factor(x)
    Ls <- lapply(levels(x), function(z){
        return(as.numeric(x == z))
    })
    Mat <- do.call(cbind, Ls)
    colnames(Mat) <- levels(x)
    if(!is.null(names(x)))
        rownames(Mat) <- names(x)
    return(Mat)
}

##****************************************************************************
##
##  show
##
setMethod("show", "FAStdIncidenceRateResults", function(object){
    callNextMethod()
    cat(paste0("Result info:\n"))
    if(length(object@sim) > 0){
        cat(paste0(" * Mean familial standardized incidence ratio: ",
                   format(mean(object@sim$fsir, na.rm=TRUE), digits=2), ".\n"))
        cat(paste0(" * lambda: \n"))
        Lam <- object@lambda
        for(i in 1:length(lambda)){
            cat(paste0("   - ", names(lambda)[i], ": ", lambda[i], "\n"))
        }
        cat(paste0())
    }else{
        cat(" * No simulation results available yet.\n")
    }
    cat(paste0(" * Number of simulations: ", object@nsim, ".\n"))
})

##****************************************************************************
##
##  plotRes
##
setMethod("plotRes", "FAStdIncidenceRateResults", function(object,
                                                           id=NULL,
                                                           family=NULL,
                                                           addLegend=TRUE,
                                                           type="density", ...){
    type <- match.arg(type, c("density", "hist"))
    if(type == "hist")
        stop("Type 'hist' is not supported (yet).")
    if(length(object@sim) == 0)
        stop("No analysis performed yet!")
    if(is.null(id))
        stop("Argument 'id' is required.")
    id <- as.character(id)
    if(!is.null(family))
        stop("Argument 'family' is not supported.")
    ## check if id is valid
    fsirs <- object$fsir
    if(!any(names(fsirs) == id))
        stop("Individual with id ", id, " not found ",
             "in the pedigree.")
    obsFsir <- fsirs[id]
    if(is.na(obsFsir))
        stop("No Familial Standardized Incidence Rate (FSIR) ",
             "calculated for individual ", id, ".")
    fam <- family(object, id=id)[1, "family"]
    if(type == "density"){
        ## Let's see whether we have the required information available.
        if(!any(names(object@sim) == "expDensity"))
            stop("Distribution of familial standardiced incidence rates ",
                 "from the simulation runs not available. You need to",
                 " run 'runSimulation' without optional argument",
                 " 'lowMem=TRUE'.")
        dens <- object@sim$expDensity[[id]]
        XL <- range(c(range(dens$x, na.rm=TRUE), obsFsir), na.rm=TRUE)
        plot(dens, main=paste0("Individual: ", id, ", family: ",
                               fam),
             xlab="Familial standardized incidence rate",
             type="h", lwd=3, col="lightgrey", xlim=XL)
        points(dens, col="grey", type="l", lwd=2)
    }
    Blue <- "#377EB8"
    abline(v=obsFsir, col=Blue)
    if(addLegend){
        legend("topright",
               legend=c(paste0("fsir     : ", format(obsFsir, digits=2)),
                        paste0("p-value : ", format(object@sim$pvalue[id],
                                                    digits=2))
                        ))
    }
})

##****************************************************************************
##
##  plotPed
##
##  the plot ped can be done for individuals and for families. In essence we
##  are just calling the plotPed from the FAData object, eventually highlighting
##  the individual and providing the FSIR as numeric values below the id.
setMethod("plotPed", "FAStdIncidenceRateResults", function(object, id=NULL,
                                                           family=NULL,
                                                           filename=NULL,
                                                           device="plot",
                                                           only.phenotyped=FALSE,
                                                           ...) {
    if(is.null(id) & is.null(family))
        stop("Either id or family has to be provided!")
    if(length(object@sim) == 0){
        ## Means we don't have any results...
        FSIR <- rep(NA, length(object$id))
    }else{
        FSIR <- object@sim$fsir
    }
    callNextMethod(object=object, id=id, family=family, filename=filename,
                   device=device, label1=FSIR, proband.id=id,
                   only.phenotyped=only.phenotyped, ...)
})

##****************************************************************************
##
##  result
##
setMethod("result", "FAStdIncidenceRateResults", function(object, method="BH"){
    method <- match.arg(method, p.adjust.methods)
    Trait <- trait(object, na.rm=TRUE)
    TraitName <- object@traitname
    if(length(TraitName) == 0)
        TraitName <- NA
    if(length(object@sim) == 0){
        MyRes <- data.frame(trait_name=TraitName,
                            total_phenotyped=length(Trait),
                            total_affected=sum(Trait!=0),
                            total_tested=0,
                            id=NA,
                            family=NA,
                            fsir=NA,
                            pvalue=NA,
                            padj=NA,
                            check.names=FALSE, stringsAsFactors=FALSE)
        warning("No simulation data available! Please run a simulation ",
                "using the 'runSimulation' method on this, or the ",
                "'fsirTest' on a 'FAData' object.")
        return(MyRes)
    }
    nind <- length(object$id)
    MyRes <- data.frame(trait_name=rep(TraitName, nind),
                        total_phenotyped=rep(length(Trait), nind),
                        total_affected=rep(sum(Trait!=0), nind),
                        total_tested=rep(sum(!is.na(object@sim$fsir)), nind),
                        id=as.character(object$id),
                        family=as.character(object$family),
                        fsir=object@sim$fsir,
                        pvalue=object@sim$pvalue,
                        padj=p.adjust(object@sim$pvalue, method=method),
                        check.names=FALSE, stringsAsFactors=FALSE)
    MyRes <- MyRes[order(MyRes$pvalue, -MyRes$fsir), ]
    rownames(MyRes) <- as.character(MyRes$id)
    return(MyRes)
})

##****************************************************************************
##
##  fsir getter method.
##
setMethod("fsir", "FAStdIncidenceRateResults",
          function(object, ...){
              if(!length(object@sim))
                  return(NULL)
              return(object@sim$fsir)
          })

##****************************************************************************
##
##  lambda getter method.
##
setMethod("lambda", "FAStdIncidenceRateResults",
          function(object, ...){
              return(object@lambda)
          })

##****************************************************************************
##
##  $ getter method.
##
setMethod("$", "FAStdIncidenceRateResults", function(x, name){
    if(name == "fsir" | name == "lambda" | name == "pvalue"
       | name == "timeInStrata"){
        ## extract local stuff
        if(length(x@sim) == 0){
            warning("No calculation result available!")
            return(NULL)
        }
        if(name == "fsir")
            return(fsir(x))
        if(name == "lambda")
            return(lambda(x))
        if(name == "pvalue")
            return(x@sim$pvalue)
        if(name == "timeInStrata")
            return(timeInStrata(x))
    }else{
        callNextMethod()
    }
})


##****************************************************************************
##
##  [ method.
##
setMethod("[", "FAStdIncidenceRateResults", function(x, i, j, ..., drop){
    stop("Subsetting of a FAStdIncidenceRateResults object is not supported!")
})

##****************************************************************************
##
##  timeInStrata getter/setter method.
##
setMethod("timeInStrata", "FAStdIncidenceRateResults",
          function(object){
              ## we're also checking that timeAtRisk, stored in its internal
              ## slot has the correct length!
              if(nrow(object@timeInStrata)==0)
                  return(object@timeInStrata)
              if(nrow(object@timeInStrata) != length(object$id))
                  stop("Length of 'timeInStrata' does not match the ",
                       "number of individuals in ",
                       "the pedigree!")
              return(object@timeInStrata)
          })

##****************************************************************************
##
##  resultForId.
##
##  extract data for an individual.
setMethod("resultForId", "FAStdIncidenceRateResults", function(object, id=NULL){
    if(is.null(id))
        stop("'id' has to be provided!")
    ## check if id is there.
    if(!any(object$id == id))
        stop("Individual with id ", id, " not in the pedigree.")
    ## check if we have any results:
    if(length(object@sim) == 0)
        stop("No simulation results available. Please run 'runSimulation' first.")
    idx <- which(object$id == id)
    return(list(id=id, fsir=object$fsir[id], pvalue=object@sim$pvalue[idx],
                timeInStrata=object@timeInStrata[idx, ],
                lambda=object@lambda))
})

##****************************************************************************
##
##  trait<-
##
##  Calling the trait replecement method from FAResult and in addition
##  reset the simulation result.
setReplaceMethod("trait", "FAStdIncidenceRateResults", function(object, value){
    object <- callNextMethod()
    ## reset the result
    object@sim <- list()
    object@nsim <- 0
    object@traitname <- character()
    return(object)
})

## Testing some stuff related to the familial standardized incidence ratio
## (FSIR) from Kerber (1995).
data(minnbreast)
##mbsub <- minnbreast
mbsub <- minnbreast[minnbreast$famid %in% c(4:19, 411, 432), ]
mbped <- mbsub[, c("famid", "id", "fatherid", "motherid", "sex")]
colnames(mbped) <- FamAgg:::.PEDCN
fad <- FAData(pedigree=mbped)
tcancer <- mbsub$cancer
names(tcancer) <- mbsub$id
trait(fad) <- tcancer

test_fsir <- function(){
    do.plot <- FALSE
    ## eventually remove that...
    affected <- fad$affected
    ped <- pedigree(fad)
    kin <- kinship(fad)
    nas <- is.na(affected)
    affected <- affected[!nas]
    ped <- ped[!nas,]
    kin <- kin[!nas, !nas]
    nas <- is.na(ped$sex)
    affected <- affected[!nas]
    ped <- ped[!nas,]
    kin <- kin[!nas, !nas]
    ## get the endage for the individuals:
    endages <- minnbreast$endage
    names(endages) <- as.character(minnbreast$id)
    endages <- endages[as.character(ped$id)]
    ## Remove all those that have an NA endage
    nas <- is.na(endages)
    endages <- endages[!nas]
    kin <- kin[!nas, !nas]
    ped <- ped[!nas, ]
    affected <- affected[!nas]
    stratMat <- FamAgg:::factor2matrix(ped$sex)
    stratMatAge <- stratMat * endages
    ## lambda would be the incidence ratio in the population.
    ## using incidence ratio from http://www.cancerresearchuk.org/health-professional/cancer-statistics/statistics-by-cancer-type/breast-cancer/incidence-invasive#heading-Zero
    ## these are the incidence rates in UK!
    FemaleBreastC <- 155.3/100000
    MaleBreastC <- 1.1/100000
    ## and for prostate cancer in UK
    ## http://www.cancerresearchuk.org/health-professional/cancer-statistics/statistics-by-cancer-type/prostate-cancer/incidence#heading-Zero
    FemaleProstateC <- 0
    MaleProstateC <- 134.3/100000
    ## the same number for the US:
    ## http://seer.cancer.gov/statfacts/html/prost.html
    ## these are new cases per year and 100000 males.
    ## FemaleProstateCUS <- 0
    ## MaleProstateCUS <- 137.9/100000

    ## Consider:
    ## shouldn't be timeInStrata the actual time at risk for each individual? This
    ## might then be interpreted for the actual example that the strata is not just
    ## 1 and 0 for male/female, but these numbers multiplied with the endage for each
    ## participant. That way, e.g. male 1 is endage years at risk.

    lambda <- c(M=(1.1+134.4)/100000, F=155.3/100000)
    ## what would be the rates estimated from the pedigree data set?
    Counts <- table(ped$sex[ped$affected > 0])
    ## relate that to the number of M/F.
    lambdaInternal <- Counts / as.numeric(table(ped$sex))

    ## That would be the FSIR with strata being male/female
    FSIRs <- FamAgg:::doFsir(affected=affected, kin=kin,
                           lambda=lambda, timeInStrata=stratMat)
    ## the same with strata being the actual time at risk in the sex strata:
    FSIRsAge <- FamAgg:::doFsir(affected=affected, kin=kin, lambda=lambda,
                              timeInStrata=stratMatAge)
    if(do.plot){
        par(mfrow=c(1, 2))
        plot(density(FSIRs, na.rm=TRUE))
        abline(v=1, col="grey")
        plot(density(FSIRsAge, na.rm=TRUE))
        abline(v=1, col="grey")
    }
    ## we get unexpectedly large FSIRs if we don't consider the age/time at risk.
    ## Thus we have to consider also the time at risk in each stratum.
    sum(FSIRs < 20 & FSIRs > 0, na.rm=TRUE)
    ## looks better for those that are
    sum(FSIRsAge > 1, na.rm=TRUE)
    sum(FSIRsAge > 2, na.rm=TRUE)

    if(do.plot){
        ## check: boxplot for FSIRs of affected and of unaffected.
        boxplot(split(FSIRsAge, f=affected), ylab="FSIR", xlab="affected")
    }

    ## Now doing it with the method.
    ## First constract time at risk strata matrix
    stratMat <- FamAgg:::factor2matrix(fad$sex)
    stratMat <- stratMat * mbsub$endage
    allFsirs <- FamAgg:::fsir(fad, trait=trait(fad), lambda=lambda,
                              timeInStrata=stratMat)
    ## compare the FSIRs with the allFsirs:
    checkEquals(FSIRsAge, allFsirs[names(FSIRsAge)])

    ##*****************************************************************
    ##
    ## Test the simulation thing.
    set.seed(18011977)
    fsirRes <- fsirTest(fad, trait=trait(fad), lambda=lambda, timeInStrata=stratMat,
                        nsim=400)
    checkEquals(FSIRsAge, fsirRes@sim$fsir[names(FSIRsAge)])
    checkEquals(FSIRsAge, fsirRes$fsir[names(FSIRsAge)])

    ##
    head(result(fsirRes))
    length(which(result(fsirRes)$pvalue < 0.05))
    fam <- result(fsirRes)[1, "family"]
    if(do.plot){
        plotPed(fsirRes, family=fam)
        plotRes(fsirRes, id=result(fsirRes)[1, "id"])
        plotPed(fsirRes, family="411")
        plotRes(fsirRes, id="16442")
        ## interestingly, id 16442 has the largest FSIR value in that pedigree,
        ## but only a poor p-value. Most likely because not much is known for this
        ## person (i.e. only one phenotyped daughter).
    }

    ## simulation with option lowMem
    set.seed(18011977)
    fsirResLM <- fsirTest(fad, trait=trait(fad), lambda=lambda, timeInStrata=stratMat,
                          nsim=400, lowMem=TRUE)
    checkEquals(fsirRes@sim$fsir, fsirResLM@sim$fsir)
    checkEquals(fsirRes@sim$pvalue, fsirResLM@sim$pvalue)

    ## checking getters.
    ## lambda
    checkEquals(fsirRes$lambda, fsirRes@lambda)
    checkEquals(fsirRes$lambda, lambda(fsirRes))
    ## timeInStrata
    checkEquals(fsirRes@timeInStrata, timeInStrata(fsirRes))
    ## fsir
    checkEquals(fsirRes@sim$fsir, fsir(fsirRes))
    checkEquals(fsir(fsirRes), fsirRes$fsir)
    ## OK, done.

    ## get the data for id 16442
    resultForId(fsirRes, id="16442")

    ## ## That below goes eventually into the vignette!
    ## ## Check the family for some of the guys with the highest risk...
    ## id <- names(sort(allFsirs, decreasing=TRUE))[1]
    ## fam <- family(fad, id=id)
    ## if(do.plot){
    ##     plotPed(fad, family=fam[1, "family"], label1=allFsirs[as.character(fam$id)])
    ## }
    ## ## OK, so male individuals have a higher FSIR, and parents of a single affected
    ## ## child that are not in kinship with other individuals in the family.

    ## id <- names(sort(allFsirs, decreasing=TRUE))[3]
    ## fam <- family(fad, id=id)
    ## if(do.plot){
    ##     plotPed(fad, family=fam[1, "family"], label1=allFsirs[as.character(fam$id)], cex=0.5)
    ## }

    ## ## Check family 432
    ## fam <- family(fad, family="432")
    ## if(do.plot){
    ##     plotPed(fad, family=fam[1, "family"], label1=allFsirs[as.character(fam$id)], cex=0.5,
    ##             only.phenotyped=TRUE)
    ## }

    ## ## Aggregate the FSIR per family.
    ## fsirPerFamMat <- aggregate(allFsirs, by=list(fad$family), mean, na.rm=TRUE)
    ## fsirPerFam <- fsirPerFamMat[, "x"]
    ## names(fsirPerFam) <- fsirPerFamMat[, 1]

    ## head(sort(fsirPerFam, decreasing=TRUE))
    ## if(do.plot){
    ##     plot(density(fsirPerFam), main="mean FSIR per family", xlab="mean FSIR")
    ##     abline(v=1, col="grey")
    ## }

    ## ## plot for some of the top families.

    ## ## Compare FSIR to FR


    ## ## binning by time interval.
    ## endages <- mbsub$endage
    ## ## stratify in < 40 > 40
}

test_fsir_sim <- function(){
    stratMat <- FamAgg:::factor2matrix(fad$sex)
    stratMat <- stratMat * mbsub$endage
    lambda <- c(M=(1.1+134.4)/100000, F=155.3/100000)
    set.seed(18011977)
    fsirRes <- fsirTest(fad, trait=trait(fad), lambda=lambda, timeInStrata=stratMat,
                        nsim=400, lowMem=FALSE)
    set.seed(18011977)
    fsirRes2 <- fsirTest(fad, trait=trait(fad), lambda=lambda, timeInStrata=stratMat,
                        nsim=400)
    set.seed(18011977)
    fsirRes2 <- fsirTest(fad, trait=trait(fad), lambda=lambda, timeInStrata=stratMat,
                        nsim=400, lowMem=TRUE)
    checkEquals(fsirRes$fsir, fsirRes2$fsir)
}

## calculate person-time at risk:
test_slice_age <- function(){
    ## given: ages, slice that into time span.
    Ages <- c(13, 65, 45, 35, 19, 34, 49, 45, 40, 39, 17)
    AgeTable <- sliceAge(Ages)
    checkEquals(max(AgeTable[, 1]), 40)
}

## functions to retrieve matched controls from pedigrees for a list of input
## individuals

##***************************************************************************
##
##   Functions to get ids from the pedigree that could be used as controls
##   for the given list of ids.
##
##   Input: ped, id and any number of optional parameters as input
##   Return:
##
##***************************************************************************

## get all ids from the pedigree matching the generation from the input id(s)
## ped is assumed to have the standard format.
## if id is "spread" across several families, ids are returned for each
## family separately.
## include.ancs and include.off allow to increase the range of generations from
## which individuals are returned (e.g. include.ancs=1 all ids form the ids'
## generation plus one generation of ancestors is returned).
## If a list of pre-calculated generations is submitted the function will run
## (obviously) faster.
##### NOTE:
## If we want to consider also the frequency of a generation being
## picked/present in id: get generationMatched for each id separately,
## concatenate the ids for all runs, make a table (-> get the frequency) and
## use that as probability for the sampling.
doGetGenerationMatched <- function(ped, id=NULL, include.anc=0,
                                   include.off=0, ...){
    if(is.null(ped))
        stop("No pedigree submitted!")
    ped <- checkPedCol(ped)
    if(is.null(id))
        stop("No id has been submitted!")
    id <- as.character(id)
    id <- id[id %in% ped$id]
    if(length(id) == 0)
        stop("None of the ids present in the pedigree!")
    names(id) <- ped[id, "family"]
    ids <- split(id, names(id))
    lapply(ids, function(z){
        gens <- doEstimateGenerationsFor2(ped,
                                          family=names(z)[1])[[1]]
        gens <- gens[!is.na(gens)]
        gensOfIds <- gens[z]
        gensOfIds <- unique(c(gensOfIds, gensOfIds - include.anc,
                              gensOfIds + include.off))
        return(names(gens)[gens %in% gensOfIds])
    })
}

##
doGetGenerationSexMatched <- function(ped, id=NULL, include.anc=0,
                                      include.off=0, ...){
    if(is.null(ped))
        stop("No pedigree submitted!")
    ped <- checkPedCol(ped)
    if(is.null(id))
        stop("No id has been submitted!")
    id <- as.character(id)
    id <- id[id %in% ped$id]
    if(length(id) == 0)
        stop("None of the ids present in the pedigree!")
    names(id) <- ped[id, "family"]
    ids <- split(id, names(id))
    lapply(ids, function(z){
        gens <- doEstimateGenerationsFor2(ped,
                                          family=names(z)[1])[[1]]
        gens <- gens[!is.na(gens)]
        gensOfIds <- gens[z]
        gensOfIds <- unique(c(gensOfIds, gensOfIds - include.anc,
                              gensOfIds + include.off))
        ctrls <- names(gens)[gens %in% gensOfIds]
        ## further subset to sex:
        subPed <- ped[ped$family == names(z)[1], , drop=FALSE]
        rownames(subPed) <- as.character(subPed$id)
        haveSex <- unique(subPed[as.character(subPed$id) %in% z, "sex"])
        haveSex <- haveSex[!is.na(haveSex)]
        if(length(haveSex) == 0)
            return(NA)
        ctrlsSex <- subPed[ctrls, "sex"]
        ctrls <- ctrls[which(ctrlsSex %in% haveSex)]
        return(ctrls)
    })
}


## just get all individuals of a family.
doGetAll <- function(ped, id=NULL, ...){
    if(is.null(ped))
        stop("No pedigree submitted!")
    ped <- checkPedCol(ped)
    if(is.null(id))
        stop("id has to be provided!")
    id <- as.character(id)
    id <- id[id %in% ped$id]
    names(id) <- ped[id, "family"]
    ids <- split(id, names(id))
    lapply(ids, function(z){
        return(as.character(ped[as.character(ped$family) == names(z)[1], "id"]))
    })
}

## just get all individuals of a family matching by sex
doGetSexMatched <- function(ped, id=NULL, ...){
    if(is.null(ped))
        stop("No pedigree submitted!")
    ped <- checkPedCol(ped)
    if(is.null(id))
        stop("id has to be provided!")
    id <- as.character(id)
    id <- id[id %in% ped$id]
    names(id) <- ped[id, "family"]
    ids <- split(id, names(id))
    lapply(ids, function(z){
        ## further subset to sex:
        subPed <- ped[ped$family == names(z)[1], , drop=FALSE]
        rownames(subPed) <- as.character(subPed$id)
        haveSex <- unique(subPed[as.character(subPed$id) %in% z, "sex"])
        haveSex <- haveSex[!is.na(haveSex)]
        if(length(haveSex) == 0)
            return(NA)
        ctrlsSex <- subPed[, "sex"]
        ctrls <- subPed[which(ctrlsSex %in% haveSex), "id"]
        return(as.character(ctrls))
    })
}

doGetExternalMatched <- function(ped, id=NULL, match.using, ...){
    ## first check match.using
    if(missing(match.using))
        stop("Function getExternalMatched requires argument 'match.using'!")
    if(!is.null(dim(match.using)))
        stop("'match.using' should be a one-dimensional vector!")
    if(is.null(names(match.using)))
        stop("'match.using' has to be a named vector with the names ",
             "corresponding to the ids of the individuals in the pedigree!")
    ## check the rest...
    if(is.null(ped))
        stop("No pedigree submitted!")
    ped <- checkPedCol(ped)
    if(is.null(id))
        stop("id has to be provided!")
    id <- as.character(id)
    id <- id[id %in% ped$id]
    names(id) <- ped[id, "family"]
    ids <- split(id, names(id))
    lapply(ids, function(z){
        infam <- as.character(ped[ped$family == names(z)[1], "id"])
        if(!all(infam %in% names(match.using)))
            stop("Could not find some of the ids in 'names(match.using)'!")
        ## Now get the value in match.using for the ids, i.e. z
        needVal <- unique(match.using[z])
        needVal <- needVal[!is.na(needVal)]
        if(length(needVal) == 0)
            return(NA)
        ctrlsVal <- match.using[infam]
        ctrls <- infam[which(ctrlsVal %in% needVal)]
        return(as.character(ctrls))
    })
}

### setting options and stuff...
.setOptions <- function(){
    FA <- list()
    FA$haplopainter <- system.file("perl/HaploPainter1.043.pl",
                                   package="FamAgg")
    options("FamAgg"=FA)
}


.checkHaplopaintRequirements <- function(messagefun=message){
    pedfile <- system.file("txt/minnbreastsub.txt", package="FamAgg")
    mbsub <- read.table(pedfile, sep="\t", header=TRUE)
    PedDf <- mbsub[, c("famid", "id", "fatherid", "motherid", "sex")]
    colnames(PedDf) <- .PEDCN
    PedDf$sex <- sanitizeSex(PedDf$sex)
    suppressMessages(
        fad <- FAData(PedDf)
    )
    ped <- buildPed(fad, id=6)
    pedDf <- buildHaplopaintDataframe(individual=ped$id, father=ped$father,
                                      mother=ped$mother, gender=ped$sex)
    tmpf <- tempfile()
    outfile <- paste0(tmpf, ".pdf")
    write.table(pedDf, tmpf, row.names=FALSE, col.names=FALSE,
                sep="\t", quote=FALSE)
    plotcall <- paste0("perl ", options()$FamAgg$haplopaint, " -b -pedfile ",
                       tmpf, " -pedformat csv -outfile ", outfile,
                       " -family 1 -bgcolor \\#ffffff -outformat pdf")
    res <- tryCatch(system(plotcall), error=function(e){
    })
    if(res == 0){
        messagefun("OK")
        return("haplopaint")
    }else{
        messagefun(paste0("FAIL\nHaplopainter not working! For requirements ",
                          "and installation see the package vignette.\nWill ",
                          "use internal plotting functions instead."))
        return("ks2paint")
    }
}

.onLoad <- function(libname, pkgname){
    .setOptions()
}

.onAttach <- function(libname, pkgname){
    .setOptions()
    FA <- getOption("FamAgg")
    FA$plotfun <- "ks2paint"
    options("FamAgg"=FA)
}

## maximal supported pedigree size for the gap package:
GAP_MAX_CLIQUE_SIZE <- 22

## here we define the pedigree plotting functions.
switchPlotfun <- function(method, check=TRUE){
    if(!missing(method)){
        method <- match.arg(method, c("ks2paint", "haplopaint"))
    }else{
        if(options()$FamAgg$plotfun=="ks2paint"){
            method <- "haplopaint"
        }else if(options()$FamAgg$plotfun=="haplopaint"){
            method <- "ks2paint"
        }
    }
    if(method=="haplopaint" && check){
        ## check if that is possible at all.
        method <- .checkHaplopaintRequirements()
    }
    FA <- getOption("FamAgg")
    FA$plotfun <- method
    message(paste0("Plotting backend: ", method, "."))
    options("FamAgg"=FA)
}

##########################
## the main plotting function that dispatches on options()$FamAgg$plotfun
## this function takes a lot of input...
doPlotPed <- function(family=NULL, individual=NULL, father=NULL, mother=NULL,
                      gender=NULL, affected=NULL, is.deceased=NULL,
                      is.sab.or.top=NULL, is.proband=NULL, is.adopted=NULL,
                      are.twins=NULL, are.consanguineous=NULL,
                      text.inside.symbol=NULL, text.beside.symbol=NULL,
                      text1.below.symbol=NULL, text2.below.symbol=NULL,
                      text3.below.symbol=NULL, text4.below.symbol=NULL,
                      filename=NULL, device="plot", res=600, ...){
    plotfun <- options()$FamAgg$plotfun
    plotfun <- match.arg(plotfun, c("ks2paint", "haplopaint"))
    if(plotfun == "haplopaint"){
        if(device == "plot"){
            device <- "pdf"
            warning("haplopaint does not support device='plot', changing ",
                    "to 'pdf'.")
        }
    }
    ## build the argument list.
    argList <- list(family=family, individual=individual, father=father,
                    mother=mother, gender=gender, affected=affected,
                    is.deceased=is.deceased, is.sab.or.top=is.sab.or.top,
                    is.proband=is.proband, is.adopted=is.adopted,
                    are.twins=are.twins,
                    are.consanguineous=are.consanguineous,
                    text.inside.symbol=text.inside.symbol,
                    text.beside.symbol=text.beside.symbol,
                    text1.below.symbol=text1.below.symbol,
                    text2.below.symbol=text2.below.symbol,
                    text3.below.symbol=text3.below.symbol,
                    text4.below.symbol=text4.below.symbol,
                    filename=filename, device=device,
                    res=res)
    addArgs <- list(...)
    ## kick out arguments we know might be passed, but are not needed
    ## further down...
    if(length(addArgs) > 0){
        addArgs <- addArgs[!(names(addArgs) %in%
                             c("prune", "max.generations.up",
                               "max.generations.down"))]
    }
    argList <- c(argList, addArgs)
    ## call the function.
    invisible(do.call(plotfun, args=argList))
}

##########################
## ks2paint uses kinship2 to plot pedigrees.
## text2.below.symbol will be plotted on the top left corner.
## text3.below.symbol will be plotted on the top right corner.
## text1.below.symbol is expected to be the age... add that below the ID.
## is.proband: highlight ids in red.
ks2paint <- function(family=NULL, individual=NULL, father=NULL, mother=NULL,
                     gender=NULL, affected=NULL, is.deceased=NULL,
                     is.sab.or.top=NULL, is.proband=NULL, is.adopted=NULL,
                     are.twins=NULL, are.consanguineous=NULL,
                     text.inside.symbol=NULL, text.beside.symbol=NULL,
                     text1.below.symbol=NULL, text2.below.symbol=NULL,
                     text3.below.symbol=NULL, text4.below.symbol=NULL,
                     filename=NULL, device="pdf", res=600, ...){
    device <- match.arg(device, c("pdf", "png", "plot"))
    ## device plot means we're just plotting, not generating a file.
    ## affected can be a vector or matrix with up to 4 columns.
    ## status (optional): 0 alive, missing, 1 dead.
    if(!is.null(is.adopted))
        warning("argument is.adopted not yet supported for kinship2 plotting.")
    if(!is.null(are.twins))
        warning("argument are.twins not yet supported for kinship2 plotting.")
    if(!is.null(are.consanguineous))
        warning("argument are.consanguineous not yet supported for kinship2 ",
                "plotting.")
    if(!is.null(is.sab.or.top))
        warning("argument is.sab.or.top not yet supported for kinship2 ",
                "plotting.")
    ## check the affected.
    if(!is.null(affected)){
        affected <- checkTransformNumeric(affected, name="affected")
        if(all(is.na(affected)))
            affected <- rep(0, length(individual))
    }else{
        affected <- rep(0, length(individual))
    }
    ## just creating the data.frame (although not using that one) to check all arguments.
    df <- buildHaplopaintDataframe(family=family, individual=individual,
                                   father=father, mother=mother, gender=gender,
                                   affected=affected, is.deceased=is.deceased,
                                   is.sab.or.top=is.sab.or.top,
                                   is.proband=is.proband, is.adopted=is.adopted,
                                   are.twins=are.twins,
                                   are.consanguineous=are.consanguineous,
                                   text.inside.symbol=text.inside.symbol,
                                   text.beside.symbol=text.beside.symbol,
                                   text1.below.symbol=text1.below.symbol,
                                   text2.below.symbol=text2.below.symbol,
                                   text3.below.symbol=text3.below.symbol,
                                   text4.below.symbol=text4.below.symbol
                                   )
    if(!is.null(is.deceased)){
        is.deceased <- checkTransformNumeric(is.deceased, name="is.deceased")
        ## should only have 0 and 1!
        is.deceased[is.na(is.deceased)] <- 0
        if(any(!(is.deceased) %in% c(0, 1)))
            stop("Only TRUE/FALSE or 1/0 allowed for is.deceased!")
    }else{
        is.deceased <- rep(0, length(individual))
    }
    ## OK, now trying to avoid a stupid bug in kinship2:
    if(any(is.na(affected)) & length(unique(affected)) == 2){
        ## append a dummy individual...
        affected <- c(affected, 0, 1)
        individual <- c(individual, "removeme1", "removeme2")
        father <- c(father, NA, NA)
        mother <- c(mother, NA, NA)
        gender <- c(gender, NA, NA)
        is.deceased <- c(is.deceased, 0, 0)
        ## have to add 2 elements to each other vector too.
        if(!is.null(text1.below.symbol))
            text1.below.symbol <- c(text1.below.symbol, NA, NA)
        if(!is.null(text2.below.symbol))
            text2.below.symbol <- c(text2.below.symbol, NA, NA)
        if(!is.null(text3.below.symbol))
            text3.below.symbol <- c(text3.below.symbol, NA, NA)
    }
    ## if we didn't get any error thus far, all should be right.
    ## generate a pedigree and plot, that's all.
    ped <- kinship2::pedigree(id=individual, dadid=father, momid=mother,
                              sex=sanitizeSex(gender))
    ## OK, let's start.
    FN <- filename
    if(device=="pdf"){
        if(is.null(FN))
            FN <- paste0(tempfile(), ".pdf")
        pdf(FN)
    }
    if(device=="png"){
        if(is.null(FN))
            FN <- paste0(tempfile(), ".png")
        png(FN)
    }
    ## checking additional input arguments.
    Args <- list(...)
    if(any(names(Args)=="cex")){
        Cex <- Args$cex
    }else{
        Cex <- 1
    }
    if(any(names(Args)=="symbolsize")){
        symbolsize <- Args$symbolsize
    }else{
        symbolsize <- 1
    }
    Coords <- plot(ped, status=is.deceased, affected=affected,
                   keep.par=TRUE, ...)
    oldxpd <- par()$xpd
    par(xpd=NA)
    ## if we do have any highlight ids or similar, i.e. text1.below.symbol etc.
    ## allow two: top left of symbol and top right of symbol.
    ## use the Coords$x, Coords$y, +/- Coords$boxw and pos=2, pos=4
    ## age below:
    ## use strheight to determine the size of the string.
    ## pos: y: Coords$y-Coores$boxh-strheight
    if(!is.null(text1.below.symbol)){
        strh <- strheight("1g", cex=Cex)
        ## Text below age line. The use of the adj parameter is similar to the
        ## kinship2 source code (plot.pedigree.R v1.8.4, line 233).
        text(x=Coords$x, y=(Coords$y+Coords$boxh-2*strh),
             labels=text1.below.symbol, cex=0.8*Cex, adj=c(0.5,1))
    }
    if(!is.null(text2.below.symbol)){
        ## top left
        text(x=(Coords$x-Coords$boxw/8), y=Coords$y, pos=2,
             labels=text2.below.symbol, cex=0.8*Cex)
    }
    if(!is.null(text3.below.symbol)){
        ## top right
        text(x=(Coords$x+Coords$boxw/8), y=Coords$y, pos=4,
             labels=text3.below.symbol, cex=0.8*Cex)
    }
    if(!is.null(text4.below.symbol))
        warning("ks2paint does not support text4.below.symbol.")
    ## eventually, it might be nicer to highlight the id of probands in a
    ## different color... that's however not that easy as we see below: we do
    ## have to add the text for highlighted ids later to that plot to overplot
    ## the ids plotted in black
    if(!is.null(is.proband)){
        is.proband <- checkArgNumericLogical(is.proband, name="is.proband")
        Colors <- rep(NA, length(is.proband))
        Colors[is.proband==1] <- "#E41A1C"    ## RColorBrewer, Set1, (red)
        ## from kinship2: pretty tedious to get all the settings...
        psize <- par('pin')  # plot region in inches
        stemp1 <- strwidth("ABC", units='inches', cex=Cex)* 2.5/3
        stemp2 <- strheight('1g', units='inches', cex=Cex)
        stemp3 <- max(strheight(ped$id, units='inches', cex=Cex))
        xrange <- range(Coords$plist$pos[Coords$plist$nid >0])
        maxlev <- nrow(Coords$plist$pos)
        ht1 <- psize[2]/maxlev - (stemp3 + 1.5*stemp2)
        ht2 <- psize[2]/(maxlev + (maxlev-1)/2)
        wd2 <- .8*psize[1]/(.8 + diff(xrange))
        boxsize <- symbolsize* min(ht1, ht2, stemp1, wd2) # box size in inches
        vscale <- (psize[2]-(stemp3 + stemp2/2 + boxsize))/ max(1, maxlev-1)
        labh  <- stemp2/vscale   # height of a text string
        boxh <- Coords$boxh
        for(i in 1:maxlev){
            for(j in 1:Coords$plist$n[i]){
                k <- Coords$plist$nid[i,j]
                text(Coords$plist$pos[i,j], i + boxh + labh*.7, ped$id[k],
                     cex=Cex, adj=c(.5,1), col=Colors[k])
            }
        }
    }
    par(xpd=oldxpd)
    if(device!="plot")
        dev.off()
    ##return(Coords)
    invisible(FN)
}


#########################
## haplopaint uses the HaploPainter perl script/function.
haplopaint <- function(family=NULL, individual=NULL, father=NULL, mother=NULL,
                       gender=NULL, affected=NULL, is.deceased=NULL,
                       is.sab.or.top=NULL, is.proband=NULL, is.adopted=NULL,
                       are.twins=NULL, are.consanguineous=NULL,
                       text.inside.symbol=NULL, text.beside.symbol=NULL,
                       text1.below.symbol=NULL, text2.below.symbol=NULL,
                       text3.below.symbol=NULL, text4.below.symbol=NULL,
                       filename=NULL, device="pdf", res=600, ...){
    device <- match.arg(device, c("ps", "pdf", "svg", "png", "txt"))
    ## first check input arguments.
    df <- buildHaplopaintDataframe(family=family, individual=individual,
                                   father=father, mother=mother, gender=gender,
                                   affected=affected, is.deceased=is.deceased,
                                   is.sab.or.top=is.sab.or.top,
                                   is.proband=is.proband, is.adopted=is.adopted,
                                   are.twins=are.twins,
                                   are.consanguineous=are.consanguineous,
                                   text.inside.symbol=text.inside.symbol,
                                   text.beside.symbol=text.beside.symbol,
                                   text1.below.symbol=text1.below.symbol,
                                   text2.below.symbol=text2.below.symbol,
                                   text3.below.symbol=text3.below.symbol,
                                   text4.below.symbol=text4.below.symbol
                                   )
    rownames(df) <- df[, "individual"]
    ## handle childless founders:
    kins <- kinship(family=df[, "family"], id=df[, "individual"],
                    dadid=df[, "father"], momid=df[, "mother"])
    cols <- colSums(kins)
    if(any(cols <= 0.5)){
        kins <- kins[cols > 0.5, cols > 0.5]
        df <- df[colnames(kins), ]
        warning("Removed ", sum(cols <= 0.5), " childless founders!")
    }
    ## Attention: if family parameter has not been provided, this is set to 1 in
    ## buildHaplopaintDataframe. This does not happen at the moment, as it is
    ## called by plotPed, where that parameter is set.
    famid <- df[1, "family"]
    if(is.null(filename))
        filename <- paste0(tempfile(), ".", device)
    if (device == "txt") {
        dfFile <- filename
        ## Write a header info
        write(paste0("#FAMID\tINDIVID\tDADID\tMOMID\tGENDER\tAFF\tDEAD\t",
                     "SABTOP\tPROBAND\tADOPT\tTWINS\tCONSAN\tTINSYM\t",
                     "TBESSYM\tT1UNDR\tT2UNDR\tT3UNDR\tT4UNDR"),
              file = filename)
    } else {
        dfFile <- tempfile()
    }
    write.table(df, file = dfFile, sep = "\t", row.names = FALSE,
                col.names = FALSE, quote = FALSE, append = (device == "txt"))
    if (device != "txt") {
        ## call haplopaint
        plotcall <- paste0("perl ", options()$FamAgg$haplopaint,
                           " -b -pedfile ", dfFile,
                           " -pedformat csv -outfile ", filename,
                           " -bgcolor \\#ffffff -outformat ", device,
                           " -resolution ", res, " -family \"", famid, "\"")
        res <- tryCatch(system(plotcall), error=function(e){return(e)})
        if (inherits(res, "simpleError")) {
            stop("Error calling HaploPainter!", res$message)
        } else {
            if (res!=0)
                stop("Error calling HaploPainter! Please check error message.")
        }
    }
    ## return the file name.
    invisible(filename)
}


#########################
## this function first checks all input parameters for correct input and
## subsequently builds a data.frame that can be used as input for haplopaint.
buildHaplopaintDataframe <- function(family=NULL, individual=NULL, father=NULL,
                                     mother=NULL, gender=NULL, affected=NULL,
                                     is.deceased=NULL, is.sab.or.top=NULL,
                                     is.proband=NULL, is.adopted=NULL,
                                     are.twins=NULL, are.consanguineous=NULL,
                                     text.inside.symbol=NULL,
                                     text.beside.symbol=NULL,
                                     text1.below.symbol=NULL,
                                     text2.below.symbol=NULL,
                                     text3.below.symbol=NULL,
                                     text4.below.symbol=NULL){
    if(is.null(individual) | is.null(father) |
       is.null(mother) | is.null(gender))
        stop("Arguments individual, father, mother and gender are required!")
    ninds <- length(individual)
    if(is.null(family))
        family <- rep(1, ninds)
    ## Fixing father & mother: Haplopaint requires 0
    father[is.na(father)] <- 0
    mother[is.na(mother)] <- 0
    ## now going through all arguments and, if present, adding them to the list
    argList <- list(family=family,
                    individual=as.character(individual),
                    father=as.character(father),
                    mother=as.character(mother)
                    )
    ## checking that sex has the correcto coding: 1:M, 2:F, 0:unknown
    gender <- as.numeric(sanitizeSex(gender))
    gender[is.na(gender)] <- 0
    argList <- c(argList, list(gender=gender))
    ## * affected: can be 0, 1 or any other number. each number is represented
    ##   by a different color: Note: we're adding +1 to each numeric affected
    ##   value! the coding is: NA -> 0, 0 -> 1, 1 -> 2!
    if(!is.null(affected)){
        affected <- checkTransformNumeric(affected, name="affected")
        if(length(which(affected < 0)))
            stop("Argument affected can not contain negative values!")
        if(any(is.na(affected))){
            ## adding 1 to each value and replacing NA with 0
            affected <- affected + 1
            affected[is.na(affected)] <- 0
        }else{
            ## adding +1 to each...
            affected <- affected + 1
        }
    }else{
        affected <- rep(0, ninds)
    }
    argList <- c(argList, list(affected=affected))
    ##
    ## * is.deceased
    ##   has to be either 0 or 1 (FALSE or TRUE)
    is.deceased <- checkArgNumericLogical(is.deceased, ninds,
                                          name="is.deceased")
    argList <- c(argList, list(is.deceased=is.deceased))
    ##
    ## * is.sab.or.top
    ##   has to be either 0 or 1 (FALSE or TRUE)
    is.sab.or.top <- checkArgNumericLogical(is.sab.or.top, ninds,
                                            name="is.sab.or.top")
    argList <- c(argList, list(is.sab.or.top=is.sab.or.top))
    ##
    ## * is.proband
    ##   has to be either 0 or 1 (FALSE or TRUE)
    is.proband <- checkArgNumericLogical(is.proband, ninds,
                                         name="is.proband")
    argList <- c(argList, list(is.proband=is.proband))
    ##
    ## * is.adopted
    ##   has to be either 0 or 1 (FALSE or TRUE)
    is.adopted <- checkArgNumericLogical(is.adopted, ninds,
                                         name="is.adopted")
    argList <- c(argList, list(is.adopted=is.adopted))
    ##
    ## * are.twins
    ##   NA, m_1, d_1 (m or d)_ followed by any text
    are.twins <- checkArgAreTwins(are.twins, ninds)
    argList <- c(argList, list(are.twins=are.twins))
    ##
    ## * are.consanguineous
    ##   NA and text specifying which individuals are consanguineous; always
    ##   two have to be consanguineous.
    are.consanguineous <- checkArgConsanguineous(are.consanguineous, ninds)
    argList <- c(argList, list(are.consanguineous=are.consanguineous))
    ##
    ## * text.inside.symbol
    ##   NA or any text
    if(!is.null(text.inside.symbol)){
        text.inside.symbol[is.na(text.inside.symbol)] <- ""
    }else{
        text.inside.symbol <- rep("", ninds)
    }
    argList <- c(argList, list(text.inside.symbol=text.inside.symbol))
    ##
    ## * text.beside.symbol
    ##   NA or any text
    if(!is.null(text.beside.symbol)){
        text.beside.symbol[is.na(text.beside.symbol)] <- ""
    }else{
        text.beside.symbol <- rep("", ninds)
    }
    argList <- c(argList, list(text.beside.symbol=text.beside.symbol))
    ##
    ## * text1.below.symbol
    ##   NA or any text
    if(!is.null(text1.below.symbol)){
        text1.below.symbol[is.na(text1.below.symbol)] <- ""
    }else{
        text1.below.symbol <- rep("", ninds)
    }
    argList <- c(argList, list(text1.below.symbol=text1.below.symbol))
    ##
    ## * text2.below.symbol
    ##   NA or any text
    if(!is.null(text2.below.symbol)){
        text2.below.symbol[is.na(text2.below.symbol)] <- ""
    }else{
        text2.below.symbol <- rep("", ninds)
    }
    argList <- c(argList, list(text2.below.symbol=text2.below.symbol))
    ##
    ## * text3.below.symbol
    ##   NA or any text
    if(!is.null(text3.below.symbol)){
        text3.below.symbol[is.na(text3.below.symbol)] <- ""
    }else{
        text3.below.symbol <- rep("", ninds)
    }
    argList <- c(argList, list(text3.below.symbol=text3.below.symbol))
    ##
    ## * text4.below.symbol
    ##   NA or any text
    if(!is.null(text4.below.symbol)){
        text4.below.symbol[is.na(text4.below.symbol)] <- ""
    }else{
        text4.below.symbol <- rep("", ninds)
    }
    argList <- c(argList, list(text4.below.symbol=text4.below.symbol))

    ## next we are going to evaluate whether we have the same length for all...
    ## if yes -> write temp data.frame and call haplopainter.
    Lengths <- unlist(lapply(argList, length))
    if(length(unique(Lengths)) != 1){
        cat("Got lengths: ", paste(paste(names(argList), ": ", Lengths),
                                   collapse=", "),  "\n")
        stop("All arguments have to have the same number of elements!")
    }
    do.call(cbind, argList)
}

## tries to transform x to numeric and, in case it's not possible, throws
## an error.
checkTransformNumeric <- function(x, name=""){
    tryCatch(x <- as.numeric(x),
             error=function(e){
                 stop("Error while checking argument ", name,
                      ": argument is not numeric!")
             },
             warning=function(w){
                 stop("Error while checking argument ", name,
                      ": argument is not numeric!")
             })
    x
}

## check the are.twins argument.
checkArgAreTwins <- function(are.twins=NULL, ninds){
    if(!is.null(are.twins)){
        if(is.character(are.twins))
            are.twins <- factor(are.twins)
        if(!is.factor(are.twins))
            stop("are.twins has to be either a character vector or factor!")
        ## checking the levels: have to start with m_ or d_
        twinlevels <- levels(are.twins)
        if(!all(substring(twinlevels, 1, 2) %in% c("m_", "d_")))
            stop("elements in are.twins have to be either NA or have to ",
                 "start with m_ (monozygotic twins) or d_ (dizygotic twins)!")
        ## each level has to be present at least twice.
        if(any(table(are.twins) < 2))
            stop("each text to specify twins has to be present at least ",
                 "twice in argument are.twins!")
        ## OK, seems to be OK
        are.twins <- as.character(are.twins)
        are.twins[is.na(are.twins)] <- ""
    }else{
        are.twins <- rep("", ninds)
    }
    are.twins
}

##
## checks whether the input argument can be transformed into a numeric with
## values 0 and 1.
checkArgNumericLogical <- function(x=NULL, ninds, name=""){
    if(!is.null(x)){
        x <- checkTransformNumeric(x, name=name)
        if(any(!(x %in% c(0, 1))))
            stop(name, " should be a logical vector or a numeric vector ",
                 "with 0 (no) and 1 (yes)!")
    }else{
        x <- rep(0, ninds)
    }
    x
}

##
##
##
checkArgConsanguineous <- function(x=NULL, ninds){
    if(!is.null(x)){
        x <- factor(x)
        ## each level has to be present exactly twice.
        if(any(table(x) != 2)){
            stop("Each text specifying consanguineous couples in argument ",
                 "are.consanguineous should be present exactly twice!")
        }
        x <- as.character(x)
        x[is.na(x)] <- ""
    }else{
        x <- rep("", ninds)
    }
    x
}

data(minnbreast)
mbsub <- minnbreast[minnbreast$famid %in% 4:19, ]
rownames(mbsub) <- mbsub$id
PedDf <- mbsub[, c("famid", "id", "fatherid", "motherid", "sex")]
colnames(PedDf) <- FamAgg:::.PEDCN
PedDf$sex <- FamAgg:::sanitizeSex(PedDf$sex)
fad <- FAData(PedDf)
## trait...
far <- fad
tcancer <- mbsub$cancer
names(tcancer) <- mbsub$id
trait(far) <- tcancer

## OK, 0.0.6
## test the argument checking function of the plot functions.
test_plot_arg_numeric_logical <- function(){
    ## test numeric input allowing only 0 and 1
    testvec <- c("0", "1", "0", "0", "0", "1")
    checkEquals(as.numeric(testvec), FamAgg:::checkArgNumericLogical(testvec, 6))
    testvec <- c(testvec, "z")
    checkException(FamAgg:::checkArgNumericLogical(testvec, 7))
    testvec <- rep(TRUE, 6)
    checkEquals(as.numeric(testvec), FamAgg:::checkArgNumericLogical(testvec, 6))
    ## check if we get a vector of 6 0s
    checkEquals(rep(0, 6), FamAgg:::checkArgNumericLogical(ninds=6))
}

test_plot_arg_are_twins <- function(){
    testvec <- c(NA, NA, "m_1", NA, "m_1", NA)
    testveccomp <- testvec
    testveccomp[is.na(testveccomp)] <- ""
    checkEquals(FamAgg:::checkArgAreTwins(testvec, 6), testveccomp)
    ## empty
    checkEquals(FamAgg:::checkArgAreTwins(ninds=6), rep("", 6))
}

test_plot_arg_consanguineous <- function(){
    testvec <- c(NA, NA, NA, "a", NA, "a")
    testveccomp <- testvec
    testveccomp[is.na(testveccomp)] <- ""
    checkEquals(FamAgg:::checkArgConsanguineous(testvec, 6), testveccomp)
    ## check error:
    testvec <- c(testvec, "b")
    checkException(FamAgg:::checkArgConsanguineous(testvec, 7))
    ## empty.
    checkEquals(FamAgg:::checkArgConsanguineous(ninds=6), rep("", 6))
}

test_kinship2_plot <- function(){
    ## data(minnbreast)
    ## mbsub <- minnbreast[minnbreast$famid %in% 4:19, ]
    ## rownames(mbsub) <- mbsub$id
    ## PedDf <- mbsub[, c("famid", "id", "fatherid", "motherid", "sex")]
    ## colnames(PedDf) <- FamAgg:::.PEDCN
    ## PedDf$sex <- FamAgg:::sanitizeSex(PedDf$sex)
    ## far <- as(FAData(pedigree=PedDf), "FAResult")
    ## ## trait...
    ## tcancer <- mbsub$cancer
    ## names(tcancer) <- mbsub$id
    ## trait(far) <- tcancer

    ped <- family(far, family="4", return.type="pedigree")
    ped <- ped["4"]
    plot(ped)

    ## directly plotting...
    fam <- family(far, family="4", return.type="data.frame")
    FamAgg:::ks2paint(family=fam$family, individual=fam$id, father=fam$father,
                      mother=fam$mother, gender=fam$sex, affected=fam$affected,
                      dev="plot")
    ## random deads
    is.deceased <- sample(c(TRUE, FALSE), nrow(fam), replace=TRUE)
    FamAgg:::ks2paint(family=fam$family, individual=fam$id, father=fam$father,
                      mother=fam$mother, gender=fam$sex, affected=fam$affected,
                      is.deceased=is.deceased, dev="plot")
}

test_haplopaint_txt_plot <- function() {
    fam <- family(far, family = "4", return.type = "data.frame")
    is.deceased <- sample(c(TRUE, FALSE), nrow(fam), replace=TRUE)
    suppressWarnings(
        tmp <- FamAgg:::haplopaint(family = fam$family, individual = fam$id,
                                   father = fam$father, mother = fam$mother,
                                   gender = fam$sex, affected = fam$affected,
                                   is.deceased = is.deceased, dev = "txt")
    )
    checkTrue(is(tmp, "character"))
    res <- read.table(tmp, sep = "\t", as.is = TRUE, header = TRUE,
                      comment.char = "")
    checkEquals(as.integer(is.deceased)[fam$id %in% res$INDIVID], res$DEAD)
    pf <- options()$FamAgg$plotfun
    FA <- options()$FamAgg
    FA$plotfun <- "haplopaint"
    options(FamAgg = FA)
    suppressWarnings(
        tmp <- plotPed(far, family = "5", device = "txt")
    )
    checkTrue(is(tmp, "character"))
    res <- read.table(tmp, sep = "\t", as.is = TRUE, header = TRUE,
                      comment.char = "")
    checkTrue(ncol(res) == 18)
    FA$plotfun <- pf
    options(FamAgg = FA)
}

notrun_test_plot <- function(){
    ## ## again, make a simple pedigree with trait.
    ## data(minnbreast)
    ## mbsub <- minnbreast[minnbreast$famid %in% 4:19, ]
    ## rownames(mbsub) <- mbsub$id
    ## PedDf <- mbsub[, c("famid", "id", "fatherid", "motherid", "sex")]
    ## colnames(PedDf) <- FamAgg:::.PEDCN
    ## PedDf$sex <- FamAgg:::sanitizeSex(PedDf$sex)
    ## far <- as(FAData(pedigree=PedDf), "FAResult")
    ## ## trait...
    ## tcancer <- mbsub$cancer
    ## names(tcancer) <- mbsub$id
    ## trait(far) <- tcancer

    checkException(FamAgg:::buildHaplopaintDataframe(family=fam$family,
                                                     individual=fam$id,
                                                     father=fam$father,
                                                     mother=fam$mother,
                                                     gender=fam$sex,
                                                     affected=fam$affected,
                                                     is.deceased=c(FALSE, TRUE)))

    ## plain function to generate...
    fam <- family(far, family="4")
    rownames(fam) <- as.character(fam$id)
    ## removing childless founders?
    ## df <- buildHaplopaintDataframe(family=fam$family, individual=fam$id, father=fam$father,
    ##                                mother=fam$mother, gender=fam$sex, affected=fam$affected)
    ## kins <- kinship(id=fam$id, momid=fam$mother, dadid=fam$father, sex=as.numeric(fam$sex))
    ## cols <- colSums(kins)
    ## if(any(cols <= 0.5)){
    ##     kins <- kins[cols > 0.5, cols > 0.5]
    ##     fam <- fam[colnames(kins), ]
    ##     warning(paste0("Removed ", sum(cols <= 0.5), " childless founders!"))
    ## }
    ## plain one...
    res <- FamAgg:::haplopaint(family=fam$family, individual=fam$id, father=fam$father,
                               mother=fam$mother, gender=fam$sex, affected=fam$affected)
    ## add more stuff... random deads...
    deads <- sample(c(TRUE, FALSE), size=nrow(fam), replace=TRUE)
    res <- FamAgg:::haplopaint(family=fam$family, individual=fam$id, father=fam$father,
                               mother=fam$mother, gender=fam$sex, affected=fam$affected,
                               is.deceased=deads)
}


## testing some larger pedigrees... whether that works at all...
notrun_test_larger_ped <- function(){
    ## data(minnbreast)
    ## mbsub <- minnbreast[minnbreast$famid %in% 4:19, ]
    ## PedDf <- mbsub[, c("famid", "id", "fatherid", "motherid", "sex")]
    ## colnames(PedDf) <- FamAgg:::.PEDCN
    ## PedDf$sex <- FamAgg:::sanitizeSex(PedDf$sex)
    ## far <- as(FAData(pedigree=PedDf), "FAResult")
    ## ## trait...
    ## tcancer <- mbsub$cancer
    ## names(tcancer) <- mbsub$id

    tcancershuffle <- sample(tcancer, length(tcancer))
    trait(far) <- tcancershuffle

}

## pedFile <- system.file("txt/Large.ped", package="FamAgg")
## cliqFile <- system.file("txt/Large-cliques.txt", package="FamAgg")
## ped <- read.table(pedFile, sep=";")
## cliqDf <- read.table(cliqFile, sep="\t")
## cliqs <- cliqDf[, 1]
## names(cliqs) <- cliqDf[, 2]
## ## building the pedigree
## fad <- FAData(ped[, 1:5])
## affs <- ped[, 6]
## affs[affs==2] <- 1
## names(affs) <- ped[, 2]


## ## OK, 0.0.6
## ## new:
## ## compare families with cliques from minnbreast_default_cliques.txt
## test_probability <- function(){
##     ## pedFile <- system.file("txt/Large.ped", package="FamAgg")
##     ## cliqFile <- system.file("txt/Large-cliques.txt", package="FamAgg")
##     ## ped <- read.table(pedFile, sep=";")
##     ## cliqDf <- read.table(cliqFile, sep="\t")
##     ## cliqs <- cliqDf[, 1]
##     ## names(cliqs) <- cliqDf[, 2]
##     ## ## building the pedigree
##     ## fad <- FAData(ped[, 1:5])
##     ## affs <- ped[, 6]
##     ## affs[affs==2] <- 1
##     ## names(affs) <- ped[, 2]
##     ## performing the test
##     if (.Platform$OS.type == "unix") {
##         far <- probabilityTest(fad, trait=affs,
##                                cliques=cliqs, nsim=500)
        
##         ## check if the cliques is the same as cliqs
##         farCliqs <- cliques(far, na.rm=TRUE)
##         checkEquals(as.character(farCliqs[names(cliqs)]), as.character(cliqs))
        
##         ## shuffle cliqs and replace; check if we get the correct one back.
##         cliques(far) <- sample(cliqs, size=length(cliqs))
##         farCliqs <- cliques(far, na.rm=TRUE)
##         checkEquals(as.character(farCliqs[names(cliqs)]), as.character(cliqs))
##         far <- runSimulation(far, nsim=500)
        
##         head(result(far))
        
##         ## testing to subset the object... which is not supported
##         checkException(far[1:10, ])
##     }
##     return(TRUE)
## }


## test_plot_probability <- function(){
##     do.plot <- FALSE
##     ## pedFile <- system.file("txt/Large.ped", package="FamAgg")
##     ## cliqFile <- system.file("txt/Large-cliques.txt", package="FamAgg")
##     ## ped <- read.table(pedFile, sep=";")
##     ## cliqDf <- read.table(cliqFile, sep="\t")
##     ## cliqs <- cliqDf[, 1]
##     ## names(cliqs) <- cliqDf[, 2]
##     ## ## building the pedigree
##     ## fad <- FAData(ped[, 1:5])
##     ## affs <- ped[, 6]
##     ## affs[affs==2] <- 1
##     ## names(affs) <- ped[, 2]
##     ## performing the test
##     if (.Platform$OS.type == "unix") {
##         far <- probabilityTest(fad, trait=affs,
##                                cliques=cliqs, nsim=500)
##         res <- result(far)
##         id <- res[1, "group_id"]
##         ped <- buildPed(far, id=id, prune=TRUE)
##         allCliqs <- cliques(far)
##         clique <- names(allCliqs)[which(allCliqs == as.character(id))]
##         ## plot the pedigree highlighting the clique.
##         ## plotPed(far, id=id, filename="~/Desktop/test.pdf", prune=TRUE)
##         switchPlotfun("ks2paint")
##         plotPed(far, id=id, prune=TRUE, device="plot")
##         ## the large huge busy pedigree.
##         if(do.plot){
##             plotPed(far, id=id, prune=FALSE, device="plot")
##         }
##         id <- res[2, "group_id"]
##         ped <- buildPed(far, id=id, prune=TRUE)
##         clique <- names(allCliqs)[which(allCliqs == as.character(id))]
##         all(clique %in% ped$id)
##         plotPed(far, id=id, prune=TRUE, device="plot")
##         ##switchPlotfun()
##     }
##     return(TRUE)
## }

## test_buildped_probability <- function(){
##     ## pedFile <- system.file("txt/Large.ped", package="FamAgg")
##     ## cliqFile <- system.file("txt/Large-cliques.txt", package="FamAgg")
##     ## ped <- read.table(pedFile, sep=";")
##     ## cliqDf <- read.table(cliqFile, sep="\t")
##     ## cliqs <- cliqDf[, 1]
##     ## names(cliqs) <- cliqDf[, 2]
##     ## ## building the pedigree
##     ## fad <- FAData(ped[, 1:5])
##     ## affs <- ped[, 6]
##     ## affs[affs==2] <- 1
##     ## names(affs) <- ped[, 2]
##     ## performing the test
##     if (.Platform$OS.type == "unix") {
##         far <- probabilityTest(fad, trait=affs,
##                                cliques=cliqs, nsim=500)
##         res <- result(far)
##         id <- res[1, "group_id"]
##         ## get the clique
##         clique <- names(cliqs)[which(as.character(cliqs) == as.character(id))]
##         ## build the full pedigree
##         FullPed <- buildPed(far, id=id)
##         ## check if we've got all of the clique in the pedigree...
##         checkEquals(length(clique), sum(clique %in% FullPed$id))
##         ## make a small subset pedigree:
##         SmallPed <- buildPed(far, id=id, prune=TRUE)
##         ## check if we've got all of the clique in the pedigree...
##         checkEquals(length(clique), sum(clique %in% SmallPed$id))
##     }
##     return(TRUE)
## }

data(minnbreast)
mbsub <- minnbreast[minnbreast$famid %in% c(4, 42, 165, 432),]
PedDf <- mbsub[, c("famid", "id", "fatherid", "motherid", "sex")]
colnames(PedDf) <- FamAgg:::.PEDCN
PedDf$sex <- FamAgg:::sanitizeSex(PedDf$sex)
## data(minnbreast)
## mbsub <- minnbreast[minnbreast$famid %in% 4:19, ]
## PedDf <- mbsub[, c("famid", "id", "fatherid", "motherid", "sex")]
## colnames(PedDf) <- FamAgg:::.PEDCN
## PedDf$sex <- FamAgg:::sanitizeSex(PedDf$sex)
## generate the FAData.
fad <- FAData(pedigree=PedDf)
## specify the trait.
tcancer <- mbsub$cancer
names(tcancer) <- mbsub$id

## OK, 0.0.6
## evaluating the kinship analysis method and for FAKinshipResult objects.
test_kinship_group <- function(){
    ##
    ## run the analysis.
    set.seed(18011977)
    far <- kinshipGroupTest(fad, trait=tcancer, traitName="cancer",
                            nsim=500)
    ## repeating the test and evaluate whether we get the same results.
    set.seed(18011977)
    far2 <- kinshipGroupTest(fad, trait=tcancer, traitName="cancer",
                        nsim=500)
    checkEquals(far, far2)

    ## replacing the trait with a random trait.
    tpreg <- mbsub$everpreg
    names(tpreg) <- mbsub$id
    ## randomizing to see if re-ordering works...
    tpregrand <- sample(tpreg, length(tpreg))
    trait(far) <- tpregrand
    checkEquals(trait(far), tpreg)

    ## testing to subset the object... which is not supported
    checkException(far[1:10, ])
    return(TRUE)
}

## question if for the plotting: how to build the pedigree: just the pheno and aff
## from the affectedKinshipGroups?
## plot the kinshipGroup.
test_plot_kinship <- function(){
    do.plot <- FALSE

    set.seed(18011977)
    far <- kinshipGroupTest(fad, trait=tcancer, traitName="cancer", nsim=500)
    res <- result(far)
    res <- res[order(res$kinship_pvalue), ]

    if(do.plot){
        switchPlotfun("ks2paint")
        plotPed(far, id="17517", prune=TRUE, device="plot", cex=0.4)
        plotPed(far, id="17517", prune=FALSE, device="plot")
    }
    ## switchPlotfun()
}


test_buildped_kinship <- function(){
    ## data(minnbreast)
    ## mbsub <- minnbreast[minnbreast$famid %in% c(4, 42, 165, 432),]
    ## PedDf <- mbsub[, c("famid", "id", "fatherid", "motherid", "sex")]
    ## colnames(PedDf) <- FamAgg:::.PEDCN
    ## PedDf$sex <- FamAgg:::sanitizeSex(PedDf$sex)
    ## ## generate the FAData.
    ## fad <- FAData(pedigree=PedDf)
    ## ## specify the trait.
    ## tcancer <- mbsub$cancer
    ## names(tcancer) <- mbsub$id

    set.seed(18011977)
    far <- kinshipGroupTest(fad, trait=tcancer, traitName="cancer", nsim=500)
    res <- result(far)
    res <- res[order(res$kinship_pvalue), ]

    FullPed <- buildPed(far, id=res[1, "group_id"])
    affGroup <- affectedKinshipGroups(far)[[res[1, "group_id"]]]
    allids <- unique(c(affGroup$aff, affGroup$pheno))
    ## check if all ids are in the pedigree.
    checkEquals(length(allids), sum(allids %in% FullPed$id))
    ## check if the same works for the pruned pedigree:
    SmallPed <- buildPed(far, id=res[1, "group_id"], prune=TRUE)
    checkEquals(length(allids), sum(allids %in% SmallPed$id))
}

notrun_test_compare_strat_normal <- function(){
    ## test the strata test and compare that to the "normal" test
    set.seed(18011977)
    fag <- kinshipGroupTest(fad, trait=tcancer, nsim=500)
    fag2 <- fag
    set.seed(18011977)
    fag2 <- runSimulation(fag2, nsim=500)
    ## Compare
    checkEquals(fag@sim$pvalueRatio, fag2@sim$pvalueRatio)
    checkEquals(fag@sim$pvalueKinship, fag2@sim$pvalueKinship)
    ## now do it with strata = 1.
    strat <- rep("A", length(fad$id))
    set.seed(18011977)
    fag2 <- FamAgg:::runSimulationStrata(fag, nsim=500, strata=strat)
    checkEquals(fag@sim$pvalueRatio, fag2@sim$pvalueRatio)
    checkEquals(fag@sim$pvalueKinship, fag2@sim$pvalueKinship)
    ## Test if the NAs are correctly removed...
    strat[!is.na(tcancer)][1:10] <- NA
    fag2 <- FamAgg:::runSimulationStrata(fag, nsim=500, strata=strat)
    ## Have to get the message that 10 phenotyped are removed.
    ## OK, that's it. Now do the same but with sex strata.
    fasex <- FamAgg:::runSimulationStrata(fag, nsim=500, strata=fad$sex)
    Col <- rep("blue", length(fad$id))
    Col[which(fad$sex == "F")] <- "red"
    plot(fasex@sim$pvalueKinship, fag@sim$pvalueKinship, xlab="Sex stratified",
         ylab="No stratification", col=Col)
    abline(0, 1)
}

## These are methods for the "genealogical index" for familial clustering [Hill, 1980]
##************************************************
##
##       FAGenIndexResult
##       Methods for result objects from the genealogical index test.
##
##************************************************
setMethod("show", "FAGenIndexResults", function(object){
    callNextMethod()
    cat(paste0("Result info:\n"))
    cat(paste0(" * Dimension of result data.frame: ",
               length(object@sim),".\n"))
    cat(paste0(" * Number of simulations: ", object@nsim, ".\n"))
    cat(paste0(" * Method for control set definition: ",
               object@controlSetMethod, ".\n"))
    if(object@perFamilyTest){
        cat(" * Analysis performed for each family.\n")
    }else{
        cat(" * Analysis performed for the whole pedigree.\n")
    }
})
## calling the trait replecement method from FAResult and in addition
## reset the simulation result.
setReplaceMethod("trait", "FAGenIndexResults", function(object, value){
    object <- callNextMethod()
    ## reset the result
    object@sim <- list()
    object@nsim <- 0
    object@traitname <- character()
    object@controlSetMethod <- "getAll"
    object@perFamilyTest <- FALSE
    return(object)
})

####
## the analysis method.
setMethod("runSimulation", "FAGenIndexResults",
          function(object, nsim = 50000, perFamilyTest = FALSE,
                   controlSetMethod = "getAll", rm.singletons = TRUE,
                   strata = NULL, ...) {
              if(length(trait(object)) == 0)
                  stop("No trait information available!")
              kin <- kinship(object)
              trait <- trait(object)
              ResList <- .genIndex(ped = pedigree(object),
                                   kin = kinship(object),
                                   trait = trait(object),
                                   perFamilyTest = perFamilyTest,
                                   controlSetMethod = controlSetMethod,
                                   nsim = nsim,
                                   prune = rm.singletons, strata = strata, ...)
              object@nsim <- nsim
              object@controlSetMethod <- controlSetMethod
              object@perFamilyTest <- perFamilyTest
              object@sim <- ResList
              object
          })

## results; get the results table.
setMethod("result", "FAGenIndexResults", function(object, method="BH"){
    method <- match.arg(method, p.adjust.methods)
    TraitName <- object@traitname
    if(length(TraitName)==0)
        TraitName <- NA
    if(length(object@sim) == 0){
        ## generate a dummy matrix...
        MyRes <- data.frame(trait_name=TraitName,
                            total_phenotyped=length(phenotypedIndividuals(object)),
                            total_affected=length(affectedIndividuals(object)),
                            entity_id=NA,
                            entity_ctrls=NA,
                            entity_affected=NA,
                            genealogical_index=NA,
                            pvalue=NA,
                            padj=NA,
                            check.names=FALSE, stringsAsFactors=FALSE
                            )
        warning("No simulation data available! This means that either no ",
                "simulation was run yet (using the genealogicalIndex function ",
                "or runSimulation method) or that the simulation returned no ",
                "result (i.e. too few affected individuals in the trait).")
        return(MyRes)
    }
    ## otherwise compile the result...
    MyRes <- do.call(rbind, lapply(object@sim, function(z){
        return(c(entity_ctrls=length(z$ctrls),
                 entity_affected=length(z$affected),
                 genealogical_index=z$meanKinship * 100000,
                 pvalue=z$pvalueKinship))
    }))
    MyRes <- data.frame(trait_name=rep(TraitName, nrow(MyRes)),
                        total_phenotyped=rep(length(phenotypedIndividuals(object)),
                                             nrow(MyRes)),
                        total_affected=rep(length(affectedIndividuals(object)),
                                           nrow(MyRes)),
                        entity_id=as.character(names(object@sim)),
                        MyRes,
                        padj=p.adjust(MyRes[, "pvalue"], method=method),
                        stringsAsFactors=FALSE, check.names=FALSE)
    MyRes <- MyRes[order(MyRes$pvalue), ]
    rownames(MyRes) <- as.character(MyRes$entity_id)
    MyRes
})


## approach:
## * per family: do the analysis family-wise: find families with.
## * "normal" analysis: whole pedigree: disable getGenerationMatched!
## prepare: prune the pedigree/family, i.e. remove unconnected individuals.
## 1) select affected, calculate mean kinship.
## 2) select matched controls: using a control selection method.
##    sample from them, using eventually a strata sampler.
## ADDON: use a stratafun? something like strata by generation, strata by generation
## and sex or alike.
##
## ped: data.frame with pedigree information (pedigree(fad), fad being a FAData)
## kin: kinship matrix (kinship(fad))
## trait: numeric or logical vector, length=nrow(ped)
## perFamilyTest: perform the test for each family separately. That way we determine
##                whether the kinship between affected in that family is higher than
##                we might expect by chance, given the kinship between individuals in
##                the same family. Thus, we test for the presence of a cluster of close
##                relatives in the family affected by the trait.
## controlSetMethod: function to select matched controls.
## nsim: number of simulations.
## prune: remove un-connected individuals from the pedigree; should always be TRUE
## strata: to do strata sampling.
## aggFun: the function to be used to aggregate the kinship of affected. Defaults to
##         mean, as in the original definition by Hill, but a kinship sum might be
##         more informative.
##
## return:
## a list with:
## + meanKinship: the mean kinship between affected.
## + pvalueKinship: the p-value.
## + expDensity: the density distribution of expected mean kinships of random samples.
## + affected: the ids of the affected.
## + ctrls: the ids of the (matched) controls.
.genIndex <- function(ped=NULL, kin=NULL, trait=NULL, perFamilyTest=FALSE,
                      controlSetMethod="getAll", nsim=50000, prune=TRUE,
                      strata=NULL, aggFun=mean, ...){
    controlSetMethod <- match.arg(controlSetMethod, c("getAll",
                                                      "getSexMatched",
                                                      "getGenerationMatched",
                                                      "getGenerationSexMatched",
                                                      "getExternalMatched"))
    if(!perFamilyTest){
        ## don't allow some matched control methods.
        if((controlSetMethod == "getGenerationMatched") |
           (controlSetMethod == "getGenerationSexMatched")){
            stop("Can not use 'getGenerationMatched' or ",
                 "'getGenerationSexMatched' functions for ",
                 "'perFamilyTest=FALSE' since the generation estimation ",
                 "procedure only allows to define within-family generations, ",
                 "not generations across families in a full pedigree.")
        }
        entityIs <- "pedigree"
        ## replace the family id with 1 for all... thus performing the test for
        ## the whole pedigree in one go.
        ped[, "family"] <- 1
    }else{
        entityIs <- "family"
    }
    if(is.null(ped))
        stop("No pedigree submitted!")
    ped <- checkPedCol(ped)
    if(is.null(trait))
        stop("No trait information submitted!")
    if(is.null(kin))
        stop("No kinship matrix submitted!")
    ## add the trait information to the pedigree data.frame
    if(length(trait) != nrow(ped))
        stop("Argument 'trait' has to have the same length then there are",
             " rows (individuals) in the pedigree 'ped'!")
    ped <- cbind(ped, AFF=trait)
    ## after all we shouldn't need this; just in case. it's for prune=TRUE to
    ## add eventually removed mates of a childless couple; but in the end we
    ## don't want them anyway, and they are NOT defined in a pedigree.
    dotList <- list(...)
    if(any(names(dotList) == "addMissingMates")){
        addMisMate <- dotList$addMissingMates
    }else{
        addMisMate <- FALSE
    }
    ## check strata
    haveStrata <- FALSE
    if(!is.null(strata)){
        if(length(strata)!=nrow(ped))
            stop("Argument 'strata' has to have the same length than there",
                 " are rows (individuals) in the pedigree 'ped'!")
        ## add strata.
        ped <- cbind(ped, STRATA=strata)
        haveStrata <- TRUE
    }
    ## First remove singletons
    if(prune){
        ped <- removeSingletons(ped)
    }
    ## OK, now we can go on: lapply on the splitted ped by family
    pedL <- split(ped, ped$family)
    Res <- lapply(pedL, function(z){
        ## Now get the controls and the affected ids. We have to eventually
        ## further subset these if we have non-phenotyped controls etc.
        affIds <- as.character(z[which(z$AFF > 0), "id"])
        ## Eventually remove affected without a value in strata:
        message("Cleaning data set (got in total ", nrow(z), " individuals):")
        if(haveStrata){
            message(" * affected individuals without valid strata values...",
                    appendLF=FALSE)
            nas <- is.na(z[affIds, "STRATA"])
            if(any(nas)){
                affIds <- affIds[!nas]
                message(" ", sum(nas), " removed.")
            }else{
                message(" none present.")
            }
        }
        ## Return if there are not enough affected.
        if(length(affIds) < 2){
            warning(paste0("Can not perform test for ",
                         entityIs, " ", z[1, "family"],
                         ": less than 2 affected individuals in the trait."))
            return(list(meanKinship=NA,
                        pvalueKinship=NA,
                        expDensity=NULL,
                        affected=affIds,
                        ctrls=NA))
        }
        ## Get the matched controls for these guys...
        ctrls <- as.character(
            unlist(do.call(what=controlSetMethod, args=list(object=z,
                                                            id=affIds, ...)),
                   use.names=FALSE)
            )
        ## Subset to phenotyped individuals, i.e. those with a valid value in
        ## "affected"
        nas <- is.na(z[ctrls, "affected"])
        message(" * not phenotyped individuals among selected controls...",
                appendLF=FALSE)
        if(any(nas)){
            ctrls <- ctrls[!nas]
            message(" ", sum(nas), " removed.")
        }else{
            message(" none present.")
        }
        if(haveStrata){
            ## Subsetting the data.frame to elements that have a non-NA value
            ## in strata.
            nas <- is.na(z[ctrls, "STRATA"])
            message(" * control individuals without valid strata values...",
                    appendLF=FALSE)
            if(any(nas)){
                ctrls <- ctrls[!nas]
                message(" ", sum(nas), " removed.")
            }else{
                message(" none present.")
            }
        }
        message("Done")
        if(length(ctrls) < 4*length(affIds))
            warning("The proportion of affected individuals among the ",
                    "matched controls is > 25% in ", entityIs, " ",
                    z[1, "family"], ".")
        ## Subset the kinship matrix and transform into a "normal" matrix,
        ## assuming that subsetting and other functions run faster on that.
        bm <- colnames(kin) %in% unique(c(affIds, ctrls))
        localKin <- as.matrix(kin[bm, bm])
        ## Remove self-self kinships; we don't want to count them.
        diag(localKin) <- NA
        ## Calculate the (observed) mean kinship between the real affected.
        localKinColNames <- colnames(localKin)
        bm <- localKinColNames %in% affIds
        obsMeanKin <- aggFun(as.vector(localKin[bm, bm]), na.rm=TRUE)
        ## OK, now that we've got the controls, do the test!
        if(!haveStrata){
            affSize <- length(affIds)
            expMeanKins <- sapply(1:nsim, function(y){
                ## pick random samples, same size than affIds
                expIds <- sample(ctrls, size=affSize)
                bm <- localKinColNames %in% expIds
                return(aggFun(as.vector(localKin[bm, bm]), na.rm=TRUE))
            })
        }else{
            affStrataCounts <- table(z[affIds, "STRATA"])
            affStrataCounts <- affStrataCounts[affStrataCounts > 0]
            phenoStrata <- z[ctrls, "STRATA"]
            expMeanKins <- sapply(1:nsim, function(y){
                ## pick random strata sample using the stratsample from the
                ## survey package
                expIdx <- stratsample(phenoStrata, affStrataCounts)
                bm <- localKinColNames %in% ctrls[expIdx]
                return(aggFun(as.vector(localKin[bm, bm]), na.rm=TRUE))
            })
        }
        return(list(meanKinship=obsMeanKin,
                    pvalueKinship=sum(expMeanKins >= obsMeanKin)/nsim,
                    expDensity=density(expMeanKins),
                    expHist=hist(expMeanKins, plot=FALSE),
                    affected=affIds, ctrls=ctrls))
    })
    Res
}

#############
## plotting method...
## plotPed representing the results from the kinship clustering test.
## plotPed for FAKinClustResult does only support plotting for id.
setMethod("plotPed", "FAGenIndexResults",
          function(object, id=NULL,
                   family=NULL, filename=NULL,
                   device="plot", ...){
              if(!is.null(id)){
                  ## get the family id for this id...
                  if(!any(as.character(object$id) == id))
                      stop("Can not find any individual with id ", id,
                           " in the pedigree!")
                  family <- as.character(object$family)[as.character(object$id) == id]
                  family <- family[1]
              }
              if(length(object@sim) > 0){
                  if(!object@perFamilyTest){
                      warning("The genealogical index test has been performed ",
                              "on the full pedigree, but the plot was only ",
                              "generated for family ", family, "!")
                      ctrls <- object@sim[[1]]$ctrls
                  }else{
                      ctrls <- object@sim[[family]]$ctrls
                  }
              }else{
                  ctrls <- NA
              }

              Blue <- "#377EB8"
              callNextMethod(object = object, id = id, family = family,
                             filename = filename, device = device,
                             proband.id = ctrls, ...)
              ## alternatively use highlight.ids
          })


setMethod("[", "FAGenIndexResults", function(x, i, j, ..., drop){
    stop("Subsetting of a FAGenIndexResults object is not supported!")
})

## This will be a crazy funky method to plot the simulation results.
setMethod("plotRes", "FAGenIndexResults",
          function(object, id=NULL,
                   family=NULL, addLegend=TRUE, type="density", ...){
              type <- match.arg(type, c("density", "hist"))
              if(length(object@sim) == 0)
                  stop("No analysis performed yet!")
              if(!object@perFamilyTest){
                  ## Just ignore id and family...
                  meanKin <- object@sim[[1]]$meanKinship
                  pval <- object@sim[[1]]$pvalueKinship
                  dens <- object@sim[[1]]$expDensity
                  hist <- object@sim[[1]]$expHist
                  affCount <- length(object@sim[[1]]$affected)
                  ctrlsCount <- length(object@sim[[1]]$ctrls)
                  family <- 1
              }else{
                  if(!is.null(id)){
                      bm <- as.character(object$id) == id
                      if(!any(bm))
                          stop("No individual with id ", id,
                               " present in the pedigree!")
                      family <- as.character(object$family)[bm][1]
                  }
                  if(is.null(family))
                      stop("With per family genealogical index ",
                           "(perFamilyTest=TRUE) either the id of an ",
                           "individual or the id of a family has to be ",
                           "specified!")
                  if(!any(names(object@sim) == family))
                      stop("No family with id ", family,
                           " present in the pedigree!")
                  meanKin <- object@sim[[as.character(family)]]$meanKinship
                  pval <- object@sim[[as.character(family)]]$pvalueKinship
                  dens <- object@sim[[as.character(family)]]$expDensity
                  hist <- object@sim[[as.character(family)]]$expHist
                  affCount <- length(object@sim[[as.character(family)]]$affected)
                  ctrlsCount <- length(object@sim[[as.character(family)]]$ctrls)
                  if(is.na(meanKin))
                      stop("No results for family ", family, " available!")
              }

              ## plot it...
              par(xpd=FALSE)
              entity <- ifelse(object@perFamilyTest, yes="Family", no="Pedigree")
              if(type == "density"){
                  plot(dens, main = paste0(entity, " ", family, "; ",
                                           object@traitname),
                       xlab="Mean kinship", type="h", lwd=3, col="lightgrey",
                       xlim=range(c(range(dens$x), meanKin)))
                  points(dens, col="grey", type="l", lwd=2)
              }
              if(type == "hist"){
                  plot(hist, main=paste0(entity, " ", family, "; ",
                                         object@traitname), xlab="Mean kinship",
                       col="lightgrey", border="grey",
                       xlim=range(c(range(hist$x), meanKin)))
              }
              Blue <- "#377EB8"
              abline(v=meanKin, col=Blue)
              if(addLegend){
                  legend(
                      "topright",
                      legend=c(paste0("GIF: ", format(100000*meanKin, digits=4)),
                               paste0("mean kinship: ", format(meanKin, digits=2)),
                               paste0("p-value     : ", format(pval, digits=3)),
                               paste0("affect count: ", affCount),
                               paste0("ctrls count : ", ctrlsCount)))
              }
          })

## load the data we're going to analyze
data(minnbreast)
mbsub <- minnbreast[minnbreast$famid %in% c(4, 5, 6, 8, 14, 16), ]
PedDf <- mbsub[, c("famid", "id", "fatherid", "motherid", "sex")]
colnames(PedDf) <- FamAgg:::.PEDCN
PedDf$sex <- FamAgg:::sanitizeSex(PedDf$sex)
## generate the FAData.
fad <- FAData(pedigree=PedDf)
## specify the trait.
tcancer <- mbsub$cancer
names(tcancer) <- mbsub$id

## per family tests: that way I can better control the control methods.
test_gen_index_per_family <- function(){
    do.plot <- FALSE

    ## family 14 looks promising!
    trait(fad) <- tcancer

    gi <- as(fad, "FAGenIndexResults")
    set.seed(18011977)
    Test <- runSimulation(gi, controlSetMethod="getAll", nsim=100, perFamilyTest=TRUE)
    All <- Test@sim[["14"]]
    All8 <- Test@sim[["8"]]
    expectAff <- as.character(c(444, 447, 450, 451, 452))
    checkEquals(sort(All$affected), sort(expectAff))

    set.seed(18011977)
    Test <- runSimulation(gi, controlSetMethod="getSexMatched", nsim=100,
                          perFamilyTest=TRUE)
    SexMatch <- Test@sim[["14"]]
    SexMatch8 <- Test@sim[["8"]]
    expectCtrls <- as.character(c(441, 444, 446, 447, 442, 443, 462, 450, 451, 452,
                                  453, 454, 468, 469))
    checkEquals(sort(SexMatch$ctrls), sort(expectCtrls))

    set.seed(18011977)
    Test <- runSimulation(gi, controlSetMethod="getGenerationMatched", nsim=100,
                          perFamilyTest=TRUE)
    GenMatch <- Test@sim[["14"]]
    expectCtrls <- as.character(c(459, 444, 445, 460, 446, 461, 447, 442, 458, 443,
                                  448, 449, 462, 450, 464, 451, 465, 452, 466, 453,
                                  467, 454))
    checkEquals(sort(GenMatch$ctrls), sort(expectCtrls))

    ## Test generation & sex matched. Note that the result is no longer significant!
    set.seed(18011977)
    Test <- runSimulation(gi, controlSetMethod="getGenerationSexMatched", nsim=100,
                          perFamilyTest=TRUE)
    GenSexMatch <- Test@sim[["14"]]
    expectCtrls <- as.character(c(444, 446, 447, 442, 443, 462, 450, 451, 452, 453,
                                  454))
    checkEquals(sort(GenSexMatch$ctrls), sort(expectCtrls))

    if(do.plot){
        par(mfrow=c(4, 1))
        plot(All$expDensity, main="getAll", type="h", lwd=2, col="lightgrey")
        points(All$expDensity, type="l", col="grey")
        abline(v=All$meanKinship)
        plot(SexMatch$expDensity, "getSexMatched", type="h", lwd=2, col="lightgrey")
        points(SexMatch$expDensity, type="l", col="grey")
        abline(v=SexMatch$meanKinship)
        plot(GenMatch$expDensity, "getGenerationMatched", type="h", lwd=2, col="lightgrey")
        points(GenMatch$expDensity, type="l", col="grey")
        abline(v=GenMatch$meanKinship)
        plot(GenSexMatch$expDensity, "getGenerationSexMatched", type="h", lwd=2, col="lightgrey")
        points(GenSexMatch$expDensity, type="l", col="grey")
        abline(v=GenSexMatch$meanKinship)
    }

    ## families with male cases: 4, 8, 9, 10, 19
    ## other interesting family: 8
    ## LLLL Test with or without strata sampling for family 8.
    plotPed(fad, family="8", cex=0.8)
    ## sex matching and getting all should not make any difference here.
    checkEquals(sort(All8$ctrls), sort(SexMatch8$ctrls))

    ## do the test again using sex as strata:
    set.seed(18011977)
    Test <- runSimulation(gi, controlSetMethod="getAll", nsim=100, perFamilyTest=TRUE,
                          strata=fad$sex)
    All8Strata <- Test@sim[["8"]]
    checkTrue(All8Strata$pvalueKinship != All8$pvalueKinship)

    ## the results for family 14 have to be identical:
    set.seed(18011977)
    Test <- runSimulation(gi, controlSetMethod="getSexMatched", nsim=100, perFamilyTest=TRUE,
                          strata=gi$sex)
    checkEquals(SexMatch$pvalueKinship, Test@sim[["14"]]$pvalueKinship)
}

## The same tests, but once for the full pedigree.
test_gen_index_pedigree <- function(){
    set.seed(18011977)
    ## use getAll.
    giAll <- genealogicalIndexTest(fad, trait=tcancer, traitName="cancer", nsim=100,
                                   controlSetMethod="getAll")
    checkTrue(result(giAll)[1, "pvalue"] < 0.05)

    ## use getSexMatched
    giSexM <- genealogicalIndexTest(fad, trait=tcancer, traitName="cancer", nsim=100,
                                    controlSetMethod="getSexMatched")
    ## it is NOT the same as getAll, since some individuals with unknown sex were
    ## removed.

    ## use getSexMatched, strata.
    giSexMStrata <- genealogicalIndexTest(fad, trait=tcancer, traitName="cancer", nsim=100,
                                          controlSetMethod="getSexMatched", strata=fad$sex)

    ## checkError for getGenerationMatched
    checkException(FamAgg:::.genIndex(pedigree(fad), kin=kinship(fad), trait=tcancer,
                                      controlSetMethod="getGenerationMatched", nsim=100))
}


test_gen_index_with_gap <- function(){
    library(gap)
    gi <- genealogicalIndexTest(fad, trait=tcancer, nsim=100, rm.singletons=FALSE)

    ## Now use gap:
    gapPed <- pedigree(fad)
    gapPed[is.na(gapPed$mother), "mother"] <- 0
    gapPed[is.na(gapPed$father), "father"] <- 0
    ## get the ids of the affected
    affIds <- names(tcancer[which(tcancer == 1)])
    giGap <- gif(gapPed[, c("id", "father", "mother")], as.numeric(affIds))
    checkEquals(result(gi)$genealogical_index, giGap[[1]])
    ## Doing the same with removal of singletons:
    gi <- genealogicalIndexTest(fad, trait=tcancer, nsim=100, rm.singletons=TRUE)
    gapPed <- pedigree(fad)
    gapPed <- removeSingletons(gapPed)
    gapPed[is.na(gapPed$mother), "mother"] <- 0
    gapPed[is.na(gapPed$father), "father"] <- 0
    affIds <- affIds[affIds %in% gapPed$id]
    giGap <- gif(gapPed[, c("id", "father", "mother")], as.numeric(affIds))
    checkEquals(result(gi)$genealogical_index, giGap[[1]])
}

##********************************************************
##
##   FAData
##   that's the constructor for the FunAggrData class
##   * pedigree: the pedigree, can be a character string or a
##     data.frame
##   * age: the age, can
##   * header: for the internal read.table function that loads data.
##   * sep:
##
##********************************************************
FAData <- function(pedigree, age, trait, traitName, header = FALSE, sep = "\t",
                   id.col = "id", family.col = "family", father.col = "father",
                   mother.col = "mother", sex.col = "sex") {
    if(missing(pedigree)){
        stop("Argument pedigree has to be specified!")
    }
    if(missing(trait))
        trait <- logical()
    FSD <- new("FAData")
    expect_cn <- c(family = family.col, id = id.col, father = father.col,
                   mother = mother.col, sex = sex.col)
    ## processing pedigree
    if(is.character(pedigree)){
        ## means, we're loading the data.frame ourselfs.
        message(paste("Reading pedigree information from file ", pedigree,
                      "..."), appendLF=FALSE)
        pedigree <- doImport(pedigree, header = header, sep = sep,
                             id.col = id.col, family.col = family.col,
                             father.col = father.col, mother.col = mother.col,
                             sex.col = sex.col)
        ## Check if we've got eventually also the trait (i.e. from a ped or
        ## fam file).
        if(missing(trait) & any(colnames(pedigree) == "trait")){
            trait <- pedigree[, "trait"]
            pedigree <- pedigree[, colnames(pedigree) != "trait"]
        }
        pedigree <- pedigree[, names(expect_cn)]
        colnames(pedigree) <- expect_cn
        message("OK\n")
    }
    if (is.matrix(pedigree)) {
        ## transform to data.frame
        pedigree <- as.data.frame(pedigree, stringsAsFactors = FALSE)
    }
    if (!(is.data.frame(pedigree) | is(pedigree, "pedigree") |
          is(pedigree, "pedigreeList"))) {
        stop("Argument pedigree has to be a data.frame, a character string ",
             "specifying the file containing the pedigree, a pedigree object ",
             "or a pedigreeList object!")
    } else {
        if (is.data.frame(pedigree)) {
            CN <- .PEDCN
            if (!noColNames(pedigree)) {
                if (!all(expect_cn %in% colnames(pedigree)))
                    stop("Expected column names ",
                         paste0("'", expect_cn, "'", collapse = ", "),
                         " in 'pedigree' but I got ",
                         paste0("'", colnames(pedigree), "'", collapse = ", "))
                pedigree <- pedigree[, expect_cn]
                colnames(pedigree) <- names(expect_cn)
            } else {
                if (ncol(pedigree)!=length(CN))
                    stop("pedigree is expected to have ", length(CN),
                         " columns but I got ", ncol(pedigree), "!")
                warning("Forcing column names ", paste(CN, collapse=";"),
                        " on pedigree.")
                colnames(pedigree) <- CN
            }
        }
    }
    ## We don't like factors.
    if (is.factor(pedigree$id)) pedigree$id <- as.character(pedigree$id)
    ## Fixing the founders:
    if (is.factor(pedigree$father))
        pedigree$father <- as.character(pedigree$father)
    if (is.factor(pedigree$mother))
        pedigree$mother <- as.character(pedigree$mother)
    if (is.character(pedigree$father))
        pedigree$father[pedigree$father == ""] <- NA
    if (is.numeric(pedigree$father))
        pedigree$father[pedigree$father == 0] <- NA
    if (is.character(pedigree$mother))
        pedigree$mother[pedigree$mother == ""] <- NA
    if (is.numeric(pedigree$mother))
        pedigree$mother[pedigree$mother == 0] <- NA
    pedigree(FSD) <- pedigree
    ## processing age
    if(!missing(age)){
        if(is.character(age)){
            message(paste("Reading age information from file ", age, "..."),
                    appendLF=FALSE)
            FN <- age
            age <- read.table(age, header=header, sep=sep, as.is=TRUE)
            message("OK\n")
            ## assuming first column is the ID, second the age.
            warning("Assuming first column in ", FN,
                    " contains IDs and second age.")
            ageNames <- age[, 1]
            age <- age[ , 2]
            names(age) <- ageNames
        }
        if(!is.numeric(age)){
            stop("Argument age has to be a named numeric vector or a character",
                 " string specifying the file containing the age!")
        }
        age(FSD) <- age
    }
    if(!missing(traitName))
        FSD@traitname <- traitName
    ## Do we have a trait???
    if (length(trait)) {
        if (!all(is.na(trait))) {
            if (length(trait) == nrow(pedigree))
                if (is.null(names(trait)))
                    names(trait) <- as.character(pedigree$id)
            trait(FSD) <- trait
        }
    }
    if (validObject(FSD))
        FSD
}

## if this setup of a global data set does not work we might want to put that into
## the functions .setUp() and .tearDown(), which are executed before and after each
## test function.
data(minnbreast)
ped <- minnbreast[, c("famid", "id", "fatherid", "motherid", "sex")]
colnames(ped) <- c("family", "id", "father", "mother", "sex")
fam4 <- ped[ped$family == "4", ]
fad <- FAData(ped[ped$family %in% c(4:6), ])

## testing functions and methods related to matched controls and sampling.
test_matched_generation <- function(){
    ## data(minnbreast)
    ## ped <- minnbreast[, c("famid", "id", "fatherid", "motherid", "sex")]
    ## colnames(ped) <- c("family", "id", "father", "mother", "sex")
    ## fam4 <- ped[ped$family == "4", ]
    ## fad <- FAData(ped[ped$family %in% c(4:20), ])

    ##
    genMatch <- FamAgg:::getGenerationMatched(pedigree(fad),
                                              id=c("4", "5", "56", "76", "19"))
    ## we get a list with the list names being the family id, and elements the
    ## ids of the individuals from the same generation.
    genMatch <- unlist(FamAgg:::getGenerationMatched(pedigree(fad),
                                                     id="4"), use.names=FALSE)
    checkEquals(sort(genMatch), sort(as.character(c(25, 4, 5, 26, 6, 27, 7,
                                                    28, 8, 29, 9, 10))))
    genMatch <- unlist(FamAgg:::getGenerationMatched(pedigree(fad), id=c("4", "2")),
                       use.names=FALSE)
    checkEquals(sort(genMatch), sort(as.character(c(25, 4, 5, 26, 6, 27, 7, 28,
                                                    8, 29, 9, 10, 1, 2))))
}

test_matched_sex_generation <- function(){
    genSexMatch <- unlist(FamAgg:::getGenerationSexMatched(pedigree(fad), id="4"),
                          use.names=FALSE)
    checkEquals(sort(genSexMatch), sort(as.character(c(4, 26, 27, 8, 9, 10))))
    ## when i use 2 it gets useless if the given ids have different sexes
    genSexMatch <- unlist(FamAgg:::getGenerationSexMatched(pedigree(fad),
                                                           id=c("4", "1")), use.names=FALSE)
    checkEquals(sort(genSexMatch), sort(as.character(c(25, 4, 5, 26, 6, 27, 7, 28, 8,
                                                       29, 9, 10, 1, 2))))
    ## it's different if both are from the same sex:
    genSexMatch <- unlist(FamAgg:::getGenerationSexMatched(pedigree(fad), id=c("25", "21")),
                          use.names=FALSE)
    checkEquals(sort(genSexMatch), sort(as.character(c(25, 5, 6, 7, 28, 29, 21))))
}

test_get_all <- function(){
    all <- unlist(FamAgg:::getAll(pedigree(fad), id="4"), use.names=FALSE)
    checkEquals(sort(all), sort(as.character(family(fad, family="4")[, "id"])))
}

test_sex_matched <- function(){
    females <- unlist(FamAgg:::getSexMatched(pedigree(fad), id="4"), use.names=FALSE)
    fam4 <- family(fad, "4")
    checkEquals(sort(females), sort(as.character(fam4[fam4$sex == "F", "id"])))
    males <- unlist(FamAgg:::getSexMatched(pedigree(fad), id="1"), use.names=FALSE)
    checkEquals(sort(males), sort(as.character(fam4[fam4$sex == "M", "id"])))
}

test_external_matched <- function(){
    ## use sex for matching; can compare that then to sex matching.
    extmatch <- unlist(FamAgg:::getExternalMatched(pedigree(fad), id="4",
                                                   match.using=fad$sex), use.names=FALSE)
    sexmatch <- unlist(FamAgg:::getSexMatched(pedigree(fad), id="4"),
                       use.names=FALSE)
    checkEquals(sort(extmatch), sort(sexmatch))
}


notrun_test_matched_generation_n_frequency <- function(){
    ## here we want to test getting generation matched controls for the ids and
    ## considering also the frequency of generation of the ids.
    ## data(minnbreast)
    ## ped <- minnbreast[, c("famid", "id", "fatherid", "motherid", "sex")]
    ## colnames(ped) <- c("family", "id", "father", "mother", "sex")
    ## fam4 <- ped[ped$family == "4", ]
    ## fad <- FAData(ped[ped$family %in% c(4:20), ])

    genFreq <- FamAgg:::getGenerationMatchedFrequency(fam4, id=c("1", "5", "6", "22"))
    genFreq <- unlist(genFreq, use.names=FALSE)
    genFreq <- table(genFreq)
    ## the control ids:
    names(genFreq)
    ## the frequency
    as.numeric(genFreq)
}

notrun_test_sex_match <- function(){
    affIds <- c("4", "11")
    idL <- FamAgg:::getGenerationMatchedByIndividual(pedigree(fad), id=affIds)
    SexMatched <- FamAgg:::doSexMatch(pedigree(fad), idList=idL)
    haveS <- fad$sex[names(SexMatched)[1]]
    needS <- fad$sex[SexMatched[[1]]]
    checkEquals(all(as.character(needS)==as.character(haveS)), TRUE)
    haveS <- fad$sex[names(SexMatched)[2]]
    needS <- fad$sex[SexMatched[[2]]]
    checkEquals(all(as.character(needS)==as.character(haveS)), TRUE)

    ## sexmatched
    matches <- FamAgg:::getSexMatchedByIndividual(ped=family(fad, "4"), id=affIds)
    haveS <- fad$sex[names(matches)[1]]
    needS <- fad$sex[matches[[1]]]
    checkEquals(all(as.character(needS)==as.character(haveS)), TRUE)
    ## next
    haveS <- fad$sex[names(matches)[2]]
    needS <- fad$sex[matches[[2]]]
    checkEquals(all(as.character(needS)==as.character(haveS)), TRUE)

}

notrun_test_stratsample <- function(){
    ## hm, check stratsample...
    Test <- family(fad, "4")
    ## require(sampling)
    ## strata(data=Test, stratanames="sex", size=c(M=1, F=4))
    ## Test <- Test[Test$sex=="M", ]
    ## strata(data=Test, stratanames="sex", size=c(M=2, F=3), method="srswor")

    ## Test <- family(fad, "4")
    ## Test <- Test[Test$sex=="F", ]
    ## strata(data=Test, stratanames="sex", size=c(M=2, F=3), method="srswor")

    ## Test <- family(fad, "4")
    ## strata(data=Test, stratanames="sex", size=c(M=2, F=3), method="srswor")
    ## Test <- Test[order(as.character(Test$sex)), ]
    ## strata(data=Test, stratanames="sex", size=c(M=2, F=3), method="srswor")
    ## ## Oh man. that's not good...
    ## ## what if we had NAs???
    ## ## have to exclude those guys... dammit
    ## Test[1:3, "sex"] <- NA
    ## strata(data=Test, stratanames="sex", size=c(M=2, F=3), method="srswor")
    ## what with str
    library(survey)
    idx <- stratsample(Test$sex, c(M=4, F=3))
    Test$sex[idx]
    Test$sex[stratsample(Test$sex, c(M=4, F=3))]

    Test$sex[stratsample(Test$sex, c(F=3, M=6))]

    ## make 1 male 2 females:
    affIds <- c("5", "8", "20")
    counts <- table(Test[affIds, "sex"])
    idx <- stratsample(Test$sex, counts)
    Test[idx, "sex"]

    ## only females
    affIds <- c("4", "8", "20")
    counts <- table(Test[affIds, "sex"])
    counts <- counts[counts > 0]
    idx <- stratsample(Test$sex, counts)
    Test[idx, "sex"]

    ## do it on the full table
    ##ped <- ped[!is.na(ped$sex), ]
    ##system.time(ped$sex[stratsample(ped$sex, c(M=4, F=3))])
    ##system.time(strata(ped, stratanames="sex", size=c(M=4, F=3), method="srswor"))
    ## stratsample is way faster.
}

## just simple utility function to test for empty/not set column names
noColNames <- function(x){
    CN <- colnames(x)
    all(CN %in% paste("V", 1:ncol(x), sep=""))
}

## takes a pedigree (data.frame, pedigree or pedigreeList) as input and
## generates a valid pedigree for the submitted ids (if no ids are specified
## it returns the ped). basically, the function subsets the full pedigree to
## the individuals ids and recursively adds parents if needed.
buildPedigree <- function(ped, ids){
    ## check pedigree and eventually add missing parents...
    if(missing(ids))
        return(ped)
    ## want to have a data.frame or matrix...
    if(is(ped, "pedigree") | is(ped, "pedigreeList"))
        ped <- ped2df(ped)
    if(!all(ids %in% rownames(ped)))
        stop("Some of the submitted ids are not in the pedigree!")
    subfam <- ped[as.character(ids), ]
    ## now recursively add parents, if needed...
    parents <- as.character(unique(c(subfam$father, subfam$mother)))
    ## parents[is.na(parents)] <- 0
    ## parents <- parents[parents!=0]
    parents <- parents[!is.na(parents)]
    while(!all(parents %in% as.character(subfam$id))){
        subfam <- ped[unique(c(as.character(subfam$id), parents)), ]
        parents <- as.character(unique(c(subfam$father, subfam$mother)))
        ## parents[is.na(parents)] <- 0
        ## parents <- parents[parents!=0]
        parents <- parents[!is.na(parents)]
    }
    subfam
}


doGetSiblings <- function(ped, id = NULL) {
    if(is.null(id))
        stop("At least one if has to be specified!")
    parents <- doGetParents(ped, id = id)
    siblings <- doGetChildren(ped, id = parents, maxlevel = 1)
    siblings
}

## gets all the ancestors for (one or more) id(s)
doGetAncestors <- function(ped, id = NULL, maxlevel = 3) {
    if (is.null(id))
        stop("At least one id has to be specified!")
    id <- as.character(id)
    allids <- NULL
    for (i in 1:maxlevel) {
        ## get the parents of all the allids
        newids <- unlist(
            ped[which(ped$id %in% id), c("father", "mother")],
            use.names = FALSE
        )
        zeros <- is.na(newids)
        ##zeros <- newids == founder
        if(all(zeros) | length(newids)==0)
            break
        newids <- newids[!zeros]
        id <- newids
        allids <- c(allids, newids)
    }
    as.character(unique(allids))
}

doGetParents <- function(ped, id = NULL) {
    doGetAncestors(ped = ped, id = id, maxlevel = 1)
}

## gets all the children for (one or more) id(s)
## note: these are only children in direct blood line.
doGetChildren <- function(ped, id=NULL, maxlevel=16){
    if(is.null(id))
        stop("At least one id has to be specified!")
    allids <- NULL
    for(i in 1:maxlevel){
        ## get all the children for the ids
        newids <- ped[which(ped$father %in% id | ped$mother %in% id), "id"]
        if(length(newids)==0)
            break
        id <- newids
        allids <- c(allids, newids)
    }
    as.character(unique(allids))
}

## check for the submitted id(s) whether a mate (spouse) is
## available (in column mother or father of the pedigree ped)
## and if yes add them to the returned list of ids.
## returned value contains
## eventually missing mates.
doGetMissingMate <- function(ped, id) {
    if(is.null(id))
        stop("Al least one id has to be specified")
    newids <- c(ped[which(ped$father %in% id), "mother"],
                ped[which(ped$mother %in% id), "father"])
    unique(newids)
}

writePed <- function(ped, file){
    Sexes <- as.numeric(sanitizeSex(ped$sex))
    Sexes[is.na(Sexes)] <- 0
    ped$sex <- Sexes
    write.table(ped, file=file, quote=FALSE, sep="\t", row.names=FALSE,
                col.names=FALSE)
}

## creates a data.frame (or matrix) for a pedigree...
ped2df <- function(ped){
    ped <- do.call(cbind, ped)
    if(!any(colnames(ped)=="famid"))
        ped <- cbind(ped, famid=rep(1, nrow(ped)))
    ped <- ped[ , c("famid", "id", "findex", "mindex", "sex")]
    ## FIXME!!! Can I get NAs as findex too?
    ## have to rematch the fater id and mother id...
    notZ <- which(ped[, "findex"] > 0)
    ped[notZ, "findex"] <- ped[ped[notZ, "findex"], "id"]
    notZ <- which(ped[, "findex"] > 0)
    ped[notZ, "mindex"] <- ped[ped[notZ, "mindex"], "id"]
    colnames(ped) <- .PEDCN
    ped <- data.frame(ped)
    rownames(ped) <- ped$id
    ped
}


df2ped <- function(ped){
    ped <- checkPedCol(ped)
    if(!any(colnames(ped)=="family"))
        ped <- cbind(ped, family=rep(1, nrow(ped)))
    ## if there is only one family, remove that one ;)
    if(length(unique(ped$family)) == 1)
        ped <- ped[, colnames(ped)!="family"]
    ##
    if(any(colnames(ped) == "affected")){
        if(any(colnames(ped) == "family")){
            peddi <- kinship2::pedigree(id=ped$id, dadid=ped$father,
                                        momid=ped$mother, famid=ped$family,
                                        sex=as.numeric(ped$sex),
                                        affected=ped$affected)
        }else{
            peddi <- kinship2::pedigree(id=ped$id, dadid=ped$father,
                                        momid=ped$mother,
                                        sex=as.numeric(ped$sex),
                                        affected=ped$affected)
        }
    }else{
        if(any(colnames(ped) == "family")){
            peddi <- kinship2::pedigree(id=ped$id, dadid=ped$father,
                                        momid=ped$mother, famid=ped$family,
                                        sex=as.numeric(ped$sex))
        }else{
            peddi <- kinship2::pedigree(id=ped$id, dadid=ped$father,
                                        momid=ped$mother,
                                        sex=as.numeric(ped$sex))
        }
    }
    peddi
}


## finds related individuals, i.e. individuals sharing the same blood line.
## should we call this "shareBloodLine" or "shareKinship"
## ped has to be the data.frame that we get with the pedigree method.
## Use `rmKinship` for fine control: omit all individuals with kinship less
## than or equal to this paramter. Since it defaults to 0, all individuals
## sharing blood line will be reported by default.
doShareKinship <- function(ped, kin, id, rmKinship=0){
    if(missing(ped) & missing(kin))
        stop("ped or kin has to be submitted!")
    if(missing(id))
        stop("At least one id has to be submitted!")
    id <- as.character(id)
    ## calculate kin from ped
    if(missing(kin)){
        kin <- kinship(id=ped$id, momid=ped$mother, dadid=ped$father,
                    sex=ped$sex)
    }
    if(!all(id %in% colnames(kin)))
        stop("id has to be present in ped or kin!")
    ## subset to id (which can be more than one)
    kin <- kin[id, , drop=FALSE]
    related <- colnames(kin)[colSums(kin>rmKinship)>0]
    related
}

####============================================================
##  doGetFounders
##
##  Returns the ids of the founders in the specified pedigree.
####------------------------------------------------------------
doGetFounders <- function(ped){
    ped[is.na(ped$mother) & is.na(ped$father), "id"]
}

####============================================================
##  doGetSingletons
##
##  Return the ids of those founders that have no child in the
##  pedigree.
####------------------------------------------------------------
doGetSingletons <- function(ped){
    founders <- ped[is.na(ped$mother) & is.na(ped$father), , drop=FALSE]
    if(nrow(founders) > 0){
        childl <- !((founders$id %in% ped$father) |
                    (founders$id %in% ped$mother))
        if(any(childl)){
            return(founders[childl, "id"])
        }
    }
    character()
}


## gets all parents.
getParents <- function(ped, id){
    if(is(ped, "pedigree") | is(ped, "pedigreeList"))
        ped <- ped2df(ped)
    ped <- checkPedCol(ped)
    if(missing(id))
        stop("id is missing!")
    if(is.null(rownames(ped)))
        rownames(ped) <- ped[, "id"]
    id <- as.character(id)
    if(!is.data.frame(ped))
        ped <- as.data.frame(ped)
    ped[id, c("father", "mother"), drop=FALSE]
}


## just a simple helper function that checks that the ped parameter
## has the required colnames, is a data.frame and has rownames set.
checkPedCol <- function(ped){
    if(missing(ped))
        stop("ped is required!")
    if(!all(c("id", "father", "mother") %in% colnames(ped)))
        stop("id, father, mother are required columns in ped.")
    if(!is.data.frame(ped))
        ped <- as.data.frame(ped)
    ## add a family column if not present...
    if(!any(colnames(ped) == "family"))
        ped <- cbind(ped, family=rep(1, nrow(ped)))
    rownames(ped) <- as.character(ped$id)
    ped
}

## Sanitize the pedigree the way we want: sex should be sanitized (if present),
## father and mother column should contain NA for founders instead of "" or 0.
sanitizePed <- function(ped){
    if(any(colnames(ped) == "sex"))
        ped$sex <- sanitizeSex(ped$sex)
    Father <- ped$father
    if(is.factor(Father))
        Father <- as.character(Father)
    if(is.numeric(Father))
        Father[which(Father == 0)] <- NA
    if(is.character(Father))
        Father[which(Father == "")] <- NA
    Mother <- ped$mother
    if(is.factor(Mother))
        Mother <- as.character(Mother)
    if(is.numeric(Mother))
        Mother[which(Mother == 0)] <- NA
    if(is.character(Mother))
        Mother[which(Mother == "")] <- NA
    ped$father <- Father
    ped$mother <- Mother
    ped
}

## Add density from values x to a density d computed by a previous call to this
## function. Make the initial call by omitting d.
## Resulting density of the partitioned dataset is a (pretty decent)
## approximation of the density obtained when submitting the whole dataset.
inc.density <- function(x, d = NULL)
{
    if( is.null(d) ) {
        ## adjust has been determined empirically.
        dd <- density(x = x, n = 512, adjust = 0.75)
    } else {
        if( length(d$x)!=512 )
            stop("Argument d has not been created by a call to this function.")
        ## Determine the density data structure that has the larger domain and
        ## add interpolated density values from the other data structure. By
        ## this, the domain can grow and will never omit values that appear
        ## later in the iterative process.
        dd <- density(x, bw = d$bw, n = 512)
        if( dd$x[512]<d$x[512] ) {
            tmp <- dd; dd <- d; d <- tmp; rm(tmp)
        }
        delta <- dd$x[2] - dd$x[1]
        f <- approxfun(d$x, d$y)
        y <- f(dd$x)
        y[is.na(y)] <- 0
        dd$y <- dd$y + y
        dd$y <- dd$y/(delta*sum(dd$y))
        dd$n <- dd$n + d$n
    }
    dd
}

## Add histogram from values x to a histogram h computed by a previous call to
## this function. Additionally, reports the number of values that produced an
## overflow in variable nreplace (i.e. values did were too large to be included
## in any bin of the histogram h). Make the initial call by omitting h.
##
## Resulting histogram of the partitioned dataset is a (pretty decent)
## approximation of the histogram obtained when submitting the whole dataset.
## Drawback: the domain of the histogram cannot be changed during addition of
## new values. We therefore rely on the first batch as a random sample of the
## population and estimate the overall domain.
inc.hist <- function(x, h = NULL)
{
    if( is.null(h) ) {
        ## Empirically determined overall size of the domain (110%).
        hh <- hist(x, breaks = 1/127*(0:127)*max(x)*1.1, plot = FALSE)
        hh$nreplace <- 0
    } else {
        if( is.null(h$nreplace) )
            stop("Argument h has not been created by a call to this function.")
        maxv <- h$breaks[length(h$breaks)]
        nreplace <- sum(x>maxv)
        if( nreplace>0 )
            x[x>maxv] <- maxv
        hh <- hist(x, breaks = h$breaks, plot = FALSE)
        hh$counts <- hh$counts + h$counts
        hh$density <- (hh$counts/sum(hh$counts))/diff(hh$breaks)
        hh$nreplace <- h$nreplace + nreplace
    }
    hh
}

## Add tabulated data from values x to a table t computed by a previous call to
## this function. Make the initial call by omitting t.
## This function produces identical results when used in the iterative way
## compared to submitting the whole dataset at once.
inc.table <- function(x, t = NULL)
{
    if( is.null(t) ) {
        tt <- table(x)
    } else {
        tt <- table(x)
        both <- intersect(names(t), names(tt))
        tt[both] <- tt[both] + t[both]
        tonly <- setdiff(names(t), names(tt))
        tt[tonly] <- t[tonly]
    }
    tt
}

## Convert 1D count table with only integers as counted objects to a histogram.
table2hist.int <- function(t)
{
    ## Only 1D tables have names(), for higher dimensional tables, it is set to
    ## NULL. dimnames() OTH works opposite.
    nms <- names(t)
    stopifnot("Table must be one-dimensional" = length(nms)>=1)
    suppressWarnings( bins <- as.integer(names(t)) )
    stopifnot("Table can be integer only." = all(as.character(bins)==nms))
    h <- list(breaks = c(bins, max(bins) + 1),
              counts = as.vector(t),
              mids = bins + 0.5,
              xname = "generic",
              equidist = TRUE)
    attr(h, "class") <- "histogram"
    h
}


#' find the subPedigree (smallest pedigree) including the individuals specified
#' with id and all eventually needed additionals
#'
#' @param ped pedigree `data.frame`.
#'
#' @param id `character` with the IDs of the individuals.
#'
#' @param all `logical(1)`
#'
#' @md
#'
#' @noRd
#'
#' @author Johannes Rainer
subPedigree <- function(ped, id = NULL, all = TRUE) {
    CL <- class(ped)
    ## handling for pedigree and pedigreeList classes: unfortunately these
    ## are not correct classes, otherwise I could define methods instead of
    ## a function...
    if (is(ped, "pedigree") | is(ped, "pedigreeList")) {
        ped <- ped2df(ped)
    }
    ped <- checkPedCol(ped)
    if (is.null(id))
        stop("id is missing!")
    id <- as.character(id)
    subgr <- connectedSubgraph(ped2graph(ped), nodes = id, mode = "all",
                               all.nodes = all, ifnotfound = NA)
    if (!is(subgr, "igraph")) {
        stop("No common pedigree for the submitted individuals found!")
    }
    inds <- names(V(subgr))
    ## get missing mates
    inds <- unique(c(inds, doGetMissingMate(ped, id = inds)))
    subped <- ped[inds, ]
    subped <- removeSingletons(subped)
    ## fix founders.
    subped[!(subped$father %in% subped$id), "father"] <- NA
    subped[!(subped$mother %in% subped$id), "mother"] <- NA
    ## Fix if we have individuals with a mother or a father, but not both.
    idx <- which(is.na(subped$father) != is.na(subped$mother))
    if (length(idx)) {
        idx_ped <- match(subped$id[idx], ped$id)
        id_msng <- ped$mother[idx_ped][is.na(subped$mother[idx])]
        id_msng <- c(id_msng, ped$father[idx_ped][is.na(subped$father[idx])])
        subped$father[idx] <- ped$father[idx_ped]
        subped$mother[idx] <- ped$mother[idx_ped]
        if (length(id_msng)) {
            subped <- rbind(subped, ped[match(id_msng, ped$id), ])
            subped[!(subped$father %in% subped$id), "father"] <- NA
            subped[!(subped$mother %in% subped$id), "mother"] <- NA
        }
    }
    if(CL == "pedigree" | CL == "pedigreeList")
        subped <- df2ped(subped)
    subped
}


## find the (eventually smallest) connected subgraph of all specified
## nodes.
## graph: igraph object.
## nodes: node (vertex) names.
## mode: to calculate the smallest distance.
## all.nodes: if all nodes should be present in the resulting graph.
## ifnotfound: by default, the function throws an error if no connected
##             subgraph is found, or, if all.nodes=TRUE, not all input nodes
##             are in the connected subgraph. If ifnotfound is specified, its
##             value is returned instead of an error.
connectedSubgraph <- function(graph, nodes, mode="all", all.nodes=TRUE,
                              ifnotfound){
    if(!is(graph, "igraph"))
        stop("graph has to be an igraph object!")
    mode <- match.arg(mode, c("all", "in", "out"))
    if(missing(nodes) | length(nodes) < 2)
        stop("At least two nodes have to be submitted!")
    nodes <- as.character(nodes)
    ## check if nodes are present in the graph
    nono <- !(nodes %in% names(V(graph)))
    if(any(nono)){
        missingNodes <- nodes[nono]
        if(all.nodes)
            stop("With argument all.nodes=TRUE all nodes have to be present ",
                 "in the graph! Nodes ", paste(missingNodes, collapse=", "),
                 " not present in the original graph.")
        nodes <- nodes[!nono]
        warning("Nodes ", paste(missingNodes, collapse=", "),
                " removed as they are not present in graph! ")
        if(length(nodes) < 2)
            stop("At least two nodes have to be present in the graph!")
    }
    ## OK, now we can start...
    ## 1) iterate over all nodes and calculate the shortest path between all.
    ## 2) get all nodes along the shortest path and add to the needed nodes.
    needNodes <- character()
    ## two versions: if mode="all" the directionality does not matter, thus we
    ## can run a faster version. for the other modes really calculate distances
    ## between all (both directions).
    if(mode == "all"){
        for(i in 1:(length(nodes)-1)){
            suppressWarnings(
                paths <- shortest_paths(graph, from=nodes[i],
                                        to=nodes[(i+1):length(nodes)],
                                        mode=mode)
            )
            needNodes <- unique(c(needNodes,
                                  unlist(lapply(paths$vpath, names))))
        }
    }else{
        for(i in 1:length(nodes)){
            suppressWarnings(
                paths <- shortest_paths(graph, from=nodes[i],
                                        to=nodes[-i],
                                        mode=mode)
            )
            needNodes <- unique(c(needNodes,
                                  unlist(lapply(paths$vpath, names))))
        }
    }
    if(length(needNodes)==0){
        if(missing(ifnotfound)){
            stop("Could not find any connected subgraph!")
        }
        warning("Could not find any connected subgraph!")
        return(ifnotfound)
    }
    ## check the subgraph.
    if(!all(nodes %in% needNodes)){
        if(all.nodes){
            if(missing(ifnotfound)){
                stop("Could not find any connected subgraph!")
            }
            warning("Could not find any connected subgraph!")
            return(ifnotfound)
        }else{
            warning("Nodes ", paste(nodes[!(nodes %in% needNodes)]),
                    " not present in the returned graph!")
        }
    }
    ## finally build the graph...
    ## subgraph is a little tedious as it requires submitting of vertex INDICES!
    induced_subgraph(graph, vids=needNodes)
}

## search for a common ancestor couple in a pedigree.
## ped: data.frame or pedigree or pedigreeList.
doGetCommonAncestor <- function(ped, id, method="min.dist"){
    method <- match.arg(method, c("min.dist", "smallest.mean.dist"))
    if(missing(id))
        stop("id has to be specified")
    if(length(id) <= 1)
        stop("The minimum number of ids has to be 2.")
    if(is(ped, "pedigree") | is(ped, "pedigreeList"))
        ped <- ped2df(ped)
    ped <- checkPedCol(ped)
    ## check if id are in ped:
    id <- as.character(id)
    if(!all(id %in% rownames(ped))){
        stop("all of the sumitted ids should be present in the pedigree!")
    }
    ## check if the ids are in the same family
    if(any(colnames(ped)=="family")){
        if(length(unique(ped[id, "family"])) > 1){
            warning("The provided ids are from different families; still",
                    " trying to find a common ancestor but I'll probably fail.")
        }else{
            ## subset the pedigree to this family...
            ped <- ped[ped$family == ped[id, "family"][1], , drop=FALSE]
        }
    }
    ## OK; done with input parameter checking.
    ## transform the pedigree into a graph
    igr <- ped2graph(ped)
    ## "in": go from children -> parents
    ## LLLL TODO can do faster!!!
    din <- distances(igr, mode="in")
    ## use the ROWS in that.
    subD <- din[id, , drop=FALSE]
    subD <- subD[, !apply(subD, MARGIN=2, function(x){
        return(all(infOrZero(x)))
    }) , drop=FALSE]
    ## check if we've got one common ancestor:
    if(all(is.infinite(colSums(subD)))){
        warning("Did not find any common ancestor!")
        return(NA)
    }
    ## remains to find the "one and only" (well, usually we expect a couple).
    subD <- subD[, !is.infinite(colSums(subD)), drop=FALSE]
    ## now get the ones that are closest:
    if(method == "min.dist"){
        dists <- apply(subD, MARGIN=2, min)
        ancs <- names(dists)[dists==min(dists)]
    }
    if(method == "smallest.mean.dist"){
        dists <- colMeans(subD)
        ancs <- names(dists)[dists==min(dists)]
    }
    ## at last trying to get the mate... if any; usually, ancs should
    ## already correspond to the ancestor couple.
    ancs <- c(ancs, doGetMissingMate(ped, ancs))
    unique(ancs)
}

## returns true if a value is 0 or infinite
infOrZero <- function(x){
    (is.infinite(x) | x == 0)
}

## transform a pedigree into a (directed) igraph
## direction is always parent->child.
## this time each individual is treated as a single node!
ped2graph <- function(ped){
    if(is(ped, "pedigree") | is(ped, "pedigreeList"))
        ped <- ped2df(ped)
    ped <- checkPedCol(ped)
    ## define end/starting points
    ped[!ped[, "father"] %in% ped$id, "father"] <- NA
    ped[!ped[, "mother"] %in% ped$id, "mother"] <- NA
    eds <- apply(ped[, c("id", "father", "mother")], MARGIN=1, function(x){
        retval <- c()
        if(!is.na(x["father"]))
            retval <- x[c("father", "id")]
        if(!is.na(x["mother"]))
            retval <- c(retval, x[c("mother", "id")])
        return(retval)
    })
    ## what the heck... I need character vertex names!
    Edges <- unlist(eds, use.names=FALSE)
    igr <- graph(edges=as.character(Edges))
    igr
}


## takes a pedigree as input and tries to identify the founders
## in case there are more than two founder pairs (i.e. founders with the same,
## largest number of offspring generations), it returns only one of them.
doFindFounders <- function(ped, family, id){
    if(!any(colnames(ped) == "family"))
        ped <- data.frame(ped, family=rep(1, nrow(ped)), stringsAsFactors=FALSE)
    families <- unique(as.character(ped$family))
    if(missing(family))
        family <- NULL
    if (missing(id))
        id <- NULL
    if(is.null(family) & is.null(id)){
        family <- families[1]
        warning("No family specified; using the first one in the pedigree: ",
                family, ".")
    }
    if (!is.null(family))
        ped <- ped[ped$family == family, , drop=FALSE]
    if (!is.null(id)) {
        return(doFindFoundersForId(ped = ped, id = id))
    } else {
        founders <- ped$id[which(is.na(ped$father) & is.na(ped$mother))]
        ## determine for each founder the number of generations of its children,
        ## grandchildren etc.
        founderGen <- doCountGenerations(ped, id=founders, direction="down")
        founders <- names(founderGen)[founderGen == max(founderGen)][1]
        founders <- c(founders, doGetMissingMate(ped, founders))
        return(unique(founders))
    }
}

##' Find founders for the pedigree of a given individual. Have to first build
##' the largest pedigree for the given individual and identify the founders in
##' that pedigree.
##'
##' @note The function returns only a single founder pair, i.e. the one with the
##' largest number of offspring generations.
##' @noRd
doFindFoundersForId <- function(ped, id) {
    if (length(id) > 1) {
        id <- id[1]
        warning("length of 'id' was > 1, but only the first id was selected!")
    }
    ## Check if we've got this id in the pedigree
    if (!any(ped[, "id"] == id))
        stop("Individual with id '", id, "' not found in the pedigree!")
    ## Building the pedigree for that individual.
    ids <- unique(c(doGetAncestors(ped = ped, id = id, maxlevel = 200), id))
    ids <- unique(c(ids, doGetChildren(ped = ped, id = ids, maxlevel = 200)))
    ids <- unique(c(ids, doGetMissingMate(ped, id = ids)))
    ped <- ped[ped[, "id"] %in% ids, , drop = FALSE]
    ## OK, now we have the pedigree for the individual, now find founders.
    founders <- ped[which(is.na(ped[, "father"]) &
                          is.na(ped[, "mother"])), "id"]
    ## determine for each founder the number of generations of its children,
    ## grandchildren etc.
    founderGen <- doCountGenerations(ped, id = founders, direction = "down")
    founders <- names(founderGen)[founderGen == max(founderGen)][1]
    founders <- c(founders, doGetMissingMate(ped, founders))
    unique(founders)
}

## calculate for each id the number of generations up- or down.
doCountGenerations <- function(ped, id=NULL, direction="down"){
    if(is.null(id))
        stop("At least one id has to be specified!")
    direction <- match.arg(direction, c("up", "down"))
    if(direction == "up")
        walkFun <- doGetAncestors
    if(direction == "down")
        walkFun <- doGetChildren
    res <- sapply(id, function(x){
        generation <- 0
        ids <- x
        repeat{
            ids <- walkFun(ped, id=ids, maxlevel=1)
            if(length(ids) == 0){
                return(generation)
            }else{
                generation <- generation + 1
            }
        }
    })
    names(res) <- id
    res
}

## returns a list of generations for each family in the pedigree.
## generations are relative to the founders.
doEstimateGenerationsFor2 <- function(ped, family=NULL){
    if(!is.null(family)){
        ped <- ped[ped$family %in% family, ]
        if(nrow(ped) == 0)
            stop("The submitted family id is not present in the pedigree!")
    }
    ## OK, now loop through the families...
    pedList <- split(ped, ped$family)
    Gens <- lapply(pedList, function(currentPed){
        founders <- doFindFounders(currentPed, family=currentPed[1, "family"])
        generation <- doGetGenerationFrom2(currentPed, founders[1])
        return(generation)
    })
    Gens
}

getAllSibsNMates <- function(ped, id){
    ## recursively fetch siblings and mates
    sibsNmates <- id
    repeat{
        sibs <- unique(c(id, doGetSiblings(ped, id=id)))
        sibsNmates <- unique(c(doGetMissingMate(ped, id=sibs), sibs))
        if(length(sibsNmates) != length(id)){
            id <- sibsNmates
        }else{
            break
        }
    }
    sibsNmates
}

doGetGenerationFrom2 <- function(ped, id, generation=0){
    if(missing(id))
        stop("id has to be specified!")
    if(missing(ped))
        stop("ped has to be specified!")
    id <- as.character(id)
    if(length(id) > 1){
        id <- id[1]
        warning("Length of 'id' is larger 1. Using only the first id ",
                "in 'id': ", id)
    }
    if(!any(ped$id %in% id))
        stop("Can not find id in the pedigree!")
    famid <- ped[ped$id == id, "family"]
    ped <- ped[ped$family == famid, , drop=FALSE]
    ## first define the connected!!! pedigree.
    pedg <- ped2graph(ped)
    if(clusters(pedg)$no > 1){
        Cl <- clusters(pedg)
        warning("Found ", Cl$no, " connected sub-graphs in the pedigree ",
                "for family ", famid, "! Will restrict the analysis to the ",
                "subgraph containing individual ", id, ".")
        vertices <- names(Cl$membership)[Cl$membership == Cl$membership[id]]
        pedg <- induced_subgraph(pedg, vids=vertices)
    }
    individuals <- names(V(pedg))
    gens4individuals <- rep(NA, length(individuals))
    names(gens4individuals) <- individuals
    if(!any(names(gens4individuals) == id))
        stop("Individual ", id, " is not part of the connected pedigree!")
    gens4individuals[id] <- 0
    fromId <- id
    Tested <- rep(FALSE, length(gens4individuals))
    repeat{
        ## get the generation of the individuals from which we start
        fromIdBool <- individuals %in% fromId
        Tested[fromIdBool] <- TRUE
        currentGens <- gens4individuals[fromIdBool]
        currentGens <- split(currentGens, currentGens)
        Res <- lapply(currentGens, allids=individuals, FUN=function(z, allids){
            ## get the ancestors and set the generation for them
            ancs <- getAncestors(ped, id=names(z), max.generations=1)
            if(length(ancs) > 0){
                ancret <- cbind(which(allids %in% ancs), (z[1] - 1))
            }else{
                ancret <- NULL
            }
            ## get the children
            kids <- getChildren(ped, id=names(z), max.generation=1)
            if(length(kids) > 0){
                kidret <- cbind(which(allids %in% kids), (z[1] + 1))
            }else{
                kidret <- NULL
            }
            return(rbind(ancret, kidret))
        })
        Res <- do.call(rbind, Res)
        if(nrow(Res) == 0){
            warning("What the heck... stopped before we got all?")
            break
        }
        ## don't want to search the same id several times... if Tested is
        ## TRUE, don't use them again.
        toTestIdx <- Res[, 1][!Tested[Res[, 1]]]
        fromId <- individuals[toTestIdx]
        gens4individuals[Res[, 1]] <- Res[, 2]
        if(sum(is.na(gens4individuals)) == 0)
            break
    }
    ## make a larger generation vector.
    allgens <- rep(NA, nrow(ped))
    names(allgens) <- ped$id
    allgens[names(gens4individuals)] <- gens4individuals
    allgens
}

##********************************************************************
##
##   Utility functions
##
##********************************************************************
sanitizeSex <- function(x){
    if(is.numeric(x)){
        ## require values 1, 2, NA
        numrange <- unique(x)
        if(!all(c(1, 2) %in% numrange))
            stop("If column sex is numeric it has to contain 1 (=male) ",
                 "and 2 (=female) values! Unknown or missing can be encoded",
                 " by NA or any other number than 1 or 2.")
        news <- rep(NA, length(x))
        news[which(x == 1)] <- "M"
        news[which(x == 2)] <- "F"
        return(factor(news, levels=c("M", "F")))
    }
    if(is.factor(x))
        x <- as.character(x)
    if(is.character(x)){
        ## remove all white spaces:
        x <- gsub(x, pattern="\\s", replacement="")
        ## that's tricky...kind of...
        ## just get the first character; want to have M and F...
        x <- substr(toupper(x), 1, 1)
        chars <- unique(x)
        if(length(chars) == 2){
            if(!all(c("M", "F") %in% chars))
                stop("Male and female individuals have to be represented",
                     " by 'M' and 'F', respectively!")
        }
        if(length(chars) == 1){
            if(!any(c("M", "F") %in% chars))
                stop("Male and female individuals have to be represented",
                     " by 'M' and 'F', respectively!")
        }
        if(length(chars[!is.na(chars)]) > 2)
            warning("All characters in column sex other than M and F",
                    " (or male, female etc) are set to NA!")
        news <- rep(NA, length(x))
        news[which(x == "M")] <- "M"
        news[which(x == "F")] <- "F"
        return(factor(news, levels=c("M", "F")))
    }
}


#####
#### NOTE:
##   isn't that the same as removeSingletons???
##
####
## remove non-connected individuals. We're assuming that ped is the pedigree
## of a SINGLE family, as we expect to get ONE graph. in case we've got 2, we're
## using the largest connected one.
## solveMultiGraph: what should we do if we get multiple graphs?
##    + use.all: just use all, i.e. return vertix names of all.
##    + use.largest: just return vertices of the largest connected sub-graph.
doPrunePed <- function(ped, addMissingMates=FALSE, solveMultiGraph="use.all"){
    .Deprecated("removeSingletons")
    solveMultiGraph <- match.arg(solveMultiGraph, c("use.all", "use.largest"))
    pedg <- ped2graph(ped)
    if(solveMultiGraph == "use.largest"){
        if(clusters(pedg)$no > 1){
            Cl <- clusters(pedg)
            warning("Found ", Cl$no,
                    " connected sub-graphs in the pedigree! Will restrict ",
                    "the analysis to the largest connected pedigree",
                    " in that family.")
            vertices <-
                names(Cl$membership)[Cl$membership ==
                                     as.numeric(sort(table(Cl$membership),
                                                     decreasing=TRUE))[1]]
            pedg <- induced_subgraph(pedg, vids=vertices)
        }
    }
    keepIds <- names(V(pedg))
    if(addMissingMates){
        missMates <- doGetMissingMate(ped, id=keepIds)
        keepIds <- unique(c(missMates, keepIds))
    }
    notThere <- !(as.character(ped$id) %in% keepIds)
    ped <- ped[!notThere, , drop=FALSE]
    if(any(notThere))
        warning("Had to remove ", sum(notThere),
                " un-connected individuals from the pedigree.")
    ped
}


## a singleton a.k.a childless founder is an individual which id is not
## present in the father or mother column of the pedigree and which has 0
## in mother and father
.removeSingletons <- function(ped){
    message("Removing singletons...", appendLF=FALSE)
    ped <- checkPedCol(ped)
    ped <- sanitizePed(ped)
    childlF <- doGetSingletons(ped)
    if(length(childlF) > 0){
        ped <- ped[!(ped$id %in% childlF), ]
        message(" ", length(childlF), " removed.")
    }else{
        message(" none present.")
    }
    ped
}

#' @title Extract pairs of individuals matching certain kinship criteria
#'
#' @name kinshipPairs
#'
#' @description
#'
#' The `kinshipPairs` function allows to extract pairs of individuals matching
#' a user-defined kinship *condition* (e.g. individuals with a kinship larger
#' than 0.0625). Such sets of paired individuals (along with paired unrelated
#' values) would enable a *familial resemblance* analysis on quantitative
#' traits (Ziegler 2010) (see examples below for details).
#'
#' By default, `kinshipPairs` returns all pairs of individuals for which the
#' `condition` on the kinship matrix matches (e.g. all pairs of individuals with
#' a kinship coefficient larger than or equal to 0.25). Individuals can thus
#' be reported multiple times (see examples below). Parameter `duplicates` can
#' be used to define a strategy to avoid such duplicated IDs. Supported are:
#'
#' - `duplicates = "keep"`: the default, return all values.
#' - `duplicates = "first"`: report only the first pair of individuals for each
#'   individual ID.
#' - `duplicates = "last"`: report only the last pair of individuals for each
#'   individual ID.
#' - `duplicates = "random"`: randomly select one pair of individuals for
#'   each individual ID.
#'
#' For any setting different than `duplicates = "keep"` each individual will
#' only be listed **once** in the resulting matrix.
#'
#' @param x A `FAData` object (or object inheriting from that).
#'
#' @param condition A `function` defining how individuals should be selected
#'     based on the object's kinship matrix. The default is to select all
#'     individuals with a kinship `>= 0.25`. Note that the diagonal of the
#'     kinship matrix (i.e. the kinship of individuals with itself) is always
#'     skipped, so no additional criteria is needed to avoid self-pairs.
#'
#' @param duplicates `character(1)` defining how to deal with duplicated IDs
#'     in the result returned by the function.
#'     See function description and examples below for more details. Defaults
#'     to `duplicates = "keep"` returning all pairs of IDs matching `condition`.
#'
#' @param family optional family identifiers if pairs should only defined for
#'     selected families. Defaults to `family = NULL` hence the full data set is
#'     considered.
#'
#' @param id optional identifiers of subsets of individuals on which the pairs
#'     should be defined. Defaults to `id = NULL` hence the full data set is
#'     considered.
#'
#' @return
#'
#' A two column `matrix` with the IDs (colnames/rownames of the kinship matrix
#' or as defined in `x$id`) of the pairs. If `duplicates` is either `"first"`,
#' `"last"` or `"random"` each ID is only returned once (i.e. no ID is reported
#' more than one time).
#'
#' @author Johannes Rainer
#'
#' @references
#'
#' Ziegler A., Koenig I. R. (2010). Familiality, Heristability, and Segregation
#' Analysis. In A Statistical Approach to Genetic Epidemiology: With Access to
#' E-Learning Platform by Friedrich Pahlke, Second Edition.
#' \doi{10.1002/9783527633654.ch6}.
#'
#' @seealso [PedigreeUtils] for other pedigree utility functions.
#'
#' @md
#'
#' @examples
#'
#' ##########################
#' ##
#' ##  Create a new FAData object
#' ##
#' ## Load the Minnesota Breast Cancer record and subset to the
#' ## first families.
#' data(minnbreast)
#' mbsub <- minnbreast[minnbreast$famid %in% 1:20, ]
#' mbped <- mbsub[, c("famid", "id", "fatherid", "motherid", "sex")]
#' ## Renaming column names
#' colnames(mbped) <- c("family", "id", "father", "mother", "sex")
#' ## Defining the optional argument age.
#' Age <- mbsub$endage
#' names(Age) <- mbsub$id
#' ## Create the object
#' fad <- FAData(pedigree=mbped, age=Age)
#'
#' ## Getting all pairs of individuals with a kinship coefficient >= 0.25
#' ## keeping all duplicates
#' rel_pairs <- kinshipPairs(fad)
#' head(rel_pairs)
#' ## As we see, we have multiple times the individual 1 etc.
#'
#' ## For an actual correlation analysis it would be better to drop duplicates.
#' ## Below we randomly select individual pairs if they occurr multiple times
#' rel_pairs <- kinshipPairs(fad, duplicates = "random")
#' head(rel_pairs)
#'
#' ## In addition we extract pairs of individuals that are much less related.
#' ## For this examples we consider all individuals with a kinship
#' ## coefficient < 0.03125 (second cousin) to be *unrelated*.
#' unrel_pairs <- kinshipPairs(fad, duplicates = "random",
#'     condition = function(z) z < 0.03125)
#' head(unrel_pairs)
#'
#' ## For a familial resemblance analysis we can now calculate the correlation
#' ## coefficient of a quantitative trait between pairs of related individuals
#' ## and compare that with the correlation coefficient calculated on unrelated
#' ## individuals. For our toy example we use the participant's age, since we
#' ## don't have any other quantitative values available.
#' cor_rel <- cor(age(fad)[rel_pairs[, 1]], age(fad)[rel_pairs[, 2]],
#'     use = "pairwise.complete.obs")
#'
#' cor_unrel <- cor(age(fad)[unrel_pairs[, 1]], age(fad)[unrel_pairs[, 2]],
#'     use = "pairwise.complete.obs")
#' cor_rel
#' cor_unrel
#'
#' ## We don't see a clear difference in the correlation, thus, the age (as
#' ## expected) has no familial component.
kinshipPairs <- function(x, condition = function(x) x >= 0.25,
                         duplicates = c("keep", "first", "last", "random"),
                         id = NULL, family = NULL) {
    duplicates <- match.arg(duplicates)
    if (!inherits(x, "FAData"))
        stop("'kinshipPairs' is currently only implemented for 'FAData'")
    if (length(family)) {
        id <- x$id[x$family %in% family]
        if (!length(id))
            stop("No individuals for the provided families found")
    }
    ks <- kinship(x)
    if (length(id)) {
        is_id <- rownames(ks) %in% id
        if (!any(is_id))
            stop("None of the specified individuals found")
        ks <- ks[is_id, is_id, drop = FALSE]
    }
    res <- condition(ks)
    if (!length(dim(res)) || ncol(ks) != ncol(res) ||
        nrow(ks) != nrow(res) || !is.logical(res[1, 1]))
        stop("'condition' function did not return a 'logical' 'matrix' with ",
             "dimensions matching the kinship matrix in 'x'")
    res[lower.tri(res, diag = TRUE)] <- NA
    keep <- which(res, arr.ind = TRUE, useNames = FALSE)

    if (nrow(keep)) {
        if (duplicates != "keep") {
            if (duplicates == "first")
                dupl_fun <- function(z) z[1, , drop = FALSE]
            if (duplicates == "last")
                dupl_fun <- function(z) z[nrow(z), , drop = FALSE]
            if (duplicates == "random")
                dupl_fun <- function(z)
                    z[sample(seq_len(nrow(z)), 1), , drop = FALSE]
            uids <- unique(as.integer(keep))
            to_test <- rep(TRUE, nrow(keep))
            res <- matrix(ncol = 2, nrow = length(uids))
            for (i in seq_along(uids)) {
                id <- uids[i]
                idx <- which((keep[, 1] == id | keep[, 2] == id) & to_test)
                if (length(idx)) {
                    pair <- dupl_fun(keep[idx, , drop = FALSE])
                    ## Ensure we're not reporting these individuals again.
                    to_test[keep[, 1] %in% pair | keep[, 2] %in% pair] <- FALSE
                    res[i, ] <- pair
                }
            }
            res <- res[!is.na(res[, 1]), , drop = FALSE]
            unname(cbind(rownames(ks)[res[, 1]], rownames(ks)[res[, 2]]))
        } else unname(cbind(rownames(ks)[keep[, 1]], rownames(ks)[keep[, 2]]))
    } else matrix(nrow = 0, ncol = 2)
}

.getMinSub <- function(){
    data(minnbreast)
    mbsub <- minnbreast[minnbreast$famid %in% 4:19, ]
    return(mbsub)
}

mbsub <- .getMinSub()

test_show <- function() {
    mbped <- mbsub[, c("famid", "id", "fatherid", "motherid", "sex")]
    colnames(mbped) <- FamAgg:::.PEDCN
    fad <- FAData(pedigree=mbped)
    res <- capture.output(show(fad))
    checkEquals(res[1], "FAData object with:")
}

test_construct <- function(){
    mbped <- mbsub[, c("famid", "id", "fatherid", "motherid", "sex")]
    colnames(mbped) <- FamAgg:::.PEDCN
    fad <- FAData(pedigree=mbped)
    tcancer <- mbsub$cancer
    names(tcancer) <- mbsub$id
    trait(fad) <- tcancer
    ## constructor with the trait
    fad <- FAData(pedigree=mbped, trait=mbsub$cancer)

    tcancershuffle <- sample(tcancer, length(tcancer))
    trait(fad) <- tcancershuffle
    ## the function internally has to rematch the ids
    checkEquals(trait(fad), tcancer)
    ## testing the $ accessor
    checkEquals(fad$trait, tcancer)
    fad@traitname <- "cancer"
    checkEquals(fad$cancer, tcancer)

    ## check if the pedigree returns the correct extended pedigree
    ## with the additional column affected
    ped <- pedigree(fad)
    fromPed <- ped$affected
    names(fromPed) <- ped$id
    checkEquals(fromPed, tcancer)

    ## test affected and phenotyped individuals
    affIds <- affectedIndividuals(fad)
    checkEquals(affIds, as.character(ped$id[which(ped$affected == 1)]))
    phenoIds <- phenotypedIndividuals(fad)
    checkEquals(phenoIds, as.character(ped$id[!is.na(ped$affected)]))

    ## check if a logical trait is handled properly
    tcancerrandom <- sample(c(rep(T, 3), rep(F, 7), NA), size=length(tcancer),
                            replace = TRUE)
    names(tcancerrandom) <- names(tcancer)
    fad1 <- FAData(pedigree=mbped, trait=tcancerrandom)
    fad2 <- FAData(pedigree=mbped)
    trait(fad2) <- tcancerrandom
    checkEquals(trait(fad1), trait(fad2))
    tcint <- as.integer(tcancerrandom)
    names(tcint) <- names(tcancerrandom)
    checkEquals(trait(fad1), tcint)
}

test_FAData_from_file <- function(){
    ## create from file
    pedfile <- system.file("txt/minnbreastsub.txt", package="FamAgg")
    checkException(fad2 <- FAData(pedfile, id.col="ID" ))
    fad2 <- FAData(pedfile, id.col="id", family.col="famid", father.col="fatherid",
                   mother.col="motherid", sex.col="sex", header=TRUE)
    ## Now read it from ped
    pedfile <- system.file("txt/minnbreastsub.ped.gz", package="FamAgg")
    fadPed <- FAData(pedfile)
    pedfile <- system.file("txt/minnbreastsub-bad.ped.gz", package="FamAgg")
    checkException(FAData(pedfile))
}


## Testing the validation methods
test_validate <- function(){
    tmpPed <- mbsub[, c("famid", "id", "fatherid", "motherid", "sex")]
    colnames(tmpPed) <- FamAgg:::.PEDCN
    ## Don't allow NAs in id, father or mother
    badPed <- tmpPed
    badPed[13, "id"] <- NA
    checkException(FAData(badPed))
    badPed <- tmpPed
    badPed[13, "father"] <- NA
    checkException(FAData(badPed))
    badPed <- tmpPed
    badPed[13, "mother"] <- NA
    checkException(FAData(badPed))
    ## Don't allow father or mother ids other than 0 or %in% $id
    badPed <- tmpPed
    badPed[13, "mother"] <- "123345"
    checkException(FAData(badPed))
    badPed <- tmpPed
    badPed[13, "father"] <- "123345"
    checkException(FAData(badPed))
}


## OK, 0.0.6
test_age <- function(){
    mbped <- mbsub[, c("famid", "id", "fatherid", "motherid", "sex")]
    colnames(mbped) <- FamAgg:::.PEDCN

    tmp <- new("FAData")
    ages <- mbsub$endage
    names(ages) <- mbped$id
    ## Setting ages to a empty object should not add age.
    age(tmp) <- ages
    checkEquals(length(age(tmp)), 0)
    checkTrue(validObject(tmp))
    ## With pedigree:
    pedigree(tmp) <- mbped
    checkTrue(validObject(tmp))
    age(tmp) <- ages
    checkEqualsNumeric(length(age(tmp)), length(ages))
    checkEqualsNumeric(length(age(tmp)), nrow(mbped))
    checkEquals(names(age(tmp)), as.character(mbped[, 2]))
}

test_pedigree <- function(){
    tmp <- new("FAData")

    ## mbsub <- .getMinSub()

    PedDf <- mbsub[, c("famid", "id", "fatherid", "motherid", "sex")]
    colnames(PedDf) <- FamAgg:::.PEDCN
    pedigree(tmp) <- PedDf
    checkTrue(is(pedigree(tmp, return.type="data.frame"), "data.frame"))
    tmp2 <- pedigree(tmp, return.type="data.frame")
    ## have to transform sex into a factor...
    PedDf$sex <- factor(PedDf$sex, levels=c("M", "F"))
    rownames(PedDf) <- as.character(PedDf$id)

    checkEquals(tmp2, FamAgg:::sanitizePed(PedDf))

    ## submit a pedigree(List) object...
    Ped <- pedigree(id=PedDf[, 2], famid=PedDf[, 1], dadid=PedDf[, 3],
                    momid=PedDf[, 4], sex=PedDf[, 5])
    pedigree(tmp) <- Ped
    checkTrue(is(pedigree(tmp, return.type="pedigree"), "pedigreeList"))
    ## check the Ped with the pedigree(tmp)
    tmp1 <- do.call(cbind, Ped)
    tmp2 <- do.call(cbind, pedigree(tmp, return.type="pedigree"))
    checkEquals(tmp1, tmp2)
}


test_subset <- function(){
    do.plot <- FALSE
    ## mbsub <- .getMinSub()

    mbped <- mbsub[, c("famid", "id", "fatherid", "motherid", "sex")]
    colnames(mbped) <- FamAgg:::.PEDCN

    ## using the constructor
    fad <- FAData(pedigree=mbped)
    ##----------------------------------
    ## subset using boolean
    fadSub <- fad[fad$family==4, ]
    pedSub <- mbped[mbped$family==4, ]
    pedSub$sex <- FamAgg:::sanitizeSex(pedSub$sex)
    pedSub <- FamAgg:::sanitizePed(pedSub)
    checkEquals(pedigree(fadSub), pedSub)
    ## checking kinship
    kinSub <- kinship(pedigree(id=pedSub$id, dadid=pedSub$father, momid=pedSub$mother,
                               famid=pedSub$family, sex=as.numeric(pedSub$sex)))
    checkEquals(kinship(fadSub), kinSub)
    ##-----------------------------------
    ## subset using character
    ids <- c("524", "471", "17", "5", "4", "6")
    pedSub <- mbped[ids, ]
    pedSub$sex <- FamAgg:::sanitizeSex(pedSub$sex)
    fadSub <- fad[ids, ]
    ## Just compare columns other than father and mother, as they have been changed!
    checkEquals(pedigree(fadSub)[, c("family", "id", "sex")], pedSub[, c("family", "id", "sex")])
    ## kinship checking makes no sense... pedigree function does not work!
    ##-----------------------------------
    ## subset using numeric
    idx <- c(4, 2, 6, 112, 12, 14)
    pedSub <- mbped[idx, ]
    pedSub$sex <- FamAgg:::sanitizeSex(pedSub$sex)
    fadSub <- fad[idx, ]
    checkEquals(pedigree(fadSub)[, c("family", "id", "sex")], pedSub[, c("family", "id", "sex")])
    ##-----------------------------------
    ## error testing
    ids <- c("a", "b", "c")
    checkException(fad[ids, ])

    ## Subset to male only:
    fam4 <- fad[fad$family == "4", ]
    fam4M <- fam4[fam4$sex == "M", ]
    checkEquals(all(is.na(fam4M$mother)), TRUE)
    if(do.plot){
        ## That should work.
        plotPed(fam4, family="4")
        ## Now, how is this going to work if we've got females?
        ## plotPed(fam4M, family="4")
    }


    ## manually defining the object.
    fad <- new("FAData")
    pedigree(fad) <- mbped
    fadSub <- fad[fad$family==4, ]
    pedSub <- mbped[mbped$family==4, ]
    pedSub$sex <- FamAgg:::sanitizeSex(pedSub$sex)
    pedSub <- FamAgg:::sanitizePed(pedSub)
    checkEquals(pedigree(fadSub), pedSub)
    ## checking kinship
    kinSub <- kinship(pedigree(id=pedSub$id, dadid=pedSub$father, momid=pedSub$mother,
                               famid=pedSub$family, sex=as.numeric(pedSub$sex)))
    checkEquals(kinship(fadSub), kinSub)

    ## with age.
    ages <- mbsub$endage
    names(ages) <- mbped$id
    fad <- FAData(pedigree=mbped, age=ages)
    ##-----------------------------------
    ## subset using boolean
    fadSub <- fad[fad$family==4, ]
    ageSub <- ages[mbped$family==4]
    checkEquals(age(fadSub), ageSub)
    ##-----------------------------------
    ## subset using character
    ids <- c("524", "471", "17", "5", "4", "6")
    fadSub <- fad[ids, ]
    ageSub <- ages[ids]
    checkEquals(age(fadSub), ageSub)
    ##-----------------------------------
    ## subset using numeric
    idx <- c(4, 2, 6, 112, 12, 14)
    fadSub <- fad[idx, ]
    ageSub <- ages[idx]
    checkEquals(age(fadSub), ageSub)

    PedDf <- mbsub[, c("famid", "id", "fatherid", "motherid", "sex")]
    colnames(PedDf) <- FamAgg:::.PEDCN
    tcancer <- mbsub$cancer
    names(tcancer) <- mbsub$id
    trait(fad) <- tcancer
    ## subset by boolean.
    pedSub <- PedDf[PedDf$family == 5, ]
    tcancerSub <- tcancer[PedDf$family == 5]
    farSub <- fad[fad$family == 5, ]
    checkEquals(tcancerSub, trait(farSub))
    checkEquals(tcancerSub, farSub$trait)

    ## subset by id.
    ids <- sample(rownames(pedigree(fad)), size=100)
    pedSub <- PedDf[ids, ]
    tcancerSub <- tcancer[ids]
    farSub <- fad[ids, ]
    checkEquals(tcancerSub, trait(farSub))
    checkEquals(tcancerSub, farSub$trait)

    ## subset by idx.
    idx <- sample(1:nrow(pedigree(fad)), size=100)
    pedSub <- PedDf[idx, ]
    tcancerSub <- tcancer[idx]
    farSub <- fad[idx, ]
    checkEquals(tcancerSub, trait(farSub))
    checkEquals(tcancerSub, farSub$trait)

}


## testing the "sex sanitation"
test_sex_columns <- function(){
    sexnum <- c(1, 1, 2, 2, 3, 1, 1, 2, 2, NA)
    expect <- factor(c("M", "M", "F", "F", NA, "M", "M", "F", "F", NA),
                     levels=c("M", "F"))
    got <- FamAgg:::sanitizeSex(sexnum)
    checkEquals(got, expect)
    sexchar <- c("male", "male", "female", "female", "other", "male", "Man", "female", "female", NA)
    got <- FamAgg:::sanitizeSex(sexchar)
    checkEquals(got, expect)
    got <- FamAgg:::sanitizeSex(expect)
    checkEquals(got, expect)
    ## now checking some errors:
    checkException(FamAgg:::sanitizeSex(c(3, 2, 5, 6)))
    checkException(FamAgg:::sanitizeSex(c("woman", "man", "woman", "man")))
}

## test the kinship...
test_kinship <- function(){
    ## mbsub <- .getMinSub()
    ped <- pedigree(id=mbsub$id, dadid=mbsub$fatherid, momid=mbsub$motherid,
                    sex=mbsub$sex, famid=mbsub$famid)
    kin <- kinship(ped)
    fad <- FAData(pedigree=ped)
    return(checkEquals(kin[fad$id, fad$id], kinship(fad)))
}

## testing the constructor.
test_FAData <- function(){
    pedFile <- system.file("txt/minnbreastsub.txt", package="FamAgg")
    checkException(fad <- FAData(pedigree=pedFile, header=TRUE))
    fad <- FAData(pedigree=pedFile, header=TRUE, family.col="famid", id.col="id",
                  father.col="fatherid", mother.col="motherid")
    mbsub <- .getMinSub()
    PedDf <- mbsub[, c("famid", "id", "fatherid", "motherid", "sex")]
    colnames(PedDf) <- FamAgg:::.PEDCN
    PedDf$sex <- FamAgg:::sanitizeSex(PedDf$sex)
    rownames(PedDf) <- as.character(PedDf$id)
    PedDf <- FamAgg:::sanitizePed(PedDf)
    checkEquals(PedDf, pedigree(fad))

    ## adding also age...
    Ages <- as.numeric(mbsub$endage)
    names(Ages) <- mbsub$id
    fad <- FAData(pedigree=pedFile, header=TRUE, family.col="famid", id.col="id",
                  father.col="fatherid", mother.col="motherid", age=Ages)
    checkEquals(Ages, age(fad))
    Ages[1] <- 120
    age(fad) <- Ages
    checkEquals(Ages, age(fad))

    ## construct by submitting a data.frame
    fad <- FAData(PedDf)
    checkEquals(pedigree(fad), PedDf)
}

test_family <- function(){
    ## mbsub <- .getMinSub()
    PedDf <- mbsub[, c("famid", "id", "fatherid", "motherid", "sex")]
    colnames(PedDf) <- FamAgg:::.PEDCN
    PedDf$sex <- FamAgg:::sanitizeSex(PedDf$sex)
    rownames(PedDf) <- as.character(PedDf$id)
    fad <- FAData(PedDf)

    PedDf <- FamAgg:::sanitizePed(PedDf)
    Test <- family(fad, id=7)
    checkEquals(Test, PedDf[PedDf$family==4, ])
}


test_getCommonAncestor <- function(){
    do.plot <- FALSE
    ## mbsub <- .getMinSub()
    PedDf <- mbsub[, c("famid", "id", "fatherid", "motherid", "sex")]
    colnames(PedDf) <- FamAgg:::.PEDCN
    PedDf$sex <- FamAgg:::sanitizeSex(PedDf$sex)
    rownames(PedDf) <- NULL
    fad <- FAData(PedDf)
    ## get common ancestors for some example families
    ancs <- c("759", "760")
    checkEquals(sort(getCommonAncestor(fad, id=c("761", "763", "772"))), sort(ancs))
    if(do.plot)
        plotPed(fad, id=c("761", "763", "772"), device="plot", prune=TRUE)
    ##
    checkException(getCommonAncestor(fad, id="761"))
    checkException(getCommonAncestor(fad, id=c("761", "bla")))
}

test_buildPed <- function() {
    do.plot <- FALSE
    ## mbsub <- .getMinSub()
    PedDf <- mbsub[, c("famid", "id", "fatherid", "motherid", "sex")]
    colnames(PedDf) <- FamAgg:::.PEDCN
    PedDf$sex <- FamAgg:::sanitizeSex(PedDf$sex)
    rownames(PedDf) <- NULL
    fad <- FAData(PedDf)
    if(do.plot){
        switchPlotfun("ks2paint")
        plotPed(fad, id="4", device="plot")
        switchPlotfun()
    }
    tmpf <- paste0(tempfile(), ".tsv")
    switchPlotfun("haplopaint", check=FALSE)
    plotPed(fad, id="4", device="txt")
    switchPlotfun("ks2paint")
    file.remove(tmpf)

    expect <- c(5, 26, 11, 12, 13)
    tmp <- buildPed(fad, id = c(26, 13, 12, 11), prune = TRUE)
    checkEquals(sort(tmp$id), sort(expect))

    expect <- c(23, 3, 24, 4, 25, 5, 26, 11, 1, 2)
    tmp <- buildPed(fad, id = c(23, 11), prune = TRUE)
    checkEquals(sort(tmp$id), sort(expect))

    ## build ped for individuals of family 14:
    ## pedigree for individual 447 should include individuals of a second
    ## branch, e.g. individual 26062
    checkTrue(any(buildPed(fad, id = "447")$id == "26062"))
    ## for individual 440 (father of 447) this second branch should be
    ## missing
    checkTrue(!any(buildPed(fad, id = "440")$id == "26062"))

    checkException(buildPed(fad, family = 2))
    res <- buildPed(fad, family = "5")
    res_2 <- family(fad, family = "5")
    checkTrue(nrow(res) < nrow(res_2))
    res_2 <- removeSingletons(res_2)
    res_2 <- res_2[order(res_2$id), ]
    res <- res[order(res$id), ]
    checkEquals(res, res_2)
}


test_dollar <- function(){
    mbped <- mbsub[, c("famid", "id", "fatherid", "motherid", "sex")]
    colnames(mbped) <- FamAgg:::.PEDCN
    far <- FAData(pedigree=mbped)
    tcancer <- mbsub$cancer
    names(tcancer) <- mbsub$id
    trait(far) <- tcancer

    checkEquals(far$affected, tcancer)
    ## basic stuff from pedigree...
    ids <- as.character(pedigree(far)$id)
    names(ids) <- as.character(ids)
    checkEquals(far$id, ids)
    ## family
    tmp <- pedigree(far)$family
    names(tmp) <- as.character(pedigree(far)$id)
    checkEquals(far$family, tmp)
    ## father
    tmp <- pedigree(far)$father
    names(tmp) <- as.character(pedigree(far)$id)
    checkEquals(far$father, tmp)
    ## mother
    tmp <- pedigree(far)$mother
    names(tmp) <- as.character(pedigree(far)$id)
    checkEquals(far$mother, tmp)

}

## Here we just want to ensure that it's OK to have either 0 or NA for founders in
## the pedigree.
test_kinship2_kinship <- function(){
    mbped <- mbsub[, c("famid", "id", "fatherid", "motherid", "sex")]
    colnames(mbped) <- FamAgg:::.PEDCN
    pediNumeric <- pedigree(id=mbped$id, famid=mbped$family, dadid=mbped$father,
                            momid=mbped$mother, sex=mbped$sex)
    kinNumeric <- kinship(pediNumeric)
    ## Replacing 0s with NAs:
    mbped$father[mbped$father == 0] <- NA
    mbped$mother[mbped$mother == 0] <- NA
    pediNumericNA <- pedigree(id=mbped$id, famid=mbped$family, dadid=mbped$father,
                              momid=mbped$mother, sex=mbped$sex)
    kinNumericNA <- kinship(pediNumeric)
    checkEquals(kinNumeric, kinNumericNA)
    ## And now character:
    mbped$id <- as.character(mbped$id)
    mbped$father <- as.character(mbped$father)
    mbped$mother <- as.character(mbped$mother)
    pediCharNA <- pedigree(id=mbped$id, famid=mbped$family, dadid=mbped$father,
                           momid=mbped$mother, sex=mbped$sex)
    kinCharNA <- kinship(pediCharNA)
    checkEquals(kinNumeric, kinCharNA)
    ## The same but with "0" for founders.
    mbped$father[is.na(mbped$father)] <- 0
    mbped$mother[is.na(mbped$mother)] <- 0
    checkException(pedigree(id=mbped$id, famid=mbped$family, dadid=mbped$father,
                            momid=mbped$mother, sex=mbped$sex))

    ## "" for "0"
    mbped$father[mbped$father == "0"] <- ""
    mbped$mother[mbped$mother == "0"] <- ""
    pediCharZ <- pedigree(id=mbped$id, famid=mbped$family, dadid=mbped$father,
                          momid=mbped$mother, sex=mbped$sex)
    kinCharZ <- kinship(pediCharZ)
    checkEquals(kinNumeric, kinCharZ)
}

test_export <- function(){
    mbped <- mbsub[, c("famid", "id", "fatherid", "motherid", "sex")]
    colnames(mbped) <- FamAgg:::.PEDCN
    far <- FAData(pedigree=mbped)
    fam4 <- far[far$family == 4, ]
    tmpF <- paste0(tempfile(), ".ped")
    export(fam4, tmpF, format="ped")
    fam4imported <- FAData(tmpF)
    checkEquals(as.character(pedigree(fam4)$family), pedigree(fam4imported)$family)
    checkEquals(as.character(pedigree(fam4)$id), pedigree(fam4imported)$id)
    checkEquals(as.character(pedigree(fam4)$father), pedigree(fam4imported)$father)
    checkEquals(as.character(pedigree(fam4)$mother), pedigree(fam4imported)$mother)
    checkEquals(as.character(pedigree(fam4)$sex), as.character(pedigree(fam4imported)$sex))
}

test_genevieve_reported_errors <- function() {
    ped <- data.frame(family = c("a", "a", "a", "a", "b", "b", "b"),
                      id = c(1, 2, 3, 4, 1, 2, 3),
                      father = c(0, 0, 1, 1, 0, 0, 1),
                      mother = c(0, 0, 2, 2, 0, 0, 2),
                      sex = c("M", "F", "M", "F", "M", "F", "F"),
                      stringsAsFactors = FALSE)
    checkException(FAData(ped))
    ## Combine family and id
    ped2 <- ped
    ped2$id <- paste0(ped$family, "-", ped$id)
    ped2$father <- paste0(ped$family, "-", ped$father)
    ped2$mother <- paste0(ped$family, "-", ped$mother)
    ## Fixing the founder
    ped2$father[ped$father == 0] <- NA
    ped2$mother[ped$mother == 0] <- NA

    fad <- FAData(ped2)
    ped3 <- ped
    ped3$id <- paste0(ped$family, "-", ped$id)
    ped3$father <- paste0(ped$family, "-", ped$father)
    ped3$mother <- paste0(ped$family, "-", ped$mother)
    ## Fixing the founder
    ped3$father[ped$father == 0] <- 0
    ped3$mother[ped$mother == 0] <- 0

    fad <- FAData(ped3)
}

data(minnbreast)
mbsub <- minnbreast[minnbreast$famid %in% 4:19, ]
PedDf <- mbsub[, c("famid", "id", "fatherid", "motherid", "sex")]
colnames(PedDf) <- FamAgg:::.PEDCN
PedDf$sex <- FamAgg:::sanitizeSex(PedDf$sex)
## generate the FAData.
fad <- FAData(pedigree=PedDf)
## specify the trait.
tcancer <- mbsub$cancer
names(tcancer) <- mbsub$id

## OK, 0.0.6
## testing the kinship sum test.
test_kinship_sum <- function(){
    do.plot <- FALSE

    ## perform the test:
    set.seed(18011977)
    far <- kinshipSumTest(fad, trait=tcancer, traitName="cancer",
                          nsim=1000)
    Result <- result(far)
    dim(Result)
    ## check the pedigree/family of this individual
    rownames(PedDf) <- as.character(PedDf$id)
    ## use the family method to extract the pedigree of that family.
    GotFam <- family(far, id=Result[1, "affected_id"])
    AffFam <- PedDf[PedDf$family == PedDf[Result[1, "affected_id"], "family"], ]
    AffFam <- FamAgg:::sanitizePed(AffFam)
    AffFam <- cbind(AffFam, affected=tcancer[AffFam$id])
    checkEquals(GotFam, AffFam)

    if(do.plot){
        plotPed(far, id=Result[1, "affected_id"])
    }

    ## now replacing the trait and re-running.
    tpreg <- mbsub$everpreg
    names(tpreg) <- mbsub$id
    ## randomizing to see if re-ordering works...
    tpregrand <- sample(tpreg, length(tpreg))
    trait(far) <- tpregrand
    checkEquals(trait(far), tpreg)
    ## running the simulation.
    far <- runSimulation(far, nsim=1000)

    ## testing to subset the object... which is not supported
    checkException(far[1:10, ])

    ## tableSimVals
    far <- kinshipSumTest(fad, trait=tcancer, traitName="cancer",
                          nsim=1000, tableSimVals = TRUE)
    checkTrue(any(names(far@sim) == "tableSimVals"))

    ##
    return(TRUE)
}

test_kinship_sum_strata <- function(){
    set.seed(18011977)
    far <- kinshipSumTest(fad, trait=tcancer, traitName="cancer",
                          nsim=500)
    ## do the same using a fake strata
    set.seed(18011977)
    farStrat <- kinshipSumTest(fad, trait=tcancer, traitName="cancer",
                               nsim=500, strata=rep("OK", length(tcancer)))
    checkEquals(far@sim$sumKinship, farStrat@sim$sumKinship)
    checkEquals(far@sim$pvalueKinship, farStrat@sim$pvalueKinship)
    ## check again, subsetting to male only.
    fadM <- fad[which(fad$sex == "M"), ]
    set.seed(18011977)
    farM <- kinshipSumTest(fadM, trait=tcancer, nsim=500)
    sexStrat <- fad$sex
    sexStrat[sexStrat == "F"] <- NA
    set.seed(18011977)
    farStrat <- kinshipSumTest(fad, trait=tcancer, nsim=500, strata=sexStrat)
    ## compare that
    checkEquals(farM@sim$sumKinship, farStrat@sim$sumKinship)
    checkEquals(farM@sim$pvalueKinship, farStrat@sim$pvalueKinship)
}

test_plot_kinclust <- function(){
    ## data(minnbreast)
    ## mbsub <- minnbreast[minnbreast$famid %in% 4:19, ]
    ## PedDf <- mbsub[, c("famid", "id", "fatherid", "motherid", "sex")]
    ## colnames(PedDf) <- FamAgg:::.PEDCN
    ## PedDf$sex <- FamAgg:::sanitizeSex(PedDf$sex)
    ## ## generate the FAData.
    ## fad <- FAData(pedigree=PedDf)
    ## ## specify the trait.
    ## tcancer <- mbsub$cancer
    ## names(tcancer) <- mbsub$id

    ## perform the test:
    set.seed(18011977)
    far <- kinshipSumTest(fad, trait=tcancer, traitName="cancer",
                          nsim=500)
    res <- result(far)

    id <- "410"

    cols <- rep(1, 32)
    cols[1] <- 2

    ## kinship2 plotting
    switchPlotfun("ks2paint")
    plotPed(far, id=id, device="plot", col=cols)
    ##switchPlotfun()
}

pedFile <- system.file("txt/Large.ped", package="FamAgg")
largePed <- read.table(pedFile, sep=";")[, 1:5]
colnames(largePed) <- c("family", "id", "father", "mother", "sex")
rownames(largePed) <- as.character(largePed$id)
largeFad <- FAData(pedigree=largePed)

data(minnbreast)
## head(minnbreast)
minPed <- minnbreast[, c("famid", "id", "fatherid", "motherid", "sex")]
colnames(minPed) <- c("family", "id", "father", "mother", "sex")
minFad <- FAData(pedigree=minPed[minPed$family %in% 4:10, ])

test_ped2graph <- function(){
    allped <- largePed

    pedids <- c("3511", "3225", "3508", "3550", "3507", "3510", "3224",
                "3052", "3250", "2969", "3407")
    ped <- largePed[pedids, ]
    ped[!(ped$father %in% ped$id), "father"] <- 0
    ped[!(ped$mother %in% ped$id), "mother"] <- 0
    plot(pedigree(id=ped$id, dadid=ped$father, momid=ped$mother, sex=ped$sex))
    igr <- ped2graph(ped)
    igr <- add_vertices(igr, 3)
    plot(igr)


    ## distance matrix: this lists us all that we can get to following the direction
    dout <- distances(igr, mode="out")
    ## got the way back: from children -> parents
    din <- distances(igr, mode="in")
    ## use the ROWS in that.
    testids <- c("3511", "3507", "3224")
    subD <- din[testids, , drop=FALSE]
    subD <- subD[, !apply(subD, MARGIN=2, function(x){
        return(all(FamAgg:::infOrZero(x)))
    }) , drop=FALSE]

    ## hugegraph
    IGR <- ped2graph(allped)
    Dist <- distances(IGR)
    any(is.infinite(Dist[pedids, pedids])) ## means, they are connected!
    testids <- c("3224", "3511", "255")
    any(is.infinite(Dist[testids, testids]))
    any(is.infinite(Dist))
    ## apparently, that thing is totally connected...
    return(TRUE)
}

test_subPedigree <- function(){
    ped <- largePed
    testids <- c("3511", "3225", "3508", "3550", "3507", "3510", "3224")
    ## get the subPedigree for these
    subped <- subPedigree(ped, id=testids)

    checkEquals(sum(testids %in% as.character(subped$id)), length(testids))

    return(TRUE)
}


test_getCommonAncestor <- function(){
    do.plot <- FALSE
    ped <- largePed
    fad <- largeFad
    ##testids <- c("3511", "3225", "3508")  ## no common graph
    testids <- c("3511", "3225", "3508", "3550", "3507", "3510", "3224")
    ancs <- c("3052", "2969")
    checkEquals(sort(getCommonAncestor(ped, id=testids)), sort(ancs))

    ## plotting the pedigree for this...
    if(do.plot){
        switchPlotfun("ks2paint")
        plotPed(fad, id=testids, device="plot", prune=TRUE)
    }

    ## 1 generation up:
    testids <- c("3508", "3507")
    ancs <- c("3250", "3225")
    checkEquals(sort(getCommonAncestor(ped, id=testids)), sort(ancs))
    if(do.plot)
        plotPed(fad, id=testids, device="plot", prune=TRUE)
    ## OK

    ## 2 generations up:
    testids <- c("3550", "3511")
    ancs <- c("3052", "2969")
    checkEquals(sort(getCommonAncestor(ped, id=testids)), sort(ancs))
    if(do.plot)
        plotPed(fad, id=testids, device="plot", prune=TRUE)
    ## OK

    ## x generations?
    testids <- c("3224", "3250")
    getCommonAncestor(ped, id=testids)
    if(do.plot)
        plotPed(fad, id=c(testids, "2042"), device="plot", prune=TRUE)
    ## lovely pedigree ;)
    ## switchPlotfun()
    ## plotPed(fad, id=c(testids, "2042"), filename="~/Desktop/test.pdf", prune=TRUE)

    ## ids from different generations:
    testids <- c(3508, 3224, 3225)
    ancs <- c("3052", "2969")
    getCommonAncestor(ped, id=testids)
    checkEquals(sort(getCommonAncestor(ped, id=testids)), sort(ancs))
    ## OK

    ## doesn't work if one is already the ancestor...
    testids <- c(3508, 3052)
    ancs <- c("3052", "2969")
    checkEquals(sort(getCommonAncestor(ped, id=testids)), sort(ancs))
    if(do.plot)
        plotPed(fad, id=c(testids, ancs), device="plot", prune=TRUE)

    ## test an example that can not work.
    testids <- c(3016, 3349)
    ancs <- getCommonAncestor(ped, testids)
    ## HECK?
    if(do.plot){
        plotPed(fad, id=c(testids), device="plot", prune=TRUE)
        plotPed(fad, id=c(testids, ancs), device="plot", prune=TRUE)
    }

    ## ????
    testids <- c(3016, 2962)
    ancs <- getCommonAncestor(ped, testids)
    if(do.plot){
        plotPed(fad, id=c(testids), device="plot", prune=TRUE)
        plotPed(fad, id=c(testids, ancs), device="plot", prune=TRUE)
    }

    ##************************************************
    ##*
    ##* test it with the minnesota breast cancer set:
    ##*
    ##table(fad$family)
    fad <- minFad
    ped <- pedigree(fad)

    if(do.plot)
        plotPed(fad, family="6", device="plot")
    testids <- c("103", "104", "95", "99")
    ancs <- c("80", "81")
    checkEquals(sort(getCommonAncestor(ped, id=testids)), sort(ancs))
    if(do.plot){
        plotPed(fad, id=c(testids, ancs), device="plot")
        plotPed(fad, id=c(testids, ancs), device="plot", prune=TRUE)
    }

    ## other example:
    testids <- c(101, 83)
    ancs <- c("107", "84")
    checkEquals(sort(getCommonAncestor(ped, id=testids)), sort(ancs))
    if(do.plot)
        plotPed(fad, id=c(testids), device="plot", prune=TRUE)

    ## can not find one.
    testids <- c("88", "93", "114")
    checkEquals(getCommonAncestor(ped, id=testids), NA)
    return(TRUE)
}

test_count_generations <- function(){
    ped <- minPed
    expect <- 3
    names(expect) <- "2"
    checkEquals(countFenerations(ped=ped, id=2), expect)

    expect <- 2
    names(expect) <- "11"
    checkEquals(countGenerations(ped=ped, id=11, direction="up"), expect)
}

test_find_founders <- function(){
    fad <- minFad
    ped <- pedigree(fad)
    family <- "4"
    checkEquals(findFounders(ped, family=family), c("1", "2"))
    ## for FAData
    fad <- FAData(ped[ped$family=="4", ])
    checkEquals(findFounders(fad, family=family), c("1", "2"))
    checkEquals(findFounders(pedigree(fad, return.type="pedigree"),
                             family=family), c("1", "2"))
}

test_find_foundersForId <- function() {
    ## Check if we get the same results for findFounders for family and for
    ## findFounders with an id within the family.
    fad <- minFad
    ped <- pedigree(fad)
    fnds <- findFounders(ped, id = 269)
    fnds_2 <- findFounders(ped, family = 10)
    checkEquals(fnds, fnds_2)
    fnds <- findFounders(ped, id = 20)
    fnds_2 <- findFounders(ped, family = 4)
    checkEquals(fnds, fnds_2)
    ## The same for FAData
    fnds <- findFounders(fad, id = 269)
    fnds_2 <- findFounders(fad, family = 10)
    checkEquals(fnds, fnds_2)
    fnds <- findFounders(fad, id = 20)
    fnds_2 <- findFounders(fad, family = 4)
    checkEquals(fnds, fnds_2)
}

test_find_siblings <- function(){
    ped <- minPed
    checkEquals(getSiblings(ped, id="11"), c("11", "12", "13"))
    ## for FAData
    fad <- FAData(ped[ped$family=="4", ])
    checkEquals(getSiblings(fad, id="11"), c("11", "12", "13"))
    checkEquals(getSiblings(pedigree(fad, return.type="pedigree"),
                            id="11"), c("11", "12", "13"))
}

test_do_get_ancestors <- function(){
    ped <- minPed
    checkEquals(getAncestors(ped, id="11"), c("5", "26", "1", "2"))
    ## for FAData
    fad <- FAData(ped[ped$family=="4", ])
    checkEquals(getAncestors(fad, id="11"), c("5", "26", "1", "2"))
    checkEquals(getAncestors(pedigree(fad, return.type="pedigree"),
                             id="11"), c("5", "26", "1", "2"))
    expect <- as.character(c(3, 4, 24, 25))
    checkEquals(sort(getAncestors(fad, c(22, 3), max.generations=1)), sort(expect))
    checkEquals(getAncestors(fad, 4, 1), FamAgg:::doGetParents(pedigree(fad), 4))
}

test_do_get_children <- function(){
    ped <- minPed
    checkEquals(getChildren(ped, id="3"), c("21", "22", "23"))
    ## for FAData
    fad <- FAData(ped[ped$family=="4", ])
    checkEquals(getChildren(fad, id="3"), c("21", "22", "23"))
    checkEquals(getChildren(pedigree(fad, return.type="pedigree"), id="3"), c("21", "22", "23"))
    expect <- as.character(c(4, 5, 6, 7, 8, 9, 10, 17))
    checkEquals(sort(getChildren(fad, id=c(1, 28, 2, 8), max.generation=1)), sort(expect))
}

test_count_generations <- function(){
    ped <- minPed
    expect <- 1
    names(expect) <- "28"
    checkEquals(countGenerations(ped, id="28"), expect)
    ## for FAData
    fad <- FAData(ped[ped$family=="4", ])
    checkEquals(countGenerations(fad, id="28"), expect)
    checkEquals(countGenerations(pedigree(fad, return.type="pedigree"),
                                 id="28"), expect)
}

test_estimate_generations <- function(){
    ped <- minPed
    fam4 <- ped[ped$family == "4", ]
    fad <- FAData(ped[ped$family %in% c(4:20), ])

    ## testing stuff...
    from8 <- c(-1, -1, 1, 0,0,0,0,0,0,0,1,1,1,1,1,1,1,1,1,1,2,2,2,1,0,0,0,0,0)
    names(from8) <- as.character(1:length(from8))
    ## gens <- FamAgg:::doGetGenerationFrom(fam4, id=8)
    ## checkEquals(from8, gens[names(from8)])
    gens2 <- generationsFrom(fam4, id=8)
    checkEquals(from8, gens2[names(from8)])

    ## artificially create a second branch for fam4:
    addon <- data.frame(family=4,
                        id=50:59,
                        father=c(0, 0, 50, 50, 0, 54, 0, 54, 57, 57),
                        mother=c(0, 0, 51, 51, 0, 53, 0, 53, 56 ,56),
                        sex=c("M", "F", "M", "F", "M", "F", "F", "M", "F", "M")
                        )
    fam4mod <- rbind(fam4, addon)
    fam4mod[fam4mod$id == "25", c("father", "mother")] <- c(54, 53)
    checkEquals(findFounders(fam4mod, family="4"), c("50", "51"))

    addon2 <- data.frame(family=4,
                         id=70:76,
                         father=c(0, 70, 70, 70, 0, 71, 71),
                         mother=c(0, 20 ,20, 20, 0, 74, 74),
                         sex=c("M", "M", "F", "M", "F", "M", "F"))
    fam4mod2 <- rbind(fam4mod, addon2)
    checkEquals(findFounders(fam4mod2, 4), c("1", "2"))

    ## these modified families are interesting to test...
    fam4from6 <- c(-1, -1, 1, 0, 0, 0, 0, 0, 0, 0, 1, 1, 1, 1, 1,
                   1, 1, 1, 1, 1, 2, 2, 2, 1, 0, 0, 0, 0, 0)
    names(fam4from6) <- as.character(1:29)
    checkEquals(fam4from6, generationsFrom(fam4, id="6")[names(fam4from6)])

    addonfrom6 <- c(-2, -2, -1, -1, -1, 0, 0, 0, 1, 1)
    names(addonfrom6) <- as.character(50:59)
    fam4modfrom6 <- c(fam4from6, addonfrom6)
    checkEquals(fam4modfrom6, generationsFrom(fam4mod, id="6")[names(fam4modfrom6)])

    addon2from6 <- c(1, 2, 2, 2, 2, 3, 3)
    names(addon2from6) <- as.character(70:76)
    fam4mod2from6 <- c(fam4modfrom6, addon2from6)
    checkEquals(fam4mod2from6, generationsFrom(fam4mod2, id="6")[names(fam4mod2from6)])

    ## estimating generations...
    Gens <- estimateGenerations(fam4)[[1]]
    checkEquals(Gens[names(fam4from6)], fam4from6+1)
    ## what for fam4mod
    findFounders(fam4mod, 4)
    Gens <- estimateGenerations(fam4mod)[[1]]
    checkEquals(Gens[names(fam4modfrom6)], fam4modfrom6+2)
    ## what for fam4mod2
    findFounders(fam4mod2, 4)
    Gens <- estimateGenerations(fam4mod2)[[1]]
    checkEquals(Gens[names(fam4mod2from6)], fam4mod2from6+1)
}

## getFounders, getSingletons
test_getFounders_getSingletons <- function(){
    ped <- minPed
    fad <- FAData(ped)
    checkEquals(getFounders(fad), getFounders(ped))
    checkEquals(getSingletons(fad), getSingletons(ped))
    ## Now check if they are really childless...
    FounderPed <- ped[ped$id %in% getFounders(ped), ]
    checkEquals(sum(FounderPed[, c("father", "mother")]), 0)
    ## Check if childless founders really have no children.
    clf <- getSingletons(fad)
    checkEquals(any(ped$father %in% clf), FALSE)
    checkEquals(any(ped$mother %in% clf), FALSE)
}

## Test the difference between removeSingletons and doPrunePed
test_singletons_vs_prune <- function(){
    ped <- minPed
    ## using prune
    system.time(
        pedPruned <- FamAgg:::doPrunePed(ped)
    )
    system.time(
        pedRemSin <- removeSingletons(ped)
    )
    pedPruned <- FamAgg:::sanitizePed(pedPruned)
    rownames(pedRemSin) <- pedRemSin$id
    rownames(pedPruned) <- pedPruned$id
    checkEquals(pedPruned, pedRemSin)
    ## OK, it's the same; now let's check what happens if we want to add missing
    ## mates on them.
    system.time(
        pedPruned <- FamAgg:::doPrunePed(ped, addMissingMates=TRUE)
    )
    checkEquals(nrow(pedPruned), nrow(pedRemSin))
    ## Eventually I might want to use the removeSingletons instead of the prune.
}

test_removeSingletons <- function() {
    ped <- minFad
    sings <- getSingletons(ped)
    pedSub <- removeSingletons(ped)
    sings_2 <- getSingletons(pedSub)
    checkTrue(length(sings_2) == 0)
    checkTrue((length(sings) + nrow(pedigree(pedSub))) == nrow(pedigree(ped)))
    trait <- sample(c(0, 1), nrow(pedigree(ped)), replace = TRUE)
    names(trait) <- ped$id
    trait(ped) <- trait
    ## Do the subsetting with the trait:
    pedSub <- removeSingletons(ped)
    checkEquals(trait(pedSub), trait[pedSub$id])
}

test_kinshipPairs <- function() {

    checkException(kinshipPairs(4), "FAData")
    checkException(kinshipPairs(minFad, condition = function(z) z * z),
                   "did not return")
    checkException(kinshipPairs(minFad, family = "other"), "families")
    res <- kinshipPairs(minFad)
    extract_values <- function(data, x) {
        res <- numeric(nrow(x))
        for (i in seq_len(nrow(x)))
            res[i] <- data[x[i, 1], x[i, 2]]
        res
    }
    ks <- extract_values(kinship(minFad), res)
    checkTrue(all(ks >= 0.25))

    res <- kinshipPairs(minFad, duplicates = "first")
    ## Each individual should be present only once!
    tab <- table(as.character(res))
    checkTrue(all(tab == 1))

    res_2 <- kinshipPairs(minFad, duplicates = "last")
    tab <- table(as.character(res_2))
    checkTrue(all(tab == 1))

    res <- kinshipPairs(minFad, duplicates = "first",
                        condition = function(z) z < 0.125)
    ks <- extract_values(kinship(minFad), res)
    checkTrue(all(ks < 0.125))
    tab <- table(as.character(res))
    checkTrue(all(tab == 1))

    ## Subset by family, check that only IDs from that family are reported.
    res <- kinshipPairs(minFad, family = 1:4)
    checkTrue(all(as.character(res) %in% minFad$id[minFad$family %in% 1:4]))

    ## Subset by ID, check that only these IDs are reported.
    ids <- 1:40
    res <- kinshipPairs(minFad, id = ids)
    checkTrue(all(as.character(res) %in% ids))
}

test_shareKinship <- function() {
    ## Have a look at the pedigree: plotPed(minFad, family = "4")
    ## Lowest kinship value between 21 and the rest of this family (#4): 0.03125
    fullKSIDs <- shareKinship(minFad, id="21")
    checkTrue(length(fullKSIDs)==25)
    ## Perform some checks on the kinship itself. a) Parents should share ks.
    checkTrue(all(c("3", "24") %in% fullKSIDs))
    ## b) siblings, too
    checkTrue(all(c("22", "23") %in% fullKSIDs))
    ## c) grandparents
    checkTrue(all(c("4", "25") %in% fullKSIDs))
    ## d) great-grandparents
    checkTrue(all(c("1", "2") %in% fullKSIDs))
    ## e) some more remote folks
    checkTrue(all(c("7", "11", "14", "17", "20") %in% fullKSIDs))
    ## f) but not any marry-ins
    checkTrue(!any(c("26", "27", "28", "29") %in% fullKSIDs))
    fullKSIDs2 <- shareKinship(minFad, id="21", rmKinship=0.01)
    ## Should still be the same.
    checkTrue(length(fullKSIDs2)==25 &&
              length( setdiff(fullKSIDs, fullKSIDs2) )==0)
    ## Ignore those with ks <= 0.03125
    ids3125 <- shareKinship(minFad, id="21", rmKinship = 0.03125)
    checkTrue(length(ids3125)==15)
    ids625 <- shareKinship(minFad, id="21", rmKinship = 0.0625)
    checkTrue(length(ids625)==7)
    ids125 <- shareKinship(minFad, id="21", rmKinship = 0.125)
    checkTrue(length(ids125)==5)
    ids25 <- shareKinship(minFad, id="21", rmKinship = 0.25)
    checkTrue(ids25=="21")
    ## Higher kinship filtering results in nested subsets.
    checkTrue(length(intersect(ids25, ids125))==1)
    checkTrue(length(intersect(ids125, ids625))==5)
    checkTrue(length(intersect(ids625, ids3125))==7)
    checkTrue(length(intersect(ids3125, fullKSIDs))==15)
    ## Get all relatives for "21" that have exactly kinship 0.125. Two ways.
    ## 1. by definition of rmKinship
    idsOnly125.1 <- setdiff(ids625, ids125)
    ## 2. by querying the kinship matrix itself.
    kin21 <- kinship(minFad)["21", ]
    idsOnly125.2 <- names(kin21[kin21==0.125])
    ## Sets must be identical.
    checkTrue( all(idsOnly125.1 %in% idsOnly125.2) &
               all(idsOnly125.2 %in% idsOnly125.1) )
}

## some other methods and methods for data.frame, pedigree or pedigreeList
## countGenerations
setMethod("countGenerations", "data.frame",
          function(object, id=NULL, direction="down"){
              object <- checkPedCol(object)
              object <- sanitizePed(object)
              doCountGenerations(object, id=id, direction=direction)
          })
setMethod("countGenerations", "pedigreeList",
          function(object, id=NULL, direction="down"){
              object <- ped2df(object)
              object <- checkPedCol(object)
              object <- sanitizePed(object)
              doCountGenerations(object, id=id, direction=direction)
          })
setMethod("countGenerations", "pedigree",
          function(object, id=NULL, direction="down"){
              object <- ped2df(object)
              object <- checkPedCol(object)
              object <- sanitizePed(object)
              doCountGenerations(object, id=id, direction=direction)
          })

## estimateGenerations
setMethod("estimateGenerations", "data.frame",
          function(object, family=NULL, ...){
              object <- sanitizePed(object)
              object <- checkPedCol(object)
              doEstimateGenerationsFor2(object, family=family)
          })
setMethod("estimateGenerations", "pedigree",
          function(object, family=NULL, ...){
              object <- ped2df(object)
              object <- checkPedCol(object)
              object <- sanitizePed(object)
              doEstimateGenerationsFor2(object, family=family)
          })
setMethod("estimateGenerations", "pedigreeList",
          function(object, family=NULL, ...){
              object <- ped2df(object)
              object <- checkPedCol(object)
              object <- sanitizePed(object)
              doEstimateGenerationsFor2(object, family=family)
          })


## findFounders
setMethod("findFounders", "data.frame",
          function(object, family = NULL, id = NULL, ...){
              object <- checkPedCol(object)
              object <- sanitizePed(object)
              doFindFounders(object, family = family, id = id)
          })
setMethod("findFounders", "pedigreeList",
          function(object, family=NULL, ...){
              object <- ped2df(object)
              object <- checkPedCol(object)
              object <- sanitizePed(object)
              doFindFounders(object, family=family)
          })
setMethod("findFounders", "pedigreeList",
          function(object, family=NULL, ...){
              object <- ped2df(object)
              object <- checkPedCol(object)
              object <- sanitizePed(object)
              doFindFounders(object, family=family)
          })

## generationsFrom
setMethod("generationsFrom", "data.frame",
          function(object, id=NULL, ...){
              object <- checkPedCol(object)
              object <- sanitizePed(object)
              doGetGenerationFrom2(object, id=id, ...)
          })
setMethod("generationsFrom", "pedigree",
          function(object, id=NULL, ...){
              object <- ped2df(object)
              object <- checkPedCol(object)
              object <- sanitizePed(object)
              doGetGenerationFrom2(object, id=id, ...)
          })
setMethod("generationsFrom", "pedigreeList",
          function(object, id=NULL, ...){
              object <- ped2df(object)
              object <- checkPedCol(object)
              object <- sanitizePed(object)
              doGetGenerationFrom2(object, id=id, ...)
          })

## getAncestors
setMethod("getAncestors", "data.frame",
          function(object, id=NULL, max.generations=3, ...){
              object <- checkPedCol(object)
              object <- sanitizePed(object)
              doGetAncestors(ped=object, id=id, maxlevel=max.generations, ...)
          })
setMethod("getAncestors", "pedigreeList",
          function(object, id=NULL, max.generations=3, ...){
              if(!is(object, "pedigree") & !is(object, "pedigreeList"))
                  stop("object should be either a 'pedigree' or ",
                       "'pedigreeList' object!")
              object <- ped2df(object)
              object <- checkPedCol(object)
              object <- sanitizePed(object)
              doGetAncestors(ped=object, id=id, maxlevel=max.generations, ...)
          })
setMethod("getAncestors", "pedigree",
          function(object, id=NULL, max.generations=3, ...){
              if(!is(object, "pedigree") & !is(object, "pedigreeList"))
                  stop("object should be either a 'pedigree' or ",
                       "'pedigreeList' object!")
              object <- ped2df(object)
              object <- checkPedCol(object)
              object <- sanitizePed(object)
              doGetAncestors(ped=object, id=id, maxlevel=max.generations, ...)
          })

## getChildren
setMethod("getChildren", "data.frame",
          function(object, id=NULL, max.generations=3, ...){
              object <- checkPedCol(object)
              object <- sanitizePed(object)
              doGetChildren(ped=object, id=id, maxlevel=max.generations, ...)
          })
setMethod("getChildren", "pedigreeList",
          function(object, id=NULL, max.generations=3, ...){
              if(!is(object, "pedigree") & !is(object, "pedigreeList"))
                  stop("object should be either a 'pedigree' or ",
                       "'pedigreeList' object!")
              object <- ped2df(object)
              object <- checkPedCol(object)
              object <- sanitizePed(object)
              doGetChildren(ped=object, id=id, maxlevel=max.generations, ...)
          })
setMethod("getChildren", "pedigree",
          function(object, id=NULL, max.generations=3, ...){
              if(!is(object, "pedigree") & !is(object, "pedigreeList"))
                  stop("object should be either a 'pedigree' or ",
                       "'pedigreeList' object!")
              object <- ped2df(object)
              object <- checkPedCol(object)
              object <- sanitizePed(object)
              doGetChildren(ped=object, id=id, maxlevel=max.generations, ...)
          })

## getCommonAncestor
setMethod("getCommonAncestor", "data.frame",
          function(object, id=NULL, method="min.dist"){
              object <- checkPedCol(object)
              object <- sanitizePed(object)
              doGetCommonAncestor(object, id=id, method=method)
          })
setMethod("getCommonAncestor", "pedigree",
          function(object, id=NULL, method="min.dist"){
              object <- ped2df(object)
              object <- checkPedCol(object)
              object <- sanitizePed(object)
              doGetCommonAncestor(object, id=id, method=method)
          })
setMethod("getCommonAncestor", "pedigreeList",
          function(object, id=NULL, method="min.dist"){
              object <- ped2df(object)
              object <- checkPedCol(object)
              object <- sanitizePed(object)
              doGetCommonAncestor(object, id=id, method=method)
          })

## getMissingMate
setMethod("getMissingMate", "data.frame",
          function(object, id=NULL, ...){
              object <- checkPedCol(object)
              object <- sanitizePed(object)
              doGetMissingMate(object, id=id)
          })
setMethod("getMissingMate", "pedigreeList",
          function(object, id=NULL, ...){
              object <- ped2df(object)
              object <- checkPedCol(object)
              object <- sanitizePed(object)
              doGetMissingMate(object, id=id)
          })
setMethod("getMissingMate", "pedigree",
          function(object, id=NULL, ...){
              object <- ped2df(object)
              object <- checkPedCol(object)
              object <- sanitizePed(object)
              doGetMissingMate(object, id=id)
          })

## getSiblings
setMethod("getSiblings", "data.frame",
          function(object, id=NULL, ...){
              object <- checkPedCol(object)
              object <- sanitizePed(object)
              doGetSiblings(object, id=id)
          })
setMethod("getSiblings", "pedigreeList",
          function(object, id=NULL, ...){
              object <- ped2df(object)
              object <- checkPedCol(object)
              object <- sanitizePed(object)
              doGetSiblings(object, id=id)
          })
setMethod("getSiblings", "pedigree",
          function(object, id=NULL, ...){
              object <- ped2df(object)
              object <- checkPedCol(object)
              object <- sanitizePed(object)
              doGetSiblings(object, id=id)
          })




##***************************************************************************
##
##   Methods to get ids from the pedigree that could be used as controls
##   for the given list of ids.
##
##   Input: ped, id and any number of optional parameters as input
##   Return: a list of ids, names of the list are the family ids.
##
##***************************************************************************
## getAll
setMethod("getAll", "data.frame",
          function(object, id=NULL, ...){
              object <- checkPedCol(object)
              return(doGetAll(ped=object, id=id, ...))
          })
setMethod("getAll", "pedigreeList",
          function(object, id=NULL, ...){
              object <- ped2df(object)
              object <- checkPedCol(object)
              return(doGetAll(ped=object, id=id, ...))
          })
setMethod("getAll", "pedigree",
          function(object, id=NULL, ...){
              object <- ped2df(object)
              object <- checkPedCol(object)
              return(doGetAll(ped=object, id=id, ...))
          })

## getExternalMatched
setMethod("getExternalMatched", "data.frame",
          function(object, id=NULL, match.using, ...){
              object <- checkPedCol(object)
              return(doGetExternalMatched(ped=object, id=id, match.using, ...))
          })
setMethod("getExternalMatched", "pedigreeList",
          function(object, id=NULL, match.using, ...){
              object <- ped2df(object)
              object <- checkPedCol(object)
              return(doGetExternalMatched(ped=object, id=id, match.using, ...))
          })
setMethod("getExternalMatched", "pedigree",
          function(object, id=NULL, match.using, ...){
              object <- ped2df(object)
              object <- checkPedCol(object)
              return(doGetExternalMatched(ped=object, id=id, match.using, ...))
          })

## getGenerationMatched
setMethod("getGenerationMatched", "data.frame",
          function(object, id=NULL, include.anc=0, include.off=0, ...){
              object <- checkPedCol(object)
              object <- sanitizePed(object)
              return(doGetGenerationMatched(ped=object, id=id, include.anc=include.anc,
                                            include.off=include.off, ...))
          })
setMethod("getGenerationMatched", "pedigreeList",
          function(object, id=NULL, include.anc=0, include.off=0, ...){
              object <- ped2df(object)
              object <- checkPedCol(object)
              object <- sanitizePed(object)
              return(doGetGenerationMatched(ped=object, id=id, include.anc=include.anc,
                                            include.off=include.off, ...))
          })
setMethod("getGenerationMatched", "pedigree",
          function(object, id=NULL, include.anc=0, include.off=0, ...){
              object <- ped2df(object)
              object <- checkPedCol(object)
              object <- sanitizePed(object)
              return(doGetGenerationMatched(ped=object, id=id, include.anc=include.anc,
                                            include.off=include.off, ...))
          })

## getGenerationSexMatched
setMethod("getGenerationSexMatched", "data.frame",
          function(object, id=NULL, include.anc=0, include.off=0, ...){
              object <- checkPedCol(object)
              object <- sanitizePed(object)
              return(doGetGenerationSexMatched(ped=object, id=id, include.anc=include.anc,
                                               include.off=include.off, ...))
          })
setMethod("getGenerationSexMatched", "pedigreeList",
          function(object, id=NULL, include.anc=0, include.off=0, ...){
              object <- ped2df(object)
              object <- checkPedCol(object)
              object <- sanitizePed(object)
              return(doGetGenerationSexMatched(ped=object, id=id, include.anc=include.anc,
                                               include.off=include.off, ...))
          })
setMethod("getGenerationSexMatched", "pedigree",
          function(object, id=NULL, include.anc=0, include.off=0, ...){
              object <- ped2df(object)
              object <- checkPedCol(object)
              object <- sanitizePed(object)
              return(doGetGenerationSexMatched(ped=object, id=id, include.anc=include.anc,
                                               include.off=include.off, ...))
          })

## getSexMatched
setMethod("getSexMatched", "data.frame",
          function(object, id=NULL, ...){
              object <- checkPedCol(object)
              object <- sanitizePed(object)
              return(doGetSexMatched(ped=object, id=id, ...))
          })
setMethod("getSexMatched", "pedigreeList",
          function(object, id=NULL, ...){
              object <- ped2df(object)
              object <- checkPedCol(object)
              object <- sanitizePed(object)
              return(doGetSexMatched(ped=object, id=id, ...))
          })
setMethod("getSexMatched", "pedigree",
          function(object, id=NULL, ...){
              object <- ped2df(object)
              object <- checkPedCol(object)
              object <- sanitizePed(object)
              return(doGetSexMatched(ped=object, id=id, ...))
          })

## getFounders
setMethod("getFounders", "data.frame",
          function(object, ...){
              object <- checkPedCol(object)
              object <- sanitizePed(object)
              return(doGetFounders(object, ...))
          })

## getSingletons
setMethod("getSingletons", "data.frame",
          function(object, ...){
              object <- checkPedCol(object)
              object <- sanitizePed(object)
              return(doGetSingletons(object, ...))
          })

## removeSingletons
setMethod("removeSingletons", "data.frame",
          function(object, ...) {
              return(.removeSingletons(object))
          })

##************************************************
##
##       FAProbResult
##
##       Main class defining/containing results and data.
##       * clique
##************************************************
setClass("FAProbResults",
         contains="FAData",
         slots=c(.cliques="factor",
                 sim="list",
                 nsim="numeric"),
         prototype=list(.cliques=factor(),
                        sim=list(),
                        nsim=0)
         )

setGeneric("probabilityTest", function(object, ...)
    standardGeneric("probabilityTest"))
setGeneric("cliques", function(object, ...)
    standardGeneric("cliques"))
setGeneric("cliques<-", function(object, value)
    standardGeneric("cliques<-"))
setGeneric("cliqueAndTrait", function(object, ...)
    standardGeneric("cliqueAndTrait"))
setGeneric("traitByClique", function(object)
    standardGeneric("traitByClique"))
##********************************************************************
##
##   Data analysis methods
##
##********************************************************************
## probabilistic test.
## based partially on code from Daniel Taliun
## x: FAData
## trait: trait data, need IDs and 0, 1 NA encoding.
## cliques: cliques data: two columns, first is "clique", second is ID.
## nsim: number of simulations.
## traitName: the name of the trait.
.prob_msg <- paste0("Due to problems with the 'gap' package in MS Windows the",
                    " 'probability test' will be removed in Bioconductor ",
                    "version 3.8. We apologize for any inconveniences.")
setMethod("probabilityTest", "FAData",
          function(object, trait, cliques, nsim=50000, traitName, ...){
              if(missing(cliques))
                  stop("cliques is missing!")
              if(missing(trait)){
                  ## check internal trait...
                  if(length(object@.trait) == 0)
                      stop("trait is missing!")
                  trait <- trait(object)
              }
              OrigTrait <- trait
              OrigCliques <- cliques
              ## building the result data object
              ## object <- as(object, "FATrait")
              object <- as(object, "FAProbResults")
              suppressMessages(
                  trait(object) <- trait
              )
              cliques(object) <- cliques
              if(!missing(traitName))
                  object@traitname <- traitName
              ## run the simulation: calls the runSimulation method for the
              ## FAProbResult class.
              object <- runSimulation(object, nsim=nsim, ...)
              return(object)
          })

##************************************************
##
##       FAProbResults
##
##
##************************************************
setMethod("show", "FAProbResults", function(object){
    ##cat("FAProbResults object with:\n")
    callNextMethod()
    ## print clique info.
    cat("Clique info:\n")
    cat(paste0(" * Total number of cliques: ",
               length(levels(cliques(object, na.rm=TRUE))), ".\n"))
    cat(paste0(" * Sum of individuals in pedigree not part of any clique: ",
        sum(is.na(cliques(object))), ".\n"))
    cat(paste0("Result info:\n"))
    cat(paste0(" * Dimension of result data.frame: ",
               dim(result(object)),".\n"))
    cat(paste0(" * Number of simulations: ", object@nsim, ".\n"))
})
## get the clique information.
setMethod("cliques", "FAProbResults", function(object, na.rm=FALSE){
    .Deprecated(msg = .prob_msg)
    cl <- object@.cliques
    if(na.rm){
        cl <- cl[!is.na(cl)]
    }
    cl <- droplevels(cl)
    return(cl)
})
setReplaceMethod("cliques", "FAProbResults", function(object, value){
    .Deprecated(msg = .prob_msg)
    if(!(is.character(value) | is.numeric(value) | is.factor(value))){
        stop("'cliques' should be a numeric or character vector or a factor!")
    }
    if(is.null(names(value)))
        stop("cliques should be a named vector with the names corresponding",
             " to the ids of the individuals in the pedigree!")
    ## Convert to character; for now
    valNames <- names(value)
    value <- as.character(value)
    names(value) <- valNames
    Pedigree <- pedigree(object)
    cliqInPed <- names(value) %in% Pedigree$id
    if(!any(cliqInPed))
        stop("None of the ids in cliques (i.e. names of the input argument",
             " cliques) can be matched to ids in the pedigree")
    value <- value[cliqInPed]
    ## create a cliques vector
    cliques <- rep(NA, nrow(Pedigree))
    names(cliques) <- Pedigree$id
    ## filling with values...
    cliques[match(names(value), names(cliques))] <- value
    cliques <- factor(cliques)
    object@.cliques <- cliques
    ## Reset results, if there are some...
    if(length(object@sim) > 0){
        message("Resetting results.")
        object@sim <- list()
    }
    ## object@traitname <- character()
    return(object)
})

## calling the trait replecement method from FAResult and in addition reset
## the simulation result.
setReplaceMethod("trait", "FAProbResults", function(object, value){
    .Deprecated(msg = .prob_msg)
    object <- callNextMethod()
    ## reset the result
    object@sim <- list()
    object@nsim <- 0
    object@traitname <- character()
    return(object)
})


## get a data.frame with the clique and the data from the trait for each
## individual, rownames are the individual IDs.
setMethod("cliqueAndTrait", "FAProbResults", function(object, na.rm=FALSE){
    .Deprecated(msg = .prob_msg)
    affClique <- data.frame(clique=cliques(object), trait=trait(object))
    if(na.rm){
        affClique <- affClique[!is.na(affClique[, 1]), ]
        affClique <- affClique[!is.na(affClique[, 2]), ]
    }
    droplevels(affClique)
})
## get a matrix with the size of the clique and number of affected, rownames
## correspond to the clique ID.
setMethod("traitByClique", "FAProbResults", function(object){
    .Deprecated(msg = .prob_msg)
    affClique <- cliqueAndTrait(object, na.rm=TRUE)
    CliqueSummary <- split(affClique, f=affClique$clique)
    CliqueSummary <- lapply(CliqueSummary, function(z){
        return(c(nrow(z), sum(z$trait)))
    })
    ## summary for the clique, i.e. size of the clique and number of affected.
    CliqueSummary <- do.call(rbind, CliqueSummary)
    colnames(CliqueSummary) <- c("size" , "affected_count")
    CliqueSummary
})

#######
## the analysis method:
setMethod("runSimulation", "FAProbResults", function(object, nsim=50000){
    .Deprecated(msg = .prob_msg)
    ## only makes sense if we've got trait and cliques
    if(length(trait(object)) == 0)
        stop("No trait information available!")
    if(length(cliques(object)) == 0)
        stop("No cliques information avaliable!")
    ## performing the test
    ## based on trait and clique...
    ## affClique <- getAffectedInClique(x)
    ## no of affected per clique and sum of clique members:
    CliqueSummary <- traitByClique(object)

    ## Dummy info...
    message("Cleaning data set (got in total ", length(object$id),
            " individuals):")
    message(" * not phenotyped individuals...", appendLF=FALSE)
    notPhen <- sum(is.na(trait(object)))
    if(notPhen > 0){
        message(" ", notPhen, " removed.")
    }else{
        message(" none present.")
    }
    message("Done.")

    ## what is the frequency???
    l_freq <- aggregate(rep(1, nrow(CliqueSummary)),
                        by=list(CliqueSummary[, "size"],
                                CliqueSummary[, "affected_count"]),
                        FUN=sum)
    ## ASK X_: what's that frequency???
    ## is it the size of a group, the number of affected and the number of
    ## times that this group size/affected count is occurring?
    colnames(l_freq) <- c("n", "affected", "freq")
    l_freq <- as.matrix(l_freq)

    ## Test if there are any cliques which size exceeds the max size
    ## supported by the gap package
    TooLarge <- l_freq[, "n"] > GAP_MAX_CLIQUE_SIZE
    if(sum(TooLarge) > 0)
        stop(sum(TooLarge), " cliques are larger than supported by the Monte ",
             "Carlo simulation in the gap package! Consider defining smaller ",
             "cliques!")
    if (.Platform$OS.type != "unix")
        warning("On Windows the 'gap' package on which this test is based on ",
                "seems to be buggy. You might encounter errors due to that.")
    fc.sim <- pfc.sim(l_freq, n.sim=nsim)
    object@nsim <- nsim
    object@sim <- fc.sim
    object
})

## results; get the results table.
setMethod("result", "FAProbResults", function(object, method="BH"){
    .Deprecated(msg = .prob_msg)
    if(length(object@sim)==0){
        stop("No simulation performed yet! Please use the probabilityTest ",
             "function or the runSimulation method to start the simulation.")
    }
    method <- match.arg(method, p.adjust.methods)
    ## generate the result table...
    ## get a data.frame with clique
    cltr <- cliqueAndTrait(object, na.rm=TRUE)
    ## get a summary matrix: size and affected per clique.
    ## note: that's the clique summary data for all individuals per clique
    ## for which we do have data in the trait!!! thus it's different from
    ## the table(cliques(Test))
    CliqSum <- traitByClique(object)
    Trait <- trait(object)
    ## subset to those which are part of any of the cliques...
    TraitNClique <- Trait[!is.na(cliques(object))]
    Trait <- Trait[!is.na(Trait)]
    TraitNClique <- TraitNClique[!is.na(TraitNClique)]
    ## Question: what is the Pedigree Size???
    TraitName <- object@traitname
    if(length(TraitName)==0)
        TraitName <- NA
    ## Get the ids and cliques
    Cl <- cliques(object)
    Cl <- Cl[!is.na(Cl)]
    ClIds <- names(Cl)
    names(ClIds) <- as.character(Cl)
    ## Get one representative ID for each clique
    ClIds <- ClIds[rownames(CliqSum)]
    MyRes <- data.frame(trait_name=TraitName,
                        total_phenotyped=length(Trait),
                        total_affected=sum(Trait!=0),
                        phenotyped=length(TraitNClique),
                        affected=sum(TraitNClique!=0),
                        group_id=rownames(CliqSum),
                        family=pedigree(object)[ClIds, "family"],
                        group_phenotyped=CliqSum[, "size"],
                        group_affected=CliqSum[, "affected_count"],
                        pvalue=object@sim$tailpu,
                        padj=p.adjust(object@sim$tailpu, method=method),
                        check.names=FALSE, stringsAsFactors=FALSE)
    rownames(MyRes) <- as.character(MyRes$group_id)
    return(MyRes)
})


#############
## plotting method...
## plotPed representing the results from the probabilistic test.
## plotPed for FAProbResults does only support plotting for id with the id
## being the group id.
## TODO: fix that: should I only plot the pedigree of the clique or of the
## full pedigree highlighing the clique?
setMethod("plotPed", "FAProbResults", function(object, id = NULL,
                                               family = NULL, filename = NULL,
                                               device = "plot", ...) {
    .Deprecated(msg = .prob_msg)
    if (!is.null(family))
        stop("Generating a pedigree for a family is not supported for ",
             "FAProbResults. See help for more information.")
    cliqs <- cliques(object)
    res <- result(object)
    if(!any(res$group_id == id))
        stop("The id should be one of the group ids (clique names; column ",
             "group_id in result(object))!")
    ## get all individuals of the group...
    clique <- names(cliqs)[which(as.character(cliqs) == as.character(id))]
    highlight.ids <- list(`*`=clique)  ## alternatively, use id=clique!
    callNextMethod(object=object, id=id, family=family,
                   filename=filename, proband.id=clique,
                   device=device,...)
})

## this buildPed simply ensures that all phenotyped in the group will be
## included in the pedigree.
## TODO: how to build the pedigree?
setMethod("buildPed", "FAProbResults", function(object, id = NULL,
                                                max.generations.up = 3,
                                                max.generations.down = 16,
                                                prune = FALSE) {
    .Deprecated(msg = .prob_msg)
    if(is.null(id))
        stop("The id of the group has to be speficied!")
    cliqs <- cliques(object)
    res <- result(object)
    if(!any(res$group_id == id))
        stop("The id should be one of the group ids (clique names; column ",
             "group_id in result(object))!")
    ## get all individuals of the group...
    clique <- names(cliqs)[which(as.character(cliqs) == as.character(id))]
    ped <- callNextMethod(object=object, id=clique, prune=prune,
                          max.generations.up=max.generations.up,
                          max.generations.down=max.generations.down)
    return(ped)
})

## this is to get all those that are related with any individuals of the group!
setMethod("shareKinship", "FAProbResults", function(object, id = NULL) {
    if(is.null(id))
        stop("The id of the group has to be speficied!")
    cliqs <- cliques(object)
    res <- result(object)
    if(!any(res$group_id == id))
        stop("The id should be one of the group ids (clique names; column ",
             "group_id in result(object))!")
    ## get all individuals of the group...
    clique <- names(cliqs)[which(as.character(cliqs) == as.character(id))]
    return(doShareKinship(kin=kinship(object), id=clique))
})

setMethod("[", "FAProbResults", function(x, i, j, ..., drop){
    stop("Subsetting of a FAProbResults object is not supported!")
})

##************************************************
##
##       FAKinSumResults
##       Most of the code here is from Chris X. Weichenberger
##
##************************************************
setMethod("show", "FAKinSumResults", function(object){
    callNextMethod()
    cat(paste0("Result info:\n"))
    cat(paste0(" * Number of rows of the result data.frame: ",
               nrow(result(object)),".\n"))
    cat(paste0(" * Number of simulations: ", object@nsim, ".\n"))
})
## calling the trait replacement method from FAResult and in addition
## reset the simulation result.
setReplaceMethod("trait", "FAKinSumResults", function(object, value){
    object <- callNextMethod()
    ## reset the result
    object@sim <- list()
    object@nsim <- 0
    object@traitname <- character()
    return(object)
})

####
## the analysis method.
setMethod("runSimulation", "FAKinSumResults",
          function(object, nsim=50000, strata=NULL, ...){
              if(length(trait(object)) == 0)
                  stop("No trait information available!")
              ind <- TRUE
              kin <- kinship(object)
              ## check strata
              ped <- pedigree(object)
              affectedIds <- as.character(ped[which(ped$affected > 0), "id"])
              phenotypedIds <-as.character(ped[!is.na(ped$affected), "id"])
              message("Cleaning data set (got in total ", nrow(ped),
                      " individuals):")
              ## Dummy for removing unaffected:
              message(" * not phenotyped individuals...")
              if(length(phenotypedIds) != nrow(ped)){
                  message(" ", nrow(ped)-length(phenotypedIds), " removed")
              }else{
                  message(" none present.")
              }
              if(!is.null(strata)){
                  if(length(strata)!=nrow(ped))
                      stop("Argument 'strata' has to have the same length than",
                           " there are individuals in the pedigree!")
                  ## add strata.
                  ped <- cbind(ped, STRATA=strata)
                  ## Subset the affectedIds and phenotypedIds to those for
                  ## which strata is not NA.
                  ped <- ped[!is.na(ped$STRATA) & !is.na(ped$affected), ,
                             drop=FALSE]
                  strata <- as.character(ped$STRATA)
                  names(strata) <- as.character(ped$id)
                  newAffIds <- as.character(ped[which(ped$affected > 0), "id"])
                  newPhenoIds <- as.character(ped[, "id"])
                  message(paste0(" * unaffected individuals without valid ",
                                 "strata values..."), appendLF=FALSE)
                  if(length(newPhenoIds)!=length(phenotypedIds)){
                      message(" ", length(phenotypedIds)-length(newPhenoIds),
                              " removed.")
                      phenotypedIds <- newPhenoIds
                  }else{
                      message(" none present.")
                  }
                  message(paste0(" * affected individuals without valid strata",
                                 " values..."), appendLF=FALSE)
                  if(length(newAffIds)!=length(affectedIds)){
                      message(" ", length(affectedIds)-length(newAffIds),
                              " removed.")
                      affectedIds <- newAffIds
                  }else{
                      message(" none present.")
                  }
              }
              message("Done")
              ## now forcing the kinship matrix to match ordering of
              ## phenotypedIds and eventually strata.
              kin <- kin[phenotypedIds, phenotypedIds]
              ## removing self-self kinship:
              diag(kin) <- NA
              object@nsim <- nsim
              ## break...
              if(length(affectedIds) <= 1){
                  warning(paste0("Trait ",
                                 object@traitname,
                                 ": only one or none affected individual!"))
                  return(object)
              }
              if(ind){
                  res <- significant.individuals.althyp(kin, affectedIds,
                                                        phenotypedIds,
                                                        nr.sim=nsim,
                                                        strata=strata,
                                                        ...)
              }else{
                  ## res <- significan.population(kin, affectedIds, phenotypedIds,
                  ##                              nr.sim=nsim)
              }
              object@sim <- res
              return(object)
          })

## results; get the results table.
setMethod("result", "FAKinSumResults",
          function(object, method="BH", cutoff=0.05, rmKinship=0){
    method <- match.arg(method, p.adjust.methods)
    TraitName <- object@traitname
    if(length(TraitName)==0)
        TraitName <- NA
    if(length(object@sim) == 0){
        ## generate a dummy matrix...
        MyRes <- data.frame(
            trait_name = TraitName,
            total_phenotyped = length(phenotypedIndividuals(object)),
            total_affected = length(affectedIndividuals(object)),
            affected_id = NA,
            family = NA,
            affected = NA,
            ksgrp = NA,
            kinship_sum = NA,
            freq = NA,
            pvalue = NA,
            padj = NA,
            check.names = FALSE, stringsAsFactors = FALSE)
        warning("No simulation data available! This means that either no ",
                "simulation was run yet (using the kinshipClusterTest ",
                "function or runSimulation method) or that the simulation ",
                "returned no result (i.e. too few affected individuals in ",
                "the trait).")
        return(MyRes)
    }
    resList <- object@sim
    affIds <- as.character(names(resList$sumKinship))
    ped <- pedigree(object)
    ped <- ped[ped$id %in% affIds, c("id", "family")]
    pedL <- split(ped, f=ped$id)
    fams <- unlist(lapply(pedL, function(z){
        return(paste(unique(z[, "family"]), collapse=", "))
    }))
    fams <- fams[affIds]
    res <- data.frame(
        trait_name=TraitName,
        total_phenotyped=length(phenotypedIndividuals(object)),
        total_affected=length(affectedIndividuals(object)),
        affected_id=affIds,
        family=fams,
        affected=rep(length(resList$affected), length(resList$sumKinship)),
        ksgrp=NA,
        kinship_sum=resList$sumKinship,
        freq=resList$frequencyKinship,
        pvalue=resList$pvalueKinship,
        padj=p.adjust(resList$pvalueKinship, method=method),
        stringsAsFactors=FALSE)
    rownames(res) <- as.character(res$affected_id)
    res <- res[order(res$pvalue, -res$kinship_sum), , drop=FALSE]
    padj <- res$padj
    names(padj) <- rownames(res)
    res$ksgrp <- define.result.groups.ks(kinship(object), padj, th=cutoff,
                                         rmKinship=rmKinship)
    return(res)
})

##' Define groups of related affected individuals for KS test.
##' @param kin Kinship matrix.
##' @param padj Named vector with adjusted P values of all affected individuals.
##' @param th  Threshold of `padj` for inclusion of affected individuals.
##' @param rmKinship Ignore all pairs with kinship <= rmKinship.
##' @return A named vector of group numbers or NAs, the names correspond to the
##' IDs of the affected individuals. An NA indicates that the affected
##' individual was above the threshold. The named vector corresponds to the
##' entries in vector `padj`.
define.result.groups.ks <- function(kin, padj, th=0.05, rmKinship=0)
{
    stopifnot("Vector padj must have associated names."=!is.null(names(padj)))
    allIDs <- names(padj)
    thIDs <- allIDs[padj<th]
    grpNr  <- 1
    grp    <- rep(NA, length(allIDs))
    names(grp) <- allIDs
    while( length(thIDs)>0 ) {
        kinIDs <- intersect(
            doShareKinship(kin=kin, id=thIDs[1], rmKinship=rmKinship), allIDs)
        ## Don't overwrite previously assigned group numbers.
        grp[is.na(grp) & names(grp) %in% kinIDs] <- grpNr
        grpNr <- grpNr + 1
        thIDs <- setdiff(thIDs, kinIDs)
    }
    grp
}

## null hypothesis. H0: sum of kinship values of affected
## with all other affected is random. Test: is the sum of kinship values of
## affected with all other affected larger or equal that we would expect by
## chance (randomly selected individuals being affected)? The result is
## essentially identical to the significant.individuals2 function.
## NOTE: this function is robust against cases in which the observed kinship
## sum was never sampled in the simulation! The significant.individuals.2
## function would return a NA for the p-value and the frequency, while the
## p-value could be non-NA if larger kinship sums were sampled in the simulation!
## strata is supposed to be ordered the same way than pool
## tableSimVals: if TRUE a table of the simulated kinship values is returned.
significant.individuals.althyp = function(ks, affected, pool, nr.sim,
                                          strata=NULL, tableSimVals = FALSE){
    if( length(affected)<=1 ) {
        res = data.frame(id=NA, kc=NA, freq=NA, pval=NA);
        return(res);
    }
    if(!all(affected %in% colnames(ks))){
        stop("Got affected ids for which I don't have a kinship value! ",
             "This shouldn't happen!")
    }
    obs <- colSums(ks[affected, affected], na.rm=TRUE)
    names(obs) <- affected

    ## We compute small chunks of simulated data in order to keep the memory
    ## footprint low. This involves strategies to compute incremental tables,
    ## densities and histograms.
    nr.sim.todo <- nr.sim
    estimates <- matrix(0, nrow = length(obs), ncol = 2,
                        dimnames = list(names(obs), c("freq", "pval")))
    expDensity <- expHist <- expTable <- NULL
    simCount <- 0
    nr.inc.sim <- 1000
    while( nr.sim.todo>0 ) {
        ## Last simulation loop: run only the leftover number of steps.
        if( nr.sim.todo<nr.inc.sim )
            nr.inc.sim <- nr.sim.todo
        ## return the actual values from the simulation.
        if(is.null(strata)){
            simVals <- bg.ks.distr(ks, affected, pool, nr.inc.sim)
        }else{
            simVals <- bg.ks.distr.strat(ks, affected, pool, nr.inc.sim, strata)
        }
        ## Perform all the incremental steps.
        simCount <- simCount + length(simVals)
        estimates <- estimates + do.call(rbind,lapply(obs, function(z){
            oFreq <- sum(simVals == z)
            oPval <- sum(simVals >= z)
            return(c(freq = if( is.na(oFreq) ) 0 else oFreq,
                     pval = if( is.na(oPval) ) 0 else oPval))
        }))
        expDensity <- inc.density(simVals, expDensity)
        expHist <- inc.hist(simVals, expHist)
        if( tableSimVals )
            expTable <- inc.table(simVals, expTable)
        nr.sim.todo <- nr.sim.todo - nr.inc.sim
    }

    if( expHist$nreplace>0 )
        warning("Histogram approximation (overflow): replaced ",
                expHist$nreplace, " value(s) with highest possible value, ",
                expHist$breaks[length(expHist$breaks)])
    sumKin <- obs
    names(sumKin) <- affected
    freqKin <- estimates[, "freq"]*100/simCount
    names(freqKin) <- affected
    pKin <- estimates[, "pval"]/simCount
    names(pKin) <- affected
    res <- list(sumKinship=sumKin,
                pvalueKinship=pKin,
                frequencyKinship=freqKin,
                expDensity=expDensity,
                expHist=expHist,
                affected=affected)
    if (tableSimVals) {
        res <- c(res, list(tableSimVals = expTable))
    }

    return(res)
}


## Am just sampling using the index 1:nrow of kinship matrix.
bg.ks.distr = function(ks, affIds, pool, nr.sim){
    ## Assuming that kinship and pool are ordered the same way!
    n <- length(affIds)
    colIdx <- 1:ncol(ks)
    bg <- lapply(1:nr.sim, function(x){
        affidx <- sample(colIdx, n, replace=FALSE)
        return(colSums(ks[affidx, affidx], na.rm=TRUE))
    })
    return(unlist(bg, use.names=FALSE))
}

## same as bg.ks.distr, but for stratified sampling.
## Important is that kinship, pool and strata are in the same ordering!
bg.ks.distr.strat = function(ks, affIds, pool, nr.sim, strata=NULL){
    ksColnames <- colnames(ks)
    names(strata) <- as.character(pool)
    affStrataCounts <- table(strata[affIds])
    affStrataCounts <- affStrataCounts[affStrataCounts > 0]
    bg <- lapply(1:nr.sim, function(x){
        ## subsetting by index is faster...
        affidx <- stratsample(strata, counts=affStrataCounts)
        return(colSums(ks[affidx, affidx], na.rm=TRUE))
    })
    return(unlist(bg, use.names=FALSE))
}



#############
## plotting method...
## plotPed representing the results from the kinship clustering test.
## plotPed for FAKinSumResults does only support plotting for id.
setMethod("plotPed", "FAKinSumResults",
          function(object, id=NULL,
                   family=NULL, filename=NULL,
                   device="plot", only.phenotyped=FALSE, ...){
                       if(!is.null(family))
                           stop("Generating a pedigree for a family is not ",
                                "supported for FAKinshipResult. See help for ",
                                "more information.")
                       callNextMethod(object=object, id=id, family=family,
                                      filename=filename,
                                      device=device, proband.id=id,
                                      only.phenotyped=only.phenotyped, ...)
                       ## alternatively use highlight.ids
                   })


setMethod("[", "FAKinSumResults", function(x, i, j, ..., drop){
    stop("Subsetting of a FAKinSumResults object is not supported!")
})

## This will be a crazy funky method to plot the simulation results.
setMethod("plotRes", "FAKinSumResults",
          function(object, id=NULL,
                   family=NULL, addLegend=TRUE, type="density", ...){
              type <- match.arg(type, c("density", "hist"))
              if(length(object@sim) == 0)
                  stop("No analysis performed yet!")
              if(is.null(id))
                  stop("The id of the affected individual for whom the ",
                       "simulation results should be displayed has to be ",
                       "specified!")
              id <- as.character(id)
              ## result on family is not allowed.
              if(!is.null(family) | length(id) > 1)
                  stop("plotRes for FAKinSumResults does only support ",
                       "specifying a single id!")
              ## check if the id is an affected for which we tested...
              if(!any(names(object@sim$pvalueKinship) == id))
                  stop("No simulation result is available for individual ", id,
                       ". id should be one of result(object)$affected_id.")
              ## OK, now we can really start doing things...
              kinSum <- object@sim$sumKinship[id]
              kinPval <- object@sim$pvalueKinship[id]
              fam <- family(object, id=id)[1, "family"]
              par(xpd=FALSE)
              if(type == "density"){
                  toplot <- object@sim$expDensity
                  XL <- range(c(range(toplot$x), kinSum))
                  plot(toplot, main=paste0("Affected: ", id, ", family: ", fam),
                       xlab="Kinship sum", type="h",
                       lwd=3, col="lightgrey", xlim=XL)
                  points(toplot, col="grey", type="l", lwd=2)
              }
              if(type == "hist"){
                  toplot <- object@sim$expHist
                  XL <- range(c(range(toplot$mids), kinSum))
                  plot(toplot, main=paste0("Affected: ", id, ", family: ", fam),
                       xlab="Kinship sum",
                       col="lightgrey", border="grey", xlim=XL)
              }
              Blue <- "#377EB8"
              abline(v=kinSum, col=Blue)
              if(addLegend){
                  legend("topright", legend=c(paste0("kinship sum: ",
                                                     format(kinSum, digits=2)),
                                              paste0("p-value     : ",
                                                     format(kinPval, digits=3))
                                              ))
              }
          })

## Methods for the classes
setMethod("pedigree", "missing", function(...){
    return(kinship2::pedigree(...))
})


##************************************************
##
##       Methods for FAData objects.
##
##************************************************
## validate the FAData object. Checks if the pedigree and age fit/match.
validateFAData <- function(object){
    Complaints <- character()
    if(!is.null(object@pedigree)){
        ## check the number of columns and column names.
        ## check the columns...
        value <- object@pedigree
        CN <- .PEDCN
        ##
        if(is.null(noColNames(value)) & ncol(value) == length(CN)){
            colnames(Value) <- CN
        }
        ## check if all expected colnames are there:
        if(all(CN %in% colnames(value))){
            ## all fine, just drop additional columns
            value <- value[, CN]
        }else{
            ## Missing one or more colnames!!!
            missingCN <- CN[!(CN %in% colnames(value))]
            Complaints <- c(Complaints, paste0("Required columns ",
                                               paste(missingCN, collapse=", "),
                                               " not present in pedigree!"))
        }
        object@pedigree$sex <- sanitizeSex(object@pedigree$sex)
        ## Check that we have a non-NA in id, father, mother
        if(any(is.na(object@pedigree$id)))
            Complaints <- c(Complaints, "No NAs in column 'id' allowed!")
        FatherId <- as.character(object@pedigree$father)
        if(!(all(FatherId[!is.na(FatherId)] %in%
                 as.character(object@pedigree$id))))
            Complaints <- c(
                Complaints,
                paste0("Some of the father IDs are not present",
                       " in the pedigree! All IDs in column 'father'",
                       " have to be either present in the pedigree, or",
                       " have to be <NA>."))
        MotherId <- as.character(object@pedigree$mother)
        if(!(all(MotherId[!is.na(MotherId)] %in%
                 as.character(object@pedigree$id))))
            Complaints <- c(
                Complaints,
                paste0("Some of the mother IDs are not present",
                       " in the pedigree! All IDs in column 'mother'",
                       " have to be either present in the pedigree, or",
                       " have to be <NA>."))
    }
    if(length(object@age)>0){
        ## has to be a named numeric vector!
        if(is.null(names(object@age))){
            Complaints <- c(Complaints,
                            "\nAge has to be a named numeric vector!")
        }
    }
    ## check the trait:
    if(length(object@.trait) > 0){
        ## trait has to have the same length than the pedigree has rows.
        if(length(object@.trait) != nrow(object@pedigree))
            Complaints <- c(
                Complaints,
                "\nlength of trait has to match the number of rows of pedigree!")
    }
    if(length(Complaints)>0){
        return(Complaints)
    }else{
        return(TRUE)
    }
}
setValidity("FAData", validateFAData)
setMethod("initialize", "FAData", function(.Object,...){
    OK <- validateFAData(.Object)
    if(is.character(OK)){
        stop(OK)
    }
    callNextMethod(.Object, ...)
})
setMethod("show", "FAData", function(object){
    cat(paste0(class(object), " object with:\n"))
    cat(paste0(" * Pedigree of length ", nrow(object@pedigree), ".\n"))
    if(!is.null(object@pedigree)){
        if(length(object@pedigree) > 0 & nrow(object@pedigree) > 0){
            ## Number of individuals.
            UnIds <- unique(object@pedigree[, "id"])
            cat(paste0(" * Number of unique individuals: ", length(UnIds), ".\n"))
            ## Number of families.
            famTab <- table(pedigree(object)[, "family"])
            cat(paste0(" * Number of families: ", length(famTab), ".\n"))
            ## Number of individuals in largest family.
            famTab <- sort(famTab)
            cat(paste0(" * Number of individuals in largest family: ",
                       famTab[length(famTab)], ".\n"))
            ## Number of individuals in the smallest family.
            cat(paste0(" * Number of individuals in smallest family: ",
                       famTab[1], ".\n"))
            ## Average number of individuals per family.
            if(length(age(object)) > 0){
                ## check for how many individuals we have a defined age...
                ages <- age(object)
                if (any(!is.na(ages)))
                    cat(paste0(" * Number of individuals with known age: ",
                               sum(!is.na(ages)), ".\n"))
            }
        }
        if(length(object@.trait) > 0){
            if(length(object@traitname)==0){
                cat("Information on an unnamed trait\n")
            }else{
                cat("Information on trait '", object@traitname, "'\n", sep = "")
            }
            cat(paste0(" * Number of non-NA values: ",
                       length(trait(object, na.rm=TRUE)),
                       ".\n"))
            cat(paste0(" * Number of non-zero values: ",
                       sum(trait(object, na.rm=TRUE)!=0),
                       ".\n"))
        }
    }
})


########
## getter/setter for pedigree.
setMethod("pedigree", "FAData", function(object, return.type="data.frame"){
    if(length(object@pedigree)==0){
        return(NULL)
    }
    return.type <- match.arg(return.type, c("data.frame", "pedigree"))
    ped <- object@pedigree
    CN <- .PEDCN
    if(ncol(ped)!=length(CN))
        stop("pedigree has more columns than expected!")
    if(noColNames(ped)){
        colnames(ped) <- CN
    }
    if(!is.factor(ped$sex)){
        ped$sex <- sanitizeSex(ped$sex)
    }
    ## should we add the trait information?
    if(length(object@.trait) > 0){
        tr <- trait(object)
        ped <- cbind(ped, affected=tr)
    }
    if(return.type=="pedigree"){
        if(any(colnames(ped) == "affected")){
            ped <- kinship2::pedigree(famid=ped[, "family"],
                                      id=ped[, "id"],
                                      dadid=ped[, "father"],
                                      momid=ped[, "mother"],
                                      sex=ped[, "sex"],
                                      affected=ped[, "affected"])
        }else{
            ped <- kinship2::pedigree(famid=ped[, "family"],
                                      id=ped[, "id"],
                                      dadid=ped[, "father"],
                                      momid=ped[, "mother"],
                                      sex=ped[, "sex"])
        }
    }
    return(ped)
})


## pedigree<- internally checks validity of the submitted value.
setReplaceMethod("pedigree", "FAData", function(object, value) {
    if (is.data.frame(value)) {
        CN <- .PEDCN
        if (!noColNames(value)) {
            ## check the colnames
            if (sum(CN %in% colnames(value)) != length(CN)) {
                stop(paste0("pedigree is expected to have column names ",
                            paste(CN, collapse=", "), "."))
            }
            value <- value[, CN]
        } else {
            ## no col names, thus assuming they fit what we expect.
            if (ncol(value)!=length(CN))
                stop(paste0("pedigree is expected to have ", length(CN),
                            " columns but I got ", ncol(value), "!"))
            colnames(value) <- CN

        }
        ## check and sanitize the sex...
        value[, "sex"] <- sanitizeSex(value$sex)
        if (length(unique(value$id)) != nrow(value))
            stop("IDs for individuals have to be unique, even for individuals",
                 " in different families!")
        rownames(value) <- as.character(value$id)
    } else if (is(value, "pedigree") | is(value, "pedigreeList")) {
        value <- ped2df(value)
    } else {
        stop("pedigree has to be a data.frame, a pedigree or a pedigreeList ",
             "object!")
    }

    ## Remove trailing and leading white spaces
    for (theCol in c("id", "family", "father", "mother")) {
        tmp <- value[, theCol]
        M <- mode(tmp)
        tmp <- gsub(tmp, pattern = "^(\\s)*", replacement = "")
        tmp <- gsub(tmp, pattern = "(\\s)*$", replacement = "")
        mode(tmp) <- M
        value[, theCol] <- tmp
        rm(tmp)
    }
    if (is.factor(value$id))
        value$id <- as.character(value$id)
    rownames(value) <- value$id
    ## Fix father/mother column. We're using NA for not present!
    if (is.factor(value$father))
        value$father <- as.character(value$father)
    if (is.factor(value$mother))
        value$mother <- as.character(value$mother)
    if (is.character(value$father))
        value$father[value$father == ""] <- NA
    if (is.character(value$father))
        value$father[value$father == "0"] <- NA
    if (is.numeric(value$father))
        value$father[value$father == 0] <- NA
    if (is.character(value$mother))
        value$mother[value$mother == ""] <- NA
    if (is.character(value$mother))
        value$mother[value$mother == "0"] <- NA
    if (is.numeric(value$mother))
        value$mother[value$mother == 0] <- NA

    object@pedigree <- value
    validObject(object)

    ## save the kinship matrix
    message("Generating the kinship matrix...", appendLF=FALSE)
    object@.kinship <- kinship2::kinship(pedigree(object,
                                                  return.type = "pedigree"))
    message("OK\n")

    return(object)
})


########
## get pedigree size
setMethod("pedigreeSize", "FAData", function(object){
    return(nrow(pedigree(object)))
})


## accessor for columns in pedigree...
setMethod("$", "FAData", function(x, name){
    tn <- x@traitname
    if(length(tn) == 0)
        tn <- ""
    if(name == "trait" | name == tn){
        return(trait(x))
    }
    vals <- eval(substitute(pedigree(x)$NAME_ARG, list(NAME_ARG=name)))
    if(length(vals) > 0){
        ## if we're accessing the id let's transform that to character
        if(name == "id")
            vals <- as.character(vals)
        ids <- pedigree(x)$id
        if(length(vals)==length(ids))
            names(vals) <- ids
    }
    return(vals)
})

## get the ids of all affected individuals in the trait
setMethod("affectedIndividuals", "FAData", function(object){
    if(length(object@.trait) == 0){
        warning("No trait information available")
        return(NULL)
    }
    tr <- trait(object, na.rm=TRUE)
    return(names(tr)[tr!=0])
})
## get the ids of all phenotyped individuals in the trait
setMethod("phenotypedIndividuals", "FAData", function(object){
    if(length(object@.trait) == 0){
        warning("No trait information available")
        return(NULL)
    }
    tr <- trait(object, na.rm=TRUE)
    return(names(tr))
})


########
## getter/setter for age.
setMethod("age", "FAData", function(object){
    ## we do have a pedigree, so we make sure we return ages in the same
    ## order than ids in the pedigree!
    if(length(object@pedigree) > 0){
        ped <- pedigree(object, return.type="data.frame")
        ages <- rep(NA, nrow(ped))
        names(ages) <- ped[, "id"]
        if(length(object@age) > 0){
            ageids <- intersect(names(ages), names(object@age))
            ages[ageids] <- object@age[ageids]
            return(ages)
        }else{
            return(ages)
        }
    }else{
        ## we don't have any pedigree, so just return ages:
        return(object@age)
    }
})
setReplaceMethod("age", "FAData", function(object, value){
    if(!is.numeric(value) | is.null(names(value)))
        stop("value has to be a named numeric vector.")
    object@age <- value
    ## check if the object is (still) valid
    validateFAData(object)
    return(object)
})
## get the trait information.
setMethod("trait", "FAData", function(object, na.rm=FALSE){
    tr <- object@.trait
    if(na.rm){
        return(tr[!is.na(tr)])
    }else{
        return(tr)
    }
})
## trait will be matched to the ids in the pedigree of object.
setReplaceMethod("trait", "FAData", function(object, value) {
    if (is.logical(value))
        value <- value + 0L
    if (is.numeric(value)) {
        ## check that trait is 0, 1, NA
        if (!all(unique(value) %in% c(0, 1, NA)))
            stop(paste0("trait should be a named logical vector or numeric",
                        " vector with values 0, 1 and NA!"))
    } else {
        stop("trait has to be either a named logical or numerical vector!")
    }
    if (all(is.na(value)))
        stop("Can not use a 'trait' with only NA values (i.e. without ",
             "phenotyped individuals)")
    if (is.null(names(value)))
        stop(paste0("trait has to be a named vector with the names",
                    " corresponding to the IDs used in the pedigree"))
    Pedigree <- pedigree(object)
    traitInPed <- names(value) %in% Pedigree$id
    ## subsetting trait to those...
    if (!any(traitInPed))
        stop(paste0("None of the ids in trait (i.e. names of the input",
                    " argument trait) can be matched to ids in the pedigree!"))
    value <- value[traitInPed]
    message(paste0(sum(traitInPed), " of in total ", length(traitInPed),
                   " trait values can be matched to IDs in the pedigree."))
    trait <- as.numeric(rep(NA, nrow(Pedigree)))
    names(trait) <- Pedigree$id
    ## filling with values...
    trait[match(names(value), names(trait))] <- value
    object@.trait <- trait
    return(object)
})



########
## getter for kinship. if not present in the .kinship slot it will be
## generated and saved.
setMethod("kinship", "FAData", function(id, ...){
    if(nrow(id@.kinship)==0){
        kin <- kinship2::kinship(pedigree(id, return.type="pedigree"))
    }else{
        kin <- id@.kinship
    }
    return(kin[as.character(id$id), as.character(id$id)])
})
#######
## get the family given an id or a family id.
setMethod("family",
          "FAData",
          function(object, id=NULL, family=NULL, return.type="data.frame"){
              if(missing(id))
                  id <- NULL
              if(missing(family))
                  family <- NULL
              return.type <- match.arg(return.type, c("data.frame", "pedigree"))
              if(is.null(id) & is.null(family))
                  stop("Either id or family has to be specified!")
              ## check first if we have id specified...
              ped <- pedigree(object)
              if(!is.null(id)){
                  ## get the ids of all families for these id(s)
                  family <- as.character(
                      ped[as.character(ped$id) %in% as.character(id), "family"]
                  )
              }
              if(!is.null(family) & length(family) > 0){
                  ped <- ped[as.character(ped$family) %in% family, ]
              }else{
                  return(NULL)
              }
              if(return.type=="pedigree")
                  if(any(colnames(ped)=="affected")){
                      ped <- pedigree(id=ped$id, dadid=ped$father,
                                      momid=ped$mother, sex=ped$sex,
                                      famid=ped$family, affected=ped$affected)
                  }else{
                      ped <- pedigree(id=ped$id, dadid=ped$father,
                                      momid=ped$mother, sex=ped$sex,
                                      famid=ped$family)
                  }
              return(ped)
          })

setMethod("buildPed", "FAData",
          function(object, id = NULL, family = NULL, max.generations.up = 3,
                   max.generations.down = 16,
                   prune = FALSE, ...){
              ped <- pedigree(object)
              if(is.null(id) & is.null(family))
                  return(ped)
              if (!is.null(family)) {
                  family <- as.character(family)
                  if (length(family) > 1)
                      stop("'family' should be the ID of a single family!")
                  have_fams <- as.character(unique(object$family))
                  if (!any(have_fams == family))
                      stop("'family' ", family, " not found in pedigree!")
                  id <- as.character(object$id)[object$family == family]
              }
              id <- as.character(id)
              notThere <- !(id %in% ped$id)
              if (any(notThere)) {
                  id <- id[!notThere]
                  warning("Removed ", sum(notThere),
                          " ids since they are not in the pedigree!")
                  if (length(id) == 0)
                      stop("No id available!")
              }
              if ((length(id) == 1) & prune) {
                  prune <- FALSE
                  warning("Setting prune=FALSE since there is only a single id!")
              }
              if (prune) {
                  ## restrict to the smalles pedigree including the ids
                  return(subPedigree(ped, id = id))
              }
              ## else grow the full pedigree
              ## first get all ancestors:
              ancs <- doGetAncestors(ped, id = id,
                                     maxlevel = max.generations.up)
              ## add eventual missing mates for the ancestors:
              mismate <- doGetMissingMate(ped, id = ancs)
              allids <- unique(c(ancs, id, mismate))
              ## next get all children:
              chlds <- doGetChildren(ped, id = allids,
                                     maxlevel = max.generations.down)
              ## Get eventually missing parents for the children: special case
              ## for one CHRIS pedigree
              allids <- unique(c(allids, chlds, doGetParents(ped, chlds)))
              ## Get all missing mates:
              allids <- unique(c(allids, doGetMissingMate(ped, allids)))
              ped <- ped[as.character(allids), , drop = FALSE]
              ## fixfounders:
              ## set all mothers and fathers that are not in id to 0
              ped[!(ped$mother %in% ped$id), "mother"] <- NA
              ped[!(ped$father %in% ped$id), "father"] <- NA
              ped <- removeSingletons(ped)
              ped
          })


## [ subsetting.
.bracketSubset <- function(x, i, j, ..., drop){
    if(!missing(j))
        stop("Subsetting by columns ('j') is not supported")
    haveRows <- nrow(pedigree(x))
    if(length(haveRows) == 0){
        warning("Can not subset an empty object.")
        return(x)
    }
    if(missing(i)){
        i <- 1:haveRows
    }
    ## check i:
    i <- .checkRowindex(i, pedigree(x))
    ## don't use the accessor here!
    pedSub <- x@pedigree[i, , drop=FALSE]
    ageSub <- numeric()
    if(length(x@age) > 0){
        ## subsetting age.
        ageSub <- x@age[names(x@age) %in% rownames(pedSub)]
    }
    ## In order to have a valid pedigree object, I have to set all father and
    ## mother IDs which are not in column $id to NA.
    pedSub[!(pedSub$father %in% pedSub$id), "father"] <- NA
    pedSub[!(pedSub$mother %in% pedSub$id), "mother"] <- NA
    ## have to use character colnames here.
    kinSub <- kinship(x)[as.character(pedSub$id), as.character(pedSub$id),
                         drop=FALSE]
    newX <- new("FAData", pedigree=pedSub, age=ageSub, .kinship=kinSub)
    ## subsetting trait...
    if(length(x@.trait) > 0){
        subTrait <- trait(x)[i]
        suppressWarnings(
            trait(newX) <- subTrait
        )
        newX@traitname <- x@traitname
    }
    validObject(newX)
    return(newX)
}
setMethod("[", "FAData", .bracketSubset)

## i can be a numeric, character or logical vector which is supposed to be used
## for subsetting data.
## data is a data.frame or matrix with rownames.
## the function checks if i is valid to subset data and returns a numeric vector
## that can be used for subsetting.
.checkRowindex <- function(i, data){
    haveRows <- nrow(data)
    if(!is.logical(i) & !is.numeric(i) & !is.character(i))
        stop("'i' has to be either a logical, numeric or character vector!")
    ## i can be boolean -> has to be the same length than haveRows
    if(is.logical(i)){
        if(length(i) != haveRows)
            stop("If 'i' is a logical vector its length has to match",
                 " the number individuals in the pedigree!")
        ## transform to numeric
        i <- which(i)
    }
    ##          character -> have to match rownames of pedigree.
    if(is.character(i)){
        iLen <- length(i)
        i <- match(i, rownames(data))
        if(all(is.na(i))){
            stop("None of the elements in 'i' matches an id of an",
                 " individual (i.e. rownames of pedigree)!")
        }
        if(any(is.na(i))){
            warning(sum(is.na(i)), " elements in 'i' can not be",
                    " matched to ids of individuals and were thus discarded.")
            i <- i[!is.na(i)]
        }
    }
    ##          numeric -> indices have to be within haveRows
    iLen <- length(i)
    i <- i[i >=1 & i <=haveRows]
    if(length(i)==0)
        stop("'i' has to be a numeric between 1 and ", haveRows, "!")
    if(iLen != length(i))
        warning("Some of the values in 'i' are outside of the",
                " allowed range [1,",haveRows, "] and were thus discarded")
    ## have it.
    return(i)
}


##
## kinship test.
## based partially on code from Daniel Taliun.
setMethod("kinshipGroupTest", "FAData",
          function(object, trait, nsim=50000, traitName, strata=NULL, ...){
              if(missing(trait)){
                  if(length(object@.trait) == 0)
                      stop("trait is missing!")
                  trait <- trait(object)
              }
              OrigTrait <- trait
              ## building the result data object
              ## object <- as(object, "FATrait")
              object <- as(object, "FAKinGroupResults")
              suppressMessages(
                  trait(object) <- trait
              )
              if(!missing(traitName))
                  object@traitname <- traitName
              ## run the simulation: calls the runSimulation method for the
              ## FAKinshipResult class.
              object <- runSimulation(object, nsim=nsim, strata=strata, ...)
              return(object)
          })

## kinship clustering test.
setMethod("kinshipSumTest", "FAData",
          function(object, trait, nsim=50000, traitName, strata=NULL, ...){
              if(missing(trait)){
                  if(length(object@.trait) == 0)
                      stop("trait is missing!")
                  trait <- trait(object)
              }
              ## object <- as(object, "FAResult")
              object <- as(object, "FAKinSumResults")
              suppressMessages(
                  trait(object) <- trait
              )
              if(!missing(traitName))
                  object@traitname <- traitName
              ## run the simulation.
              object <- runSimulation(object, nsim=nsim, strata=strata, ...)
              return(object)
          })

## genealogical index
setMethod("genealogicalIndexTest", "FAData",
          function(object, trait, nsim=50000, traitName,
                   perFamilyTest=FALSE, controlSetMethod="getAll",
                   rm.singletons=TRUE, strata=NULL, ...){
              if(missing(trait)){
                  if(length(object@.trait) == 0)
                      stop("trait is missing!")
                  trait <- trait(object)
              }
              ## building the result data object
              ## object <- as(object, "FAResult")
              object <- as(object, "FAGenIndexResults")
              suppressMessages(
                  trait(object) <- trait
              )
              if(!missing(traitName))
                  object@traitname <- traitName
              ## run the simulation: calls the runSimulation method for the
              ## FAProbResult cla ss.
              runSimulation(object, nsim=nsim,
                                      perFamilyTest=perFamilyTest,
                                      controlSetMethod=controlSetMethod,
                                      rm.singletons=rm.singletons,
                                      strata=strata, ...)
          })


##****************************************************************************
##
##  familial incidence rate as described in Kerber
##
##
setMethod("familialIncidenceRate", "FAData",
          function(object, trait=NULL, timeAtRisk=NULL){
              ## .FR is defined in Methods-FAIncidenceRatio.R
              FR <- .FR(ped=pedigree(object), kin=kinship(object), trait=trait,
                        timeAtRisk=timeAtRisk, perFamilyTest=FALSE)
              Res <- rep(NA, length(object$id))
              names(Res) <- object$id
              FR <- FR[[1]]
              Res[names(FR)] <- FR
              return(Res)
          })

##****************************************************************************
##
##  familialIncidenceRateTest method.
##
setMethod("familialIncidenceRateTest", "FAData",
          function(object, trait=NULL, nsim=50000, traitName=NULL,
                   timeAtRisk=NULL, strata=NULL, ...){
              if(is.null(trait)){
                  if(length(object@.trait) == 0)
                      stop("trait is missing!")
                  trait <- trait(object)
              }
              object <- as(object, "FAIncidenceRateResults")
              suppressMessages(
                  trait(object) <- trait
              )
              if(!is.null(traitName))
                  object@traitname <- traitName
              ## run the simulation
              object <- runSimulation(object, nsim=nsim, timeAtRisk=timeAtRisk,
                                      strata=strata, ...)
              return(object)
          })


##****************************************************************************
##
##  fsir method.
##  familial standardized incidence rate as described in Kerber
##
setMethod("fsir", "FAData", function(object, trait=NULL, lambda=NULL,
                                     timeInStrata=NULL){
    ## First we need to do a lot of checking and testing.
    if(is.null(trait)){
        ## check internal trait...
        if(length(object@.trait) == 0)
            stop("trait is missing!")
        trait <- trait(object)
    }
    ## Check lambda
    if(is.null(lambda))
        stop("lambda missing!")
    if(is.null(names(lambda)))
        stop(paste0("lambda has to be a named vector with the names",
                    "corresponding to the strata names!"))
    ## Check timeInStrata
    if(is.null(timeInStrata))
        stop("timeInStrata missing!")
    if (!is.matrix(timeInStrata))
        stop("timeInStrata has to be a matrix!")
    if(nrow(timeInStrata) != length(object$id))
        stop(paste0("timeInStrata has to have the same number of rows as",
                    " there are individuals in the pedigree!"))
    if(length(lambda)!=ncol(timeInStrata))
        stop("length of lambda has to match the number of columns of timeInStrata!")
    if(!all(colnames(timeInStrata) %in% names(lambda)))
        stop("Names of lambda does not match the colnames of timeInStrata!")
    lambda <- lambda[colnames(timeInStrata)]
    ## Done.
    suppressMessages(
        trait(object) <- trait
    )
    trait <- trait(object)   # that way we ensure that we have the same ordering.
    kin <- kinship(object)
    ## Just to be on the save side... ensure that the ordering of id/Trait
    ## matches the kin
    kin <- kin[names(trait), names(trait)]
    diag(kin) <- 0
    ## Now start subsetting the data:
    message("Cleaning data set (got in total ", nrow(kin), " individuals):")
    message(" * not phenotyped individuals...", appendLF=FALSE)
    ## * NA in trait
    nas <- is.na(trait)
    if(any(nas)){
        trait <- trait[!nas]
        kin <- kin[!nas, !nas]
        timeInStrata <- timeInStrata[!nas, , drop=FALSE]
        message(" ", sum(nas), " removed.")
    }else{
        message(" none present.")
    }
    ## * NA in timeInStrata.
    nas <- apply(timeInStrata, MARGIN=1, function(z){
        any(is.na(z))
    })
    message(" * individuals with missing time in strata...", appendLF=FALSE)
    if(any(nas)){
        trait <- trait[!nas]
        kin <- kin[!nas, !nas]
        timeInStrata <- timeInStrata[!nas, , drop=FALSE]
        message(" ", sum(nas), " removed.")
    }else{
        message(" none present.")
    }
    ## Anyway removing singletons here, since they result in NA values!
    message(" * singletons (also caused by previous subsetting)...",
            appendLF=FALSE)
    ## * Not related, i.e. individuals with a kinship sum of 0
    nas <- colSums(kin) == 0
    if(any(nas)){
        trait <- trait[!nas]
        kin <- kin[!nas, !nas]
        timeInStrata <- timeInStrata[!nas, , drop=FALSE]
        message(" ", sum(nas), " removed.")
    }else{
        message(" none present.")
    }
    message("Done")

    ## Well done. Now let's do the test:
    fsirs <- doFsir(affected=trait, kin=kin, lambda=lambda,
                    timeInStrata=timeInStrata)
    ## Prepare the results vector.
    allIds <- object$id
    allFsirs <- rep(NA, length(allIds))
    names(allFsirs) <- allIds
    allFsirs[names(trait)] <- as.numeric(fsirs)
    return(allFsirs)
})

##****************************************************************************
##
##  fsirTest method.
##
setMethod("fsirTest", "FAData",
          function(object, trait=NULL, nsim=50000, traitName=NULL, lambda=NULL,
                   timeInStrata=NULL, strata=NULL, ...){
              if(is.null(trait)){
                  if(length(object@.trait) == 0)
                      stop("trait is missing!")
                  trait <- trait(object)
              }
              object <- as(object, "FAStdIncidenceRateResults")
              suppressMessages(
                  trait(object) <- trait
              )
              if(!is.null(traitName))
                  object@traitname <- traitName
              ## run the simulation
              object <- runSimulation(object, nsim=nsim, lambda=lambda,
                                      timeInStrata=timeInStrata,
                                      strata=strata, ...)
              return(object)
          })



.checkLabels <- function(label, fam){
    if(is.null(label))
        return(NULL)
    if(length(label) == nrow(fam)){
        names(label) <- fam$id
    }else{
        ## do we have names?
        if(is.null(names(label))){
            warning("Label vector does not have names and was thus discarded!")
            return(NULL)
        }
    }
    ## check and match by names.
    subset <- names(label) %in% fam$id
    if(sum(subset) == 0){
        warning(paste0("None of the names of label match to ids in the",
                       " pedigree! Argument label was thus discarded!"))
        return(NULL)
    }
    label <- label[subset]
    NAs <- is.na(label)
    if(is.numeric(label)){
        label <- format(label, digits=2)
        label[NAs] <- ""
    }
    realLabel <- rep(NA, nrow(fam))
    names(realLabel) <- fam$id
    realLabel[names(label)] <- label
    return(realLabel)
}

##********************************************************************
##
##   Plotting
##
##********************************************************************
## plotting method...
## if family not NULL: plot the pedigree for the FULL family.
## if id is not NULL: build the pedigree around that id and plot that.
## highlight.ids: character vector of ids or named list with character vector(s)
setMethod("plotPed", "FAData",
          function(object, id = NULL, family = NULL, filename = NULL,
                   device = "plot", symbol.related = NA, proband.id = NULL,
                   highlight.ids = NULL, only.phenotyped = FALSE,
                   label1 = age(object), label2 = NULL, label3 = NULL, ...) {
              ## if id was defined build the pedigree for that individual,
              ## otherwise plot the full family.
              Args <- list(...)
              if (!is.null(id)) {
                  ## only extract arguments we might need for buildPed:
                  max.generations.up <- 3
                  max.generations.down <- 16
                  prune <- FALSE
                  if (any(names(Args) == "max.generations.up"))
                      max.generations.up <- Args$max.generations.up
                  if (any(names(Args) == "max.generations.down"))
                      max.generations.down <- Args$max.generations.down
                  if (any(names(Args) == "prune"))
                      prune <- Args$prune
                  fam <- buildPed(object, id = id,
                                  max.generations.up = max.generations.up,
                                  max.generations.down = max.generations.down,
                                  prune = prune)
              } else {
                  fam <- family(object, id = id, family = family)
              }
              if (nrow(fam) == 0)
                  stop("No data left for plotting after sub-setting.")
              ## Ensure that the pedigree has the same order...
              fam <- fam[order(match(fam$id, pedigree(object)$id)), ]
              ## check if we've got affected
              if (any(colnames(fam) == "affected")) {
                  affected <- fam[, "affected"]
                  haveAffected <- TRUE
              } else {
                  affected <- rep(NA, nrow(fam))
                  names(affected) <- as.character(fam$id)
                  haveAffected <- FALSE
              }
              if (is.null(symbol.related))
                  symbol.related <- NA
              ## checking labels.
              label1 <- .checkLabels(label1, fam)
              label2 <- .checkLabels(label2, fam)
              label3 <- .checkLabels(label3, fam)
              ## do not have affected... obviously...
              is.proband <- rep(FALSE, nrow(fam))
              names(is.proband) <- as.character(fam$id)
              if (!missing(proband.id)) {
                  proband.id <- as.character(proband.id)
                  is.proband[names(is.proband) %in% proband.id] <- TRUE
                  if (sum(is.proband) != length(proband.id))
                      warning("Not all probands specified in proband.id are ",
                              "in the pedigree!")
              }
              text.inside.symbol <- rep("", nrow(fam))
              names(text.inside.symbol) <- as.character(fam$id)
              if (!is.null(id)) {
                  ## get individuals that share kinship
                  related <- shareKinship(object, id = id)
                  related <- related[related %in% fam$id]
                  ## could also use shareKinship(object, id), but that's not
                  ## that efficient!
                  related <- related[related %in% names(text.inside.symbol)]
                  text.inside.symbol[related] <- symbol.related
              }
              text2.below.symbol = NULL
              text3.below.symbol = NULL
              text4.below.symbol = NULL
              if (!is.null(highlight.ids)) {
                  if (is.character(highlight.ids)) {
                      highlight.ids <- list(`*`=highlight.ids)
                  }
                  if (is.list(highlight.ids)) {
                      ## support at max 3 highlight lines.
                      for (i in 1:min(c(3, length(highlight.ids)))) {
                          if (is.null(names(highlight.ids)[i])) {
                              symb <- "*"
                          } else {
                              symb <- names(highlight.ids)[i]
                          }
                          texts <- rep("", nrow(fam))
                          names(texts) <- as.character(fam$id)
                          texts[names(texts) %in% highlight.ids[[i]]] <- symb
                          if (i == 1)
                              text2.below.symbol <- texts
                          if (i == 2)
                              text3.below.symbol <- texts
                          if (i == 3)
                              text4.below.symbol <- texts
                      }
                  } else {
                      warning("Discarding argument highlight.ids. It",
                              " should be a character vector of ids or a",
                              " (named) list of character vectors with ids.")
                  }
              }
              ## Note that label1 to label3 has precedence to any other argument!
              if (!is.null(label2))
                  text2.below.symbol <- label2
              if (!is.null(label3))
                  text3.below.symbol <- label3
              ## If we want to plot only phenotyped individuals...
              if (only.phenotyped & haveAffected){
                  ## use the buildPed...
                  keepIds <- fam[!is.na(fam[, "affected"]), "id"]
                  subped <- subPedigree(fam, id=as.character(keepIds), all=FALSE)
                  plotMe <- as.character(fam$id) %in% as.character(subped$id)
              } else {
                  plotMe <- rep(TRUE, nrow(fam))
              }
              ## OK, now plot!!!
              res <- doPlotPed(family = fam$family[plotMe],
                               individual = fam$id[plotMe],
                               father = fam$father[plotMe],
                               mother = fam$mother[plotMe],
                               gender = fam$sex[plotMe],
                               is.proband = is.proband[plotMe],
                               text1.below.symbol = label1[plotMe],
                               text2.below.symbol = text2.below.symbol[plotMe],
                               text3.below.symbol = text3.below.symbol[plotMe],
                               text4.below.symbol = text4.below.symbol[plotMe],
                               text.inside.symbol = text.inside.symbol[plotMe],
                               affected = affected[plotMe],
                               filename = filename, device = device, ...)
              invisible(res)
          })





##********************************************************************
##
##   Pedigree utilities
##
##********************************************************************
setMethod("countGenerations", "FAData",
          function(object, id=NULL, direction="down", ...){
              if(nrow(pedigree(object)) == 0)
                  stop("No pedigree available!")
              doCountGenerations(pedigree(object), id=id, direction=direction)
          })
setMethod("estimateGenerations", "FAData",
          function(object, family=NULL, ...){
              if(nrow(pedigree(object)) == 0)
                  stop("No pedigree available!")
              doEstimateGenerationsFor2(pedigree(object), family=family)
          })
setMethod("findFounders", "FAData",
          function(object, family = NULL, id = NULL, ...){
              if(nrow(pedigree(object)) == 0)
                  stop("No pedigree available!")
              doFindFounders(pedigree(object), family = family, id = id)
          })
setMethod("generationsFrom", "FAData",
          function(object, id=NULL, ...){
              if(nrow(pedigree(object)) == 0)
                  stop("No pedigree available!")
              doGetGenerationFrom2(pedigree(object), id=id, ...)
          })
setMethod("getAncestors", "FAData",
          function(object, id=NULL, max.generations=3, ...){
              if(nrow(pedigree(object)) == 0)
                  stop("No pedigree available!")
              doGetAncestors(pedigree(object), id=id,
                             maxlevel=max.generations, ...)
          })
setMethod("getChildren", "FAData",
          function(object, id=NULL, max.generations=16, ...){
              if(nrow(pedigree(object)) == 0)
                  stop("No pedigree available!")
              doGetChildren(pedigree(object), id=id,
                            maxlevel=max.generations, ...)
          })
setMethod("getMissingMate", "FAData",
          function(object, id=NULL, ...){
              if(nrow(pedigree(object)) == 0)
                  stop("No pedigree available!")
              doGetMissingMate(pedigree(object), id=id, ...)
          })
setMethod("getSiblings", "FAData",
          function(object, id=NULL, ...){
              if(nrow(pedigree(object)) == 0)
                  stop("No pedigree available!")
              doGetSiblings(pedigree(object), id=id, ...)
          })

setMethod("shareKinship", "FAData",
          function(object, id=NULL, rmKinship=0){
              if(is.null(id))
                  stop("id has to be specified!")
              doShareKinship(kin=kinship(object), id=id, rmKinship=rmKinship)
          })

setMethod("getCommonAncestor", "FAData",
          function(object, id, method="min.dist"){
              doGetCommonAncestor(pedigree(object), id=id, method=method)
          })



##***************************************************************************
##
##   Methods to get ids from the pedigree that could be used as controls
##   for the given list of ids.
##
##***************************************************************************
## getAll
setMethod("getAll", "FAData",
          function(object, id=NULL, ...){
              doGetAll(pedigree(object), id=id, ...)
          })
setMethod("getExternalMatched", "FAData",
          function(object, id=NULL, match.using, ...){
              doGetExternalMatched(pedigree(object), id=id, match.using, ...)
          })
setMethod("getGenerationMatched", "FAData",
          function(object, id=NULL, include.anc=0, include.off=0, ...){
              doGetGenerationMatched(pedigree(object), id=id,
                                     include.anc=include.anc,
                                     include.off=include.off, ...)
          })
setMethod("getGenerationSexMatched", "FAData",
          function(object, id=NULL, include.anc=0, include.off=0, ...){
              doGetGenerationSexMatched(pedigree(object), id=id,
                                        include.anc=include.anc,
                                        include.off=include.off, ...)
          })
setMethod("getSexMatched", "FAData",
          function(object, id=NULL, ...){
              getSexMatched(pedigree(object), id=id, ...)
          })


####============================================================
##
##  export
##
##  Export the pedigree information.
##
####------------------------------------------------------------
setMethod("export", "FAData", function(object, con, format="ped", ...){
    if(missing(con))
        stop("The file name has to be specified!")
    if(missing(format))
        stop("The format has to be specified (either 'ped' or 'fam')!")
    format <- match.arg(format, c("ped", "fam"))
    ped <- pedigree(object)
    if(format == "ped" | format == "fam"){
        ped <- doProcessDf2Ped(ped)
        write.table(ped, file=con, quote=FALSE, sep="\t", row.names=FALSE,
                    col.names=FALSE)
    }
})


####============================================================
##  getFounders
##
##  return the ids of the founders in the pedigree
####------------------------------------------------------------
setMethod("getFounders", "FAData", function(object, ...){
    return(doGetFounders(pedigree(object)))
})

####============================================================
##  getSingletons
##
##  return the id of the childless founders.
####------------------------------------------------------------
setMethod("getSingletons", "FAData", function(object, ...){
    return(doGetSingletons(pedigree(object)))
})

############################################################
## removeSingletons
##
setMethod("removeSingletons", "FAData", function(object, ...) {
    ped <- removeSingletons(pedigree(object))
    if (nrow(ped) == nrow(pedigree(object)))
        return(object)
    ## In addition to replace the pedigree, we have also to replace
    if (length(object@.trait) > 0) {
        the_trait <- object@.trait
        object@.trait <- numeric()
    } else
        the_trait <- numeric()
    pedigree(object) <- ped
    ## the trait
    if (length(the_trait) > 0)
        trait(object) <- the_trait[rownames(ped)]
    return(object)
})

####============================================================
##  import ped files
##
##  Function to import the ped file format used in plink: Expected
##  columns:
##  + Family ID
##  + Sample ID
##  + Paternal ID (0 if father isn't in dataset)
##  + Maternal ID (0 if mother isn't in dataset)
##  + Sex (1=male; 2=female; other=unknown)
##  + Affection (0=unknown; 1=unaffected; 2=affected)
##  + Genotypes (space or tab separated, 2 for each marker. 0, -9,
##    non-numeric=missing)
##  We will ignore the Genotype information though.
####------------------------------------------------------------
doImportPed <- function(x, sep="\t", header=FALSE, ...){
    df <- read.table(x, sep=sep, header=header)
    doProcessPed2Df(df)
}

####============================================================
##  import fam files
##
##  Function to import the fam file format used in plink. The fam
##  file corresponds to the first 6 columns of the ped file.
####------------------------------------------------------------
doImportFam <- function(x, sep="\t", header=FALSE, ...){
    df <- read.table(x, sep=sep, header=header)
    doProcessPed2Df(df)
}

####============================================================
##  import generic txt files
##
####------------------------------------------------------------
doImportGeneric <- function(x, sep="\t", header=TRUE, id.col="id",
                            family.col="family", father.col="father",
                            mother.col="mother", sex.col="sex", ...) {
    df <- read.table(x, sep=sep, header=header)
    if (header) {
        ## check if I have all required column names...
        if(!all(c( id.col, family.col, father.col, mother.col, sex.col) %in%
                colnames(df)))
            stop("One or more required column names not found! Expect: ",
                 family.col, ", ", id.col, ", ", father.col, ", ",
                 mother.col, ", ", sex.col, "!")
        df <- df[, c(family.col, id.col, father.col, mother.col, sex.col)]
        colnames(df) <- .PEDCN
    }
    df
}

####============================================================
##  process a data.frame corresponding to a ped file and re-format
##  it according to our needs. The expected columns are:
##  + Family ID
##  + Sample ID
##  + Paternal ID (0 if father isn't in dataset)
##  + Maternal ID (0 if mother isn't in dataset)
##  + Sex (1=male; 2=female; other=unknown)
##  + Affection (0, -9, non-numeric=unknown; 1=unaffected; 2=affected)
##
####------------------------------------------------------------
doProcessPed2Df <- function(x){
    ## Subset the ped data.frame to it's first 6 columns.
    if(ncol(x) < 6)
        stop("A fam/ped file is expected to have at least 6 columns! ",
             "The present one has however only ", ncol(x), "!")
    ## Extract family ID.
    FamId <- as.character(x[, 1])
    ## Extract individual ID.
    IId <- as.character(x[, 2])
    ## Checking father id and mother id; should be ALL in the pedigree!
    FatherId <- as.character(x[, 3])
    ## Set all empty ("") and 0s to NA
    FatherId[which(FatherId == "" | FatherId == "0" )] <- NA
    if(!all(FatherId[!is.na(FatherId)] %in% IId))
        stop("If provided (i.e. different from '0') the father ID has",
             " to correspond to an ID in the pedigree!")
    MotherId <- as.character(x[, 4])
    MotherId[which(MotherId == "" | MotherId == "0" )] <- NA
    if(any(!(MotherId[!is.na(MotherId)] %in% IId)))
        stop("If provided (i.e. different from '0') the mother ID has",
             " to correspond to an ID in the pedigree!")
    ## Check the sex: 1=male, 2=female; other=unknown.
    Sex <- rep(NA, nrow(x))
    Sex[which(x[, 5] == "1")] <- 1
    Sex[which(x[, 5] == "2")] <- 2
    ## Affected:
    ## 0, -9 or any non-numeric are mapped to NA (unknown)
    ## 1 is mapped to 0 (unaffected)
    ## 2 is mapped to 1 (affected)
    Affected <- x[, 6]
    ## Check if we've got case/control or numeric. We're not supporting the latter!
    if(is.numeric(Affected)){
        amNa <- is.na(Affected)
        if(!all(Affected[!amNa] %in% c(1, 2, 0, -9)))
            stop("FamAgg supports only categorical case/control phenotype",
                 " information (i.e. values 1, 2, 0 or -9 in phenotype ",
                 "column 6 of the data file)!")
    }
    ## Convert to character and fix to only NA, 0 and 1.
    Affected <- as.character(Affected)
    NewAff <- rep(NA, length(Affected))
    NewAff[Affected == "1"] <- 0
    NewAff[Affected == "2"] <- 1
    ## Compile the data.frame
    df <- data.frame(family=FamId, id=IId, father=FatherId, mother=MotherId,
                     sex=Sex, trait=NewAff, stringsAsFactors=FALSE)
    df$sex <- sanitizeSex(df$sex)
    df
}

####============================================================
##  Format a pedigree data.frame as extracted from FAData in "ped"
##  format.
####------------------------------------------------------------
doProcessDf2Ped <- function(x){
    ## Assuming correct column names (family, id, father, mother, sex,
    ## affected)
    if(!any(colnames(x) == "affected"))
        x <- cbind(x, affected=rep(NA, nrow(x)))
    x <- x[, c("family", "id", "father", "mother", "sex", "affected")]
    x$father[is.na(x$father)] <- 0
    x$mother[is.na(x$mother)] <- 0
    x$sex <- as.numeric(x$sex)
    x$sex[is.na(x$sex)] <- 0
    x$affected <- x$affected + 1
    x$affected[is.na(x$affected)] <- 0
    x
}


####============================================================
##  doImport
##
##  The main import function that switches based on the file extension.
####------------------------------------------------------------
doImport <- function(file, ...){
    extension <- .fileExtension(file)
    file <- path.expand(file)
    if(extension == "fam")
        return(doImportFam(file, ...))
    if(extension == "ped")
        return(doImportPed(file, ...))
    ## Fall back to "generic import"
    doImportGeneric(file, ...)
}

.fileExtension <- function(file){
    if(!grepl("\\.", file))
        stop("Unable to identify extension for file '", file, "'")
     ext <- sub(".*\\.", "", sub("\\.gz$|\\.gzip$", "", basename(file)))
    if(ext=="")
        stop("Unable to identify extension for file '", file, "'")
    tolower(ext)
}

## Methods for the classes
setMethod("cliques", "missing", function(...){
    return(igraph::cliques(...))
})

##************************************************
##
##       FAKinGroupResults
##
##
##************************************************
setMethod("show", "FAKinGroupResults", function(object){
    callNextMethod()
    cat(paste0("Result info:\n"))
    cat(paste0(" * Dimension of result data.frame: ",
               dim(result(object)),".\n"))
    cat(paste0(" * Number of simulations: ", object@nsim, ".\n"))
})
setMethod("affectedKinshipGroups", "FAKinGroupResults", function(object){
    return(object@affectedKinshipGroups)
})
setMethod("independentGroupCount", "FAKinGroupResults", function(object,
                                                                 method="jo"){
    method <- match.arg(method, c("jo", "daniel"))
    if(length(object@affectedKinshipGroups) == 0)
        stop("No affected kinship groups present!")
    affectedKins <- object@affectedKinshipGroups
    indGroupCount <- 0
    ## on the long run we want to get rid of this...
    if(method=="jo"){
        AffIds <- lapply(affectedKins, function(z){return(z$aff)})
        for(i in 1:length(AffIds)){
            ## check: any of the group's ids also in any of the other groups?
            if(length(intersect(unlist(AffIds[i]), unlist(AffIds[-i]))) == 0)
                indGroupCount <- indGroupCount + 1
        }
    }
    if(method=="daniel"){
        temp <- c()
        for(i in 1:length(affectedKins)) {
            if(length(intersect(temp, affectedKins[[i]]$aff)) == 0) {
                indGroupCount <- indGroupCount + 1
            }
            temp <- c(temp, affectedKins[[i]]$aff)
        }
    }
    return(indGroupCount)
})
## calling the trait replacement method from FAResult and in addition reset the
## simulation result.
setReplaceMethod("trait", "FAKinGroupResults", function(object, value){
    object <- callNextMethod()
    ## reset the result
    object@affectedKinshipGroups <- list()
    object@sim <- list()
    object@nsim <- 0
    object@traitname <- character()
    return(object)
})


####
## The analysis method.
setMethod("runSimulation", "FAKinGroupResults", function(object, nsim=50000,
                                                         strata=NULL){
    if(length(trait(object)) == 0)
        stop("No trait information available!")
    kin <- kinship(object)

    ## Get the ids of the affected and of the phenotyped.
    affectedIds <- affectedIndividuals(object)
    phenotypedIds <- phenotypedIndividuals(object)
    ## Ensure again that we do have pedigree information for these.
    affectedIds <- intersect(colnames(kin), affectedIds)
    phenotypedIds <- intersect(colnames(kin), phenotypedIds)

    ## break...
    if(length(affectedIds) <= 1){
        warning("Trait ", object@traitname, ": only one or none affected ",
                "individual!")
        return(object)
    }
    message("Cleaning data set (got in total ", nrow(kin), " individuals):")
    ## Dummy for removing unaffected:
    message(" * not phenotyped individuals...")
    if(length(phenotypedIds) != nrow(kin)){
        message(" ", nrow(kin)-length(phenotypedIds), " removed")
    }else{
        message(" none present.")
    }
    ## Strata. First check that strata has the correct length, then ensure
    ## that all of the affected and phenotyped ids have a non-NA value in
    ## strata.
    if(!is.null(strata)){
        if(length(strata) != length(object$id))
            stop("Argument 'strata' has to have the same length than there ",
                 "are individuals in the pedigree!")
        nas <- is.na(strata)
        names(strata) <- object$id
        newAffIds <- affectedIds[affectedIds %in% names(strata)[!nas]]
        message(" * affected individuals without valid strata values...",
                appendLF=FALSE)
        if(length(newAffIds) != length(affectedIds)){
            message(" ", length(affectedIds)-length(newAffIds), " removed.")
            affectedIds <- newAffIds
        }else{
            message(" none present.")
        }
        newPheIds <- phenotypedIds[phenotypedIds %in% names(strata)[!nas]]
        message(" * unaffected individuals without valid strata values...",
                appendLF=FALSE)
        if(length(newPheIds) != length(phenotypedIds)){
            message(" ", length(phenotypedIds)-length(newPheIds), " removed.")
            phenotypedIds <- newPheIds
        }else{
            message(" none present.")
        }
        ## Subset strata and ensure same ordering than phenotypedIds
        strata <- strata[phenotypedIds]
    }
    message("Done")

    ## Subset the kinship matrix
    ## Transform the Matrix into a matrix... needs more memory, but runs faster.
    affectedIds <- as.character(affectedIds)
    phenotypedIds <- as.character(phenotypedIds)
    kinAffected <- as.matrix(kin[affectedIds, affectedIds])
    kinPhenotyped <- as.matrix(kin[phenotypedIds, phenotypedIds])
    ## Remove self-self kinship
    diag(kinAffected) <- NA
    diag(kinPhenotyped) <- NA

    ## Get groups of affected related individuals around each affected
    ## individual affectedKins is then a list one element for each affected
    ## individual with $aff being all affected individuals having kinship with
    ## the individual and $phe all phenotyped individuals up to a kinship
    ## being smaller or equal to the smallest kinship of the individual with
    ## any other affected.
    ## $kinfreq: a table with the frequency (counts) of kinship values
    ##           (smaller 0.5) the names of the table correspond to the kinship
    ##           values increasingly ordered.
    affectedKins <- lapply(as.list(affectedIds), function(z){
        ## process all affected around the current affected
        currentKin <- kinAffected[, z]   ## that's a numeric now...
        ## get all individuals that share kinship...
        currentKin <- currentKin[which(currentKin > 0)]
        affIds <- c(z, names(currentKin))
        ## remove the self-self kinship
        ## currentKin <- currentKin[-(match(z, affIds))]
        minKin <- ifelse(length(currentKin) > 0,
                         yes=min(currentKin, na.rm=TRUE), no=NA)
        meanKin <- ifelse(length(currentKin) > 0,
                          yes=mean(currentKin, na.rm=TRUE), no=NA)
        ## process all phenotyped around the current affected
        currentKin <- kinPhenotyped[, z]
        ## Note: which works nicely here even if minMin is NA
        phenIds <- names(currentKin[which(currentKin >= minKin)])
        ## the kinship frequencies:
        ## returning a table with the count of all kinship values < 0.5; names
        ## of the table are the kinship values, increasingly ordered.
        if(length(affIds) >= 2){
            idx <- match(affIds, colnames(kinPhenotyped))
            temp <- kinPhenotyped[idx, idx]
            kinFreq <- table(as.vector(temp))
        }else{
            kinFreq <- NULL
        }
        return(list(aff=sort(affIds), pheno=sort(phenIds), kinfreq=kinFreq,
                    meankin=meanKin, minkin=minKin))
    })
    names(affectedKins) <- affectedIds

    ## remove duplicated entries...
    ## re-order affectedKins by phenotyped size... that way we select below the
    ## smaller kinship group if two groups have the same affected individuals.
    ## We select the smaller group to get more conservative p-values in the
    ## permutation below <- CHRIS
    affectedKins <- affectedKins[order(unlist(lapply(affectedKins, function(z){
        length(z$phe)
    })), decreasing=FALSE)]
    tmp <- lapply(affectedKins, function(z){z$aff})
    affectedKins <- affectedKins[match(unique(tmp), tmp)]  ## match returns only the first hit
    ## remove groups with a single affected
    affectedKins <- affectedKins[unlist(lapply(affectedKins, function(z){
        return(length(z$aff))
    })) > 1]
    ## re-order the affectedKins
    commonIds <- affectedIds[affectedIds %in% names(affectedKins)]
    affectedKins <- affectedKins[commonIds]

    ## break...
    if(length(affectedKins) == 0){
        warning(paste0("Trait ",
                       object@traitname,
                       ": No family with more than one affected member found!"))
        return(object)
    }

    ## Finally, do the simulations.
    ## 1) Sample number of affected individuals (length(affectedIds)) from all
    ##    phenotyped (phenotypedIds).
    ## 2) Loop over the affected kin groups (affectedKins)
    ##    2.1) get those phenotyped in the affected kinship group that match the
    ##         sampled ids.
    ##    2.2) if the expected overlap is >= than the observed -> +1 to
    ##         frequency ratio.
    ##    2.3) get the kinship matrix for the overlap (if overlap >=2)
    ##    2.4) make a frequency table of the expected kinship values and check:
    ##         a) if the largest expected kinship value is > any of the
    ##            observed kinship values: -> +1 to the kinship frequency.
    ##         b) if the largest expected kinship value == the largest observed
    ##            kinship value: compare their counts and if the expected
    ##            count >= the observed count: -> +1 to the kinship frequency.
    ## In other words:
    ## Define random sets of affected individuals by randomly selecting X
    ## individuals among all phenotyped individuals, with X being the total
    ## number of affected cases. Then, for each group, iteratively determine
    ## the number of individuals from the group which have been labeled as
    ## affected in the simulation and their kinship value.
    ExpRatio <- ExpKin <- rep(0, length(affectedKins))
    expTable <- expDensity <- vector("list", length(affectedKins))

    nr.sim.todo <- nsim
    while( nr.sim.todo>0 ) {
        ## Step size.
        ## We compute small chunks of simulated data in order to keep the memory
        ## footprint low. This involves strategies to add data to fixed vectors
        ## and compute incremental tables and densities. A set of tables is
        ## ultimately transformed to a histogram.
        ## Last simulation loop: run only the leftover number of steps.
        nr.inc.sim <- min(1000, nr.sim.todo)
        if(is.null(strata)){
            Sims <- lapply(1:nr.inc.sim, function(z){
                return(sample(phenotypedIds, length(affectedIds)))
            })
        }else{
            affStrataCounts <- table(strata[affectedIds])
            affStrataCounts <- affStrataCounts[affStrataCounts > 0]
            Sims <- lapply(1:nr.inc.sim, function(z){
                idx <- stratsample(strata, counts=affStrataCounts)
                return(phenotypedIds[idx])
            })
        }

        BigRes <- vector("list", nr.inc.sim)
        for( i in 1:nr.inc.sim ) {
            ExpRes <- do.call(rbind, lapply(affectedKins, FUN=function(z){
                Res <- c(0, 0, 0)
                expectedAffInKin <- intersect(Sims[[i]], z$phe)
                Res[3] <- length(expectedAffInKin)
                if(length(expectedAffInKin) >= length(z$aff)){
                    Res[1] <- 1
                }
                ## check frequency...
                if(length(expectedAffInKin) >= 2){
                    ## that's the slowest part of the function... subsetting of the
                    ## kinship matrix...
                    ## we get some speedup if we subset a matrix, not a Matrix!
                    expectedAffInKin <- match(expectedAffInKin,
                                              colnames(kinPhenotyped))
                    expectedKin <- kinPhenotyped[expectedAffInKin, expectedAffInKin]
                    diag(expectedKin) <- NA
                    ## get the table with the frequencies of kinship values...
                    expectedKinFreqs <- table(as.vector(expectedKin))
                    observedKinFreqs <- z$kinfreq
                    if(length(expectedKinFreqs) > 0 & length(observedKinFreqs) > 0){
                        ## names of the table are the kinship values, ordered increasingly by size.
                        expKinShipValue <- names(expectedKinFreqs[length(expectedKinFreqs)])
                        obsKinShipValue <- names(observedKinFreqs[length(observedKinFreqs)])
                        if(as.numeric(expKinShipValue) >= as.numeric(obsKinShipValue)){
                            ## if the kinship value is the same, compare the counts.
                            if(expKinShipValue==obsKinShipValue){
                                if(expectedKinFreqs[expKinShipValue] >=
                                   observedKinFreqs[obsKinShipValue])
                                    Res[2] <- 1
                            }else{
                                ## means it's larger, so add +1
                                Res[2] <- 1
                            }
                        }
                    }
                }
                return(Res)
            }))
            ExpRatio <- ExpRatio + ExpRes[, 1]
            ExpKin   <- ExpKin   + ExpRes[, 2]
            BigRes[[i]] <- ExpRes[, 3]
        }
        ## BigRes is now a list of length nr.inc.sim, each element a vector
        ## of length affectedKins.
        ExpVals <- do.call(rbind, BigRes)

        for( i in 1:ncol(ExpVals) ) {
            expDensity[[i]] <- inc.density(ExpVals[, i], expDensity[[i]])
            expTable[[i]] <- inc.table(ExpVals[, i], expTable[[i]])
        }
        names(expDensity) <- names(expTable) <- colnames(ExpVals)

        nr.sim.todo <- nr.sim.todo - nr.inc.sim
    }
    ## Done with the simulation. Compute the P values, histograms, and
    ## densities and get the hell outta here.
    ExpRatio <- ExpRatio / nsim
    ExpKin <- ExpKin / nsim
    names(ExpRatio) <- names(affectedKins)
    names(ExpKin) <- names(affectedKins)
    expDensity <- expDensity[names(affectedKins)]
    expHist <- lapply(expTable, FUN=table2hist.int)
    expHist <- expHist[names(affectedKins)]

    object@affectedKinshipGroups <- affectedKins
    object@sim <- list(pvalueKinship=ExpKin, pvalueRatio=ExpRatio,
                       expDensity=expDensity, expHist=expHist)
    object@nsim <- nsim

    return(object)
})



setMethod("result", "FAKinGroupResults", function(object, method="BH"){
    ## generate the result table...
    method <- match.arg(method, p.adjust.methods)
    Trait <- trait(object, na.rm=TRUE)
    TraitName <- object@traitname
    if(length(TraitName)==0)
        TraitName <- NA
    if(length(object@sim)==0){
        ## generate a dummy matrix...
        MyRes <- data.frame(trait_name=TraitName,
                            total_phenotyped=length(Trait),
                            total_affected=sum(Trait!=0),
                            phenotyped=0,
                            affected=0,
                            group_id=NA,
                            family=NA,
                            group_phenotyped=NA,
                            group_affected=NA,
                            ratio_pvalue=NA,
                            ratio_padj=NA,
                            mean_kinship=NA,
                            kinship_pvalue=NA,
                            kinship_padj=NA,
                            check.names=FALSE, stringsAsFactors=FALSE)
        warning(paste0("No simulation data available! This means that",
                       " either no simulation was run yet (using the ",
                       "kinshipTest function or runSimulation method) ",
                       "or that the simulation returned no result."))
        return(MyRes)
    }
    affKin <- affectedKinshipGroups(object)
    ped <- pedigree(object)
    ped <- ped[ped$id %in% names(affKin), c("id", "family")]
    pedL <- split(ped, f=ped$id)
    fams <- unlist(lapply(pedL, function(z){
        return(paste(unique(z[, "family"]), collapse=", "))
    }))
    fams <- fams[names(affKin)]
    MyRes <- data.frame(trait_name = rep(TraitName, length(affKin)),
                        total_phenotyped = rep(length(Trait), length(affKin)),
                        total_affected = rep(sum(Trait!=0), length(affKin)),
                        phenotyped = rep(
                            length(unique(unlist(lapply(affKin,
                                                        function(z)z$phe)))),
                            length(affKin)),
                        affected = rep(
                            length(unique(unlist(lapply(affKin,
                                                        function(z)z$aff)))),
                            length(affKin)),
                        group_id = names(affKin),
                        family = fams,
                        group_phenotyped = unlist(lapply(affKin, function(z){
                            length(z$phe)})),
                        group_affected = unlist(lapply(affKin, function(z){
                            length(z$aff)})),
                        ratio_pvalue = object@sim$pvalueRatio,
                        ratio_padj = p.adjust(object@sim$pvalueRatio,
                                              method=method),
                        mean_kinship = unlist(lapply(affKin,
                                                     function(z) z$meankin)),
                        kinship_pvalue = object@sim$pvalueKinship,
                        kinship_padj = p.adjust(object@sim$pvalueKinship,
                                                method=method),
                        check.names=FALSE, stringsAsFactors=FALSE
                        )
    MyRes <- MyRes[order(MyRes$ratio_pvalue, MyRes$kinship_pvalue,
                         -MyRes$mean_kinship), ]
    rownames(MyRes) <- as.character(MyRes$group_id)
    return(MyRes)
})


#############
## plotting method...
## plotPed representing the results from the kinship test.
## plotPed for FAKinGroupResults does only support plotting for id.
setMethod("plotPed", "FAKinGroupResults", function(object, id=NULL,
                                                   family=NULL, filename=NULL,
                                                   device="plot", ...){
    if(!is.null(family))
        stop("Generating a pedigree for a family is not supported for ",
             "FAKinGroupResults. See help for more information.")
    affGroup <- affectedKinshipGroups(object)
    if(!any(names(affGroup) == id))
        stop("The id should be one of the names of the affected kinship ",
             "groups (i.e. names(affectedKinshipGroups(object)))!")
    id <- as.character(id)
    affGroup <- affGroup[[id]]
    callNextMethod(object = object, id = id, family = family,
                   filename = filename, device = device,
                   proband.id = unique(c(affGroup$aff, affGroup$pheno)), ...)
})


## this buildPed simply ensures that all phenotyped in the group will be
## included in the pedigree.
setMethod("buildPed", "FAKinGroupResults",
          function(object, id = NULL, max.generations.up = 3,
                   max.generations.down = 16, prune = FALSE) {
              if (is.null(id))
                  stop("The id of the group has to be speficied!")
              affGroup <- affectedKinshipGroups(object)
              id <- as.character(id)
              if (!any(names(affGroup) == id))
                  stop("The id should be one of the names of the affected ",
                       "kinship groups (i.e. ",
                       "names(affectedKinshipGroups(object)))!")
              affGroup <- affGroup[[id]]
              phenoAndAff <- as.character(unique(c(affGroup$aff,
                                                   affGroup$pheno)))
              kin <- kinship(object)
              ped <- family(object, id=id)
              whichs <- which(colnames(kin) %in% ped$id)
              ## get the kinship of the kin with all other in the pedigree
              kin <- kin[id, whichs]
              keep <- names(kin)[kin >= affGroup$minkin]
              keep <- unique(c(keep, affGroup$pheno, affGroup$aff))
              ped <- callNextMethod(object=object, id=keep, prune=prune,
                                    max.generations.up=max.generations.up,
                                    max.generations.down=max.generations.down)
              return(ped)
          })

## this is to get all those that are related with any individuals of the group!!!
setMethod("shareKinship", "FAKinGroupResults",
          function(object, id = NULL, rmKinship = 0) {
    if (is.null(id))
        stop("The id of the group has to be speficied!")
    affGroup <- affectedKinshipGroups(object)
    id <- as.character(id)
    if (!any(names(affGroup) == id))
        stop("The id should be one of the names of the affected kinship ",
             "groups (i.e. names(affectedKinshipGroups(object)))!")
    affGroup <- affGroup[[id]]
    allids <- unique(c(affGroup$aff, affGroup$pheno))
    ## get all individuals of the group...
    doShareKinship(kin=kinship(object), id=allids, rmKinship=rmKinship)
})

## plot the pedigree for the kinship group...
## the function does add eventually missing parents.
plotPedForKinshipResult <- function(object, groupid=NULL, filename=NULL,
                                    device="pdf", ...){
    if(is.null(groupid))
        stop("No id specified!")
    if(length(groupid) > 1){
        groupid <- groupid[1]
        warning("length groupid > 1; using only the first element!")
    }
    groupid <- as.character(groupid)
    if(!any(names(affectedKinshipGroups(object)) == groupid))
        stop("groupid should be one of the names of the affected kinship ",
             "groups (i.e. names(affectedKinshipGroups(object)))!")
    affs <- affectedKinshipGroups(object)[[groupid]]$aff
    phens <- affectedKinshipGroups(object)[[groupid]]$pheno
    fam <- buildPedigree(family(object, id=groupid), ids=unique(c(affs, phens)))
    cat("nrow fam ", nrow(fam), " length affs phens: ",
        length(unique(c(affs, phens))), "\n")
    ## check if we've got affected
    if(any(colnames(fam)=="affected")){
        affected <- fam[, "affected"]
    }else{
        affected <- rep(NA, nrow(fam))
        names(affected) <- fam$id
    }
    ## check ages...
    ages <- age(object)[fam$id]
    ages[!is.na(ages)] <- paste(format(ages[!is.na(ages)], digits=3), "years")
    ## ages[!is.na(ages)] <- sprintf("%d years", ages[!is.na(ages)])
    ages[is.na(ages)] <- ""
    ## do not have affected... obviously...
    is.proband <- rep(FALSE, nrow(fam))
    names(is.proband) <- fam$id
    if(!is.null(groupid))
        is.proband[as.character(groupid)] <- TRUE
    ## how is the code from Daniel labeling the affected:
    ## - any phenotyped: 1
    ## - any affected: 4
    ## - any affected in the affected group: 2.
    ## OK, now plot!!!
    res <- doPlotPed(family=fam$family, individual=fam$id, father=fam$father,
                     mother=fam$mother, gender=fam$sex, is.proband=is.proband,
                     text1.below.symbol=ages, affected=affected,
                     filename=filename, device=device, ...)
    res
}

setMethod("[", "FAKinGroupResults", function(x, i, j, ..., drop){
    stop("Subsetting of a FAKinGroupResults object is not supported!")
})

## This will be a crazy funky method to plot the simulation results.
setMethod("plotRes", "FAKinGroupResults", function(object, id = NULL,
                                                   family = NULL,
                                                   addLegend = TRUE,
                                                   type = "density", ...) {
    type <- match.arg(type, c("density", "hist"))
    if(length(object@sim) == 0)
        stop("No analysis performed yet!")
    if(is.null(id))
        stop("The id of the affected individual for whom the simulation ",
             "results should be displayed has to be specified!")
    id <- as.character(id)
    ## result on family is not allowed.
    if(!is.null(family) | length(id) > 1)
        stop("plotRes for FAKinGroupResults does only support specifying ",
             "a single id!")
    ## check if the id is an affected for which we tested...
    if(!any(names(object@sim$pvalueKinship) == id))
        stop("No simulation result is available for individual ", id,
             ". id should be one of result(object)$group_id.")
    ## OK, now we can really start doing things...
    affGroup <- affectedKinshipGroups(object)[[id]]
    meanKin <- affGroup$meankin
    affCount <- length(affGroup$aff)
    pheCount <- length(affGroup$phe)
    kinPval <- object@sim$pvalueKinship[id]
    fam <- family(object, id=id)[1, "family"]
    ratioP <- object@sim$pvalueRatio[id]
    kinP <- object@sim$pvalueKinship[id]
    par(xpd=FALSE)
    if(type == "density"){
        toplot <- object@sim$expDensity[[id]]
        XL <- range(c(range(toplot$x), affCount))
        plot(toplot, main=paste0("Affected: ", id, ", family: ", fam),
             xlab="Affected count", type="h",
             lwd=3, col="lightgrey", xlim=XL)
        points(toplot, col="grey", type="l", lwd=2)
    }
    if(type == "hist"){
        toplot <- object@sim$expHist[[id]]
        XL <- range(c(range(toplot$mids), affCount))
        plot(toplot, main=paste0("Affected: ", id, ", family: ", fam),
             xlab="Affected count",
             col="lightgrey", border="grey", xlim=XL)
    }
    Blue <- "#377EB8"
    abline(v=affCount, col=Blue)
    if(addLegend){
        legend("topright",
               legend=c(paste0("affect. count: ", affCount),
                        paste0("pheno. count:  ", pheCount),
                        paste0("ratio p-value: ", format(ratioP, digits=3)),
                        paste0("mean kinship:  ", format(meanKin, digits=3)),
                        paste0("kinhip p-value: ", format(kinPval, digits=3))
                        ))
    }
})

## This is the implementation of the FI (familial incidence ratio) and the FSIR
## (familial standardized incidence ratio) from Kerber (1995)

## calculate the time at risk
## startDate: can be either the birth date or the start date for the study.
## endDate: the end date of the study.
## incidenceDate: the date of the incidence.
## deathDate: the date of death.
## the time at risk will be the minimum (earliest) date of (endDate,
## incidenceDate, deathDate) minus the startDate
## need at least endData, in that case endDate is assumed to be "all in one"
## allowNegative: if TRUE return also negative differences. If FALSE, negative
## values are replaced by 0
## Note: we will subtract half of incidence.subtract for each affected individual.
estimateTimeAtRisk <- function(startDate = NULL, startDateFormat = "%Y-%m-%d",
                               endDate = NULL, endDateFormat = "%Y-%m-%d",
                               incidenceDate = NULL,
                               incidenceDateFormat = "%Y-%m-%d",
                               deathDate = NULL, deathDateFormat = "%Y-%m-%d",
                               allowNegative = FALSE, affected = NULL,
                               incidenceSubtract = 0.5) {
    if (is.null(startDate))
        stop("'startDate' has to be specified (either the start of the study,",
             " or the birth date).")
    if (!is(startDate, "Date")) {
        message("Transforming startDate into Date format...", appendLF=FALSE)
        startDate <- as.Date(startDate, format=startDateFormat)
        message("OK")
    }
    ## check if we've got at least one of the following:
    if (is.null(endDate)) {
        stop("'endDate' has to be specified!")
    } else {
        if(is.null(incidenceDate) & is.null(deathDate))
            message("'endDate' is assumed to specify, for each individual,",
                    " either the end year of the study, the individual's death",
                    " date or the individual's incidence date.")
        if (length(endDate) != length(startDate))
            stop("'startDate' and 'endDate' have to have the same length!")
    }
    message("Transforming endDate into Date format...", appendLF=FALSE)
    endDate <- as.Date(endDate, format=endDateFormat)
    message("OK")
    if (!is.null(affected)) {
        if (length(affected) != length(startDate)) {
            affected <- NULL
            warning("length of affected does not match length of startData; ",
                    "dropping.")
        } else {
            affected <- as.logical(affected)
        }
    }
    ## if we've got an incidenceDate, process that.
    if (!is.null(incidenceDate)) {
        message("Transforming incidenceDate into Date format...",
                appendLF=FALSE)
        incidenceDate <- as.Date(incidenceDate, format=incidenceDateFormat)
        message("OK")
        if(length(incidenceDate) != length(startDate))
            stop("'incidenceDate', 'startDate' and 'endDate' all have to have ",
                 "the same length!")
        ## now get the earlier time point: incidence or end:
        DF <- data.frame(endDate, incidenceDate)
        endDate <- as.Date(apply(DF, MARGIN=1, min, na.rm=TRUE))
        affected <- rep(FALSE, length(incidenceDate))
        affected[!is.na(incidenceDate)] <- TRUE
    }
    if (!is.null(deathDate)) {
        message("Transforming deathDate into Date format...", appendLF=FALSE)
        deathDate <- as.Date(deathDate, format=deathDateFormat)
        message("OK")
        if(length(deathDate) != length(startDate))
            stop("'deathDate', 'startDate' and 'endDate' all have to have the ",
                 "same length!")
        ## now get the earlier time point: incidence or end:
        DF <- data.frame(endDate, deathDate)
        endDate <- as.Date(apply(DF, MARGIN=1, min, na.rm=TRUE))
    }
    ## OK, now we're set.
    Diff <- endDate - startDate
    if (!is.null(affected)) {
        Diff[affected] <- Diff[affected] - incidenceSubtract
    }
    Diff <- as.numeric(Diff)
    if(!allowNegative)
        Diff[which(Diff < 0)] <- 0
    Diff
}

## takes a numeric vector of ages as input and slices it into
sliceAge <- function(x, slices=c(0, 40, Inf)){
    SliceDiff <- diff(slices)
    Res <- lapply(x, function(z){
        tmp <- z - slices
        tmp <- tmp[1:(length(tmp)-1)]
        bm <- which(tmp > SliceDiff)
        tmp[bm] <- SliceDiff[bm]
        tmp[which(tmp < 0)] <- 0
        return(tmp)
    })
    Res <- do.call(rbind, Res)
    CN <- paste0(slices[1:(length(slices)-1)], ", ", slices[2:length(slices)])
    CN <- paste0("(", CN, "]")
    colnames(Res) <- CN
    return(Res)
}

##****************************************************************************
##
##  timeAtRisk getter/setter method.
##
setMethod("timeAtRisk", "FAIncidenceRateResults",
          function(object){
              ## we're also checking that timeAtRisk, stored in its internal
              ## slot has the correct length!
              if(length(object@timeAtRisk)==0)
                  return(object@timeAtRisk)
              if(length(object@timeAtRisk) != length(object$id))
                  stop("Length of 'timeAtRisk' does not match the number of ",
                       "individuals in the pedigree!")
              return(object@timeAtRisk)
          })
setReplaceMethod("timeAtRisk", "FAIncidenceRateResults",
                 function(object, value) {
                     ## check if we've got the pedigree, otherwise does not
                     ## make sense to store the timeAtRisk.
                     if(length(object$id) == 0)
                         stop("No pedigree data in 'object'! Add the pedigree ",
                              "data before setting 'timeAtRisk'.")
                     if(length(object$id) != length(value))
                         stop("Length of 'timeAtRisk' does not match the ",
                              "number of individuals in the pedigree!")
                     object@timeAtRisk <- value
                     object
                 })


##****************************************************************************
##
##  familialIncidenceRate getter method.
##
setMethod("familialIncidenceRate", "FAIncidenceRateResults",
          function(object, ...){
              if(length(object@sim) == 0)
                  return(NULL)
              return(object@sim$fir)
          })

##****************************************************************************
##
##  $ getter method.
##
setMethod("$", "FAIncidenceRateResults", function(x, name){
    if(name == "fir" | name == "tar" | name == "timeAtRisk"
       | name == "pvalue"){
        ## extract local stuff
        if(length(x@sim) == 0){
            warning("No calculation result available!")
            return(NULL)
        }
        if(name == "fir")
            return(x@sim$fir)
        if(name == "tar")
            return(timeAtRisk(x))
        if(name == "timeAtRisk")
            return(timeAtRisk(x))
        if(name == "pvalue")
            return(x@sim$pvalue)
    }else{
        callNextMethod()
    }
})


##****************************************************************************
##
##  runSimulation
##
##  We can pass optional argument 'lowMem' to the downstream function that
##  requires less memory and runs (eventually) faster.
##  The second optional argument is 'prune' that removes unconnected
##  individuals from the pedigree prior testing.
setMethod("runSimulation", "FAIncidenceRateResults",
          function(object, nsim=50000, timeAtRisk=NULL,
                   strata=NULL, ...){
              if(length(trait(object)) == 0)
                  stop("No trait information available!")
              SimRes <- .FRSimulation(ped=pedigree(object),
                                      kin=kinship(object),
                                      trait=trait(object),
                                      timeAtRisk=timeAtRisk,
                                      nsim=nsim,
                                      strata=strata,
                                      prune=FALSE,
                                      ...)
              timeAtRisk(object) <- timeAtRisk
              object@nsim <- nsim
              object@sim <- SimRes
              return(object)
          })

##****************************************************************************
##
##  trait<-
##
##  Calling the trait replecement method from FAResult and in addition
##  reset the simulation result.
setReplaceMethod("trait", "FAIncidenceRateResults", function(object, value){
    object <- callNextMethod()
    ## reset the result
    object@sim <- list()
    object@nsim <- 0
    object@traitname <- character()
    return(object)
})

##****************************************************************************
##
##  familial incidence rate along with Monte Carlo simulations to
##  estimate the significance of the ratio.
##  Note: we skipped here the perFamilyTest option as that makes a) no sense,
##  and b) would make the simulations eventually problematic (sampling from
##  small sample sets).
##
##  lowMem: if TRUE a less memory demanding code is run; density information
##          is then however not available.
##****************************************************************************
.FRSimulation <- function(ped=NULL, kin=NULL, trait=NULL, timeAtRisk=NULL,
                          prune=TRUE, nsim=50000, strata=NULL,
                          lowMem=FALSE, ...){
    ## Input argument checking.
    if(is.null(ped))
        stop("No pedigree submitted!")
    if(is.null(kin))
        stop("No kinship matrix submitted!")
    if(is.null(trait))
        stop("No trait information submitted!")
    if(length(trait) != nrow(ped))
        stop("Argument 'trait' has to have the same length then there",
             " are rows (individuals) in the pedigree 'ped'!")
    ped <- cbind(ped, AFF=trait)
    allIds <- as.character(ped$id)
    if(is.null(timeAtRisk)){
        stop("Argument 'timeAtRisk' has to be submitted!")
    }
    if(length(timeAtRisk) != nrow(ped))
        stop("Argument 'timeAtRisk' has to have the same length than there",
             " are rows (individuals) in the pedigree 'ped'!")
    timeAtRisk <- as.numeric(timeAtRisk)
    ped <- cbind(ped, TAR=timeAtRisk)
    if(!is.null(strata)){
        if(length(strata)!=nrow(ped))
            stop("Argument 'strata' has to have the same length than there",
                 " are individuals in the pedigree!")
        ped <- cbind(ped, STRAT=strata)
    }
    ## NOTE: We're not removing singletons here, as we are anyway removing them
    ## further down!
    ## Subset the data set to individuals with valid values for trait, timeAtRisk
    ## and strata.
    nas <- is.na(ped$AFF)
    message("Cleaning data set (got in total ", nrow(ped), " individuals):")
    message(" * not phenotyped individuals...", appendLF=FALSE)
    if(any(nas)){
        ped <- ped[!nas, , drop=FALSE]
        message(" ", sum(nas), " removed.")
    }else{
        message(" none present.")
    }
    ## time at risk
    nas <- is.na(ped$TAR)
    message(" * individuals with unknown time at risk...", appendLF=FALSE)
    if(any(nas)){
        ped <- ped[!nas, , drop=FALSE]
        message(" ", sum(nas), " removed.")
    }else{
        message(" none present.")
    }
    ## Strata
    if(!is.null(strata)){
        nas <- is.na(ped$STRAT)
        message(" * individuals without valid strata values...", appendLF=FALSE)
        if(any(nas)){
            ped <- ped[!nas, , drop=FALSE]
            message(" ", sum(nas), " removed.")
        }else{
            message(" none present.")
        }

    }
    ## Anyway removing singletons here, since they result in NA values!
    message(" * singletons (also caused by previous subsetting)...",
            appendLF=FALSE)
    origSize <- nrow(ped)
    suppressMessages(
        ped <- removeSingletons(ped)
    )
    if(nrow(ped) != origSize){
        message(" ", origSize-nrow(ped), " removed.")
    }else{
        message(" none present.")
    }
    message("Done")

    ## Calculate the kinship for the observed.
    idx <- match(as.character(ped$id), colnames(kin))
    if(any(is.na(idx)))
        stop("Some of the individuals in the pedigree are missing in the ",
             "kinship matrix!")
    kin <- as.matrix(kin[idx, idx])
    ## Remove self-self kinship and set all affected > 1 to 1.
    diag(kin) <- 0
    Denomi <- ped$TAR %*% kin
    affected <- rep(0, nrow(ped))
    affected[ped$AFF > 0] <- 1
    nAff <- sum(affected)
    nAll <- length(affected)
    sampleFrom <- 1:length(affected)
    obsFr <- as.vector((affected %*% kin) / Denomi)
    names(obsFr) <- ped$id
    ##
    ## Memory saving version
    if(lowMem){
        largerEqual <- rep(0, length(obsFr))
        if(is.null(strata)){
            for(i in 1:nsim){
                simIdx <- sample(sampleFrom, nAff)
                simaffs <- rep(0, nAll)
                simaffs[simIdx] <- 1
                tmp <- as.vector((simaffs %*% kin) / Denomi)
                tmp[is.na(tmp)] <- 0
                largerEqual <- largerEqual + (tmp >= obsFr)
            }
        }else{
            affStrataCounts <- table(ped[ped$AFF > 0 , "STRAT"])
            affStrataCounts <- affStrataCounts[affStrataCounts > 0]
            for(i in 1:nsim){
                simIdx <- stratsample(ped$STRAT, counts=affStrataCounts)
                simaffs <- rep(0, nAll)
                simaffs[simIdx] <- 1
                tmp <- as.vector((simaffs %*% kin) / Denomi)
                tmp[is.na(tmp)] <- 0
                largerEqual <- largerEqual + (tmp >= obsFr)
            }
        }
        PVals <- largerEqual/nsim
        denses <- NULL
    }else{
        ## full blown version
        ## Simulations: generate random sets of affected individuals, same size
        ## then the observed individuals.
        if(is.null(strata)){
            SimFr <- lapply(1:nsim, function(z){
                simIdx <- sample(sampleFrom, nAff)
                simaffs <- rep(0, nAll)
                simaffs[simIdx] <- 1
                return(as.vector((simaffs %*% kin) / Denomi))
                ## Would be less memory demanding to just return whether the
                ## expected is larger than the observed
            })
        }else{
            affStrataCounts <- table(ped[ped$AFF > 0 , "STRAT"])
            affStrataCounts <- affStrataCounts[affStrataCounts > 0]
            SimFr <- lapply(1:nsim, function(z){
                simIdx <- stratsample(ped$STRAT, counts=affStrataCounts)
                simaffs <- rep(0, nAll)
                simaffs[simIdx] <- 1
                return(as.vector((simaffs %*% kin) / Denomi))
            })
        }
        ## Performing the stuff on the large matrix.
        ## Want to compare the observed FR against the simulated FOR EACH
        ## INDIVIDUAL.
        ## Why: that way we keep the kinship the same and just calculate an
        ## expexted FR for random occurence of cases.
        SimFr <- do.call(cbind, SimFr)
        PVals <- rowSums(SimFr >= obsFr)/nsim
        denses <- apply(SimFr, MARGIN=1, density)
    }
    ## Result should match all ids in the (original) pedigree.
    fr <- rep(NA, length(allIds))
    names(fr) <- allIds
    idx <- match(names(obsFr), allIds)
    fr[idx] <- obsFr
    ## p-values
    pv <- rep(NA, length(allIds))
    names(pv) <- allIds
    pv[idx] <- PVals
    ## density
    if(!is.null(denses)){
        dens <- vector("list", length(allIds))
        names(dens) <- allIds
        dens[idx] <- denses
        Res <- list(fir=fr, pvalue=pv, expDensity=dens)
    }else{
        Res <- list(fir=fr, pvalue=pv)
    }
    Res
}

## prune: remove un-connected individuals from the pedigree AFTER removing
##        individuals without valid value in trait or a missing value in
##        timeAtRisk.
## UPDATE: disable perFamilyTest!!!
.FR <- function(ped=NULL, kin=NULL, trait=NULL, timeAtRisk=NULL,
                prune=TRUE, ...){
    perFamilyTest <- FALSE
    ## subset the data to all the individuals for which we do have the
    ## required data.
    ## * trait not NA
    ## * timeAtRist not NA
    ## * did not get removed after pruning for un-connected individuals.
    if(is.null(ped))
        stop("No pedigree submitted!")
    if(is.null(kin))
        stop("No kinship matrix submitted!")
    if(is.null(trait))
        stop("No trait information submitted!")
    if(length(trait) != nrow(ped))
        stop("Argument 'trait' has to have the same length then there",
             " are rows (individuals) in the pedigree 'ped'!")
    ped <- cbind(ped, AFF=trait)
    if(is.null(timeAtRisk)){
        stop("Argument 'timeAtRisk' has to be submitted!")
    }
    if(length(timeAtRisk) != nrow(ped))
        stop("Argument 'timeAtRisk' has to have the same length than",
             " there are rows (individuals) in the pedigree 'ped'!")
    timeAtRisk <- as.numeric(timeAtRisk)
    ped <- cbind(ped, TAR=timeAtRisk)
    if(perFamilyTest){
        entityIs <- "family"
    }else{
        entityIs <- "pedigree"
        ped[, "family"] <- 1
    }
    ## NOTE: don't do that here, since we're going to remove them anyway
    ## further below!
    pedL <- split(ped, ped$family)
    Res <- lapply(pedL, function(z){
        allIds <- as.character(z$id)
        ## remove NAs:
        ## NA in affected/trait
        nas <- is.na(z$AFF)
        message("Cleaning data set (got in total ", nrow(z), " individuals):")
        message(" * not phenotyped individuals...", appendLF=FALSE)
        if(any(nas)){
            z <- z[!nas, , drop=FALSE]
            message(" ", sum(nas), " removed.")
        }else{
            message(" none present.")
        }
        ## NA in time at risk:
        nas <- is.na(z$TAR)
        message(" * individuals with unknown time at risk...", appendLF=FALSE)
        if(any(nas)){
            z <- z[!nas, , drop=FALSE]
            message(" ", sum(nas), " removed.")
        }else{
            message(" none present.")
        }
        message(" * singletons (also caused by previous subsetting)...", appendLF=FALSE)
        origSize <- nrow(z)
        suppressMessages(
            z <- removeSingletons(z)
        )
        if(nrow(z) != origSize){
            message(" ", origSize-nrow(z), " removed.")
        }else{
            message(" none present.")
        }
        message("Done")

        ## Subset the kinship matrix and ensure correct ordering.
        idx <- match(as.character(z$id), colnames(kin))
        if(any(is.na(idx)))
            stop("Some of the individuals in the pedigree are missing in the ",
                 "kinship matrix!")
        kinSub <- as.matrix(kin[idx, idx])
        ## remove self-self kinship.
        diag(kinSub) <- 0
        affected <- rep(0, nrow(z))
        affected[z$AFF > 0] <- 1
        ## The magic formula (3) in Kerber 1995 as an R matrix multiplication.
        FR <- as.vector((affected %*% kinSub)/(z$TAR %*% kinSub))
        names(FR) <- z$id
        retFR <- rep(NA, length(allIds))
        names(retFR) <- allIds
        retFR[names(FR)] <- FR
        return(retFR)
    })
    Res
}


setMethod("[", "FAIncidenceRateResults", function(x, i, j, ..., drop){
    stop("Subsetting of a FAIncidenceRateResults object is not supported!")
})

##****************************************************************************
##
##  plotPed
##
##  the plot ped can be done for individuals and for families. In essence we
##  are just calling the plotPed from the FAData object, eventually highlighting
##  the individual and providing the FR as numeric values below the id.
setMethod("plotPed", "FAIncidenceRateResults",
          function(object, id=NULL, family=NULL, filename=NULL,
                   device="plot", only.phenotyped=FALSE, ...){
              if(is.null(id) & is.null(family))
                  stop("Either id or family has to be provided!")
              if(length(object@sim) == 0){
                  ## Means we don't have any results...
                  FR <- rep(NA, length(object$id))
              }else{
                  FR <- object@sim$fir
              }
              callNextMethod(object=object, id=id, family=family,
                             filename=filename,
                             device=device, label1=FR, proband.id=id,
                             only.phenotyped=only.phenotyped, ...)
          })


##****************************************************************************
##
##  show
##
setMethod("show", "FAIncidenceRateResults", function(object){
    callNextMethod()
    cat(paste0("Result info:\n"))
    if(length(object@sim) > 0){
        cat(paste0(" * Mean familial incidence ratio: ",
                   format(mean(object@sim$fir, na.rm=TRUE), digits=2), ".\n"))
        cat(paste0(" * Mean time at risk: ",
                   format(mean(timeAtRisk(object), na.rm=TRUE), digits=2), ".\n"))
    }else{
        cat(" * No simulation results available yet.\n")
    }
    cat(paste0(" * Number of simulations: ", object@nsim, ".\n"))
})

##****************************************************************************
##
##  plotRes
##
setMethod("plotRes", "FAIncidenceRateResults",
          function(object, id=NULL, family=NULL, addLegend=TRUE,
                   type="density", ...){
              type <- match.arg(type, c("density", "hist"))
              if(type == "hist")
                  stop("Type 'hist' is not supported (yet).")
              if(length(object@sim) == 0)
                  stop("No analysis performed yet!")
              if(is.null(id))
                  stop("Argument 'id' is required.")
              id <- as.character(id)
              if(!is.null(family))
                  stop("Argument 'family' is not supported.")
              ## check if we've got id at all.
              fr <- object@sim$fir
              if(!any(names(fr) == id))
                  stop("Individual with id ", id, " not found ",
                       "in the pedigree.")
              obsFr <- fr[id]
              if(is.na(obsFr))
                  stop(paste0("No Familial Incidence Rate value for individual ",
                              id, " calculated."))
              fam <- family(object, id=id)[1, "family"]
              if(type == "density"){
                  ## Let's see whether we have the required information available.
                  if(!any(names(object@sim) == "expDensity"))
                      stop("Distribution of familial incidence rates from ",
                           "the simulation runs not available. You need to",
                           " run 'runSimulation' without optional argument",
                           " 'lowMem=TRUE'.")
                  dens <- object@sim$expDensity[[id]]
                  XL <- range(c(range(dens$x, na.rm=TRUE), obsFr), na.rm=TRUE)
                  plot(dens, main=paste0("Individual: ", id, ", family: ",
                                         fam), xlab="Familial incidence rate",
                       type="h", lwd=3, col="lightgrey", xlim=XL)
                  points(dens, col="grey", type="l", lwd=2)
              }
              Blue <- "#377EB8"
              abline(v=obsFr, col=Blue)
              if(addLegend){
                  legend("topright",
                         legend=c(paste0("fir     : ", format(obsFr, digits=2)),
                                  paste0("p-value : ",
                                         format(object@sim$pvalue[id],
                                                digits=2))
                                  ))
              }
          })

##****************************************************************************
##
##  result
##
setMethod("result", "FAIncidenceRateResults", function(object, method="BH"){
    method <- match.arg(method, p.adjust.methods)
    Trait <- trait(object, na.rm=TRUE)
    TraitName <- object@traitname
    if(length(TraitName) == 0)
        TraitName <- NA
    if(length(object@sim) == 0){
        MyRes <- data.frame(trait_name=TraitName,
                            total_phenotyped=length(Trait),
                            total_affected=sum(Trait!=0),
                            total_tested=0,
                            id=NA,
                            family=NA,
                            fir=NA,
                            pvalue=NA,
                            padj=NA,
                            check.names=FALSE, stringsAsFactors=FALSE)
        warning(paste0("No simulation data available! Please run a simulation ",
                       "using the 'runSimulation' method on this, or the ",
                       "'familialIncidenceRateTest' on a 'FAData' object."))
        return(MyRes)
    }
    ## @sim$fir has to be in the same order than the pedigree.
    nind <- length(object$id)
    MyRes <- data.frame(trait_name=rep(TraitName, nind),
                        total_phenotyped=rep(length(Trait), nind),
                        total_affected=rep(sum(Trait!=0), nind),
                        total_tested=rep(sum(!is.na(object@sim$fir)), nind),
                        id=as.character(object$id),
                        family=as.character(object$family),
                        fir=object@sim$fir,
                        pvalue=object@sim$pvalue,
                        padj=p.adjust(object@sim$pvalue, method=method),
                        check.names=FALSE, stringsAsFactors=FALSE)
    MyRes <- MyRes[order(MyRes$pvalue, -MyRes$fir), ]
    rownames(MyRes) <- as.character(MyRes$id)
    MyRes
})

## incidence ratio test as defined by Kerber 1995
data(minnbreast)
mbsub <- minnbreast[minnbreast$famid %in% 4:19, ]
PedDf <- mbsub[, c("famid", "id", "fatherid", "motherid", "sex")]
colnames(PedDf) <- FamAgg:::.PEDCN
PedDf$sex <- FamAgg:::sanitizeSex(PedDf$sex)
## generate the FAData.
fad <- FAData(pedigree=PedDf)
## specify the trait.
tcancer <- mbsub$cancer
names(tcancer) <- mbsub$id
## the time at risk...
tar <- mbsub$endage


test_incidence_ratio <- function(){

    fr <- FamAgg:::.FR(ped=pedigree(fad), kin=kinship(fad), trait=tcancer,
                       timeAtRisk=tar)
    ## the same but per family
    frFam <- FamAgg:::.FR(ped=pedigree(fad), kin=kinship(fad), trait=tcancer,
                          timeAtRisk=tar, perFamilyTest=TRUE)
    fr3 <- familialIncidenceRate(fad, trait=tcancer, timeAtRisk=tar)
    checkEquals(fr[[1]], fr3)
    ## The argument rm.singletons has been removed, so no need for the test below.
    ## Using the method: check if rm.singletons has an effect.
    ## fr <- familialIncidenceRate(fad, trait=tcancer, timeAtRisk=tar, rm.singletons=FALSE)
    ## fr2 <- familialIncidenceRate(fad, trait=tcancer, timeAtRisk=tar, rm.singletons=TRUE)
    ## checkEquals(sum(fr, na.rm=TRUE)==sum(fr2, na.rm=TRUE), TRUE)
    ## Note: we're getting the same results here as we are summing kinship coefs. singletons have
    ## a kinship of 0 with any other, thus they don't add to the value anyway.

    ## So, it's not the same if we run this within family or for the whole pedigree...
    ## frVals <- unlist(fr, use.names=FALSE)
    ## frFamVals <- unlist(frFam, use.names=FALSE)
    ## checkEquals(frVals[!is.na(frVals)], frFamVals[!is.na(frFamVals)])
    ## checkEquals(unlist(fr, use.names=FALSE), unlist(frFam, use.names=FALSE))
    ## fr[[1]][1:10]
    ## frFam[["4"]]
}

test_estimate_time_at_risk <- function(){
    sdates <- c("2012-04-17", "2014-05-29", "1999-12-31", "2002-10-10")
    edates <- c("2015-09-15", "2015-09-15", "2005-09-15", "2015-09-15")
    idates <- c(NA, NA, "2007-07-13", "2013-12-23")
    ddates <- c(NA, NA, NA, "2014-03-14")

    checkException(estimateTimeAtRisk(sdates))
    ests <- estimateTimeAtRisk(startDate=sdates, endDate=edates)
    ests2 <- estimateTimeAtRisk(startDate=sdates, endDate=edates,
                                incidenceDate=idates)
    ests3 <- estimateTimeAtRisk(startDate=sdates, endDate=edates,
                                incidenceDate=idates, deathDate=ddates)
}

test_fr_simulation <- function(){
    doPlot <- FALSE
    fr <- FamAgg:::.FR(ped=pedigree(fad), kin=kinship(fad), trait=tcancer,
                       timeAtRisk=tar)
    fr <- fr[[1]]
    fr2 <- familialIncidenceRate(fad, trait=tcancer,
                                 timeAtRisk=tar)
    checkEquals(fr, fr2[names(fr)])
    ## Use the familialIncidenceRateTest
    set.seed(18011977)
    Test <- FamAgg:::.FRSimulation(ped=pedigree(fad), kin=kinship(fad), trait=tcancer,
                                   timeAtRisk=tar, prune=TRUE, nsim=1000)
    set.seed(18011977)
    frRes <- familialIncidenceRateTest(fad, trait=tcancer,
                                       timeAtRisk=tar, nsim=1000)
    checkEquals(frRes@sim$fir, fr2)
    ## Do the simulation using dummy strata.
    set.seed(18011977)
    frStrat <- familialIncidenceRateTest(fad, trait=tcancer,
                                                  timeAtRisk=tar, nsim=1000,
                                                  strata=rep(1, length(fad$id)))
    checkEquals(frRes@sim$fir, frStrat@sim$fir)
    checkEquals(frRes@sim$pvalue, frStrat@sim$pvalue)
    ## Repeat using the low mem version.
    set.seed(18011977)
    frLM <- familialIncidenceRateTest(fad, trait=tcancer,
                                               timeAtRisk=tar, nsim=1000,
                                               lowMem=TRUE)
    checkEquals(frRes@sim$fir, frLM@sim$fir)
    checkEquals(frRes@sim$pvalue, frLM@sim$pvalue)
    ##
    ## plotting...
    if(doPlot){
        res <- result(frRes)
        plotPed(frRes, id="4")
        plotPed(frRes, family=19)
        plotPed(frRes, id=res[1, "id"])
        ## plotRes.
        plotRes(frRes, id="4")
        plotRes(frRes, id=res[1, "id"])
    }

    ## Testing the $ accessors.
    checkEquals(frRes$fir, frRes@sim$fir)
    checkEquals(frRes$tar, FamAgg:::timeAtRisk(frRes))

    ## Testing with and without rm.singletons.
    fr <- familialIncidenceRateTest(fad, trait=tcancer, nsim=400,
                                    timeAtRisk=tar, rm.singletons=TRUE)
    fr2 <- familialIncidenceRateTest(fad, trait=tcancer, nsim=400,
                                     timeAtRisk=tar, rm.singletons=FALSE)
    checkEquals(fr$fir, fr2$fir)
}

## Generic method definitions.
## A
setGeneric("affectedIndividuals", function(object)
    standardGeneric("affectedIndividuals"))
setGeneric("affectedKinshipGroups", function(object)
    standardGeneric("affectedKinshipGroups"))
setGeneric("age", function(object)
    standardGeneric("age"))
setGeneric("age<-", function(object, value)
    standardGeneric("age<-"))

## B
setGeneric("binomialTest", function(object, ...)
    standardGeneric("binomialTest"))
setGeneric("buildPed", function(object, ...)
    standardGeneric("buildPed"))

## C
setGeneric("countGenerations", function(object, id=NULL, ...)
    standardGeneric("countGenerations"))

## D

## E
setGeneric("estimateGenerations", function(object, family=NULL, ...)
    standardGeneric("estimateGenerations"))
setGeneric("export", function(object, ...)
    standardGeneric("export"))

## F
setGeneric("familialIncidenceRate", function(object, trait=NULL,
                                             timeAtRisk=NULL, ...)
    standardGeneric("familialIncidenceRate"))
setGeneric("familialIncidenceRateTest", function(object, ...)
    standardGeneric("familialIncidenceRateTest"))
setGeneric("findFounders", function(object, ...)
    standardGeneric("findFounders"))
setGeneric("fsir", function(object, trait=NULL, lambda=NULL,
                            timeInStrata=NULL, ...)
    standardGeneric("fsir"))
setGeneric("fsirTest", function(object, ...)
    standardGeneric("fsirTest"))

## G
setGeneric("genealogicalIndexTest",
           function(object, trait, nsim=50000, traitName, perFamilyTest=FALSE,
                    controlSetMethod="getAll", rm.singletons=TRUE,
                    strata=NULL, ...)
               standardGeneric("genealogicalIndexTest"))
setGeneric("generationsFrom", function(object, id=NULL, ...)
    standardGeneric("generationsFrom"))
setGeneric("getAncestors", function(object, id=NULL, ...)
    standardGeneric("getAncestors"))
setGeneric("getChildren", function(object, id=NULL, ...)
    standardGeneric("getChildren"))
setGeneric("getCommonAncestor", function(object, id=NULL, ...)
    standardGeneric("getCommonAncestor"))
setGeneric("getFounders", function(object, ...)
    standardGeneric("getFounders"))
setGeneric("getMissingMate", function(object, ...)
    standardGeneric("getMissingMate"))
setGeneric("getSiblings", function(object, id=NULL, ...)
    standardGeneric("getSiblings"))
setGeneric("getSingletons", function(object, ...)
    standardGeneric("getSingletons"))
setGeneric("getAll", function(object, id=NULL, ...)
    standardGeneric("getAll"))
setGeneric("getExternalMatched", function(object, id=NULL, match.using, ...)
    standardGeneric("getExternalMatched"))
setGeneric("getGenerationMatched", function(object, id=NULL, include.anc=0,
                                            include.off=0, ...)
    standardGeneric("getGenerationMatched"))
setGeneric("getGenerationSexMatched", function(object, id=NULL, include.anc=0,
                                               include.off=0, ...)
    standardGeneric("getGenerationSexMatched"))
setGeneric("getSexMatched", function(object, id=NULL, ...)
    standardGeneric("getSexMatched"))

## H

## I
setGeneric("independentGroupCount", function(object, ...)
    standardGeneric("independentGroupCount"))

## J

## K
setGeneric("kinship", function(id, ...)
    standardGeneric("kinship"))
setGeneric("kinshipGroupTest", function(object, ...)
    standardGeneric("kinshipGroupTest"))
setGeneric("kinshipSumTest", function(object, ...)
    standardGeneric("kinshipSumTest"))

## L
setGeneric("lambda", function(object, ...)
    standardGeneric("lambda"))

## M

## N

## O
setGeneric("oldResult", function(object, ...)
    standardGeneric("oldResult"))

## P
setGeneric("pedigree", function(object,...)
    standardGeneric("pedigree"))
setGeneric("pedigree<-", function(object, value)
    standardGeneric("pedigree<-"))
setGeneric("pedigreeSize", function(object)
    standardGeneric("pedigreeSize"))
setGeneric("phenotypedIndividuals", function(object)
    standardGeneric("phenotypedIndividuals"))
setGeneric("plotPed", function(object, id=NULL, family=NULL, filename=NULL,
                               device="pdf", ...)
    standardGeneric("plotPed"))
setGeneric("plotRes", function(object, id=NULL, family=NULL, ...)
    standardGeneric("plotRes"))

## Q

## R
setGeneric("removeSingletons", function(object, ...)
    standardGeneric("removeSingletons"))
setGeneric("result", function(object, ...)
    standardGeneric("result"))
setGeneric("resultForId", function(object, id=NULL)
    standardGeneric("resultForId"))
setGeneric("runSimulation", function(object, nsim, ...)
    standardGeneric("runSimulation"))

## S
setGeneric("shareKinship", function(object, ...)
    standardGeneric("shareKinship"))

## T
setGeneric("timeAtRisk", function(object, ...)
    standardGeneric("timeAtRisk"))
setGeneric("timeAtRisk<-", function(object, value)
    standardGeneric("timeAtRisk<-"))
setGeneric("timeInStrata", function(object, ...)
    standardGeneric("timeInStrata"))
setGeneric("trait", function(object, ...)
    standardGeneric("trait"))
setGeneric("trait<-", function(object, value)
    standardGeneric("trait<-"))

## U
## V
## W
## X
## Y
## Z

## Definition of all classes.

## the column names for the pedigree
.PEDCN <- c("family", "id", "father", "mother", "sex")

##************************************************
##
##       FAData
##
##       Main class defining/containing all required data.
##       *
##************************************************
setClass("FAData",
         slots=c(
             pedigree="data.frame",
             age="numeric",
             .kinship="dsCMatrix",
             .trait="numeric",
             traitname="character"
         ),
         prototype=list(
             pedigree = data.frame(family = character(),
                                   id = character(),
                                   father = character(),
                                   mother = character(),
                                   sex = factor(levels = c("F", "M"))),
             age=numeric(),
             .kinship=new("dsCMatrix"),
             .trait=numeric(),
             traitname=character()
         )
         )

##************************************************
##
##       FAResult
##
##       main result class...
## setClass("FATrait",
##          contains="FAData",
##          slots=c(
##              ##             "VIRTUAL",
##              .trait="numeric",
##              traitname="character"
##          ),
##          prototype=list(.trait=numeric(),
##                         traitname=character()
##                         )
##          )

##************************************************
##
##       FAKinGroupResult
##
##       sim: the results from the simulation.
##       nsim: the number of simulations.
##       indGroupCount: number of independent affected kinship groups.
##       affectedKinGroups: list of kinship groups of each affected individual.
##************************************************
setClass("FAKinGroupResults",
         contains="FAData",
         slots=c(
             sim="list",
             nsim="numeric",
             affectedKinshipGroups="list"
         ),
         prototype=list(
             sim=list(),
             nsim=0,
             affectedKinshipGroups=list()
         )
         )

##************************************************
##
##       FAKinSumResult
##
##       sim: the results from the simulation.
##       nsim: the number of simulations.
##************************************************
setClass("FAKinSumResults",
         contains="FAData",
         slots=c(
             sim="list",
             nsim="numeric"
         ),
         prototype=list(
             sim=list(),
             nsim=0
         )
         )


##************************************************
##
##       FAGenIndexResults
##
##       sim: the results from the simulation.
##       nsim: the number of simulations.
##************************************************
setClass("FAGenIndexResults",
         contains="FAData",
         slots=c(
             sim="list",
             nsim="numeric",
             controlSetMethod="character",
             perFamilyTest="logical"
         ),
         prototype=list(
             sim=list(),
             nsim=0,
             controlSetMethod="getAll",
             perFamilyTest=FALSE
         )
         )


##************************************************
##
##       FAIncidenceRateResults
##
##       sim: the results from the simulation.
##       nsim: the number of simulations.
##************************************************
setClass("FAIncidenceRateResults",
         contains="FAData",
         slots=c(
             sim="list",
             nsim="numeric",
             timeAtRisk="numeric"
         ),
         prototype=list(
             sim=list(),
             nsim=0,
             timeAtRisk=numeric()
         )
         )

##************************************************
##
##       FAStdIncidenceRateResults
##
##       sim: the results from the simulation.
##       nsim: the number of simulations.
##************************************************
setClass("FAStdIncidenceRateResults",
         contains="FAData",
         slots=c(
             sim="list",
             nsim="numeric",
             timeInStrata="matrix",
             lambda="numeric"
         ),
         prototype=list(
             sim=list(),
             nsim=0,
             timeInStrata=matrix(),
             lambda=numeric()
         )
         )

setClass("FABinTestResults",
         contains = "FAData",
         slots = c(result = "data.frame")
        ,
         prototype = list(
             result = data.frame(total_phenotyped = integer(),
                                 total_affected = integer(),
                                 family = character(),
                                 phenotyped = integer(),
                                 affected = integer(),
                                 pvalue = numeric(),
                                 prob = numeric(),
                                 check.names = FALSE, stringsAsFactors = FALSE)
         )
         )

---
title: "Pedigree Analysis and Familial Aggregation"
graphics: yes
output:
  BiocStyle::html_document:
    toc_float: true
vignette: >
  %\VignetteIndexEntry{Pedigree Analysis and Familial Aggregation}
  %\VignetteEngine{knitr::rmarkdown}
  %\VignetteEncoding{UTF-8}
  %\VignetteDepends{FamAgg,BiocStyle}
  %\VignettePackage{FamAgg}
  %\VignetteKeywords{Genetics}
bibliography: references.bib
csl: biomed-central.csl
references:
- id: dummy
  title: no title
  author:
  - family: noname
    given: noname
---

```{r  echo = FALSE, results = "hide" }
isWindows <- .Platform$OS.type != "unix"
library(FamAgg)
library(BiocStyle)
```

**Package**: `r BiocStyle::Biocpkg("FamAgg")`<br />
**Authors**: `r packageDescription("FamAgg")$Author`<br />
**Modified**: `r file.info("FamAgg.Rmd")$mtime`<br />
**Compiled**: `r date()`


# Introduction

This package provides basic pedigree analysis and plotting utilities as well as
a variety of methods to evaluate familial clustering of cases from a given
trait. Identification of families or groups of individuals within families with
significant aggregation of cases can aid also in the selection of interesting
and promising individuals for whole genome or exome sequencing projects.

For kinship coefficient calculations and pedigree plotting the package relies
and extends the functionality of the `kinship2` package [@Sinnwell:2014kd].

If you use this package please cite Rainer et al. [@Rainer:2016hk].


# Basic pedigree operations

In the examples below we perform some simple pedigree operations, such as
plotting the pedigree for an individual or family, finding the closest common
ancestor for a set of individuals in a pedigree or retrieving the identifiers
(IDs) of all ancestors for an individual. Basic pedigree information is stored
in `FAData` objects, thus we first generate such an object from a subset of the
Minnesota Breast Cancer Study provided by the `kinship2` package. In the example
below, we generate the `FAData` providing a `data.frame` with the pedigree data,
alternatively, the pedigree information could be imported from a file (see
Section [3](#org99a8d9e)).  Upon data set creation the kinship matrix (i.e. a
matrix containing the kinship coefficient between each pair of individuals in
the whole pedigree) is internally calculated using the functionality from the
`kinship2` package [@Sinnwell:2014kd].

```{r  libraries, warning = FALSE, message = FALSE }
library(FamAgg)

data(minnbreast)
## Subsetting to only few families of the whole data set.
mbsub <- minnbreast[minnbreast$famid %in% 4:14, ]
mbped <- mbsub[, c("famid", "id", "fatherid", "motherid", "sex")]
## Renaming column names.
colnames(mbped) <- c("family", "id", "father", "mother", "sex")
## Defining the optional argument age.
endage <- mbsub$endage
names(endage) <- mbsub$id
## Create the object.
fad <- FAData(pedigree = mbped, age = endage)
```

We can access all the pedigree information stored in this object using the
`pedigree` method, but also using `$`. The row names of the pedigree
`data.frame` as well as the names of the vectors returned by `$` are the IDs of
the individuals in the pedigree.

```{r  access-data }
## Use the pedigree method to access the full pedigree
## data.frame,
head(pedigree(fad))

## or access individual columns using $.
## The ID of the father (0 representing "founders"):
head(fad$father)
## Mother:
head(fad$mother)
## Sex:
head(fad$sex)

## We can also access the age of each individual, if
## provided.
head(age(fad))
```

To extract the pedigree for a single family we can use the `family` method,
specifying either the ID of the family or the ID of an individual in the family.

```{r  family }
## Extract the pedigree information from family "4"...
nrow(family(fad, family = 4))

head(family(fad, family = 4))

## ...which is the same as extracting the family pedigree
## for an individual of this family.
head(family(fad, id = 3))

## Note that IDs are internally always converted to character,
## thus, using id=3 and id="3" return the same information.
head(family(fad, id = "3"))
```

Alternatively, we could subset the `FAData` to individuals of a single family.

```{r  subsetting }
## Subset the object to a single family.
fam4 <- fad[fad$family == "4", ]
table(fam4$family)
```

To explore this family we can plot its pedigree. By default, the plotting
capabilities of the `kinship2` package are used to plot pedigrees, but
alternatively, if all required dependencies are available, the `HaploPainter`
[@Thiele:2005] perl script (<http://haplopainter.sourceforge.net/>) can be
used instead. The `switchPlotfun` function can be used to switch the plotting
back-end. Available arguments are `ks2paint` and `haplopaint` for `kinship2` and
`HaploPainter` plotting, respectively. Note however, that `HaploPainter` only
allows to export plots to a file, while `kinship2` plotting allows, in addition
to export the plot, also to show it as a *standard* `R` plot.

Below we use the `switchPlotfun` to ensure the use of `kinship2` plotting
(usually not required) and plot the full available pedigree of individual `3`.
If the age of individuals is available, it will be plotted below the
individual's ID.

```{r  plotPed, message=FALSE, fig.align='center', warning = FALSE }
switchPlotfun("ks2paint")
## By supplying device="plot", we specify that we wish to visualize the
## pedigree in an R plot. This is the default for "ks2paint", anyway.
plotPed(fad, id = 3, device = "plot")
```

The pedigree for an individual or a list of individuals can be extracted using
the `buildPed` method. By default the method first tries to identify all parents
up to 3 generations in the pedigree, and subsequently all children of the
individuals and all identified parents.

```{r  buildPed, message = FALSE }
## Build the pedigree for individual 3.
fullPed <- buildPed(fad, id = "3")
nrow(fullPed)
```

Alternatively, we can extract the smallest possible pedigree for a list of
individuals by specifying `prune=TRUE`. Internally, the function transforms the
pedigree into a graph, tries to find all paths between the individuals and
returns the sub-graph of all individuals along with individuals along the paths
between them.

```{r  buildPed-prune, message = FALSE }
## Find the subpedigree for individuals 21, 22 and 17.
buildPed(fad, id = c(21, 22, 17), prune = TRUE)
```

And the pedigree plot for that subset of the whole family:

```{r  plotPed-3ids, message=FALSE, fig.align='center', warning = FALSE }
plotPed(fad, id = c(21, 22, 17), prune = TRUE)
```

Note that the pedigree returned by the `buildPed` method for an individual might
be different than the pedigree of a whole family. The pedigree returned by
`buildPed` contains only individuals that share kinship with the specified
individual. To exemplify this, we plot the pedigree for the family `14` in the
Minnesota Breast Cancer data set. Note that the individuals in the pedigree plot
depicted as diamonds are individuals with unknown gender. (The message "Did not
plot&#x2026;" is issued by the `kinship2` plotting function and indicates singletons
that are assigned to the family but do neither have parents nor children.)

```{r  plotPed-family-14, message=FALSE, fig.align='center', warning = FALSE }
plotPed(fad, family = "14", cex = 0.4)
```

In this family, founder `441` is the founder of two family branches. Building
the pedigree for individual `440` will not include any of the individuals of the
second branch, as he does not share kinship with any of them. The pedigree built
for `447` on the other hand contains also individuals from the second branch as
she shares kinship with them (*via* her mother `441`).

```{r  buildPed-for-individual, message = FALSE }
## Check if we have individual 26064 from the second branch in the pedigree
## of individual 440.
any(buildPed(fad, id = "440")$id == "26064")

## What for the pedigree of 447?
any(buildPed(fad, id = "447")$id == "26064")
```

A family pedigree may consist of many founder couples (i.e. individuals for
which neither father nor mother is defined in the pedigree). To identify the
pedigree's founder couple (being the couple with the largest number of offspring
generations in the pedigree) the `findFounders` method can be used. Note that
the function returns always only one couple, even if there might be two founder
couples in the family pedigree with the same number of offspring generations.

```{r  findFounders, message = FALSE }
## Find founders for family 4.
findFounders(fad, "4")
```

Alternatively, it might be of interest to determine the closest common ancestor
between individuals in a pedigree. Below we use the `getCommonAncestor` method
to identify the common ancestor for individuals `21`, `22` and `17` (which we
know from the pedigree a bit above are `1` and `2`).

```{r  getCommonAncestors, message = FALSE }
## Find the closest common ancestor.
getCommonAncestor(fad, id = c(21, 22, 17))
```

Other useful methods are `getChildren`, `getAncestors` and `getSiblings`, that
return the children (or all offspring generations up to a specified level), the
parents (or all ancestors) or the siblings for the specified individuals,
respectively.

```{r  getChildren-and-others, message = FALSE }
## Get the children of ID 4.
getChildren(fad, id = "4", max.generations = 1)

## Get the offsprings.
getChildren(fad, id = "4")

## Get all ancestors.
getAncestors(fad, id = "4")

## Get the siblings.
getSiblings(fad, id = c("4"))
```

In the whole Minnesota Breast Cancer data set there are 426 families
corresponding to 426 founders that had cancer during the screening phase between
1944 and 1952. In the code block below we identify the affected founders per
family.

```{r  affected-founders, message = FALSE }
## Add the trait information to the FAData object.
cancer <- mbsub$cancer
names(cancer) <- as.character(mbsub$id)
trait(fad) <- cancer

## Identify the affected founders.
## First all affected individuals.
affIds <- affectedIndividuals(fad)
## Identify founders for each family.
founders <- lapply(unique(fad$family), function(z){
    return(findFounders(fad, family = z))
})
names(founders) <- unique(fad$family)

## Track the affected founder.
affFounders <- lapply(founders, function(z){
    return(z[z %in% affIds])
})
## Interestingly, not all founders are affected! It seems in some cases
## parents of the affected participants in the screening phase have also
## been included.
affFounders <- affFounders[unlist(lapply(affFounders, length)) > 0]

## The number of families analyzed.
length(founders)

## The number of families with affected founder.
length(affFounders)

```

Unexpectedly, only in few families one of the founders is affected. For the
other families additional (unaffected) ancestors might have been added at a
later time point.

Next we get the number of affected individuals that are related to these
affected founders.

```{r  affected-for-founders, message = FALSE }
kin2affFounders <- shareKinship(fad, unlist(affFounders))

## How many of these are affected?
sum(kin2affFounders %in% affIds)

## How many affected are not related to an affected founder?
sum(!(affIds %in% kin2affFounders))

```


## Pedigree analysis methods

In this section we perform some more advanced pedigree operations. First, we
identify all individuals in the pedigree that share kinship with individual `4`.

```{r  shareKinship, message = FALSE }
## Get all individuals sharing kinship with individual 4.
shareKinship(fad, id = "4")
```

It is also possible to prune remote relatives. If we don't want to include grand
children, (first) cousins and everbody else more remotely related to individual
`4`, we use the option `rmKinship`. Essentially, only siblings, children, and
parents remain.

```{r  shareKinshipRm, message = FALSE }
## Get all individuals sharing kinship with individual 4, but only with kinship
## higher than 0.125 (exclude first cousins, grand children, great grand
## parents etc, i.e. everybody with kinship 0.125 or lower)
shareKinship(fad, id = "4", rmKinship = 0.125)
```

Next, we determine generations within the pedigree. Generations can only be
estimated for a single family, since in most instances e.g. the year of birth is
not available. Thus, generations are estimated considering the relation between
individuals, starting from the founder couple, i.e. generation 0, assigning
generation 1 to their children and all the mates of their children and so
on. The `estimateGenerations` method calculates such generation numbers for each
family defined in the object (or for a single family, if the family ID is
provided). The result is returned as a list with the list names corresponding to
the family ID and the list elements being the estimated generation numbers (with
names corresponding to the ID of the respective individual).

```{r  estimageGenerations, message = FALSE }
## Estimate generation levels for all families.
estimateGenerations(fad)[1:3]
```

Individuals without generation level (i.e. with an `NA`) are not connected to
any other individual in the pedigree (and thus most likely represent errors in
the pedigree).

In addition, it is also possible to calculate generation levels relative to a
(single) specified individual:

```{r  generationsFrom, message = FALSE }
gens <- generationsFrom(fad, id = "4")
```

We can render these generation numbers into the pedigree:

```{r  plotPed-with-generations, message=FALSE, fig.align='center', warning = FALSE }
plotPed(fad, family = 4, label2 = gens)
```


## Additional plotting options

If a trait information is available it might be of interest to highlight
affected individuals in the pedigree. Trait information should always be coded
as `0` (or `FALSE`) for unaffected and `1` (or `TRUE`) for affected. In the
example below, we use the *cancer* information from the Minnesota Breast Cancer
Study.

```{r  set-trait, results='hide', message=FALSE }
## Extract the cancer trait information.
tcancer <- mbsub$cancer
names(tcancer) <- mbsub$id
## Set the trait.
trait(fad) <- tcancer

```

We can now extract the trait information from the object or identify directly
the phenotyped or affected individuals.

```{r  affectedIndividuals, message = FALSE }
## Extract the trait information.
head(trait(fad))

## We can also extract the IDs of the affected individuals.
head(affectedIndividuals(fad))

## Or the IDs of the phenotyped individuals.
head(phenotypedIndividuals(fad))
```

Plotting a `FAData` object with trait information results in a pedigree plot
with highlighted affected individuals (for `kinship2` pedigree plotting:
affected, unaffected and not phenotyped are represented as filled symbols, open
symbols and symbols with a question mark inside, respectively).

```{r  plotPed-with-trait, message=FALSE, fig.align='center', warning = FALSE }
## Plotting the pedigree for family "9".
plotPed(fad, family = "9")
```

In addition, we can manually highlight individuals using the `highlight.ids`
argument. For `kinship2` pedigree plotting, a list of length 2 is supported as
argument `highlight.ids`, with the first element being plotted on the top left
corner of the symbol and the second element on the top right corner.

```{r  plotPed-trait-highlight, message=FALSE, fig.align='center', warning = FALSE }
## Plotting the pedigree for family "9".
plotPed(fad, family = "9", highlight.ids = list(a = c("185", "201", "198"),
						b = c("193")))
```

An alternative way to highlight individuals or add text to the plot is to use
the arguments `label1`, `label2` and `label3` or the `plotPed` method.


## Graph utilities

Pedigrees can also be transformed to graphs using the `ped2graph` function. That
way all graph theory methods implemented in e.g. the `igraph` package can be
applied to pedigrees.

```{r  ped2graph, message = FALSE }
## Transform the full pedigree to a graph.
fullGraph <- ped2graph(pedigree(fad))

## In addition, build the graph for a single family.
singleFam <- ped2graph(family(fad, family=4))
```

We can plot these pedigrees also as graph and could use any of the layout
methods provided in the `igraph` package.

```{r  plot-igraph, fig.align='center', message = FALSE, fig.cap = "Pedigrees represented as graphs." }
## Build the layout.
plot(fullGraph)
lay <- layout_(singleFam, on_grid())
plot(singleFam, layout = lay)
```

The `connectedSubgraph` function implemented in the `FamAgg` package provides
additional functionality to find the smallest connected subgraph of a list of
submitted nodes (i.e. individuals).

In the code below we want to extract the smallest possible connected subgraph of
the pedigree-graph of family 4 containing individuals `7`, `8`, `27` and `17`.

```{r  connectedSubgraph, message = FALSE }
subgr <- connectedSubgraph(singleFam, nodes = c("7", "8", "27", "17"))
```

This is in principle what the `buildPed` method with the option `prune=TRUE`
does to find the smallest pedigree for a set of individuals, only that
`buildPed` ensures that also eventually missing parents are added.

```{r  plot-subgraph, subgraph-plot, fig.align='center' }
## Plot the graph.
plot(subgr)
## Similar to buildPed/plotPed with prune=TRUE.
plotPed(fad, id=c("7", "8", "17", "27"), prune=TRUE)
```


# Importing and exporting pedigree data<a id="org99a8d9e"></a>

Besides providing the pedigree data as a `data.frame`, the `FAData` constructor
can also read pedigree data from various file formats, such as plink
[@Purcell:2007dg] *ped* or *fam* files
(<http://pngu.mgh.harvard.edu/~purcell/plink/data.shtml>) or generic text files.

```{r  import, message=FALSE }
## Import a "ped" file.
pedFile <- system.file("txt/minnbreastsub.ped.gz", package = "FamAgg")
## Quick glance at the file.
readLines(pedFile, n = 1)
fad <- FAData(pedFile)

head(pedigree(fad))

```

Alternatively, we can import pedigree data from generic input files.

```{r  import-generic, message=FALSE }
## Create the FAData by reading data from a txt file.
pedFile <- system.file("txt/minnbreastsub.txt", package = "FamAgg")
fad <- FAData(pedigree = pedFile, header = TRUE, id.col = "id",
	      family.col = "famid", father.col = "fatherid",
	      mother.col = "motherid")

```

And we can export pedigree data again using the `export` method. In the example
below, we subset the whole pedigree to the pedigree of family 4 and export this
as a *ped* file.

```{r  export }
tmpF <- tempfile()

## Subset the pedigree to family 4
fam4 <- fad[fad$family == 4, ]

## Export data in ped format.
export(fam4, tmpF, format = "ped")
```


# Testing for familial aggregation

Familial aggregation aims to identify families within large ancestral pedigrees
that show a non-random aggregation of traits.

As an example, we analyze here data from the Minnesota Breast Cancer Record,
which is provided by the `kinship2` package. In brief, this data set consists of
genealogical information from 426 unrelated founders diagnosed with breast
cancer whose families entered a longitudinal study on cancer in the state of
Minnesota (USA) in 1944. Cancer cases are encoded with a `1` in column `cancer`
in the `minnbreast` `data.frame`. Note however that, besides breast cancer, also
prostate cancer cases are reported. This unfortunately causes a systematic bias
in the data set as families were only included if a founder was diagnosed with
breast cancer, but all occurrences of both breast and prostate cancer are
reported. Based on this bias many of the results below should be taken with
caution. Another important information is provided in column `endage`, which
represents either the age of cancer onset, the age at the end of the study or
the age at death of the participant.

Note that, to reduce computation time, we perform the analysis only on a subset
of families from the Minnesota Breast Cancer record and reduce the number of
simulation runs. We specifically selected some families with a high percentage
of cancer cases, thus, the analysis presented here is biased. Also, in a real
analysis you should increase the `nsim` argument.

```{r  famagg-setup, warning=TRUE, message=FALSE }
library(FamAgg)
set.seed(18011977)
data(minnbreast)
## Subset the dataset to reduce processing time.
mbsub <- minnbreast[minnbreast$famid %in% c(4:100, 173, 432), ]
## Uncomment the line below to use the whole dataset instead.
## mbsub <- minnbreast

## Define the number of simulations we perform.
## nsim <- 10000
nsim <- 1000

mbped <- mbsub[, c("famid", "id", "fatherid", "motherid", "sex")]
## Renaming column names.
colnames(mbped) <- c("family", "id", "father", "mother", "sex")
## Create the FAData object.
fad <- FAData(pedigree = mbped)

## Define the trait.
tcancer <- mbsub$cancer
names(tcancer) <- as.character(mbsub$id)
```

In the following section we analyze the data set first using the *genealogical
index* method [@Hill:1980tz] (Section [4.1](#org1dbf9f5)), then we estimate the
per-individual risk of disease using the *familial incidence rate* (FIR, also
abbreviated as *FR* in the original work) [@Kerber:1995cx] (Section
[4.2](#org7913823)) and apply our *kinship sum test* to identify affected individuals
exhibiting a higher relationship to other affected individuals than what would
be expected by chance (Section [4.3](#orgb323644)). Subsequently, we apply our
*kinship group test* (Section [4.4](#orgaac6b02)) that allows to identify highly
clustered affected individuals within families.

The *genealogical index of familiality* and the *familial incidence rate* test are
well established methods while the *kinship sum test* and the *kinship group test*
are novel approaches presented here for the first time.


## *Genealogical index of familiality* <a id="org1dbf9f5"></a>

We next calculate the *genealogical index of familiality* (GIF) [@Hill:1980tz]
(referred to as the *genealogical index* in the original work) for cancer
occurrence in a subset of the Minnesota Breast Cancer Record data set. For a
given trait (e.g. whether or not an individual was diagnosed with a certain type
of cancer), the method computes the mean kinship between affected individuals
(cases) in the whole pedigree along with mean kinship values of randomly drawn
sets of individuals. The distribution of average kinship values among the
control sets is then used to estimate the probability that the observed level of
kinship among the cases is due to chance.

Below, we perform the analysis using the `genealogicalIndexTest` method on the
`cancer` trait. In its default setting, the `genealogicalIndexTest` function uses
all phenotyped individuals in the pedigree as control population from which sets
of random samples equal in size to the number of affected are drawn.

Note that by default the function excludes all singletons (i.e. unconnected
individuals in the pedigree) from the analysis. Changing the argument
`rm.singletons` to `FALSE` will estimate the GIF on the full data set.

```{r  gif, warning=FALSE, message=FALSE }
## Calculate the genealogical index of familiality.
gi <- genealogicalIndexTest(fad, trait = tcancer,
			    traitName = "cancer", nsim = nsim)

## Display the result.
result(gi)
```

The column *genealogical index* of the result `data.frame` shown above represents
the mean kinship between all pairs of affected individuals in the pedigree
multiplied by `100000` for easier interpretation.  Thus, according to the GIF
test, a clustering of cancer cases is present in the analyzed pedigree. The
output messages from the method call indicate that some individuals have been
excluded from the test since they were either not phenotyped in the trait
(i.e. have a missing value in trait), or are not *connected* in the family
pedigree (do not share kinship with any other individual in the pedigree after
removing non-phenotyped individuals).

The genealogical index of familiality implementation in this package adds some
more flexibility to the original approach. The definition of the appropriate set
of control individuals from which random samples are drawn can be specified with
the `controlSetMethod` argument. Also, it is possible to perform a stratified
sampling, e.g. if the group of affected cases in a pedigree consists of 5 female
and 3 male individuals, submitting the sex of each individual in the pedigree
with the argument `strata` (i.e. `strata=fad$sex`, with `fad` being the `FAData` object
on which the analysis is performed) allows the function to define random control
sets with the same proportion of male/female individuals.

In the next example, we use the `getSexMatched` function to define the set of
control individuals and also the `getExternalMatched` submitting the gender
information of each individual. The results from both approaches are essentially
identical, and in the present data set not that useful, as the Minnesota Breast
Cancer data set lists both, breast cancer and prostate cancer in column `cancer`,
thus, the set of control individuals will contain all individuals with known
sex.

```{r  gif-2, warning=FALSE, eval=FALSE }
## Calculate the genealogical index of familiality using random sampling from
## a sex matched control set.
giSexMatch <- genealogicalIndexTest(fad, trait = tcancer,
				    traitName = "cancer", nsim = nsim,
				    controlSetMethod = "getSexMatched")

## Use an external vector to perform the matching.
## The results are essentially identical.
giExtMatch <- genealogicalIndexTest(fad, trait = tcancer,
				    traitName = "cancer", nsim = nsim,
				    controlSetMethod = "getExternalMatched",
				    match.using = fad$sex)
```

Note that any matching or stratified sampling can lead to the exclusion of
individuals with missing values in either the matching criteria or the strata.

In the Minnesota Breast Cancer data set, the number of prostate cancer cases is
much lower than the number of breast cancer cases, thus, simple random sampling
might result in an biased genealogical index of familiality estimate since about
the same proportion of male and female individuals will be sampled. To account
for such cases a stratified sampling, as performed below, can be used instead.

```{r  gif-3, message=FALSE }
## Evaluate the proportion of male and femal cases.
table(gi$sex[affectedIndividuals(gi)])

## We can use the gender information to perform stratified sampling, i.e.
## in each permutation a random set of 3 male and 15 females will be selected.
giStrata <- genealogicalIndexTest(fad, trait = tcancer,
				  traitName = "cancer", nsim = nsim,
				  strata = fad$sex)

result(giStrata)
```

Finally, we plot the result from the simulation. The blue vertical line in the
plot below represents the mean kinship value between all affected individuals in
the pedigree. The distribution of mean kinship values from the 1000 randomly
drawn sets are shown in grey color.

```{r  gif-4-plot, mbreast-genealogical-index-result, message=FALSE, warning=FALSE, fig.align='center' }
## Plot the result.
plotRes(giStrata)
```

The genealogical index of familiality can also be estimated by the `gif`
function from the `gap` R-package. Below we calculate the estimate using both
methods and compare the resulting estimate. Note that the `gif` method reports
only the genealogical index of familiality estimate but does not estimate
significance.

```{r  gif-gap, message=FALSE }
library(gap)

## Adding the trait information, so the extracted pedigree data.frame will
## also contain a column "affected" with that information.
trait(fad) <- tcancer

## Extract the pedigree and re-format it for the gif function.
pedi <- pedigree(fad)
## Remove singletons.
pedi <- removeSingletons(pedi)
pedi[is.na(pedi$father), "father"] <- 0
pedi[is.na(pedi$mother), "mother"] <- 0

## Identify the affected individuals.
affIds <- as.numeric(pedi$id[which(pedi$affected == 1)])

## Execute the gif method contained in the gap package.
gifRes <- gif(pedi[, c("id", "father", "mother")], affIds)

## Calculate the GIF using FamAgg's genealogicalIndexTest.
gifT <- genealogicalIndexTest(fad, trait = tcancer, nsim = 100)

## Comparing the results:
all.equal( result(gifT)$genealogical_index,  gifRes[[1]] )
```

Thus, the GIF estimate from the `gap` package is identical to the one from the
`FamAgg` package.

In the examples above, we tested for an enrichment of cancer cases in the full
data set, i.e. across all families. In addition, we can perform the test
individually for each family, by setting the `perFamilyTest` parameter of the
`genealogicalIndexTest` to `TRUE`, and thus test for a clustering of cancer
cases within each family.

```{r  gif-5, message=FALSE, warning=FALSE }
## Perform the analysis (no strata etc) separately for each family.
giFam <- genealogicalIndexTest(fad, trait = tcancer, nsim = nsim,
			       perFamilyTest = TRUE,
			       traitName = "Cancer")

## Display the result from the analysis.
head(result(giFam))
```


## *Familial incidence rate* (FIR)<a id="org7913823"></a>

A per-individual risk of e.g. disease can be calculated using the *familial
incidence rate* (FIR, abbreviated as *FR* in the original work)
[@Kerber:1995cx]. This measure considers the kinship of each individual with
any affected in a given trait in the pedigree and the time at risk for each
individual. Thus, the FIR is an estimate for the risk per gene-time for each
individual given the disease-experience in the cohort.

As *time at risk* for each individual we use the `endage` column in the
Minnesota Breast Cancer data set, which represents the participant's age at the
last follow-up or at cancer incidence. This estimate of time at risk is rather
crude and in a real life situation a better, more accurate, estimate that is
based e.g. on the birth dates and dates of last follow up or incidence might be
used instead. See the help of functions `estimateTimeAtRisk` and `sliceAge` for
details and options related to *time at risk*.

```{r  fir-1, warning=FALSE, message = FALSE }
## Estimate the risk for each individual using the familial incidence
## rate method. We use the "endage" provided in the Minnesota Breast Cancer
## Record as a measure for time at risk.
fr <- familialIncidenceRate(fad, trait = tcancer, timeAtRisk = mbsub$endage)

```

A note on singletons: for all per-individual measures unconnected individuals
within the pedigree are automatically excluded from the calculations as no
kinship-based statistics can be estimated for them (they do, by definition, not
share kinship with any other individual in the pedigree, thus their kinship
coefficient with any other individual in the pedigree will be `0`). Note also
that the removal of e.g. not phenotyped individuals prior to the calculation can
also *generate* singletons, that additionally become removed. This removal
results in an estimate with the value `NA` for all singletons as well as not
phenotyped individuals.

Next, we calculate the mean FIR within each family and plot this information.

```{r  fir-2, mbreast-mean-fr-per-family, message=FALSE, warning=FALSE, fig.align='center' }
## Split the FIR by family and average the values within each.
frFam <- split(fr, f = fad$family)
frFamAvg <- lapply(frFam, mean, na.rm = TRUE)

## Sort and plot the averages.
frFamAvg <- sort(unlist(frFamAvg), decreasing = TRUE)
plot(frFamAvg, type = "h", xaxt = "n", xlab = "", ylab = "mean FIR",
     main = "Per family averaged familial incidence rate")
axis(side = 1, las = 2, at = 1:length(frFamAvg), label = names(frFamAvg))

```

Not unexpectedly, individuals in some families have on average a higher familial
incidence rate, and thus a higher risk of cancer than others.

In the next example, we calculate the familial incidence rate assessing in
addition the significance of each estimate using Monte Carlo simulations. This
extension to the original approach from Kerber [@Kerber:1995cx] does also
allow stratified sampling.

```{r  fir-3, warning=FALSE, message=FALSE }
## Estimate the risk for each individual using the familial incidence
## rate method. We use the endage provided in the Minnesota Breast Cancer
## Record as a measure for time at risk.
frTest <- familialIncidenceRateTest(fad, trait = tcancer,
				    traitName = "cancer",
				    timeAtRisk = mbsub$endage,
				    nsim = nsim)

```

The familial incidence rate can be extracted easily from the result object using
the `familialIncidenceRate` method or using `$fir`. Also, the empirical p-value
from the simulation analysis and the time at risk can be accessed using the `$`
operator (i.e. using `$pvalue`, `$tar` or `$timeAtRisk`, respectively).

```{r  fir-4 }
head(familialIncidenceRate(frTest))
head(frTest$fir)

```

Finally, we inspect the results from the analysis.

```{r  fir-5 }
head(result(frTest))

```

We can also identify the families containing individuals with a significant FIR.

```{r  fir-6 }
frRes <- result(frTest)
frSig <- frRes[which(frRes$padj < 0.05), ]

## Split by family.
frFam <- split(frSig, frSig$family)
frRes <- data.frame(family = names(frFam),
		    no_sign_fir = unlist(lapply(frFam, nrow)))
## Determine the number of phenotyped and affected individuals per family.
noPheNAff <- sapply(names(frFam), function(z){
    fam <- family(frTest, family = z)
    return(c(no_pheno = sum(!is.na(fam$affected)),
	     no_aff = length(which(fam$affected == 1))
	     ))
})
frRes <- cbind(frRes, t(noPheNAff))

## Display the number of phenotyped and affected individuals as well as
## the number of individuals within the families with a significant FIR.
frRes[order(frRes[, "no_sign_fir"], decreasing = TRUE), ]
```

We have an enrichment of affected cases in families 173, 13 and 432.


## *Kinship sum test* <a id="orgb323644"></a>

Next, we use the *kinship sum test* that evaluates familial aggregation based on
the sum of kinship values between affected cases. The test identifies affected
individuals exhibiting a higher relationship to other affected individuals than
would be expected by chance. By specifying the `strata` we perform
sex-stratified random sampling, i.e. ensure that the proportion of male and
female individuals in each randomly sampled group matches the corresponding
proportions in the *real*, observed, affected.

```{r  kinsum-1, message = FALSE }
## Perform the kinship sum test.
kinSum <- kinshipSumTest(fad, trait = tcancer, traitName = "cancer",
			 nsim = nsim, strata = fad$sex)
head(result(kinSum))

```

Next, we identify those individuals that have a significant kinship sum
accepting a 10% false discovery rate (FDR).

```{r  kinsum-2, message = FALSE }
## Extract the IDs of the individuals with significant kinship. By default,
## the raw p-values are adjusted for multiple hypothesis testing using the
## method from Benjamini and Hochberg.
kinSumRes <- result(kinSum)
kinSumIds <- as.character(kinSumRes[kinSumRes$padj < 0.1, "affected_id"])

## From which families are these?
table(kinSumRes[kinSumIds, "family"])
```

Thus, most of the identified significant individuals are from two families.
Next, we compare the FIR scores of affected or unaffected (but phenotyped)
individuals in this family to the FIR scores of affected or unaffected
individuals of all other families.

```{r  kinsum-3, mbreast-family-432-FIR-compared-to-others, message=FALSE, warning=FALSE, fig.align='center' }
## Get the familial ratio of the significant in this family, of all in
## this family, and of all others.
famId <- kinSumRes[1, "family"]

## Extract the family.
fam <- family(kinSum, family = famId)

## Stratify individuals in affected/unaffected.
strat <- rep("All, unaff.", length(kinSum$id))
strat[which(kinSum$affected > 0)] <- "All, aff."
strat[kinSum$id %in% fam$id] <- paste0("Fam ", famId, ", unaff.")
strat[kinSum$id %in% fam$id[which(fam$affected > 0)]] <-
    paste0("Fam ",famId,", aff.")

famData <- data.frame(fr = fr, group = strat)
boxplot(fr~group, data = famData, na.rm = TRUE, ylab = "FIR",
	col = rep(c("#FBB4AE", "#B3CDE3"), 2))

```

As expected, the familial incidence rate (i.e., in the present data set, the
risk of individuals to get cancer, given their kinship to other cancer cases)
for individuals (whether affected or yet unaffected) in this family is higher
than in the data set analyzed here.

Next, we plot the pedigree of this family.

```{r  kinsum-4, mbreast-family-432-affected, message=FALSE, warning=FALSE, fig.align='center' }
## Plot the pedigree for the family of the selected individual removing
## all individuals that were not phenotypes.
plotPed(kinSum, id = kinSumIds[1], cex = 0.3, only.phenotyped = TRUE)


```

And finally, also plot the kinship sum for the individuals with the largest
kinship sum in relation to the *expected* kinship sums from the Monte Carlo
simulations.

```{r  kinsum-5, mbreast-family-432-affecte-res, message=FALSE, warning=FALSE, fig.align='center' }
plotRes(kinSum, id = kinSumIds[1])
```


## *Kinship group test* <a id="orgaac6b02"></a>

Here we apply the *kinship group test* to the data set. This test first defines
for each affected individual a group of individuals considering only individuals
that are as closely related as the most distant affected individual.  For each
of these kinship groups two tests are then performed, one by comparing the mean
kinship among affected in the group with the mean kinship from Monte Carlo
simulations (ratio test) and one evaluating the largest observed kinship value
between affected individuals with those of random samples from the simulation
(kinship group test).

In the example below we specify again the `strata` argument and thus perform
sex-stratified random sampling.

```{r  kingroup-1, message=FALSE }
## Calculate the kinship test.
kinGroup <- kinshipGroupTest(fad, trait = tcancer,
			     traitName = "cancer",
			     nsim = nsim, strata = fad$sex)
head(result(kinGroup))

```

The kinship group test finds a significant aggregation of cases in families 13,
72, 173 and 432. In fact, as we see further below, the test identified a
subgroup in the latter which shows with an exceptional high proportion of cases.

Below, we summarize the results further by listing the total number of families
in the pedigree and the number of families in which kinship groups with
significant kinship p-value and significant ratio p-value (both at a 5% FDR).

```{r  kingroup-2, message = FALSE }
kinGroupRes <- result(kinGroup)
## Create a data.frame with the summarized results.
resTab <- data.frame(total_families = length(unique(kinGroup$family)),
		     ratio_sign = length(unique(
			 kinGroupRes[kinGroupRes$ratio_padj < 0.05, "family"]
		     )),
		     kinship_sign = length(unique(
			 kinGroupRes[kinGroupRes$kinship_padj < 0.05, "family"]
		     ))
		     )
resTab
```

The most significant kinship group identified by the kinship group test is shown
in the figure below. The mother (individual `17609`) of the nuclear family
representing this group and all her daughters have cancer (see figure
below). This mother is however not directly related to the affected founder of
this family, individual `17517`, but did marry her son (id `17530`; see figure above
for the full pedigree of this family `432`).

We are also submitting the familial incidence rate values calculated above with
argument `label1` which are then displayed below the ID of each individual in the
plot.

```{r  kingroup-3, mbreast-family-432-affecte-res-kinship, message=FALSE, warning=FALSE, fig.align='center' }
plotPed(kinGroup, id = kinGroupRes[kinGroupRes$family == "432",
				   "group_id"][1],
	prune = TRUE, label1 = fr)
```


## Binomial test

The binomial test evaluates whether the number of affected in a family (or the
whole pedigree) is significantly higher than what would be expected by chance
(given a probability of being affected in a trait). In contrast to most other
methods this test does not take the degree of kinship between individuals into
account and is hence independent of the family structure in the pedigree. We can
perform this type of test using the `binomialTest` function on any `FAData` object
or any object extending it. Below we use the binomial test to evaluate a
significant enrichment of affected individuals in any family in the pedigree.

```{r  bintest-1, message = FALSE }
binRes <- binomialTest(fad, trait = tcancer, traitName = "Cancer")

binResTab <- result(binRes)
head(binResTab)
```

The probability used on the binomial test is shown in column `"prob"` and is in
essence the ratio between the affected and phenotyped in the pedigree
(i.e. 154/2202). This might be an overestimation, especially if the provided
pedigree is not representative of the population. A population-based probability
can however be provided with argument `prob`. Below we test specifically whether
we have families in which the number of individuals with breast cancer is
significantly higher than expected. To this end we set the trait status of all
male individuals to `NA` and repeat the test providing the probability of
developing breast cancer during in women, which, according to the U.S. Breast
Cancer Statistics (from breastcancer.org) is 1 out of 8 in their life time.

```{r  bintest-2, message = FALSE }
## Set the trait status to NA for all male individuals.
tcancer[fad$sex == "M" | is.na(fad$sex)] <- NA

## Perform the test providing also the population probability
binRes <- binomialTest(fad, trait = tcancer, prob = 1/8)

binResTab <- result(binRes)
head(binResTab)
```

Below we plot the pedigree for the family with the strongest enrichment with
affected individuals.

```{r  bintest-3, message = FALSE, fig.align = "center", fig.pos = "h!" }
plotPed(binRes, family = 173)
```


# References

\name{getAll}
\Rdversion{1.1}
\alias{getAll}
\alias{getAll,data.frame-method}
\alias{getAll,pedigree-method}
\alias{getAll,pedigreeList-method}
\alias{getAll,FAData-method}
\alias{getExternalMatched}
\alias{getExternalMatched,data.frame-method}
\alias{getExternalMatched,pedigree-method}
\alias{getExternalMatched,pedigreeList-method}
\alias{getExternalMatched,FAData-method}
\alias{getGenerationMatched}
\alias{getGenerationMatched,data.frame-method}
\alias{getGenerationMatched,pedigree-method}
\alias{getGenerationMatched,pedigreeList-method}
\alias{getGenerationMatched,FAData-method}
\alias{getGenerationSexMatched}
\alias{getGenerationSexMatched,data.frame-method}
\alias{getGenerationSexMatched,pedigree-method}
\alias{getGenerationSexMatched,pedigreeList-method}
\alias{getGenerationSexMatched,FAData-method}
\alias{getSexMatched}
\alias{getSexMatched,data.frame-method}
\alias{getSexMatched,pedigree-method}
\alias{getSexMatched,pedigreeList-method}
\alias{getSexMatched,FAData-method}

\title{Define sets of control individuals for one or more given individuals}

\description{
  These functions allow to define sets of (eventually matched) control
  individuals for one or more given individuals using pedigree data. By
  default, controls from the same family than the specified
  individual(s) are returned.
  They are used e.g. in the \code{\link{genealogicalIndexTest}} method.
}
\usage{

\S4method{getAll}{data.frame}(object, id=NULL, ...)

\S4method{getExternalMatched}{data.frame}(object, id=NULL, match.using, ...)

\S4method{getGenerationMatched}{data.frame}(object, id=NULL,
                                           include.anc=0, include.off=0, ...)

\S4method{getGenerationSexMatched}{data.frame}(object, id=NULL,
                                               include.anc=0, include.off=0, ...)

\S4method{getSexMatched}{data.frame}(object, id=NULL, ...)

}
\arguments{

  (in alphabetic order)

  \item{id}{
    A character vector with ids of individuals for whom matched
    control individuals (from the same family) should be defined.
  }

  \item{include.anc}{
    For \code{getGenerationMatched} and \code{getGenerationSexMatched}:
    number of ancestor generations of individuals \code{id} in which
    control individuals should be defined, in addition to the actual
    generation of the individuals \code{id}.
    By default the functions define control individuals only in the
    actual generation of the individuals in \code{id}.
  }

  \item{include.off}{
    For \code{getGenerationMatched} and \code{getGenerationSexMatched}:
    number of offspring generations of individuals \code{id} in which
    control individuals should be defined, in addition to the actual
    generation of the individuals \code{id}.
    By default the functions define control individuals only in the
    actual generation of the individuals in \code{id}.
  }

  \item{match.using}{
    For \code{getExternalMatched}: a named vector for matching of
    controls. The names of the vector have to correspond to the ids of
    the individuals in the pedigree (but can have a different ordering).
  }

  \item{object}{
    Either a \code{data.frame}, \code{pedigree} or a \code{pedigreeList}
    object specifying the pedigree. If a \code{data.frame} is submitted,
    the columns \code{id}, \code{family}, \code{father}, \code{mother} and
    \code{sex} are required.
  }

  \item{...}{
    Additional arguments; not used at present.
  }

}
\section{Get matched controls}{

  \describe{

    \item{getAll}{
      Simply returns the ids of all individuals in the family
      (i.e. individuals with the same family id in the pedigree) of the
      specified ids in \code{id}.
      Returns a list with the ids (of type \code{character}) of the
      controls. The names of the list correspond to the family id.
    }

    \item{getExternalMatched}{
      Returns the ids of matched individuals from the same family for
      the specified ids in \code{id}.
      The \code{match.using} vector is for the matching, i.e. the
      function first extracts the values for the individuals in
      \code{id} from \code{match.using} and returns the ids of all
      individuals whose value in \code{match.using} matches the value of
      the individuals specified by \code{id}. Individuals with a missing
      value in \code{match.using} are excluded.
      Returns a list with the ids (of type \code{character}) of the
      controls. The names of the list correspond to the family id.
    }

    \item{getGenerationMatched}{
      Returns the ids of individuals matched by the generation of the
      individuals in \code{id}. The function returns the ids of all
      individuals from the same generation(s) than the individuals in
      \code{id}. The arguments \code{include.anc} and \code{include.off}
      can be used to increase the range of generation from which
      individuals are selected.
      Returns a list with the ids (of type \code{character}) of the
      controls. The names of the list correspond to the family id.
    }

    \item{getGenerationSexMatched}{
      Same as \code{getGenerationMatched}, but matches in addition
      individuals by sex (see \code{getSexMatched}).
      Returns a list with the ids (of type \code{character}) of the
      controls. The names of the list correspond to the family id.
    }

    \item{getSexMatched}{
      Returns ids of individuals from the same family matching the sex
      of the individuals in \code{id}. Individuals with sex being
      \code{NA} are excluded. The result is essentially identical to the
      \code{getAll} if \code{id} contains ids of male and female
      individuals.
      Returns a list with the ids (of type \code{character}) of the
      controls. The names of the list correspond to the family id.
    }

  }
}
\value{
  Refer to the method and function description above for detailed
  information on the returned result object.
}
\author{
  Johannes Rainer.
}
\seealso{
  \code{\link[kinship2]{pedigree}},
  \code{\linkS4class{FAData}},
  \code{\linkS4class{FAProbResults}},
  \code{\linkS4class{FAKinGroupResults}},
  \code{\linkS4class{FAKinSumResults}},
  \code{\linkS4class{FAGenIndexResults}},
  \code{\link{genealogicalIndexTest}}
}
\examples{
##########################
##
##  Defining a small pedigree
##
## load the Minnesota Breast Cancer record and subset to the
## first families.
data(minnbreast)
mbsub <- minnbreast[minnbreast$famid==4 | minnbreast$famid==5, ]
mbped <- mbsub[, c("famid", "id", "fatherid", "motherid", "sex")]
## renaming column names
colnames(mbped) <- c("family", "id", "father", "mother", "sex")

## Get ids of individuals from the same generation than "4"
getGenerationMatched(mbped, id="4")

## Get the ids of gemale individuals from the same generation than
## "4" and "22"
getGenerationSexMatched(mbped, id=c("4", "22"))

## Generate a FAData
fad <- FAData(mbped)

## Plot the pedigree so we can evaluate the results of the function
plotPed(fad, family="4")

## We're using getExternalMatched on the sex of the individuals
## in the pedigree, thus the results are identical to the
## getSexMatched function.
## Extracting the sex using $sex returns a named vector just
## as we need for getExternalMatched.
head(fad$sex)
getExternalMatched(fad, id="4", match.using=fad$sex)

getSexMatched(fad, id="4")

}
\keyword{classes}

\name{plotPed}
%\Rdversion{1.1}
%\docType{class}
\alias{doPlotPed}
\alias{switchPlotfun}

\title{Pedigree plotting functions}

\description{
  Plot a pedigree for a family or an individual.
}
\usage{

doPlotPed(family=NULL, individual=NULL, father=NULL, mother=NULL, gender=NULL,
        affected=NULL, is.deceased=NULL, is.sab.or.top=NULL, is.proband=NULL,
        is.adopted=NULL, are.twins=NULL, are.consanguineous=NULL,
        text.inside.symbol=NULL, text.beside.symbol=NULL,
        text1.below.symbol=NULL, text2.below.symbol=NULL,
        text3.below.symbol=NULL, text4.below.symbol=NULL,
        filename=NULL, device="plot", res=600, ...)

switchPlotfun(method, check=TRUE)

}
\arguments{

  \item{family}{
    (Optional) character or numeric vector specifying the family id.
  }

  \item{individual}{
    (Required) character or numeric vector with the ids of the
    individuals.
  }

  \item{father}{
    (Required) character or numeric vector with the id of the father for
    each individual.
  }

  \item{mother}{
    (Required) character or numeric vector with the id of the mother for
    each individual.
  }

  \item{gender}{
    (Required) character, factor or numeric vector specifying the
    gender, with \code{1} or any string starting with \code{"m"} coding
    for male and \code{2} or any string starting with \code{"f"} for
    male; \code{NA} codes for unknown.
  }

  \item{affected}{
    (Optional) numeric or logical vector specifying if the individual is
    affected, \code{0} or \code{FALSE} for not affected, \code{1} or
    \code{TRUE} for affected, \code{NA} for not phenotyped.
  }

  \item{is.deceased}{
    (Optional) numeric of logical vector specifying whether the
    individual is deceased.
  }

  \item{is.sab.or.top}{
    (Optional) numeric or logical vector specifying if the individual is
    the result of a spontaneous abortion or termination of pregnancy.
  }

  \item{is.proband}{
    (Optional) numeric or logical vector specifying whether the
    individual is declared as proband (i.e. the first affected family
    member coming to medical attention).
  }

  \item{is.adopted}{
    (Optional) numeric or logical vector specifying if the individual
    has been adopted.
  }

  \item{are.twins}{
    (Optional) character vector spefifying twins in the
    family. Individuals sharing the same string are recognized as
    twins. The string has to start either with \code{"m_"} or
    \code{"d_"} for monozygotic or dizygotic twins, followed by the
    unique identifier for the twins.
  }

  \item{are.consanguineous}{
    (Optional) character vector specifying cosanguineous couples.
  }

  \item{text.inside.symbol}{
    (Optional) character vector with text to place inside symbols.
  }

  \item{text.beside.symbol}{
    (Optional) character vector with text to place beside symbols.
  }

  \item{text1.below.symbol}{
    (Optional) character vector with text to place below symbols.
  }

  \item{text2.below.symbol}{
    (Optional) character vector with text to place below symbols.
  }

  \item{text3.below.symbol}{
    (Optional) character vector with text to place below symbols.
  }

  \item{text4.below.symbol}{
    (Optional) character vector with text to place below symbols.
  }

  \item{filename}{
    (Optional) file name for the plot. If not specified the result is
    plotted to a temporary file.
  }

  \item{device}{
    The format of the output file. Can be \code{"ps"}, \code{"pdf"},
    \code{"svg"}, \code{"png"} or \code{"txt"} if \code{HaploPainter} is
    used to create the plot, or \code{"pdf"}, \code{"png"} or \code{"plot"}
    if \code{kinship2} is used for plotting. If the \code{HaploPainter}
    backend is not installed, it is still possible to produce
    \code{HaploPainter} input files using \code{devive = "txt"} for later
    invocation of \code{HaploPainter}: this is achieved by calling
    \code{switchPlotfun("haplopaint", check = FALSE)}, which will not check
    for the presence of a \code{HaploPainter} executable. Note: if
    \code{"plot"} is specified the plot is displayed instead of exported to
    a file.
  }

  \item{res}{
    (Optional) the resolution of the image if saved to a bitmap device.
  }

  \item{method}{
    The method which should be used for plotting, either
    \code{"ks2paint"} (uses \code{kinship2} for plotting) or
    \code{"haplopaint"} (uses \code{HaploPaint}). If not specified, the
    functions switches between the methods.
  }

  \item{check}{
    A logical indicating whether the plotting backends (currently
    applied only to \code{HaploPaint}) is installed and working. Defaults
    to \code{TRUE}, such that it is guaranteed that a call to
    \code{doPlotPed} will at least technically succeed. The test is omitted
    by setting this argument to \code{FALSE}.
  }

  \item{...}{
    For \code{plotPed}: additional arguments submitted to the plotting
    function \code{doPlotPed}.
  }


}
\details{
  %% The \code{plotPed} method is simply a wrapper for the \code{doPlotPed}
  %% function, thus all arguments can be used in both. Note that the
  %% \code{plotPed} method for \code{\linkS4class{FAData}} and
  %% \code{\linkS4class{FAResult}} objects are different, i.e. do not
  %% require all the input arguments above, but use the data stored within
  %% the objects to draw the pedigree.

  All arguments passed to the \code{doPlotPed} function have to have the
  same length (with the exception of arguments \code{filename},
  \code{device} and \code{res}) and their order has to
  match the order of the individuals.

  The arguments of the \code{doPlotPed} function represent the input
  parameters supported by \code{HaploPainter}; for more information
  about \code{HaploPainter} refer to http://haplopainter.sourceforge.net/.

  By default, \code{doPlotPed} uses the \code{kinship2} package for
  plotting, but can also be configured by the \code{switchPlotFun} to
  use \code{HaploPainter} instead.
  \code{HaploPainter} is a perl script/tool for pedigree plotting bundled in
  the package that requires however some dependencies that might not be
  present on every system. Thus, the package checks on startup whether
  all requirements for \code{HaploPainter} are available. This check can
  be skipped by using \code{check=FALSE} when calling \code{switchPlotFun}.
  While using this argument is generally not recommended, it is of use
  when only writing \code{HaploPainter} input files, which does not make
  use of the \code{HaploPainter} plotting backend.

  If \code{HaploPainter} is used, the plot can only be exported to a pdf
  or png device, while, if \code{kinship2} is used, the plot can also be
  directly plotted and displayed (if \code{device="plot"} is specified).

  \code{HaploPainter} plotting supports also \code{device = "txt"} in
  which case the pedigree data are exported (in the HaploPainter file
  format) as a tabulator delimited file - no plot is created, the
  name of the file is returned. This can even be done without a
  \code{HaploPainter} executable by calling
  \code{switchPlotFun("haplopaint", check=FALSE)}.

  Also, the arguments of this function match the arguments for
  \code{HaploPainter} and not all settings can be directly matched to
  settings in \code{kinship2} plotting. The list below lists all
  arguments specific to \code{HaploPainter} and how, if at all, they are
  displayed or used in \code{kinship2} plotting:
  \describe{
    \item{is.sab.or.top}{
      Not supported yet.
    }
    \item{is.proband}{
      The id of individuals which are marked as probands are highlighted
      in red.
    }
    \item{is.adopted}{
      Not supported yet.
    }
    \item{are.twins}{
      Not supported yet.
    }
    \item{are.consanguineous}{
      Not supported yet.
    }
    \item{text2.below.symbol}{
      The text will be plotted on the top left corner of the symbol of
      the respective individual.
    }
    \item{text3.below.symbol}{
      The text will be plotted on the top right corner of the symbol of
      the respective individual.
    }
  }
}
\value{
  \describe{
    \item{switchMethod}{
      A character string representing the plotting function to be used.
    }

    %% \item{plotPed}{
    %%   The name of the file to which the plot was exported.
    %% }

    \item{doPlotPed}{
      The name of the file to which the plot was exported.
    }
  }
}
\author{
  Johannes Rainer.
}
\seealso{
  \code{\link[kinship2]{plot.pedigree}},
  \code{\link{plotPed,FAData-method}},
}
\examples{

## load the Minnesota Breast Cancer record and subset to the
## first families.
data(minnbreast)
family <- minnbreast[minnbreast$famid==4, ]

## draw a pedigree and export it to a pdf file; the file name is
## returned by the function.
doPlotPed(family=family$famid, individual=family$id, father=family$fatherid,
          mother=family$motherid, gender=family$sex, device="pdf")

## switch to the plotting functions of the kinship2 package
switchPlotfun("ks2paint")

## plot the same pedigree, but display it
doPlotPed(family=family$famid, individual=family$id, father=family$fatherid,
          mother=family$motherid, gender=family$sex, device="plot")

## Finally, generate an input file that can be used for interactive or
## scripted HaploPainter pedigree drawing.
switchPlotfun("haplopaint", check=FALSE)
doPlotPed(family=family$famid, individual=family$id, father=family$fatherid,
          mother=family$motherid, gender=family$sex, device="txt",
          filename="haplopainter.tsv")

}
\keyword{plot}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/utils.R
\name{kinshipPairs}
\alias{kinshipPairs}
\title{Extract pairs of individuals matching certain kinship criteria}
\usage{
kinshipPairs(
  x,
  condition = function(x) x >= 0.25,
  duplicates = c("keep", "first", "last", "random"),
  id = NULL,
  family = NULL
)
}
\arguments{
\item{x}{A \code{FAData} object (or object inheriting from that).}

\item{condition}{A \code{function} defining how individuals should be selected
based on the object's kinship matrix. The default is to select all
individuals with a kinship \verb{>= 0.25}. Note that the diagonal of the
kinship matrix (i.e. the kinship of individuals with itself) is always
skipped, so no additional criteria is needed to avoid self-pairs.}

\item{duplicates}{\code{character(1)} defining how to deal with duplicated IDs
in the result returned by the function.
See function description and examples below for more details. Defaults
to \code{duplicates = "keep"} returning all pairs of IDs matching \code{condition}.}

\item{id}{optional identifiers of subsets of individuals on which the pairs
should be defined. Defaults to \code{id = NULL} hence the full data set is
considered.}

\item{family}{optional family identifiers if pairs should only defined for
selected families. Defaults to \code{family = NULL} hence the full data set is
considered.}
}
\value{
A two column \code{matrix} with the IDs (colnames/rownames of the kinship matrix
or as defined in \code{x$id}) of the pairs. If \code{duplicates} is either \code{"first"},
\code{"last"} or \code{"random"} each ID is only returned once (i.e. no ID is reported
more than one time).
}
\description{
The \code{kinshipPairs} function allows to extract pairs of individuals matching
a user-defined kinship \emph{condition} (e.g. individuals with a kinship larger
than 0.0625). Such sets of paired individuals (along with paired unrelated
values) would enable a \emph{familial resemblance} analysis on quantitative
traits (Ziegler 2010) (see examples below for details).

By default, \code{kinshipPairs} returns all pairs of individuals for which the
\code{condition} on the kinship matrix matches (e.g. all pairs of individuals with
a kinship coefficient larger than or equal to 0.25). Individuals can thus
be reported multiple times (see examples below). Parameter \code{duplicates} can
be used to define a strategy to avoid such duplicated IDs. Supported are:
\itemize{
\item \code{duplicates = "keep"}: the default, return all values.
\item \code{duplicates = "first"}: report only the first pair of individuals for each
individual ID.
\item \code{duplicates = "last"}: report only the last pair of individuals for each
individual ID.
\item \code{duplicates = "random"}: randomly select one pair of individuals for
each individual ID.
}

For any setting different than \code{duplicates = "keep"} each individual will
only be listed \strong{once} in the resulting matrix.
}
\examples{

##########################
##
##  Create a new FAData object
##
## Load the Minnesota Breast Cancer record and subset to the
## first families.
data(minnbreast)
mbsub <- minnbreast[minnbreast$famid \%in\% 1:20, ]
mbped <- mbsub[, c("famid", "id", "fatherid", "motherid", "sex")]
## Renaming column names
colnames(mbped) <- c("family", "id", "father", "mother", "sex")
## Defining the optional argument age.
Age <- mbsub$endage
names(Age) <- mbsub$id
## Create the object
fad <- FAData(pedigree=mbped, age=Age)

## Getting all pairs of individuals with a kinship coefficient >= 0.25
## keeping all duplicates
rel_pairs <- kinshipPairs(fad)
head(rel_pairs)
## As we see, we have multiple times the individual 1 etc.

## For an actual correlation analysis it would be better to drop duplicates.
## Below we randomly select individual pairs if they occurr multiple times
rel_pairs <- kinshipPairs(fad, duplicates = "random")
head(rel_pairs)

## In addition we extract pairs of individuals that are much less related.
## For this examples we consider all individuals with a kinship
## coefficient < 0.03125 (second cousin) to be *unrelated*.
unrel_pairs <- kinshipPairs(fad, duplicates = "random",
    condition = function(z) z < 0.03125)
head(unrel_pairs)

## For a familial resemblance analysis we can now calculate the correlation
## coefficient of a quantitative trait between pairs of related individuals
## and compare that with the correlation coefficient calculated on unrelated
## individuals. For our toy example we use the participant's age, since we
## don't have any other quantitative values available.
cor_rel <- cor(age(fad)[rel_pairs[, 1]], age(fad)[rel_pairs[, 2]],
    use = "pairwise.complete.obs")

cor_unrel <- cor(age(fad)[unrel_pairs[, 1]], age(fad)[unrel_pairs[, 2]],
    use = "pairwise.complete.obs")
cor_rel
cor_unrel

## We don't see a clear difference in the correlation, thus, the age (as
## expected) has no familial component.
}
\references{
Ziegler A., Koenig I. R. (2010). Familiality, Heristability, and Segregation
Analysis. In A Statistical Approach to Genetic Epidemiology: With Access to
E-Learning Platform by Friedrich Pahlke, Second Edition.
\doi{10.1002/9783527633654.ch6}.
}
\seealso{
\link{PedigreeUtils} for other pedigree utility functions.
}
\author{
Johannes Rainer
}

\name{FAData-class}
\Rdversion{1.1}
\docType{class}
\alias{FAData-class}
\alias{FAData}
\alias{$}
\alias{$,FAData-method}
\alias{[,FAData,ANY-method}
\alias{[,FAData,ANY,ANY,ANY-method}
\alias{affectedIndividuals}
\alias{affectedIndividuals,FAData-method}
\alias{age}
\alias{age,FAData-method}
\alias{age<-}
\alias{age<-,FAData-method}
\alias{buildPed}
\alias{buildPed,FAData-method}
\alias{export}
\alias{export,FAData-method}
\alias{family}
\alias{family,FAData-method}
\alias{kinship}
\alias{kinship,FAData-method}
\alias{pedigree}
\alias{pedigree,FAData-method}
\alias{pedigree,missing-method}
\alias{pedigree<-}
\alias{pedigree<-,FAData-method}
\alias{pedigreeSize}
\alias{pedigreeSize,FAData-method}
\alias{phenotypedIndividuals}
\alias{phenotypedIndividuals,FAData-method}
\alias{plotPed}
\alias{plotPed,FAData-method}
\alias{show}
\alias{show,FAData-method}
\alias{trait}
\alias{trait,FAData-method}
\alias{trait<-}
\alias{trait<-,FAData-method}

\title{Pedigree data information}

\description{
  \code{FAData} objects conveniently store pedigree along with trait
  information. This object is the central data structure from the
  \code{FamAgg} package. Basic usage pedigree analysis methods are
  described on this page and on the \code{\link{PedigreeUtils}} help
  page, familial aggregation analysis methods on the
  \code{\link{PedigreeAnalysis}} help page.

  See the section about the \code{pedigree} \code{data.frame} below for
  a detailed description of the encoding of missing trait data or
  founder individuals in \code{FamAgg}.
}

\section{Objects from the Class}{
  \code{FAData} objects are created by the constructor function
  \code{FAData} and should not be directly created by a call to
  \code{new}.
}
\usage{

\S4method{affectedIndividuals}{FAData}(object)

\S4method{age}{FAData}(object)

\S4method{age}{FAData}(object) <- value

\S4method{buildPed}{FAData}(object, id=NULL, family = NULL, max.generations.up=3,
                            max.generations.down=16, prune=FALSE, ...)

\S4method{export}{FAData}(object, con, format="ped", ...)

FAData(pedigree, age, trait, traitName, header=FALSE, sep="\t", id.col="id",
       family.col="family", father.col="father", mother.col="mother",
       sex.col="sex")

\S4method{family}{FAData}(object, id=NULL, family=NULL,
                          return.type="data.frame")

\S4method{kinship}{FAData}(id, ...)

\S4method{pedigree}{FAData}(object, return.type="data.frame")

\S4method{pedigree}{FAData}(object) <- value

\S4method{pedigreeSize}{FAData}(object)

\S4method{phenotypedIndividuals}{FAData}(object)

\S4method{plotPed}{FAData}(object, id=NULL, family=NULL, filename=NULL,
                           device="plot", symbol.related=NA,
                           proband.id=NULL, highlight.ids=NULL,
                           only.phenotyped=FALSE,
                           label1=age(object), label2=NULL, label3=NULL,
                           ...)

\S4method{show}{FAData}(object)

\S4method{trait}{FAData}(object, na.rm=FALSE)

\S4method{trait}{FAData}(object) <- value

}
\arguments{

  (in alphabetic order)

  \item{age}{
    For \code{FAData}: either a character(1) specifying the file name
    from which the age should be read or a named numeric vector of ages
    with the names corresponding to the ids of the individuals in the
    pedigree.
  }

  \item{con}{
    For \code{export}: the file name or connection to a file to which
    the pedigree information should be exported.
  }

  \item{device}{
    For \code{plotPed}: the device of file format in which the plot
    should be saved. See details for allowed values.
  }

  \item{family}{
    For \code{buildPed}: the id of the family for which the pedigree
    should be returned.
    For \code{family}: the id of the family for which the pedigree
    should be returned (full pedigree of the family).
    For \code{plotPed}: the id of the family for which the pedigree
    should be plotted.
  }

  \item{family.col}{
    For \code{FAData}: the name of the column containing the id of the
    families.
  }

  \item{father.col}{
    For \code{FAData}: the name of the column containing the id of the
    father.
  }

  \item{filename}{
    For \code{plotPed}: a character string specifying the name of the
    file to which the plot should be saved. If none is submitted, the
    plot is saved to a temporary file.
  }

  \item{format}{
    For \code{export}: the format in which the pedigree should be
    exported. At present only \code{"ped"} and \code{"fam"} are
    exported, i.e. the file formats from plink
    (\url{http://pngu.mgh.harvard.edu/~purcell/plink/data.shtml}).
  }

  \item{header}{
    For \code{FAData}: only used if argument \code{pedigree} is a
    \code{character(1)}, i.e. the file name from which the pedigree
    should be read. The \code{header} argument is passed to the
    \code{read.table} function, i.e. should be set to \code{TRUE} if the
    file contains column headers.
  }

  \item{highlight.ids}{
    A list of character vector(s) of ids that should be labeled. The
    name(s) of the character vector(s) is/are used as the text to label
    the individuals (the text is shown below the symbol of the
    individuals). Up to 3 character vectors are
    supported. Alternatively, a single character vector of ids can be
    submitted in which case the individuals are labeled with an asterisc
    (\code{"*"}).
  }

  \item{id}{
    For method \code{kinship}: the \code{FAData} object from which
    the kinship matrix should be extracted, for all other methods the id
    of the individual.

    For method \code{plotPed}: the id of the individual for which the
    pedigree should be built (see \code{buildPed}) and plotted.

    Note: \code{id} can be a \code{numeric} or a
    \code{character}. Numeric ids are internally converted to
    \code{character}.
  }

  \item{id.col}{
    For \code{FAData}: the name of the column containing the id of the
    individuals.
  }

  \item{label1}{
    For \code{plotPed}: labels that should be plotted below the symbol
    for each individual. Should be either a named vector with names
    corresponding to the ids of the individuals in the pedigree or a
    vector of the same length than individuals that are to be
    plotted. For the former it is sufficient to just specify the labels
    for the individuals that should be shown.
  }

  \item{label2}{
    For \code{plotPed}: see \code{label1}. The labels are plotted in the
    second line below the symbol if \code{HaploPainter} is used to
    generate the plot, or on the top left corner of the individual's
    symbol for \code{kinship2} plotting.
  }

  \item{label3}{
    For \code{plotPed}: see \code{label1}. The labels are plotted in the
    third line below the symbol if \code{HaploPainter} is used to
    generate the plot, or on the top right corner of the individual's
    symbol for \code{kinship2} plotting.
  }

  \item{max.generations.down}{
    For \code{buildPed}: the maximal number of generations to look for
    children.
  }

  \item{max.generations.up}{
    For \code{buildPed}: the maximal number of generations to look for
    ancestors.
  }

  \item{mother.col}{
    For \code{FAData}: the name of the column containing the id of the
    mother.
  }

  \item{na.rm}{
    For \code{trait}: whether missing values in trait should be returned
    or not.
  }

  \item{object}{
    The \code{FAData} object.
  }

  \item{only.phenotyped}{
    Wheter only phenotyped individuals, i.e. individuals with a non-NA
    value in column \code{affected} (the trait information). Requires
    this information to be present.
  }

  \item{pedigree}{
    For \code{FAData}: either a \code{data.frame} with the pedigree
    information or a \code{character(1)} specifying the file name from
    which the pedigree should be read. See description below for more
    details.
  }

  \item{proband.id}{
    For \code{plotPed}: character vector with the id(s) of one ore more
    individuals that should be highlighted as probands.
    \code{HaploPainter} indicates probands with a \code{"P"} next to the
    symbol and an arrow pointing to the symbol.
  }

  \item{prune}{
    For \code{buildPed}: whether the smallest possible (connected)
    pedigree for the submitted ids should be build. This makes only
    sense if more than one id is submitted.
  }

  \item{return.type}{
    Either \code{"data.frame"} or \code{"pedigree"} if the pedigree
    information should be returned as a \code{data.frame} or
    \code{pedigreeList} object as defined in the \code{kinship2}
    package.
  }

  \item{sep}{
    For \code{FAData}: only used if argument \code{pedigree} is a
    \code{character(1)}, i.e. the file name from which the pedigree
    should be read. The \code{sep} argument is passed to the
    \code{read.table} function and specifies the field separator.
  }

  \item{sex.col}{
    For \code{FAData}: the name of the column spefifying the sex of the
    individuals.
  }

  \item{symbol.related}{
    For \code{plotPed}: the symbol which should be used to label
    individuals sharing kinship with the id for which the pedigree is
    generated and plotted.
  }

  \item{trait}{
    For \code{FAData}: a numeric vector with \code{0}, \code{1} and
    \code{NA} or a logical vector indicating unaffected (but
    phenotyped), affected and not phenotyped individuals.
  }

  \item{traitName}{
    For \code{FAData}: an optional name for the trait.
  }

  \item{value}{
    For \code{age<-}: a named numeric vector. The names (at least some
    of them) have to match the ids in the pedigree of the object.

    For \code{pedigree<-}:

    For \code{trait<-}: a named numeric vector with \code{0}, \code{1}
    and \code{NA} or a logical vector with \code{FALSE}, \code{TRUE},
    \code{NA} for not affected, affected and not tested. The names (at
    least some of them) have to match the ids in the pedigree of the
    object.

  }

  \item{...}{
    Additional arguments to be passed to the plotting functions
    (\code{\link{doPlotPed}} for \code{plotPed}).
  }

}
\section{Slots}{
  \describe{
    \item{age}{
      A (named) numerical vector with the age of the individuals. It is
      suggested to use the getter and setter methods described below to
      access this slot.
    }

    \item{pedigree}{
      A \code{data.frame} with the pedigree. It is suggested to use the
      getter and setter methods described below to access this slot.
    }

    \item{.kinship}{
      The kinship matrix for the kinship of each individual in the
      pedigree with each other. This slot should not be accessed
      directly, but the \code{kinship} method should be used
      instead.
    }

    \item{traitname}{
      The name of the trait being stored in the object.
    }

    \item{.trait}{
      A numerical vector with the trait information, \code{0}, \code{1},
      \code{NA}, for phenotyped but not affected, affected and not
      tested, respectively. This slot should not be accessed directly,
      but the \code{trait} and \code{trait<-} methods should be used
      instead that ensure that the data is matched to the information in
      the pedigree.
    }

  }
}

\section{Constructors, importing and exporting data}{
  \describe{
    \item{FAData}{
      Constructor function to create a new \code{FAData} instance. In
      addition to submitting the pedigree information as
      \code{data.frame}, \code{pedigree} or \code{pedigreeLinst} it is
      possible to specify the name of the file from which the pedigree
      information should be read.
      The recognizes and imports plink \emph{ped} and \code{fam} files
      (\url{http://pngu.mgh.harvard.edu/~purcell/plink/data.shtml}) or from
      generic text files.
      For the latter, arguments
      \code{header}, \code{sep}, \code{family.col}, \code{id.col},
      \code{father.col}, \code{mother.col} and \code{sex.col} allow to
      further specify which columns of the file contain what information
      etc.
      If argument \code{pedigree} is a \code{data.frame}, the column
      names \code{"family"}, \code{"id"}, \code{"father"},
      \code{"mother"} and \code{"sex"} are expected. Any additional
      columns are dropped.

      The sex is expected to be encoded either as a numeric 1 (male),
      2 (female) with any other number or \code{NA} representing
      unknown, or as a character vector or factor with \code{"M"},
      \code{"m"}, \code{"Male"} or \code{"male"} for male and \code{"F"},
      \code{"f"}, \code{"Female"} or \code{"female"} for female.
    }

    \item{export}{
      Export pedigree data to a file.
    }
  }
}

\section{Accessors and subsetting}{
  \describe{

    \item{object$name}{
      Access \code{name} column in the pedigree of the \code{FAData}
      object. The function returns a named vector wirh the names
      corresponding to the ids of the individuals or \code{NULL} if
      \code{name} does not correspoind to a column name in the pedigree.
      The trait data can be accessed either by \code{object$trait} or
      \code{object$affected}.
    }

    \item{age}{
      Returns the age of the individuals as a named numeric vector.
      If the pedigree is set, the order of the values corresponds always
      to the ordering of the individuals in the pedigree with \code{NA}
      for individuals for which the age is unknown. In case the age was
      never set it returns a vector of NAs with length equal to the
      number of individuals.
    }

    \item{age<-}{
      Setter for the age. Value has to be a named numeric vector.
    }

    \item{pedigree}{
      Returns the pedigree either as a \code{data.frame} or a
      \code{pedigreeList} object (defined in the \code{kinship2}
      package) depending on the value of the parameter
      \code{return.type} (i.e. either \code{return.type="data.frame"} or
      \code{return.type="pedigree"}). If \code{pedigree} is called on
      any other object than a \code{FAData} object (or any object
      that inherits from that object), the
      \code{\link[kinship2]{pedigree}} method from the
      \code{kinship2} package is called.

      For the default return type (i.e. \code{return.type="data.frame"})
      a \code{data.frame} is returned with the following columns:
      \code{"family"}: the ID of the family,
      \code{"id"}: the ID of the individual,
      \code{"father"}: the ID of the individual's father. Founder
      individuals, i.e. individuals for whom the father and mother is
      not known in the data set, contain a \code{NA} in this column.
      \code{"mother"}: the ID of the individual's father. Founder
      individuals, i.e. individuals for whom the father and mother is
      not known in the data set, contain a \code{NA} in this column.
      \code{"sex"}: the sex of the individuals encoded as a
      \code{factor} with levels \code{"M"} and \code{"F"} for male and
      female, or \code{NA} for not known.
      If trait information is available in the object the returned
      \code{data.frame} will also contain a column named \code{affected}
      with the information whether the individual is affected
      (\code{1}), not affected (\code{0}) or was not tested/phenotyped
      \code{NA}.
    }

    \item{pedigree<-}{
      Setter for the pedigree slow. Value can be a \code{data.frame}
      with columns containing the family id, individual id, father id,
      mother id and sex (in this order) or a \code{pedigree} or
      \code{pedigreeList} object as defined in the \code{kinship2}
      package.
    }

    \item{object[i, ]}{
      Subsets the \code{FAData} object to individuals specified with
      \code{i} which can be a logical, numeric or character vector. For
      the latter, the elements have to be the ids of the individuals
      (i.e. rownames of \code{pedigree(object)}). Returns the sub-setted
      object.
      Note that subsetting other than by family might result in a
      non-valid pedigree (e.g. if mother or father ID are not available
      in the sub-setted pedigree).
    }

    \item{trait}{
      Get the trait vector from the object. By default, the ordering
      is the same as pedigree, setting argument \code{na.rm=TRUE}
      removes all \code{NA} values, thus the ordering and length might
      be different. Returns a named vector with the names corresponding
      to the ids of the individuals.
    }

    \item{trait<-}{
      Setter for the trait slot. Can be a named numeric vector (values
      \code{0}, \code{1} and \code{NA}) or logical vector (values
      \code{FALSE}, \code{TRUE} and \code{NA}) with the names matching
      the ids of the individuals in the pedigree. The method internally
      matches and re-orders the trait vector to match the ordering of
      the ids in the pedigree.
    }

  }
}

\section{Basic usage}{
  \describe{

    \item{affectedIndividuals}{
      Returns a character vector with the ids of the affected
      individuals, i.e. the id of the individuals with a value other
      than 0 or \code{NA} in the trait.
      If no trait data is available the method returns \code{NULL}.
    }

    \item{buildPed}{
      Builds a pedigree for the specified id(s) containing generations
      defined by \code{max.generations.up} and
      \code{max.generations.down} and returns it as a
      \code{data.frame}. The pedigree contains all individuals in the
      family sharing kinship with the input individual(s) and mates
      needed to complete the pedigree.
      For \code{prune=TRUE} the function tries to
      find the smallest connected pedigree for all the submitted ids.
    }

    \item{family}{
      Returns the pedigree for a full family. In contrast to
      \code{buildPed} which constructs a (sub)pedigree for a specific
      individual, this method returns the pedigree of the complete
      family for an individual (if \code{id} is specified). The function
      returns either a \code{data.frame} or a \code{pedigreeList} with
      the pedigree for the family.
    }

    \item{kinship}{
      Extracts the pre-calculated kinship matrix, i.e. a symmetric
      matrix with the kinship between all individuals in the
      pedigree. The matrix is calculated using the \code{kinship} method
      provided by the \code{kinship2} package [Sinwell (2014)].
      The function returns a \code{dsCMatrix} from the \code{Matrix}
      package.
    }

    \item{pedigreeSize}{
      Returns the size, i.e. the number of individuals (rows) in the
      pedigree.
    }

    \item{phenotypedIndividuals}{
      Returns a character vector with the ids of the phenotyped
      individuals, i.e. the id of all individuals that have a non-NA
      value in thetrait.
      If no trait data is available the method returns \code{NULL}.
    }

    \item{plotPed}{
      Creates the pedigree for the submitted id(s) or family and plots
      it (i.e. saves it to the specified file). See details above for
      more information.
      Returns the file name of the file to which the pedigree plot was
      exported or \code{NULL} for \code{kinship2} plotting and
      \code{device="plot"}.

      For \code{HaploPainter} plotting and \code{device = "txt"} the name
      of the file to which the plotting data has been exported is returned.

      See \code{\link{doPlotPed}} for more information.
    }

  }
}
\section{Pedigree analysis methods}{
  Methods for familial aggregation and other pedigree analysis methods
  are described on the \link{PedigreeAnalysis} help page.
}

\section{Pedigree utilities}{
  A variety of different pedigree utilities are defined for
  \code{FAData} objects. For the full list of methods see the
  \link{PedigreeUtils} help page.
}

\details{

  See sections below for a description of the individual methods.

  The \code{buildPed} method is a combination of the methods
  \code{getAncestors}, \code{getChildren} and \code{getMissingMate},
  i.e. it first gets all ancestors for the specified id(s), determines
  then the children of all of the ids (submitted ids and their
  ancestors) and at last looks for any missing mates/spouses to complete
  the pedigree.

  The \code{plotPed} function uses either the external perl program
  \code{HaploPainter} or the plotting capabilities of the
  \code{kinship2} package.
  With \code{HaploPainter}, as it is an external too, it is not possible
  to display the plot directly, but each plot is automatically saved to
  a file (either \code{"pdf"}, \code{"ps"}, \code{"svg"} or
  \code{"png"}; can be specified with the \code{device}
  parameter). \code{HaploPainter} plotting supports also
  \code{device = "txt"} in which case the data table is exported (in the format
  expected by \code{HaploPainter}) to a tabulator delimited text file
  and the name of this text file is returned - no plot is created.
  Plotting with \code{kinship2} (the default) allows to display the plot
  (\code{device="plot"}) or export it to a file (\code{device="pdf"} or
  \code{device="png"}).

  The \code{\link{switchPlotfun}} function can be used to change the plotting
  system.

}
\note{
  The ids of individuals, father, mother and family can be either
  numeric or characters, internally, all ids will however be handled as
  characters.

  The \code{pedigree<-} setter method removes all white spaces in
  columns \code{"id"}, \code{"family"}, \code{"father"} and
  \code{"mother"} of the pedigree.
}
\value{
  Refer to the method and function description above for detailed
  information on the returned result object.
}
\author{
  Johannes Rainer.
}
\references{
  Sinwell JP, Therneau TM & Schaid DJ (2014) The kinship2 R package for
  pedigree data. \emph{Human heredity} 78:91-93.
}
\seealso{
  \code{\link[kinship2]{pedigree}},
  \code{\linkS4class{FAProbResults}},
  \code{\linkS4class{FAKinGroupResults}},
  \code{\linkS4class{FAKinSumResults}},
  \code{\linkS4class{FAGenIndexResults}},
  \code{\link{doPlotPed}},
  \code{\link{PedigreeUtils}},
  \code{\link{getAll}},
  \code{\link{PedigreeAnalysis}}
}
\examples{
##########################
##
##  Create a new FAData object
##
## Load the Minnesota Breast Cancer record and subset to the
## first families.
data(minnbreast)
mbsub <- minnbreast[minnbreast$famid==4 | minnbreast$famid==5, ]
mbped <- mbsub[, c("famid", "id", "fatherid", "motherid", "sex")]
## Renaming column names
colnames(mbped) <- c("family", "id", "father", "mother", "sex")
## Defining the optional argument age.
Age <- mbsub$endage
names(Age) <- mbsub$id
## Create the object
fad <- FAData(pedigree=mbped, age=Age)

fad

## Extract the ids directly...
head(fad$id)

## Extract the kinship matrix
dim(kinship(fad))

## What's the size of the pedigree?
pedigreeSize(fad)

## Importing a "ped" file.
pedFile <- system.file("txt/minnbreastsub.ped.gz", package="FamAgg")
## Quick glance at the file.
readLines(pedFile, n=1)
fad <- FAData(pedFile)

head(pedigree(fad))

## Creating the FAData reading data from a txt file.
pedFile <- system.file("txt/minnbreastsub.txt", package="FamAgg")
fad <- FAData(pedigree=pedFile, header=TRUE, id.col="id",
              family.col="famid", father.col="fatherid",
              mother.col="motherid")
## Adding the age
age(fad) <- Age
fad
## List all families in the pedigree along with the number of
## individuals
table(fad$family)

##########################
##
##  Basic usage
##
## Extracting the pedigree information
ped <- pedigree(fad)
## By default the pedigree is returned as a data.frame.
class(ped)
head(ped)

## In addition, we can extract the pedigree as a pedigreeList
pedigree(fad, return.type="pedigree")

## Return the ids of all ancestors of individual 6
## up to 3 generations
getAncestors(fad, id="6")

## Build the pedigree for individual 6: this includes all of its
## children and all of its ancestors up to the maximal number of
## specified generations.
buildPed(fad, id=6)
## Which is a sub-pedigree of the complete family:
family(fad, id=6)

## In addition we can specify manually some ids in the pedigree and
## generate the smallest possible pedigree containing all ids:
buildPed(fad, id=c(6, 23, 28), prune=TRUE)

## Get the list of all ids sharing kinship with individuals
## 5 and 9
shareKinship(fad, id=c("5", "9"))

## Subset the fad to family "4"
subFad <- fad[fad$family == "4", ]
subFad

## Export the pedigree from this family to a ped file
tmpFile <- tempfile()
export(subFad, con=tmpFile, format="ped")

head(read.table(tmpFile, sep="\t"))

##########################
##
##  Plotting
##
## Plot the pedigree for individual 6.
plotPed(fad, id=6)

## Alternatively, exporte it to a temporary file
pfile <- plotPed(fad, id=6, device="pdf")
pfile

## Highlighting some of the individuals:
## first get to know which other individuals are in the pedigree
plotPed(fad, id=6, highlight.ids=list(hello=c(1, 2, 4)))


##########################
##
##  Adding trait data
##
fad <- FAData(pedigree=mbped, age=Age)
tcancer <- mbsub$cancer
names(tcancer) <- mbsub$id
trait(fad) <- tcancer
## Now we can plot the pedigree also showing the affected status.
plotPed(fad, id=6)


## Alternatively, create the FAData with the trait data
fad <- FAData(pedigree=mbped, trait=mbsub$cancer, traitName="cancer")
plotPed(fad, id=6)

}
\keyword{classes}

\name{FAKinGroupResults-class}
\Rdversion{1.1}
\docType{class}
\alias{FAKinGroupResults-class}
\alias{FAKinGroupResults}
\alias{[,FAKinGroupResults,ANY-method}
\alias{[,FAKinGroupResults,ANY,ANY,ANY-method}
\alias{affectedKinshipGroups}
\alias{affectedKinshipGroups,FAKinGroupResults-method}
\alias{buildPed,FAKinGroupResults-method}
%\alias{independentGroupCount}
%\alias{independentGroupCount,FAKinGroupResults-method}
\alias{plotPed,FAKinGroupResults-method}
\alias{plotRes,FAKinGroupResults-method}
\alias{result,FAKinGroupResults-method}
\alias{runSimulation,FAKinGroupResults-method}
\alias{shareKinship,FAKinGroupResults-method}
\alias{trait<-,FAKinGroupResults-method}


\title{Kinship group test}

\description{
  The \code{FAKinGroupResults} object contains the results from the
  \emph{kinship test}. This test performs a familial aggregation
  analysis on a subset of individuals within a family. Two actual tests
  are conducted, a \emph{ratio test} that evaluates whether the number
  of affected individuals within the group is higher than expected by
  chance, and a \emph{kinship test} that compares the largest kinship
  value between affected in the group to the one between randomly
  sampled individuals.

  For more details see \code{\link{kinshipGroupTest}}.
}
\section{Objects from the Class}{
  \code{FAKinGroupResults} objects are created by the
  \code{\link{kinshipGroupTest}} method for \code{\linkS4class{FAData}}
  objects.
}
\usage{

\S4method{affectedKinshipGroups}{FAKinGroupResults}(object)

\S4method{buildPed}{FAKinGroupResults}(object, id=NULL, max.generations.up=3,
                                     max.generations.down=16, prune=FALSE)

% \S4method{independentGroupCount}{FAKinGroupResults}(object, method="jo")

\S4method{plotPed}{FAKinGroupResults}(object, id=NULL, family=NULL,
                                    filename=NULL, device="plot", ...)

\S4method{plotRes}{FAKinGroupResults}(object, id=NULL, family=NULL,
                                    addLegend=TRUE, type="density", ...)

\S4method{result}{FAKinGroupResults}(object, method="BH")

\S4method{runSimulation}{FAKinGroupResults}(object, nsim=50000, strata=NULL)

\S4method{shareKinship}{FAKinGroupResults}(object, id=NULL, rmKinship=0)

\S4method{trait}{FAKinGroupResults}(object) <- value

}
\arguments{

  (in alphabetic order)

  \item{addLegend}{
    For \code{plotRes}: if a legend should be added to the plot.
  }

  \item{device}{
    For \code{plotPed}: see \code{\link{plotPed}} for more details.
  }

  \item{family}{
    For \code{plotPed}: not supported.
  }

  \item{filename}{
    For \code{plotPed}: the file name to which the pedigree plot should
    be exported. See \code{\link{plotPed}} for more details.
  }

  \item{id}{
    For \code{buildPed}, \code{plotPed}, \code{plotRes} and
    \code{shareKinship}: the id of the kinship group (i.e. one of the
    ids in column \code{"group_id"} of the result table \code{result(object)}).

    Note: \code{id} can be a \code{numeric} or a
    \code{character}. Numeric ids will be internally converted to
    \code{character}.
  }

  \item{max.generations.down}{
    For \code{buildPed}: the maximal number of generations to look for
    children.
  }

  \item{max.generations.up}{
    For \code{buildPed}: the maximal number of generations to look for
    ancestors.
  }

  \item{method}{
    The multiple hypothesis testing method. All methods supported by
    \code{\link{p.adjust}} are allowed.
    %% The method to calculate the number of independent groups of affected
    %% individuals in the pedigree. Method \code{"jo"} returns the number
    %% of independent groups, i.e. groups that do not share a single
    %% affected individual. Method \code{"daniel"} uses the original code
    %% from Daniel Taliun.
  }

  \item{nsim}{
    Number of simulations.
  }

  \item{object}{
    The \code{FAKinGroupResults} object.
  }

  \item{prune}{
    For \code{buildPed}: whether the full pedigree should be returned
    (\code{prune=FALSE}) or the pedigree should be reduced to a smaller
    pedigree containing only individuals in the kinship group
    (\code{prune=TRUE}); see details for more information. Note: the
    \code{plotPed} method does also support this parameter.
  }

  \item{rmKinship}{
    For \code{shareKinship}: Restrict reporting pairs to those that have
    kinship values >\code{rmKinship}. See \code{shareKinship} below for more
    details.
  }

  \item{strata}{
    For \code{runSimulation}: a numeric, character of factor
    characterizing each individual in the pedigree. The length of this
    vector and the ordering has to match the pedigree.
    This vector allows to perform stratified random sampling. See
    details on the \code{\link{PedigreeAnalysis}} help page or examples
    for more information.
  }

  \item{type}{
    For \code{plotRes}: either \code{"density"} (the default) or
    \code{"hist"} specifying whether the distribution of expected values
    from the simulation should be visualized as a density plot or
    histogram.
  }

  \item{value}{
    For \code{trait<-}: can be a named numeric, character or factor
    vector. The names (at least some of them) have to match the ids in
    the pedigree of the object.
  }

  \item{...}{
    For \code{plotPed}: additional arguments to be submitted to the
    internal \code{buildPed} call and to \code{\link{plotPed}}.
  }
}
\section{Extends}{
  Class \code{\linkS4class{FAData}} directly.
}
\section{Slots}{
  \describe{
    \item{nsim}{
      Number of simulations.
    }
    \item{sim}{
      The list containing the result of the simulation. Element
      \code{"pvalueKinship"} contains the p-values from the kinship test,
      \code{"pvalueRatio"} the p-values from the ratio test,
      \code{"expDensity"} and \code{"expHist"} \code{density} and
      \code{hist} objects representing the background distribution from
      the Monte Carlo simulation.
      This slot should not be directly accessed, use the \code{result}
      method to extract result information.
    }
    \item{affectedKinshipGroups}{
      A list of lists, each element representing one kinship group,
      \code{$aff} the ids of all affected persons in that group sharing
      kinship with the indivudual (being the id of the group),
      \code{$phe} ids of all phenotyped individuals in the pedigree with
      kinship to the individual up to a kinship being smaller or equal
      to the smallest kinship of the indivudal with any other affected
      in that group. \code{$kinfreq}: a table with the frequency
      (counts) of kinship values (smaller 0.5). \code{$meankin}: the
      mean kinship in that group.
    }
  }
}
\section{Methods and Functions}{
  \describe{

    \item{affectedKinshipGroups}{
      Get groups of affected individuals in the pedigree along with
      all phenotyped individuals with kinship larger or equal to the
      smallest kinship between affected individuals in the
      group. Returns a \code{list} of lists with elements \code{aff},
      \code{phe}, \code{kinfreq} and \code{meankin} for each list item:
      \code{aff}: character vector with the ids of all affected in the
      group, \code{phe}: character vector with the ids of all phenotyped
      in the group, \code{kinfreq}: a \code{table} with the frequency
      (counts) of kinship values (self-self kinships removed; the names of the
      table correnspond to the kinship values ordered increasingly), and
      \code{meankin}: the mean kinship value in the group. The names of
      the list correspond to the id of the affected individual for which
      the affected kinship group was determined.
    }

    \item{buildPed}{
      Builds the pedigree for the submitted id (which represents the id
      of the group, i.e. one of
      \code{names(affectedKinshipGroups(object))}, respectively ids in
      column \code{"group_id"} of \code{result(object)}). Building a
      pedigree by submitting the family id is not supported. See details
      below for more information. The resulting pedigree is returned as
      a \code{data.frame}.
      %% Builds the pedigree for the submitted id (which represents the id
      %% of the group, i.e. one of
      %% \code{names(affectedKinshipGroups(object))}). In contrast to the
      %% same method for \code{\linkS4class{FAData}} objects, this method
      %% includes only individuals in the pedigree that are in the selected
      %% group of individuals with eventually included missing mates for
      %% family founders.
    }

    %% \item{independentGroupCount}{
    %%   Returns the number of independent groups of affected individuals.
    %% }

    \item{plotPed}{
      Plots a pedigree for one of the affected kinship groups in the
      simulation results. The ids of all individuals of the affected
      kinship group are highlighted in the plot in red.
      See \code{\link{plotPed}} for more details on
      the plotting and details below for additional settings.
    }

    \item{plotRes}{
      Plots the distribution of counts of randomly sampled affected
      counts within the kinship group along with the
      actually observed kinship sum.
    }

    \item{result}{
      Returns the result from the simulation as a \code{data.frame} with
      columns:
      \code{"trait_name"}: the name of the trait.
      \code{"total_phenotyped"}: the total number of phenotyped
      individuals in the trait.
      \code{"total_affected"}: the total number of affected individuals
      in the trait.
      \code{"phenotyped"}: the number of phenotyped individuals in all
      analyzed kinship groups.
      \code{"affected"}: the number of affected individuals in all
      analyzed kinship groups.
      \code{"group_id"}: the id for the kinship group (represents the id
      of one of the affected individuals in the group).
      \code{"family"}: the family id of the affected/group.
      \code{"group_phenotyped"}: the number of phenotyped individuals in the
      current kinship group.
      \code{"group_affected"}: the number of affected individuals in the
      current kinship group.
      \code{"ratio_pvalue"}: the p-value from the ratio test.
      \code{"ratio_padj"}: the p-value from the ratio test adjusted for
      multiple hypothesis testing using the method specified with
      argument \code{method}.
      \code{"mean_kinship"}: the mean kinship value between all
      individuals in the current kinship group.
      \code{"kinship_pvalue"}: the p-value from the kinship test.
      \code{"kinship_padj"}: the p-value from the kinship test adjusted
      for multiple hypothesis testing using the method specified with
      argument \code{method}.

      The \code{data.frame} is sorted by column \code{"ratio_pvalue"},
      its rownames correspond to the \code{"group_id"}.
    }

    \item{runSimulation}{
      Performs the simulation analysis based on the pedigree and trait
      information stored in the object. Optionally allows to perform
      stratified sampling. Returns a \code{FAKinGroupResults}
      with the results of the simulation.
    }

    \item{shareKinship}{
      Returns a character vector with ids of all individuals that share
      kinship with any of the individuals in the kinship group
      identified by the argument \code{id}. If \code{rmKinship} is
      specified, only individuals with a kinship >\code{rmKinship}
      to the group defined by \code{id} will be reported. This esentially
      restricts only the inclusion of individuals outside of the group.
      Everyone inside the group will be reported independently of the
      threshold defined by \code{rmKinship}. This feature has mainly been
      implemented for reasons of API compatibility with the remaining
      versions of \code{shareKinship}.
    }

    \item{trait<-}{
      Set the trait information. This method will reset all simulation
      results saved in the \code{sim} slot.
    }

  }
}
\details{
  Calling the \code{runSimulation} method on a \code{FAKinGroupResults}
  object is the same as calling the \code{\link{kinshipGroupTest}} on a
  \code{\linkS4class{FAData}} object. In the first case the
  simulation is performed using the trait information
  data stored internally in the object, while in the latter case the
  trait information have to be submitted to the function
  call.

  A call to the setter methods \code{trait<-} resets any simulation
  results present in the \code{sim} slot, thus, the object can be
  re-used to perform a simulation analysis using the new trait data.

  The \code{buildPed} method returns by default the full pedigree (all
  ancestors and all children) up to the maximal number of
  generations. By setting \code{prune=TRUE} the method restricts the
  pedigree to all individuals with a kinship >= the minimal kinship
  between the individual (with the id equal to the group id) and any
  other affected individual in its pedigree.

  The \code{plotPed} method allows to plot the pedigree for a kinship
  group. This pedigree consists of the full pedigree (all ancestors and
  children) of all individuals in the kinship group. Similar to the
  \code{buildPed} method the pedigree can be restricted to the kinship
  group (and eventual missing parents etc) by setting \code{prune=TRUE}.
}
\note{
  Subsetting (using the \code{[} operator) is not supported.
}
\value{
  Refer to the method and function description above for detailed
  information on the returned result object.
}
\author{
  Johannes Rainer, Daniel Taliun
}
\seealso{
  \code{\linkS4class{FAData}},
  \code{\link{kinship}},
  \code{\link{trait}},
  \code{\link{probabilityTest}},
  \code{\link{kinshipSumTest}},
  \code{\link{genealogicalIndexTest}},
  \code{\link{familialIncidenceRateTest}},
  \code{\link{fsirTest}},
  \code{\link{buildPed}},
  \code{\link{plotPed}},
  \code{\link{switchPlotfun}}
}
\examples{
##########################
##
##  Perform the simulation analysis
##
## Load the test data.
data(minnbreast)

## Subset to some families and generate the pedigree data.frame
mbsub <- minnbreast[minnbreast$famid == 165 | minnbreast$famid == 432, ]
PedDf <- mbsub[, c("famid", "id", "fatherid", "motherid", "sex")]
colnames(PedDf) <- c("family", "id", "father", "mother", "sex")

## Generate the FAData.
fad <- FAData(pedigree=PedDf)

## Specify the trait.
tcancer <- mbsub$cancer
names(tcancer) <- mbsub$id

## Perform the kinship group test.
far <- kinshipGroupTest(fad, trait=tcancer, traitName="cancer", nsim=1000)
res <- result(far)
head(res)

## Plot the pedigree for the most significant kinship group
plotPed(far, id=res[1, "group_id"])

## The full pedigree for this affected individual and its kinship group is
## large:
nrow(buildPed(far, id=res[1, "group_id"]))

## We can however restrict it to a reduced pedigree containing only the
## kinship group and all individuals with a kinship >= the smallest kinship
## between the individual and any other affected individual in the pedigree:
nrow(buildPed(far, id=res[1, "group_id"], prune=TRUE))

## By specifying prune=TRUE we can restrict the pedigree plot to these
## individuals
plotPed(far, id=res[1, "group_id"], prune=TRUE)

## Get the ids of all individuals sharing kinship with any of the inddividuals
## in that kinship group:
shareKinship(far, id=res[1, "group_id"])

## Plot the simulation analysis result for the ratio test.
plotRes(far, id=res[1, "group_id"], type="hist")

}
\keyword{classes}

\name{FAKinSumResults-class}
\Rdversion{1.1}
\docType{class}
\alias{FAKinSumResults-class}
\alias{FAKinSumResults}
\alias{[,FAKinSumResults,ANY-method}
\alias{[,FAKinSumResults,ANY,ANY,ANY-method}
\alias{plotPed,FAKinSumResults-method}
\alias{plotRes,FAKinSumResults-method}
\alias{result,FAKinSumResults-method}
\alias{runSimulation,FAKinSumResults-method}
\alias{trait<-,FAKinSumResults-method}


\title{Kinship sum test}

\description{
  The \code{FAKinSumResults} object contains the results from a kinship
  cluster test which evaluates familial aggregation based on the sum of kinship
  values between affected cases. This test highlights individuals that
  exhibit a higher than chance relationship to other affected
  individuals, therefore highlighting individuals within families
  aggregating the phenotype. To achieve this, for each affected
  individual the sum of kinship values to all other affected cases is
  computed. In a Monte Carlo simulation this is repeated with the same
  number of cases and the resulting background distribution is used to
  compute p-values for the kinship sums obtained from the observed
  cases.
}
\section{Objects from the Class}{
  \code{FAKinSumResults} objects are created by the
  \code{\link{kinshipSumTest}} method on a
  \code{\linkS4class{FAData}} object.
}
\usage{

\S4method{plotPed}{FAKinSumResults}(object, id=NULL, family=NULL,
                                     filename=NULL, device="plot",
                                     only.phenotyped=FALSE, ...)

\S4method{plotRes}{FAKinSumResults}(object, id=NULL, family=NULL,
                                     addLegend=TRUE, type="density", ...)

\S4method{result}{FAKinSumResults}(object, method="BH", cutoff=0.05, rmKinship=0)

\S4method{runSimulation}{FAKinSumResults}(object, nsim=50000,
strata=NULL, ...)

\S4method{trait}{FAKinSumResults}(object) <- value

}
\arguments{

  (in alphabetic order)

  \item{addLegend}{
    For \code{plotRes}: if a legend should be added to the plot.
  }

  \item{cutoff}{
    For \code{result}: P-value cutoff for clustering of kinship-related
    affected individuals based on their \code{padj} value. Individuals with
    p-value lower than this cutoff will be included in the grouping. Each group
    will have at least one individual that satisfies this threshold, as this is
    the one that started that group. Others are included due to kinship to this
    particular individual. This is especially useful for large pedigrees with
    inaccurate or missing family assignment.
  }

  \item{device}{
    For \code{plotPed}: see \code{\link{plotPed}} for more details.
  }

  \item{family}{
    For \code{plotPed}: not supported.
  }

  \item{filename}{
    For \code{plotPed}: the file name to which the pedigree plot should
    be exported. See \code{\link{plotPed}} for more details.
  }

  \item{id}{
    For \code{plotPed} and \code{plotRes}: the id of the indiviual
    (i.e. affected individual in the result \code{data.frame}) for which
    the pedigree or the simulation result should be plotted.
    Note: \code{id} can be a \code{numeric} or a
    \code{character}. Numeric ids will be internally converted to
    \code{character}.
  }

  \item{method}{
    The multiple hypothesis testing method. All methods supported by
    \code{\link{p.adjust}} are allowed.
  }

  \item{nsim}{
    Number of simulations.
  }

  \item{object}{
    The \code{FAKinSumResults} object.
  }

  \item{only.phenotyped}{
    For \code{plotPed}: Wheter only phenotyped individuals,
    i.e. individuals with a non-NA value in column \code{affected} (the
    trait information). If \code{TRUE}, the function removes all
    non-phenotyped individuals, keeping only those that are required for
    the pedigree to be complete.
  }

  \item{rmKinship}{
    For \code{result}: When assigning kinship groups, skip pairs of cases
    with kinship <= \code{rmKinship}.
  }

  \item{strata}{
    For \code{runSimulation}: a numeric, character of factor
    characterizing each individual in the pedigree. The length of this
    vector and the ordering has to match the pedigree.
    This vector allows to perform stratified random sampling. See
    details on the \code{\link{PedigreeAnalysis}} help page or examples
    for more information.

  }

  \item{type}{
    For \code{plotRes}: either \code{"density"} (the default) or
    \code{"hist"} specifying whether the distribution of expected values
    from the simulation should be visualized as a density plot or
    histogram.
  }

  \item{value}{
    For \code{trait<-}: can be a named numeric, character or factor
    vector. The names (at least some of them) have to match the ids in
    the pedigree of the object.
  }

  \item{...}{
    For \code{plotPed}: additional arguments to be submitted to the
    internal \code{buildPed} call and to \code{\link{plotPed}}.
  }

}
\section{Extends}{
  Class \code{\linkS4class{FAData}} directly.
}
\section{Slots}{
  \describe{
    \item{nsim}{
      Number of simulations.
    }
    \item{sim}{
      The result of the simulation. This slot should not be accessed
      directly, use the \code{result} method to extract result information.
    }
  }
}
\section{Methods and Functions}{
  \describe{

    \item{plotPed}{
      Plots a pedigree for one of the affected individuals in the
      simulation results. The id of the selected affected individual
      (specified with argument \code{id}) is highlighted in red.
      See \code{\link{plotPed}} for more details.
    }

    \item{plotRes}{
      Plots the distribution of kinship sums between random sets of
      samples individuals from the Monte Carlo simulation along with the
      actually observed kinship sum for the affected individual
      specified with parameter \code{id}. For \code{id} only affected
      individuals for which the analysis has been performed are
      allowed. The ids of these individuals are listed in column
      \code{"affected_id"} of the \code{data.frame} returned by
      \code{result}.

    }

    \item{result}{
      Returns the result from the simulation as a \code{data.frame} with
      columns:
      \code{"trait_name"}: the name of the trait.
      \code{"total_phenotyped"}: total number of phenotyped individuals in
      the trait.
      \code{"total_affected"}: total number of affected individuals in the
      trait.
      \code{"affected_id"}: the id of the affected individual for whom
      the test has been performed.
      \code{"family"}: the family id of the affected.
      \code{"ksgrp"}: Numeric identifier that specifies a group of affected
      individuals related by kinship. Group assignment starts with the top
      ranking individual (by \code{padj}), \code{NA} is assigned to those
      that did not pass the threshold \code{cutoff} supplied to \code{result}.
      If parameter \code{rmKinship} is passed, assignment is restricted to
      kinship values >\code{rmKinship} between the top ranking individual
      that founded this group and the rest. Kinship-related individuals that
      have a lower kinship value will be left unassigned, therefore they may
      end up in a separate group.
      \code{"kinship_sum"}: the sum of kinship values.
      \code{"freq"}: the expected frequency of the kinship sum from the
      simulation.
      \code{"pvalue"}: the p-value for the significance of the kinship
      sum.
      \code{"padj"}: the p-value adjusted for multiple hypothesis
      testing (with the method specified with argument \code{method}).

      The returned \code{data.frame} is sorted by column
      \code{"pvalue"}, its row names correspond to column
      \code{"affected_id"}.
    }

    \item{runSimulation}{
      Performs the simulation analysis based on the pedigree and trait
      information stored in the object. Returns a
      \code{FAKinSumResults} object with the results of the simulation.
    }

    \item{trait<-}{
      Set the trait information. This method will reset all simulation
      results saved in the \code{sim} slot.
    }

  }
}
\details{
  Calling the \code{runSimulation} method on a \code{FAKinSumResults}
  object is the same as calling the \code{\link{kinshipSumTest}} on a
  \code{\linkS4class{FAData}} object. In the first case the
  simulation is performed using the trait information
  data stored internally in the object, while in the latter case the
  trait information have to be submitted to the function
  call.

  A call to the setter methods \code{trait<-} resets any simulation
  results present in the \code{sim} slot, thus, the object can be
  re-used to perform a simulation analysis using the new trait data.

  The expected frequency (column \code{"freq"}) in the result
  \code{data.frame} is \code{NA} if the corresponding sum of kinship
  coefficients reported in column \code{"kinship_sum"} was never sampled
  in the simulation. Still, a p-value can be reported.

  The \code{plotPed} function does not support to draw pedigrees for
  individuals for which no simulation test has been performed. To draw a
  pedigree for any individual (with or without trait information, being
  affected or not in the trait) refer to the \code{plotPed} method for
  \code{\linkS4class{FAData}} objects.
}
\note{
  Subsetting (using the \code{[} operator) is not supported.
}
\value{
  Refer to the method and function description above for detailed
  information on the returned result object.
}
\author{
  Johannes Rainer, Christian Weichenberger
}
\seealso{
  \code{\linkS4class{FAData}},
  \code{\link{kinship}},
  \code{\link{trait}},
  \code{\link{probabilityTest}},
  \code{\link{kinshipGroupTest}},
  \code{\link{kinshipSumTest}},
  \code{\link{genealogicalIndexTest}},
  \code{\link{familialIncidenceRateTest}},
  \code{\link{fsirTest}},
  \code{\link{plotPed}}
}
\examples{
##########################
##
##  Perform the simulation analysis
##
## Load the test data.
data(minnbreast)

## Subset to some families and generate a pedigree data.frame.
mbsub <- minnbreast[minnbreast$famid == 4 | minnbreast$famid == 5 |
                  minnbreast$famid == 6 | minnbreast$famid == 7, ]
PedDf <- mbsub[, c("famid", "id", "fatherid", "motherid", "sex")]
colnames(PedDf) <- c("family", "id", "father", "mother", "sex")

## Generate the FAData.
fad <- FAData(pedigree=PedDf)

## Specify the trait.
tcancer <- mbsub$cancer
names(tcancer) <- mbsub$id

## Perform the test:
far <- kinshipSumTest(fad, trait=tcancer, traitName="cancer",
                      nsim=1000)
head(result(far))

## Plot the pedigree for one of affected individuals. The id of the affected
## individual is highlighted in red.
plotPed(far, id=result(far)$affected_id[1])

## Replace the trait, this will delete all simulation results
## on the existing FAKinSumResults object
tpreg <- mbsub$everpreg
names(tpreg) <- mbsub$id
trait(far) <- tpreg

## The analysis can be performed using the runSimulation method.
far <- runSimulation(far, nsim=1000)
head(result(far))

## Plot the pedigree of one of the affected; note that "affected" now
## indicates whether the individual was ever pregnant.
plotPed(far, id="9")

## Plot also the result from the simulation run.
plotRes(far, id="9")

}
\keyword{classes}

\name{FAProbResults-class}
\Rdversion{1.1}
\docType{class}
\alias{[,FAProbResults,ANY-method}
\alias{[,FAProbResults,ANY,ANY,ANY-method}
\alias{FAProbResults-class}
\alias{FAProbResults}
\alias{buildPed,FAProbResults-method}
\alias{cliqueAndTrait}
\alias{cliqueAndTrait,FAProbResults-method}
\alias{cliques}
\alias{cliques,FAProbResults-method}
\alias{cliques<-}
\alias{cliques<-,FAProbResults-method}
\alias{plotPed,FAProbResults-method}
\alias{result}
\alias{result,FAProbResults-method}
\alias{runSimulation}
\alias{runSimulation,FAProbResults-method}
\alias{shareKinship,FAProbResults-method}
\alias{trait<-,FAProbResults-method}
\alias{traitByClique}
\alias{traitByClique,FAProbResults-method}


\title{DEPRECATED: Probability test}

\description{
  The \code{FAProbResults} object contains the results from the
  \emph{probability test}.
  The probability test is only a convience method that calls the \code{gap} package's
  method \code{pfc.sim} to compute probabilities of familial clustering
  of phenotypes [Yu and Zelterman (2002)]. One drawback of that method
  is that it is limited to families with at most 22 individuals. Thus,
  pedigrees need to be split with specialized software such as Jenti
  [Falchi and Fuchsberger ea. (2008)], which within large families
  define cliques that can then be used as input to this algorithm.

  \emph{DEPRECATION WARNING:}
  
  Due to problems of the \code{gap} package on MS Windows systems, this
  test will be removed in the next Bioconductor release (3.8).
}
\section{Objects from the Class}{
  \code{FAProbResults} objects are created by the
  \code{\link{probabilityTest}} method of \code{\linkS4class{FAData}}
  objects.
}
\usage{

\S4method{buildPed}{FAProbResults}(object, id=NULL, max.generations.up=3,
                                  max.generations.down=16, prune=FALSE)

\S4method{cliqueAndTrait}{FAProbResults}(object, na.rm=FALSE)

\S4method{cliques}{FAProbResults}(object, na.rm=FALSE)

\S4method{cliques}{FAProbResults}(object) <- value

\S4method{plotPed}{FAProbResults}(object, id=NULL, family=NULL,
                                 filename=NULL, device="plot", ...)

\S4method{result}{FAProbResults}(object, method="BH")

\S4method{runSimulation}{FAProbResults}(object, nsim=50000)

\S4method{shareKinship}{FAProbResults}(object, id=NULL)

\S4method{trait}{FAProbResults}(object) <- value

\S4method{traitByClique}{FAProbResults}(object)

}
\arguments{

  (in alphabetic order)

  \item{device}{
    For \code{plotPed}: see \code{\link{plotPed}} for more details.
  }

  \item{family}{
    For \code{plotPed}: not supported.
  }

  \item{filename}{
    For \code{plotPed}: the file name to which the pedigree plot should
    be exported. See \code{\link{plotPed}} for more details.
  }

  \item{id}{
    For \code{buildPed}, \code{plotPed}, \code{shareKinship}: the id
    (character or numerif) of the clique (i.e. one of the ids in column
    \code{"group_id"} of the result table \code{result(object)}).

    Note: \code{id} can be a \code{numeric} or a
    \code{character}. Numeric ids will be internally converted to
    \code{character}.
  }

  \item{max.generations.down}{
    For \code{buildPed}: the maximal number of generations to look for
    children.
  }

  \item{max.generations.up}{
    For \code{buildPed}: the maximal number of generations to look for
    ancestors.
  }

  \item{method}{
    The multiple hypothesis testing method. All methods supported by
    \code{\link{p.adjust}} are allowed.
  }

  \item{na.rm}{
    Whether \code{NA} elements should be returned or not.
  }

  \item{nsim}{
    Number of simulations.
  }

  \item{object}{
    The \code{FAProbResults} object.
  }

  \item{prune}{
    For \code{buildPed}: whether the full pedigree should be returned
    (\code{prune=FALSE}) or the pedigree should be reduced to the
    individuals in the corresponding clique (\code{prune=TRUE}). Note: the
    \code{plotPed} method does also support this parameter.
  }

  \item{value}{
    For \code{cliques<-}: can be a named numeric, character or factor
    vector. The names (at least some of them) have to match the ids in
    the pedigree of the object.
  }

  \item{...}{
    For \code{plotPed}: additional arguments to be submitted to the
    internal \code{buildPed} call and to \code{\link{plotPed}}.
  }

}
\section{Extends}{
  Class \code{\linkS4class{FAData}} directly.
}
\section{Slots}{
  \describe{
    \item{nsim}{
      Number of simulations.
    }

    \item{sim}{
      The result of the simulation. This slot should not be directly
      accessed, use the \code{result} method to extract result information.
    }

    \item{.cliques}{
      A factor with the assignment of individuals to cliques. This slot
      should not be accessed directly, but the \code{cliques} and
      \code{cliques<-} methods should be used instead that ensure that
      the data is matched to the information in the pedigree.
    }

  }
}
\section{Methods and Functions}{
  \describe{

    \item{cliqueAndTrait}{
      Get a \code{data.frame} with the clique ID and the value from the
      trait for each individual. If \code{na.rm=TRUE} all rows
      (individuals) with either a missing clique ID or trait value are
      removed.
    }

    \item{buildPed}{
      Builds the pedigree for the submitted id (which represents the id
      of the group, i.e. the clique (e.g. column \code{"group_id"} of
      \code{result(object)}). By default the method builds the full
      pedigree for all individuals in the clique but the argument
      \code{prune} allows to reduce it to the individuals of the clique.
      Building a pedigree by submitting the family id is not supported.
      The pedigree is returned as a \code{data.frame}.
    }

    \item{cliques}{
      Returns a \code{factor} vector representing the clique/group
      assignment of the individuals. By default, the ordering
      is the same as pedigree, setting argument \code{na.rm=TRUE}
      removes all \code{NA} values, thus the ordering and length might
      differ (e.g. if some individuals are not part of any clique).
    }

    \item{cliques<-}{
      Setter for the cliques vector. Can be a named numeric vector, character
      vector or factor with the names matching the ids of the
      individuals in the pedigree. The method internally matches and
      re-orders the cliques vector to match the ordering of the ids in
      the pedigree.
      In addition, the function resets eventually stored simulation
      results in the \code{sim} slot.
    }

    \code{plotPed}{
      Plots a pedigree for one of the cliques in the
      simulation results. The ids of all individuals of the selected
      clique are highlighted in red.
      See \code{\link{plotPed}} for more details on
      the plotting and details below for additional settings.
    }

    \item{result}{
      Returns the result from the simulation as a \code{data.frame} with
      columns:
      \code{trait_name}: the name of the trait.
      \code{total_phenotyped}: the total number of phenotyped
      individuals in the trait.
      \code{total_affected}: the total number of affected individuals in
      the trait.
      \code{phenotyped}: the number of phenotyped individuals in
      all group (i.e. cliques specified by the \code{clique} parameter).
      \code{affected}: the number of affected individuals in
      all group (i.e. cliques specified by the \code{clique} parameter).
      \code{group_id}: the id of the group (clique).
      \code{family}: the id of the family in which the clique was defined.
      \code{group_phenotyped}: the number of phenotyped individuals in the
      current group.
      \code{group_affected}: the number of affected individuals in the current
      group.
      \code{pvalue}: the p-value from the Monte Carlo simulation.
      \code{padj}: the p-value adjusted for multiple hypothesis testing
      using the method specified with argument \code{method}.
    }

    \item{runSimulation}{
      Performs the simulation analysis based on the pedigree, trait and
      clique information stored in the object. Returns a
      \code{FAProbResults} object with the results of the simulation.
    }

    \item{shareKinship}{
      Returns a character vector with the ids of all individuals that
      share kinship with any of the individuals in the clique identified
      by the argument \code{id}.
    }

    \item{trait<-}{
      Set the trait information. This method will reset all simulation
      results saved in the \code{sim} slot.
    }

    \item{traitByClique}{
      Summarize the trait information by clique. Returns a \code{matrix}
      with the size of the clique (i.e. individuals in the clique with
      available trait information) and count of individuals in the
      clique with a trait value other than zero (column
      \code{affected_size}). The clique ids are used as rownames of the
      \code{matrix}.
    }
  }
}
\details{
  Calling the \code{runSimulation} method on a \code{FAProbResults}
  object is the same as calling the \code{probabilityTest} on a
  \code{\linkS4class{FAData}} object. In the first case the
  simulation is performed using the clique and trait information
  data stored internally in the object, while in the latter case the
  clique and trait information have to be submitted to the function
  call.

  A call to the setter methods \code{trait<-} or \code{cliques<-} resets
  any simulation results present in the \code{sim} slot, thus, the
  object can be re-used to perform a simulation analysis using the new
  trait or clique data.
}
\note{
  Subsetting (using the \code{[} operator) is not supported.
}
\value{
  Refer to the method and function description above for detailed
  information on the returned result object.
}
\author{
  Johannes Rainer, Daniel Taliun
}
\references{
  Yu C & Zelterman D (2002) Statistical inference for familial disease
  clusters. \emph{Biometrics}, pp 481-491

  Falchi M & Fuchsberger C (2008) Jenti: an efficient tool for mining
  complex inbred genealogies. \emph{Bioinformatics}, pp 724-726
}
\seealso{
  \code{\linkS4class{FAData}},
  \code{\link{buildPed}},
  \code{\link{plotPed}},
  \code{\link{trait}},
  \code{\link{probabilityTest}},
  \code{\link{kinshipGroupTest}},
  \code{\link{kinshipSumTest}},
  \code{\link{genealogicalIndexTest}},
  \code{\link{familialIncidenceRateTest}},
  \code{\link{fsirTest}}
}
\keyword{classes}

\name{FAGenIndexResults-class}
\Rdversion{1.1}
\docType{class}
\alias{FAGenIndexResults-class}
\alias{FAGenIndexResults}
\alias{[,FAGenIndexResults,ANY-method}
\alias{[,FAGenIndexResults,ANY,ANY,ANY-method}
\alias{plotPed,FAGenIndexResults-method}
\alias{plotRes}
\alias{plotRes,FAGenIndexResults-method}
\alias{result,FAGenIndexResults-method}
\alias{runSimulation,FAGenIndexResults-method}
\alias{trait<-,FAGenIndexResults-method}


\title{Genealogical Index}

\description{
  The genealogical index [Hill, 1980], also referred to as the
  \emph{genealogical index of familiality} (GIF) in the literature, is a
  method to identify familial clustering of diseases or other traits.
  For a given trait,
  the method computes the mean kinship between affected in the
  whole pedigree along with mean kinships of randomly drawn sets of
  individuals. The distribution of average kinship values among the
  control sets is used to estimate the probability that the observed
  level of kinship among the cases is due to chance.
}
\details{
  This implementation differs from the original method from Hill as it
  allows, in addition to perform per family analyses, to use also
  stratified sampling and allows a more flexible definition of the set
  of matched control individuals.
  The \code{controlSetMethod} parameter allows to specify a method to
  define the matched control set (e.g. matched by sex or matched by any
  externally provided vector).

  Stratified sampling allows to even further fine tune the selection of
  matched controls. Assuming that in a pedigree the group of affected
  consists of 5 females and 3 male individuals, passing the sex of all
  individuals to the function (e.g. \code{strata=fad$sex}, with
  \code{fad} being the \code{\linkS4class{FAData}} object containing the
  pedigree to be analyzed) results in random sets with the same
  proportion of male/female individuals (i.e. 5 females, 3 males).

  Note that, if \code{strata} is specified, all individuals with a
  missing value in \code{strata} (also affected individuals) are
  excluded from the analysis.

  Note that by default singletons (i.e. unconnected individuals in
  the pedigree) are removed from the pedigree prior the
  analysis. Set \code{rm.singletons=FALSE} if you do not want them
  to be removed.

  By default, the genealogical index is calculated on the whole
  pedigree, but it is also possible to evaluate within-family clustering
  of cases by specifying \code{perFamilyTest=TRUE}. In that case, it is
  also possible to use the \code{\link{getGenerationMatched}} and
  \code{\link{getGenerationSexMatched}} functions to define the set of
  matched controls from which random samples will be taken.

  A call to the setter methods \code{trait<-} resets any simulation
  results present in the \code{sim} slot, thus, the object can be
  re-used to perform a simulation analysis using the new trait data.


}
\section{Objects from the Class}{
  \code{FAGenIndexResults} objects are created calling the
  \code{\link{genealogicalIndexTest}} method on a
  \code{\linkS4class{FAData}} object.
}
\usage{

\S4method{plotPed}{FAGenIndexResults}(object, id=NULL, family=NULL,
                                     filename=NULL, device="plot", ...)

\S4method{plotRes}{FAGenIndexResults}(object, id=NULL, family=NULL,
                                     addLegend=TRUE, type="density", ...)

\S4method{result}{FAGenIndexResults}(object, method="BH")

\S4method{runSimulation}{FAGenIndexResults}(object, nsim=50000,
                                           perFamilyTest=FALSE,
                                           controlSetMethod="getAll",
                                           rm.singletons=TRUE, strata=NULL, ...)

\S4method{trait}{FAGenIndexResults}(object) <- value

}
\arguments{

  (in alphabetic order)

  \item{addLegend}{
    For \code{plotRes}: if a legend should be added to the plot.
  }

  \item{controlSetMethod}{
    For \code{runSimulation}: the method (i.e. name of the function)
    that should be used to define the set of (eventually matched)
    control individuals from which the random samples are
    taken. Supported functions are \code{\link{getAll}},
    \code{\link{getSexMatched}} and
    \code{\link{getExternalMatched}}. For \code{perFamilyTest=TRUE} also
    \code{\link{getGenerationMatched}} and
    \code{\link{getGenerationSexMatched}} are supported. Note: for
    \code{getExternalMatched}, a numeric, character or factor vector to
    be used for the matching has to be submitted to \code{runSimulation}
    as additional argument \code{match.using}.
  }

  \item{device}{
    For \code{plotPed}: see \code{\link{plotPed}} for more details.
  }

  \item{family}{
    For \code{plotPed}: the family for which the pedigree should be
    plotted.
    For \code{plotRes}: the family for which the genealogical index
    analysis simulation results should be shown. Only supported if
    \code{perFamilyTest=TRUE}.
  }

  \item{filename}{
    For \code{plotPed}: the file name to which the pedigree plot should
    be exported. See \code{\link{plotPed}} for more details.
  }

  \item{id}{
    For \code{plotPed}: the id of an indiviual from a family for which
    the pedigree should be plotted.
    For \code{plotRes}: the id of an individual from a family for which
    the genealogical index analysis simulation results should be
    shown. Only supported if \code{perFamilyTest=TRUE}.
  }

  \item{method}{
    The multiple hypothesis testing method. All methods supported by
    \code{\link{p.adjust}} are allowed.
  }

  \item{nsim}{
    Number of simulations.
  }

  \item{perFamilyTest}{
    For \code{runSimulation}: whether the test should be performed
    on the whole pedigree (default) or separately within each
    family. In the latter case the test evaluates the presence of
    clustered affected individuals within each family.
  }

  \item{rm.singletons}{
    For \code{runSimulation}: whether unconnected individuals in the
    pedigree (i.e. singletons) should be removed.
  }

  \item{object}{
    The \code{FAGenIndexResults} object.
  }

  \item{strata}{
    For \code{runSimulation}: a numeric, character of factor
    characterizing each individual in the pedigree. The length of this
    vector and the ordering has to match the pedigree.
    This vector allows to perform stratified random sampling. See
    details or examples for more information.
  }

  \item{type}{
    For \code{plotRes}: either \code{"density"} (the default) or
    \code{"hist"} specifying whether the distribution of expected values
    from the simulation should be visualized as a density plot or
    histogram.
  }

  \item{value}{
    For \code{trait<-}: can be a named numeric, character or factor
    vector. The names (at least some of them) have to match the ids in
    the pedigree of the object.
  }

  \item{...}{
    For \code{plotPed}: additional arguments to be submitted to the
    internal \code{buildPed} call and to \code{\link{plotPed}}.

    For \code{runSimulation}: additional arguments passed to the choosen
    \code{controlSetMethod} function (e.g. \code{match.using} for
    \code{\link{getExternalMatched}}).
  }

}
\section{Extends}{
  Class \code{\linkS4class{FAData}} directly.
}
\section{Slots}{
  \describe{
    \item{controlSetMethod}{
      A character specifying the name of the method used to define the
      set of control individuals from which random samples were taken.
    }
    \item{nsim}{
      Number of simulations.
    }
    \item{perFamilyTest}{
      Logical indicating whether a per-family test was performed.
    }
    \item{sim}{
      The result of the simulation. This slot should not be accessed
      directly, use the \code{result} method to extract result information.
    }
  }
}
\section{Methods and Functions}{
  \describe{

    \code{plotPed}{
      Plots a pedigree for one of the affected individuals in the
      simulation results. The id of the selected affected individual
      (specified with argument \code{id}) is highlighted in red.
      See \code{\link{plotPed}} for more details.
    }

    \code{plotRes}{
      Plots the results from a genealogical index simulation
      analysis. The distribution of the mean kinship values of the
      randomly drawn controls are displayed as a grey density plot, the
      observed mean kinship value of all affected as a blue vertical
      line.
    }

    \item{result}{
      Returns the result from the simulation as a \code{data.frame} with
      columns:
      \code{"trait_name"}: the name of the trait.
      \code{"total_phenotyped"}: total number of individuals in the
      pedigree phenotyped in the analyzed trait.
      \code{"total_affected"}: total number ofindividuals in the
      pedigree that are affected in the analyzed trait (i.e. number of
      cases).
      \code{"entity_id"}: the id for the analyzed entity, being either
      the whole pedigree (in which case the id will be \code{"1"}) or
      the id of the family (if \code{perFamilyTest=TRUE}).
      \code{"entity_ctrls"}: the number of (matched) control individuals
      from which the random samples were drawn.
      \code{"entity_affected"}: the number of affected individuals in
      the entity. This number can differ from the number of affected, if
      \code{strata} was specified and some of the affected have a
      missing value in \code{strata}.
      \code{"genealogical_index"}: the genealogical index of familiality
      (gif), i.e. the mean kinship value between all affected in the
      entity (pedigree or family). To be consistent with the original
      implementations, the genealogical index is the mean kinship
      multiplied with 100000.
      \code{"pvalue"}: the p-value for the significance of the mean
      kinship.
      \code{"padj"}: the p-value adjusted for multiple hypothesis
      testing (with the method specified with argument \code{method}).

      The returned \code{data.frame} is sorted by column
      \code{"pvalue"}, its rownames correspond to column
      \code{"entity_id"}.
    }

    \item{runSimulation}{
      Performs the simulation analysis based on the pedigree and trait
      information stored in the object. Returns a
      \code{FAGenIndexResults} object with the results of the simulation.
    }

    \item{trait<-}{
      Set the trait information. This method will reset all simulation
      results saved in the \code{sim} slot.
    }

  }
}
\note{
  Subsetting (using the \code{[} operator) is not supported.
}
\value{
  Refer to the method and function description above for detailed
  information on the returned result object.
}
\references{
  Hill, J. R. (1980) A survey of cancer sites by kinship in the Utah Mormon
  population. In Cairns J, Lyon JL, Skolnick M (eds):
  \emph{Cancer Incidence in Defined Populations. Banbury Report 4.}
  Cold Spring Harbor, NY: Cold Spring Harbor Laboratory Press, pp 299--318.
}
\author{
  Johannes Rainer
}
\seealso{
  \code{\linkS4class{FAData}},
  \code{\link{trait}},
  \code{\link{probabilityTest}},
  \code{\link{kinshipGroupTest}},
  \code{\link{kinshipSumTest}},
  \code{\link{familialIncidenceRateTest}},
  \code{\link{fsirTest}},
  \code{\link{plotPed}}
}
\examples{
##########################
##
##  Perform the simulation analysis
##
## Load the Minnesota Breast Cancer data set.
data(minnbreast)

## Subset to some families and generate a pedigree data.frame
mbsub <- minnbreast[minnbreast$famid == 4 | minnbreast$famid == 14 |
                  minnbreast$famid == 6 | minnbreast$famid == 8, ]
PedDf <- mbsub[, c("famid", "id", "fatherid", "motherid", "sex")]
colnames(PedDf) <- c("family", "id", "father", "mother", "sex")

## Generate the FAData.
fad <- FAData(pedigree=PedDf)

## Specify the trait.
tcancer <- mbsub$cancer
names(tcancer) <- mbsub$id

## Perform the test with default settings, i.e. use all individuals
## in the pedigree as control set from which random samples are drawn
## and perform the analysis on the whole pedigree.
gi <- genealogicalIndexTest(fad, trait=tcancer, traitName="cancer",
                            nsim=1000,)
## Just show some information
gi

## Show the results
result(gi)

## Plot the observed mean kinship and the distribution of the mean kinship of
## random samples.
plotRes(gi)

## Plot the pedigree for one of the families. All individuals
## used as matched control set are highlighted in red.
plotPed(gi, family="8")

## Repeat the analysis using the sex as strata. This will result in stratified
## random sampling with the number of female and male individuals selected in
## each permutation corresponding to the numbers below
table(gi$sex[affectedIndividuals(gi)])
giStrata <- runSimulation(gi, nsim=1000, strata=gi$sex)
result(giStrata)


## Alternatively, we can use "getSexMatched" as the function to define the set
## of control individuals. Just, in the present case both male and females
## individuals will be selected since also there are male and female individuals
## among the affected cases.
giPerFam <- runSimulation(gi, nsim=1000, controlSetMethod="getSexMatched",
                          perFamilyTest=TRUE)
result(giPerFam)

## For those families in which there are only female cases, random samples
## were drawn among only female individuals (within the same family). These
## are highlighted in red in the pedigree plot:
plotPed(giPerFam, family="14", cex=0.5)

## Plot the simulation result for this family:
plotRes(giPerFam, family="14")

}
\keyword{classes}

\name{PedigreeUtils}
\Rdversion{1.1}
\alias{PedigreeUtils}
\alias{connectedSubgraph}
\alias{countGenerations}
\alias{countGenerations,FAData-method}
\alias{countGenerations,data.frame-method}
\alias{countGenerations,pedigreeList-method}
\alias{countGenerations,pedigree-method}
%% estimateGenerations
\alias{estimateGenerations}
\alias{estimateGenerations,FAData-method}
\alias{estimateGenerations,data.frame-method}
\alias{estimateGenerations,pedigreeList-method}
\alias{estimateGenerations,pedigree-method}
%% findFounders
\alias{findFounders}
\alias{findFounders,FAData-method}
\alias{findFounders,data.frame-method}
\alias{findFounders,pedigreeList-method}
\alias{findFounders,pedigree-method}
%% generationsFrom
\alias{generationsFrom}
\alias{generationsFrom,FAData-method}
\alias{generationsFrom,data.frame-method}
\alias{generationsFrom,pedigreeList-method}
\alias{generationsFrom,pedigree-method}
%% getAncestors
\alias{getAncestors}
\alias{getAncestors,FAData-method}
\alias{getAncestors,data.frame-method}
\alias{getAncestors,pedigreeList-method}
\alias{getAncestors,pedigree-method}
%% getChildren
\alias{getChildren}
\alias{getChildren,FAData-method}
\alias{getChildren,data.frame-method}
\alias{getChildren,pedigreeList-method}
\alias{getChildren,pedigree-method}
%% getCommonAncestor
\alias{getCommonAncestor}
\alias{getCommonAncestor,FAData-method}
\alias{getCommonAncestor,data.frame-method}
\alias{getCommonAncestor,pedigreeList-method}
\alias{getCommonAncestor,pedigree-method}
%% getFounders
\alias{getFounders}
\alias{getFounders,FAData-method}
\alias{getFounders,data.frame-method}
%% getMissingMate
\alias{getMissingMate}
\alias{getMissingMate,FAData-method}
\alias{getMissingMate,data.frame-method}
\alias{getMissingMate,pedigreeList-method}
\alias{getMissingMate,pedigree-method}
%% getSiblings
\alias{getSiblings}
\alias{getSiblings,FAData-method}
\alias{getSiblings,data.frame-method}
\alias{getSiblings,pedigreeList-method}
\alias{getSiblings,pedigree-method}
%% getSingletons
\alias{getSingletons}
\alias{getSingletons,FAData-method}
\alias{getSingletons,data.frame-method}
%% removeSingletons
\alias{removeSingletons}
\alias{removeSingletons,FAData-method}
\alias{removeSingletons,data.frame-method}

\alias{ped2graph}
\alias{subPedigree}
\alias{cliques,missing-method}

\alias{shareKinship}
\alias{shareKinship,FAData-method}


\title{Basic pedigree utilities}

\description{
  Utility functions to access, modify or subset pedigrees. Most of these
  functions can be applied to simple \code{data.frame} in pedigree
  format or \code{pedigree} or \code{pedigreeList} objects defined in
  the \code{kinship2} package.
}
\usage{

\S4method{cliques}{missing}(object, ...)

connectedSubgraph(graph, nodes, mode="all", all.nodes=TRUE, ifnotfound)

\S4method{countGenerations}{FAData}(object, id=NULL, direction="down", ...)

\S4method{estimateGenerations}{FAData}(object, family=NULL, ...)

\S4method{findFounders}{FAData}(object, family=NULL, id = NULL, ...)

\S4method{generationsFrom}{FAData}(object, id=NULL, ...)

\S4method{getAncestors}{FAData}(object, id=NULL, max.generations=3, ...)

\S4method{getChildren}{FAData}(object, id=NULL, max.generations=16, ...)

\S4method{getCommonAncestor}{FAData}(object, id, method="min.dist")

\S4method{getFounders}{FAData}(object, ...)

\S4method{getMissingMate}{FAData}(object, id=NULL, ...)

\S4method{getSiblings}{FAData}(object, id=NULL, ...)

\S4method{getSingletons}{FAData}(object, ...)

ped2graph(ped)

\S4method{removeSingletons}{FAData}(object, ...)

\S4method{removeSingletons}{data.frame}(object, ...)

subPedigree(ped, id=NULL, all=TRUE)

\S4method{shareKinship}{FAData}(object, id, rmKinship=0)

}
\arguments{

  (in alphabetic order)

  \item{all}{
    For \code{subPedigree}: if all individuals have to be present in the
    sub-pedigree.
  }

  \item{all.nodes}{
    For \code{connectedSubgraph}: if all nodes have to be present in the
    resulting graph, or only those that are connected with each other.
  }

  \item{direction}{
    For \code{countGenerations}: whether the number of ancestor (\code{"up"})
    generations or offspring (\code{"down"}) generation should be counted.
  }

  \item{family}{
    A character or numeric representing the family id.
    For \code{doFindFounders}: the id of the family in the pedigree for
    which the founders should be identified. Uses the first family in
    the pedigree if not specified.
    For \code{estimateGenerations}: optional id of the family if
    generation numbers should only be calculated for one
    family. Otherwiseif, the generations are estimated for all families
    (separately) in the object.
  }

  \item{graph}{
    An \code{igraph} graph object.
  }

  \item{id}{
    A character or numeric vector length 1 or longer specifying the
    id(s) of the individual(s).
    For \code{generationsFrom} and \code{findFounders} only a single id
    should be submitted.
  }

  \item{ifnotfound}{
    For \code{connectedSubgraph}: if not defined, the function throws an
    error if no subgraph can be specified. If defined, its value is
    returned if no subgraph was found.
  }

  \item{max.generations}{
    For \code{getAncestors} and \code{getChildren}: the maximal
    number of ancestor or offspring generations that should be returned.
  }

  \item{method}{
    For \code{getCommonAncestor}: the method by which the closest
    common ancestor sould be identified. Either \code{"min.dist"}
    (ancestor with the minimal distance to any of the individuals) or
    \code{"smallest.mean.dist"} (ancestor with the smallest mean
    distance to any of the individuals).
  }

  \item{mode}{
    For \code{connectedSubgraph}: either \code{"all"}, \code{"in"},
    \code{"out"} specifying how distances and paths between individual
    nodes should be determined. See help for function
    \code{shortest_paths} in package \code{igraph} for more details.
  }

  \item{nodes}{
    For \code{connectedSubgraph}: A character vector of node (vertex) names
    for which the subgraph should be defined.
  }

  \item{object}{
    For \code{cliques}: passed to the \code{cliques} function
    from the \code{igraph} package.
    For all other methods: either a \code{\linkS4class{FAData}} object
    (or any object inheriting from it), a \code{data.frame},
    \code{pedigree} or \code{pedigreeList} objects (the latter being
    defined in the \code{kinship2} package).
  }

  \item{ped}{
    Either a \code{data.frame} or a \code{pedigree} object specifying
    the pedigree. If a \code{data.frame} is submitted, the columns
    \code{id}, \code{family}, \code{father}, \code{mother} and
    \code{sex} are required.
  }

  \item{rmKinship}{
    For \code{shareKinship}: threshold for inclusion in being reported.
    Pairs of individuals with a kinship less or equal than this value
    will be omitted. This can be used to remove very distant relatives.
  }

  \item{...}{
    For \code{cliques}: additional arguments passed to the
    \code{cliques} function from the \code{igraph} package.
  }

}
\section{Basic pedigree utilities}{

  \describe{

    \item{countGenerations}{
      Count the generations up- or down the pedigree for the specified
      individual(s), i.e. determine the number of ancestor or offspring
      generations defined in the pedigree for the specified
      individual(s).
      Returns a named numeric vector, names corresponding to the
      individual's id, with the number of generations for each specified
      individual.
    }

    \item{findFounders}{
      Identifies the founder couple with the largest number of offspring
      generations in the pedigree. The provided pedigree
      object/data.frame can contain pedigrees of multiple families,
      thus, to identify the founder pair for a family its ID can be
      provided with the \code{family} parameter. Alternatively, the ID
      of an individual can be specified, in which case the founder pair
      of the (full) pedigree of the specified individual is identified.
      If two or more couples have the same, largest number of
      offspring generations, the first couple is selected.
      Returns a character vector of length 2 with the ids of the founder
      individuals.
    }

    \item{getFounders}{
      Returns the ids of all founders in the pedigree. A founder is an
      individual from which neither father nor mother is known in the
      pedigree.
    }

    \item{getSingletons}{
      Returns the ids of all singletons, i.e. individuals in the
      pedigree that are not connected to any other individual (have no
      parents in the pedigree and no children).
    }

    \item{getAncestors}{
      Identify and return the ids of ancestor generations (up to
      \code{max.generations}) for the specified individual(s).
    }

    \item{getChildren}{
      Identify and return the ids of offspring generations (up to
      \code{max.generations}) for the specified individual(s).
    }

    \item{getCommonAncestor}{
      Finds the closest common ancestor between specified individuals (2
      or more ids are required).
      Returns a character vector with the ids of the ancestors or \code{NA} if
      no common ancestor was found.
    }

    \item{getMissingMate}{
      The function evaluates if in the sub-pedigree defined by the
      specified ids one or more mates (spouse) are
      missing and if so it returns their ids.
    }

    \item{getSiblings}{
      Get siblings for the specified id(s). Returns their ids as
      character, or numeric vector.
    }

    \item{removeSingletons}{
      Removes all unconnected individuals (i.e. singletons) from the
      pedigree. Returns a \code{data.frame} with the pedigree cleaned
      from all singletons. Note that, due to internal sanitizing,
      columns \code{"father"} and \code{"mother"} in the resulting
      \code{data.frame} have a \code{NA} for individuals for which the
      father or mother is not known in the pedigree.
    }

    \item{subPedigree}{
      Finds the smallest pedigree containing all specified individuals.
      Depending on the input, a \code{data.frame}, \code{pedigree}
      or \code{pedigreeList}.
    }

  }
}

\section{Advanced pedigree methods}{
  \describe{

    \item{estimateGenerations}{
      Estimates generation levels/numbers for each, or only one, family
      in the object. Generation numbers are always relative to the
      founder couple (defined by \code{\link{findFounders}}).
      Returns (always) a named list of generation numbers. The names of
      the list represent the family id, the names of the numeric vector
      of generations the id of the individuals in the family.
    }

    \item{generationsFrom}{
      Determine generations starting from the specified
      individual. Siblings including their mates and all other in the
      same generation () are assigned generation 0, ancestor generations
      (all their parents etc) negative generation numbers, decreasing with
      ancestor level and their offspring positive numbers, increasing
      with each generation.
      Generations are only estimated within the family of the
      individual, also, if the pedigree consists of un-connected
      sub-pedigree, generation numbers will only be calculated for the
      sub-pedigree containing the specified individual.
      The function returns a named numeric vector of generation numbers,
      the names corresponding to the ids of the individuals in the
      specified individual's family. Not connected individuals in the
      family get a \code{NA} generation number.
    }

    \item{shareKinship}{
      Finds all related individuals (individuals sharing kinship >
      \code{rmKinship} with the individual) for the specified
      individual(s) in the pedigree and returns their ids as a character
      vector.
    }
  }
}

\section{Graph theory related functions}{
  \describe{
    \item{cliques}{
      Wrapper method passing all arguments to the \code{cliques}
      function from the \code{igraph} package.
    }

    \item{connectedSubgraph}{
      Finds the (eventually smallest) connected subgraph of all
      specified nodes.
      Returns an \code{igraph} object representing the subgraph of the
      specified nodes.
    }

    \item{ped2graph}{
      Transforms the pedigree into a (directed) graph with the direction
      of the edges being always from parent to child.
      An \code{igraph} object.
    }
  }
}
\value{
  Refer to the method and function description above for detailed
  information on the returned result object.
}
\author{
  Johannes Rainer.
}
\seealso{
  \code{\link[kinship2]{pedigree}},
  \code{\linkS4class{FAData}},
  \code{\linkS4class{FAProbResults}},
  \code{\linkS4class{FAKinGroupResults}},
  \code{\linkS4class{FAKinSumResults}},
  \code{\link{PedigreeAnalysis}}
  \code{\link{kinshipPairs}}
}
\examples{
##########################
##
##  Defining a small pedigree
##
## load the Minnesota Breast Cancer record and subset to the
## first families.
data(minnbreast)
mbsub <- minnbreast[minnbreast$famid==4 | minnbreast$famid==5, ]
mbped <- mbsub[, c("famid", "id", "fatherid", "motherid", "sex")]
## renaming column names
colnames(mbped) <- c("family", "id", "father", "mother", "sex")

## plot the pedigree for family 4 to get an overview.
switchPlotfun(method="ks2paint")
fam4 <- mbped[mbped$family==4, ]
doPlotPed(individual=fam4$id, father=fam4$father, mother=fam4$mother,
          gender=fam4$sex, device="plot")

## find the closest common ancestor between individuals 23, 3 and 8
getCommonAncestor(fam4, id=c(23, 3, 8))

## create the smallest sub-pedigree for individuals 21, 22 and 25
subPedigree(fam4, id=c(21, 22, 25))
## plot that
fam4sub <- subPedigree(fam4, id=c(21, 22, 25))
doPlotPed(individual=fam4sub$id, father=fam4sub$father, mother=fam4sub$mother,
          gender=fam4sub$sex, device="plot")

#########################
##
##  Basic pedigree utils
##
## Note: the same methods can be applied to a data.frame representing
## a pedigree, or a FAData, pedigree or pedigreeList object.

## Find the founder couple for family 4
findFounders(fam4, family=4)

## Alternatively, find the founders for the pedigree in which ibdividual 20 is a
## member
findFounders(fam4, id = 20)

## Return all founders in the pedigree.
getFounders(fam4)

## Get all founders without children (i.e. singletons).
getSingletons(fam4)

## Clean the pedigree from all singletons
fam4noS <- removeSingletons(fam4)
nrow(fam4)
nrow(fam4noS)

## Count the offspring generations for individual "4"
countGenerations(fam4, id="4")

## Get the ids of all ancestors for that individual
getAncestors(fam4, id="4")

## Get the ids of the children of this individual
getChildren(fam4, id="4", max.generations=1)

## Get the ids of the complete offspring for this individuals
getChildren(fam4, id="4")

## Create a FAData object from the pedigree data.frame
fad <- FAData(fam4)
## get the list of all ids sharing kinship with individuals
## 5 and 9
shareKinship(fad, id=c("5", "9"))

## Count the numbers of generations of ancestors for individual 12
countGenerations(fad, id="12", direction="up")

## Count the numbers of offspring generations for individuals 2 and 29
countGenerations(fad, id=c("2", "29"))

## Get all brothers/sisters for individual 9
getSiblings(fad, id="9")

## Determine generation levels starting from individual "9"
generationsFrom(fad, id="9")

## Estimate generations relative to the founder couple for each
## family in the submitted object, a data.frame in the example below
estimateGenerations(mbped)


#########################
##
##  Graph utilities
##
## Convert the pedigree into a graph
pgraph <- ped2graph(fam4)
plot(pgraph)

## Make a subgraph containing nodes 10, 22, 12 and 14
sgraph <- connectedSubgraph(pgraph, c("10", "22", "12", "14"))
plot(sgraph)

}
\keyword{classes}

\name{FABinTestResults-class}
\Rdversion{1.1}
\docType{class}
\alias{FABinTestResults-class}
\alias{FABinTestResults}
\alias{[,FABinTestResults,ANY-method}
\alias{[,FABinTestResults,ANY,ANY,ANY-method}
\alias{result,FABinTestResults-method}
\alias{trait<-,FABinTestResults-method}


\title{Binomial test for familial aggregation}

\description{
  The \code{FABinTestResults} object contains the results from a
  simple binomial to test whether the number of affected in a trait are
  higher than expected by chance. For more details on the method please
  see \code{\link{binomialTest}}.

}
\section{Objects from the Class}{
  \code{FABinTestResults} objects are returned by the
  \code{\link{binomialTest}} function.
}
\usage{

\S4method{result}{FABinTestResults}(object, method="BH")

\S4method{trait}{FABinTestResults}(object) <- value

}
\arguments{

  (in alphabetic order)

  \item{method}{
    The multiple hypothesis testing method. All methods supported by
    \code{\link{p.adjust}} are allowed.
  }

  \item{object}{
    The \code{FABinTestResults} object.
  }

  \item{value}{
    For \code{trait<-}: can be a named numeric, character or factor
    vector. The names (at least some of them) have to match the ids in
    the pedigree of the object.
  }

}
\section{Extends}{
  Class \code{\linkS4class{FAData}} directly.
}
\section{Slots}{
  \describe{
    \item{result}{
      The results \code{data.frame}.
    }
  }
}
\section{Methods and Functions}{
  \describe{

    \item{result}{
      Returns the result from the test as a \code{data.frame} with
      columns:
      
      \code{"trait_name"}: the name of the trait.
      
      \code{"total_phenotyped"}: total number of phenotyped individuals in
      the trait.
      
      \code{"total_affected"}: total number of affected individuals in the
      trait.
      
      \code{"family"}: the family id. If a global test is used (i.e. if
      the pedigree consists of a single family, or \code{global = TRUE}
      was provided, the column shows \code{"full pedigree"}).
      
      \code{phenotyped}: the number of phenotyped individuals in the
      family.

      \code{affected}: the number of affected individuals in the family.

      \code{pvalue}: the p-value from the binomial test (conducted using
      the \code{\link{binom.test}} function).

      \code{prob}: the probability of being affected. Either a
      \emph{local} probability calculated based on all affected and
      phenotyped individuals in the whole pedigree, or a \emph{global}
      (population) probability that has to be provided with argument
      \code{prob}.

      \code{padj}: the p-value adjusted for multiple hypothesis testing
      using the method defined with argument \code{"method"}.
    
    }

    \item{trait<-}{
      Set the trait information. This method will reset all simulation
      results saved in the \code{sim} slot.
    }

  }
}
\details{

  A call to the setter methods \code{trait<-} resets any test
  results present in the \code{result} slot, thus, the object can be
  re-used to perform a simulation analysis using the new trait data.

}
\note{
  Subsetting (using the \code{[} operator) is not supported.
}
\value{
  Refer to the method and function description above for detailed
  information on the returned result object.
}
\author{
  Johannes Rainer, Christian Weichenberger
}
\seealso{
  \code{\linkS4class{FAData}},
  \code{\link{kinship}},
  \code{\link{trait}},
  \code{\link{probabilityTest}},
  \code{\link{kinshipGroupTest}},
  \code{\link{kinshipSumTest}},
  \code{\link{genealogicalIndexTest}},
  \code{\link{familialIncidenceRateTest}},
  \code{\link{fsirTest}},
  \code{\link{plotPed}}
}
\examples{
##########################
##
##  Perform the analysis
##
## Load the test data.
data(minnbreast)

## Subset to some families and generate a pedigree data.frame.
mbsub <- minnbreast[minnbreast$famid == 4 | minnbreast$famid == 5 |
                    minnbreast$famid == 6 | minnbreast$famid == 7 |
                    minnbreast$famid == 8, ]
PedDf <- mbsub[, c("famid", "id", "fatherid", "motherid", "sex")]
colnames(PedDf) <- c("family", "id", "father", "mother", "sex")

## Generate the FAData.
fad <- FAData(pedigree=PedDf)

## Specify the trait.
tcancer <- mbsub$cancer
names(tcancer) <- mbsub$id

## Perform the test:
bir <- binomialTest(fad, trait=tcancer, traitName="cancer")
result(bir)

## Calculating the probability of being affected from the whole data set.
prob <- sum(minnbreast$cancer, na.rm = TRUE) / sum(!is.na(minnbreast$cancer))
bir <- binomialTest(fad, trait = tcancer, prob = prob)
result(bir)

## Plot the pedigree of the family with the smallest p-value.
plotPed(bir, family = "8")

}
\keyword{classes}

\name{PedigreeAnalysis}
\Rdversion{1.1}
\alias{PedigreeAnalysis}
\alias{estimateTimeAtRisk}
\alias{factor2matrix}
\alias{familialIncidenceRate}
\alias{familialIncidenceRate,FAData-method}
\alias{familialIncidenceRateTest}
\alias{familialIncidenceRateTest,FAData-method}
\alias{fsir}
\alias{fsir,FAData-method}
\alias{fsirTest}
\alias{fsirTest,FAData-method}
\alias{genealogicalIndexTest}
\alias{genealogicalIndexTest,FAData-method}
\alias{kinshipGroupTest}
\alias{kinshipGroupTest,FAData-method}
\alias{kinshipSumTest}
\alias{kinshipSumTest,FAData-method}
\alias{probabilityTest}
\alias{probabilityTest,FAData-method}
\alias{sliceAge}
\alias{binomialTest}

\title{Pedigree analysis and familial aggregation methods}

\description{
  Various functions to perform pedigree analyses and to investigate
  familial clustering of e.g. cancer cases.
}
\usage{

binomialTest(object, trait, traitName, global = FALSE, prob = NULL,
    alternative = c("greater", "less", "two.sided"))

estimateTimeAtRisk(startDate=NULL, startDateFormat="\%Y-\%m-\%d",
                   endDate=NULL, endDateFormat="\%Y-\%m-\%d",
                   incidenceDate=NULL, incidenceDateFormat="\%Y-\%m-\%d",
                   deathDate=NULL, deathDateFormat="\%Y-\%m-\%d",
                   allowNegative=FALSE, affected=NULL,
                   incidenceSubtract=0.5)

factor2matrix(x)

\S4method{familialIncidenceRate}{FAData}(object, trait=NULL,
                                         timeAtRisk=NULL)

\S4method{familialIncidenceRateTest}{FAData}(object, trait=NULL,
                                            nsim=50000, traitName=NULL,
                                            timeAtRisk=NULL,
                                            strata=NULL, ...)

\S4method{fsir}{FAData}(object, trait=NULL, lambda=NULL, timeInStrata=NULL)

\S4method{fsirTest}{FAData}(object, trait=NULL, nsim=50000, traitName=NULL,
                            lambda=NULL, timeInStrata=NULL,
                            strata=NULL, ...)

\S4method{genealogicalIndexTest}{FAData}(object, trait, nsim=50000,
                                         traitName, perFamilyTest=FALSE,
                                         controlSetMethod="getAll",
                                         rm.singletons=TRUE, strata=NULL, ...)

\S4method{kinshipGroupTest}{FAData}(object, trait, nsim=50000,
                                    traitName, strata=NULL, ...)

\S4method{kinshipSumTest}{FAData}(object, trait, nsim=50000,
                                  traitName, strata=NULL, ...)

\S4method{probabilityTest}{FAData}(object, trait, cliques,
                                   nsim=50000, traitName,
                                   ...)

sliceAge(x, slices=c(0, 40, Inf))
%connectedSubgraph(graph, nodes, mode="all", all.nodes=TRUE, ifnotfound)


}
\arguments{

  (in alphabetic order)

  \item{affected}{
    For \code{estimateTimeAtRisk}: optional parameter specifying which
    of the individuals are affected. This is useful if only \code{endDate} is
    specified, but not the \code{incidenceDate}. See method description
    for further details.
  }

  \item{allowNegative}{
    For \code{estimateTimeAtRisk}: if \code{FALSE} any negative time
    periods are set to \code{0}.
  }

  \item{alternative}{
    For \code{binomialTest}: the alternative hypothesis. See
    \code{\link{binom.test}} for more details. Defaults to
    \code{"greater"}, i.e. tests whether in a family a larger number of
    affected is present than expected by chance (given a global
    probability).
  }
  
  \item{cliques}{
    A named numeric or characted vector or factor with the names
    corresponding to ids of the individuals in the pedigree. The ids
    will be internally matched and sub-set to the ids available in the
    pedigree.
  }

  \item{controlSetMethod}{
    For \code{genealogicalIndexTest}: the method (i.e. name of the function)
    that should be used to define the set of (eventually matched)
    control individuals from which the random samples are
    taken. Supported functions are \code{\link{getAll}},
    \code{\link{getSexMatched}} and
    \code{\link{getExternalMatched}}. For \code{perFamilyTest=TRUE} also
    \code{\link{getGenerationMatched}} and
    \code{\link{getGenerationSexMatched}} are supported.
    Note: for \code{getExternalMatched}, a numeric, character or factor vector to
    be used for the matching has to be submitted
    as additional argument \code{match.using}.
  }

  \item{deathDate}{
    For \code{estimateTimeAtRisk}: the date of death.
  }

  \item{deathDateFormat}{
    For \code{estimateTimeAtRisk}: the format in which the dates are
    submitted. See \code{\link{as.Date}} for more information.
  }

  \item{endDate}{
    For \code{estimateTimeAtRisk}: the end date, which can be the end
    date for the study or, if \code{deathDate} and \code{incidenceDate}
    are not specified, the earliest time point of: date of incidence,
    death or end of study.
  }

  \item{endDateFormat}{
    For \code{estimateTimeAtRisk}: the format in which the dates are
    submitted. See \code{\link{as.Date}} for more information.
  }

  \item{global}{
    For \code{binomialTest}: whether the binomial test should be applied
    to the whole pedigree, or family-wise (default). If
    \code{global = TRUE} the population probability has to be provided
    with parameter \code{prob}.
  }
  
  \item{incidenceDate}{
    For \code{estimateTimeAtRisk}: the date of the incidence for an
    individual, i.e. the date when the status was changed from
    un-affected to affected in the to be analyzed trait.
  }

  \item{incidenceDateFormat}{
    For \code{estimateTimeAtRisk}: the format in which the dates are
    submitted. See \code{\link{as.Date}} for more information.
  }

  \item{incidenceSubtract}{
    For \code{estimateTimeAtRisk}: the amount of time (of the time unit
    of the time at risk) that should be
    subtracted from the calculated time at risk for affected
    individuals. See method description below for more details.
  }

  \item{lambda}{
    Numeric vector with the incidence rates per stratum from the
    population. The length of this vector has to match the number of
    columns of argument \code{timeInStrata}.
  }

  \item{nsim}{
    The number of simulations.
  }

  \item{object}{
    The \code{FAData} object.
  }

  \item{perFamilyTest}{
    For \code{genealogicalIndexTest}: whether the test should be performed
    on the whole pedigree (default) or separately within each
    family. In the latter case the test evaluates the presence of
    clustered affected individuals within each family.
  }

  \item{prob}{
    For \code{binomialTest}: the hypothesized probability of success
    (being affected) from/for the whole population.
  }
  
  \item{rm.singletons}{
    For \code{genealogicalIndexTest}:
    whether unconnected individuals in the pedigree (singletons) should
    be removed from the pedigree prior to the analysis.
  }

  \item{slices}{
    For \code{sliceAge}: a numeric vector defining the
    age-slices. Similar to argument \code{vec} for
    \code{\link{findInterval}}. Defines the minimum and maximum age for
    the age slices, i.e. first number corresponds to the lower boundary
    of the first age slice, the second number to the upper boundary of
    the first and lower boundary of the second age slice and so on.
  }

  \item{startDate}{
    For \code{estimateTimeAtRisk}: the date of the start of the
    study. Can also be the birth date.
  }

  \item{startDateFormat}{
    For \code{estimateTimeAtRisk}: the format in which the dates are
    submitted. See \code{\link{as.Date}} for more information.
  }

  \item{strata}{
    For \code{genealogicalIndexTest}, \code{kinshipGroupTest} and
    \code{kinshipSumTest}: a numeric, character or factor
    characterizing each individual in the pedigree. The length of this
    vector and the ordering has to match the pedigree.
    This vector allows to perform stratified random sampling. See
    details for more information.
  }

  \item{timeAtRisk}{
    A numeric vector specifying the \emph{time at risk} for each
    individual. The definition for this variable is taken from Kerber
    (1995). See description of the method below for more information.
    \code{timeAtRisk} has to have the same number of elements than there are
    individuals in the pedigree and it is assumed that the ordering of
    the vector matches the order of the individuals in the pedigree.
  }

  \item{timeInStrata}{
    For \code{fsir} and \code{fsirTest}: a numeric matrix specifying the time at
    risk for each individual in each strata. Columns represent the
    strata, rows the individuals, each cell the time at risk for the
    individual in the respective strata.
  }

  \item{trait}{
    A named numeric vector (values \code{0}, \code{1} and \code{NA}) or
    logical vector (values \code{FALSE}, \code{TRUE} and \code{NA}) with
    the names matching the ids of the individuals in the pedigree. The
    method internally matches and re-orders the trait vector to match
    the ordering of the ids in the pedigree.

    If trait is not specified, the trait information stored within the
    \code{FAData} object is used.
  }

  \item{traitName}{
    The name of the trait (optional).
  }

  \item{x}{
    For \code{sliceAge}: a numeric vector representing the age of
    individuals.
    For \code{factor2matrix}: a factor that should be converted into a
    matrix.
  }

  \item{...}{
    For \code{genealogicalIndexTest}: additional arguments passed to the choosen
    \code{controlSetMethod} function (e.g. \code{match.using} for
    \code{\link{getExternalMatched}}).

    For \code{familialIncidenceRateTest}: use \code{lowMem=TRUE} for
  very large pedigrees. This will use a faster and less memory demanding
  p-value estimation.
  }

}
\details{
  Stratified sampling: some of the familial aggregation methods allow to
  use stratified sampling for the Monte Carlo simulations. In stratified
  sampling, the same number of random samples will be selected within each
  class/stratum then there are among the affected. As example, if 5
  female and 2 male individuals are affected in the analysed trait and
  sex stratified sampling is performed, in each permuatation the same
  number of random samples in each group (i.e. 5 females and 2 males)
  are selected.

  A note on singletons: for all per-individual measures, unconnected individuals
  within the pedigree are automatically excluded from the calculations as no
  kinship based statistic can be estimated for them since they do, by definition,
  not share kinship with any other individual in the pedigree.

}
\section{Familial aggregation methods}{

  \describe{

    \item{binomialTest}{
      Evaluate whether the number of affected in a trait are higher than
      expected by chance using a simple binomial test. In contrast to
      most other methods presented here, this does not use the kinship
      between affected individuals, but simply performs a binomial test
      for each family considering the numbers of affected within the
      family, the size of the family and the global probability of being
      affected. The latter is by default calculated on the data set
      (ratio between the total number of affected in the pedigree and
      the total number of phenotyped individuals), can however also be
      specified with the \code{prob} argument.

      The test is performed using the \code{\link{binom.test}}.

      The function returns a \code{\linkS4class{FABinTestResults}} object.
    }
    
    \item{familialIncidenceRate}{
      Calculate the familial incidence rate (FIR, or FR) as defined in [Kerber
      1995], formula (3). The FIR is an estimate for the risk per
      gene-time for each individual for a certain disease (trait) given
      the disease experience in the cohort. The measure considers the
      kinship of each individual with any affected individual in the
      pedigree and the time at risk for each individual.

      Internally, the function first excludes individuals from the test
      which have a missing value (\code{NA}) either in the argument
      \code{trait} or in the argument \code{timeAtRisk}. Next, the thus
      reduced pedigree, is further cleaned by removing all resulting
      singletons (i.e. individuals that do not share kinship with any
      other individual in the above reduced data set).

      The method returns a vector with the FIR value for each
      individual. Individuals that were excluded from the test as
      described above habe an FIR of \code{NA}.
    }

    \item{familialIncidenceRateTest}{
      Calculates the familial incidence rate for each individual and in
      addition assesses the significance of these based on Monte Carlo
      simulations. See \code{\linkS4class{FAIncidenceRateResults}} for
      more details.

      The method returns a \code{\linkS4class{FAIncidenceRateResults}}
      object.
    }

    \item{fsir}{
      Calculate the familial standardized incidence rate (FSIR) as
      defined in [Kerber, 1995], formula (4). The FSIR weights the
      disease status of relatives based on their degree of relatedness
      with the proband [Kerber, 1995]. Formally, the FSIR is defined as
      the standardized incidence ratio (SIR) or standardized morality
      ratio in epidemiology, i.e. as the ratio between the observed and
      expected number of cases, only that both are in addition also
      weighted by the degree of relatedness (i.e. kinship value) between
      individuals in the pedigree.

      Similar to \code{familialIncidenceRate}, the function excludes
      individuals with missing values in any of the arguments
      \code{trait}, \code{timeInStrata} (and optionally \code{strata})
      and all individuals that do not share any kinship with any other
      individual in the pedigree after removing the above individuals.

      The method returns a vector with the FSIR value for each
      individual. Individuals excluded as above describe have a FSIR value
      of \code{NA}.
    }

    \item{fsirTest}{
      Calculates the familial standardized incidence rate (FSIR) for
      each individual and in addition assesses the significance of these
      based on Monte Carlo simulations. See
      \code{\linkS4class{FAStdIncidenceRateResults}} for more details.

      The method returns a \code{\linkS4class{FAStdIncidenceRateResults}}
      object.
    }

    \item{genealogicalIndexTest}{
      Performs the \emph{genealogical index} analysis from [Hill
      1980] (also known as the \emph{genealogical index of
      familiality} or \code{genetic index of familiality}) to identify
      familial clustering of traits (e.g. cancers etc).

      This test calculates the mean kinship among affected individuals in a
      pedigree along with mean kinships of equal sized random control
      sets drawn form the pedigree. The distribution of average kinship
      values among these random sets is used to estimate the probability
      that the observed mean kinship between the affected individuals is due
      to chance.
      The \code{controlSetMethod} argument allows to specify the method
      to define sets of matched control individuals in a pedigree or
      family.

      Note that by default singletons (i.e. unconnected individuals in
      the pedigree) are removed from the pedigree prior the
      analysis. Set \code{rm.singletons=FALSE} if you do not want them
      to be removed.

      The method can also be performed separately for each family within
      the larger pedigree (\code{perFamilyTest=TRUE} to evaluate the
      presence of clustered affected within each family).
      In this case it is also possible to
      use \code{controlSetMethod="getGenerationMatched"} or
      \code{controlSetMethod="getGenerationSexMatched"}, which allows to
      draw random control samples from the same generation(s) than the
      affected are.

      Stratified random sampling can be performed with the \code{strata}
      argument. See details for more information.

      The function returns a \code{\linkS4class{FAGenIndexResults}} object.
    }

    \item{kinshipGroupTest}{
      Performs a familial aggregation test on a subset of a family. The
      idea behind this test is to narrow down the set of controls for
      each affected individual by considering only individuals that are
      as closely related as the most distant affected individual. This
      strategy incorporates more the family structure of the cases and
      is meant to be an alternative to the \code{kinshipSumTest}
      method.

      Initially, for an affected individual \emph{i} a group \emph{C(i)}
      is created that contains all individuals that share kinship as far as
      the most distantly related affected individual. This cluster can be
      interpreted as a circle that is centered at individual \emph{i} with
      radius equal to the most distantly related case. Therefore, the
      cluster defines a narrowed, individual-specific set of individuals
      in which the phenotype is assumed to have been passed on.
      Groups consisting of the same set of affected individuals are
      reduced to a single group (i.e. the group with the smallest total
      number of individuals).

      Based on this definition of groups \emph{C(i)}, we compute two
      statistics by performing Monte Carlo simulations (which optionally
      allow to perform stratified random sampling). During each
      simulation step affected cases are randomly sampled from the
      population.

      1. The ratio test counts per group \emph{C(i)} the number of times
      we observe a higher number of affected individuals in the simulation
      than in the observed case. Dividing this number by the number of
      simulation steps yields immediately the p-value that describes the
      event to observe by chance a higher number of affected individuals
      than in the given case.

      2. The kinship test addresses the degree of relatedness within the
      simulated set by a counting method where we count the number of times
      in a simulation step there is a pair of affected individuals that are
      more closely related than in the observed group \emph{C(i)}. In case
      the closest degree of relatedness is equal in both the simulation
      step and the observed case, we look at the number of pairs found in
      both and count it if this number is higher in the simulation
      step. Again, dividing this count by the number of simulation steps
      readily yields a p-value.

      See also the method \code{runSimulation} for
      \code{\linkS4class{FAKinGroupResults}}.

      The function returns a \code{\linkS4class{FAKinGroupResults}} object.
    }

    \item{kinshipSumTest}{
      Performs a test for familial aggregation based on the sum of kinship
      values between affected cases. This test highlights individuals that
      exhibit a higher than chance relationship to other affected
      individuals, therefore highlighting individuals within families
      aggregating the phenotype. To achieve this, for each affected
      individual the sum of kinship values to all other affected cases is
      computed. In a Monte Carlo simulation this is repeated with the same
      number of cases (and optionally stratified with the \code{strata}
      argument), and the resulting background distribution is used to
      compute p-values for the kinship sums obtained from the observed
      cases. See also the method \code{runSimulation} for
      \code{\linkS4class{FAKinSumResults}}.

      The function returns a \code{\linkS4class{FAKinSumResults}} object.
    }

    \item{probabilityTest}{
      DEPRECATED: this test will be removed in Bioconductor version 3.8
      due to problems and incompatibilities of the \code{gap} package on
      MS Windows systems.
      
      This is only a convience method that calls the \code{gap} package's
      method \code{pfc.sim} to compute probabilities of familial clustering
      of phenotypes [Yu and Zelterman (2002)]. One drawback of that method
      is that it is limited to families with at most 22 individuals. Thus,
      pedigrees need to be split with specialized software such as Jenti
      [Falchi and Fuchsberger ea. (2008)], which within large families
      define cliques that can then be used as input to this algorithm.

      See also method \code{runSimulation} for
      \code{\linkS4class{FAProbResults}}.

      The function returns a \code{\linkS4class{FAProbResults}} object.
    }


  }
}
\section{Utility functions}{

  \describe{

    \item{factor2matrix}{
      Converts a factor into a matrix with columns corresponding to the
      levels and values (cell row i, column j) being either 0 or 1
      depending on whether the ith factor was of the level j. See
      examples below for in or \code{FAStdIncidenceRateResults}.
    }

    \item{estimateTimeAtRisk}{
      Function to calculate the time at risk based on the start date of
      the study or the birth date of an individual (\code{startDate})
      and the study's end date (\code{endDate}), the date of an
      incidence (e.g. date of diagnosis of a cancer
      \code{incidenceDate}) or the death of the individual
      (\code{deathDate}). The time at risk for each individual is
      calculated as the minimal time period between \code{startDate} and
      any of \code{endDate}, \code{incidenceDate} or
      \code{deathDate}. Thus it is also possible to provide just the
      \code{endDate} along with the \code{startDate}, in which case the
      \code{endDate} should be the earliest time point of: end date of
      the study, incidence date or date of death.

      For affected individuals (those for which either an incidence date
      is provided or the value in the optional argument \code{affected}
      is \code{TRUE} or bigger than 0), by default half of the time unit
      is subtracted. For example, a individual that has an incidence
      after 2 days is 1.5 days at risk. The proportion of the time unit
      to subtract can be specified with the argument
      \code{incidenceSubtract}.

      The function returns a numeric vector with the time at risk in
      days.
    }

    \item{sliceAge}{
      Generates a matrix with columns corresponding to age slices/strata
      defined by argument \code{slices} and rows to individuals. Each
      cell in a row represents the time spent by the individual in the
      age slice/strata. See example below.
    }

  }
}

\value{
  Refer to the method and function description above for detailed
  information on the returned result object.
}

\author{
  Johannes Rainer, Daniel Taliun, Christian Weichenberger.
}
\references{
  Rainer J, Talliun D, D'Elia Y, Domingues FS and Weichenberger CX
  (2016) FamAgg: an R package to evaluate familial aggregation of traits
  in large pedigrees. \emph{Bioinformatics}.
  
  Hill, J.R. (1980) A survey of cancer sites by kinship in the Utah Mormon
  population. In Cairns J, Lyon JL, Skolnick M (eds):
  \emph{Cancer Incidence in Defined Populations. Banbury Report 4.}
  Cold Spring Harbor, NY: Cold Spring Harbor Laboratory Press, pp 299--318.

  Kerber, R.A. (1995) Method for calculating risk associated with family
  history of a disease. \emph{Genet Epidemiol}, pp 291--301.

  Yu, C. and Zelterman, D. (2002) Statistical inference for familial disease
  clusters. \emph{Biometrics}, pp 481--491

  Falchi, M. and Fuchsberger, C. (2008) Jenti: an efficient tool for mining
  complex inbred genealogies. \emph{Bioinformatics}, pp 724--726
}

\seealso{
  \code{\link[kinship2]{pedigree}},
  \code{\linkS4class{FAData}},
  \code{\linkS4class{FAProbResults}},
  \code{\linkS4class{FAKinGroupResults}},
  \code{\linkS4class{FAKinSumResults}},
  \code{\linkS4class{FAIncidenceRateResults}}
}
\examples{
##########################
##
##  Defining a small pedigree
##
## load the Minnesota Breast Cancer record and subset to the
## first families.
data(minnbreast)
mbsub <- minnbreast[minnbreast$famid==4 | minnbreast$famid==5 |
                    minnbreast$famid==14 | minnbreast$famid==8, ]
mbped <- mbsub[, c("famid", "id", "fatherid", "motherid", "sex")]
## renaming column names
colnames(mbped) <- c("family", "id", "father", "mother", "sex")
## create the FAData object
fad <- FAData(pedigree=mbped)

## We specify the cancer trait.
tcancer <- mbsub$cancer
names(tcancer) <- mbsub$id

##########################
##
## Familial Incidence Rate
##
## Calculate the FR for each individual given the affected status of
## each individual in trait cancer and the time at risk for each
## participant. We use column "endage" in the minnbreast data.frame
## that specifies the age at the last follow-up or incident cancer as a
##rather impresice estimate for time at risk.
fr <- familialIncidenceRate(fad, trait=tcancer, timeAtRisk=mbsub$endage)

## Plot the distribution of familial rates
plot(density(fr, na.rm=TRUE))

## Perform in addition Monte Carlo simulations to assess the significance
## for the familial incidence rates.
frRes <- familialIncidenceRateTest(fad, trait=tcancer,
                                   timeAtRisk=mbsub$endage,
                                   nsim=500)
head(result(frRes))


##########################
##
## Familial Standardized Incidence Rate:
## Please see examples of FAStdIncidenceRateResults.



##########################
##
##  Perform familial aggregation analyses using the genealogical index
##
gi <- genealogicalIndexTest(fad, trait=tcancer, traitName="cancer",
                            nsim=500)
result(gi)
## A significant clustering of cancer cases was identified in the
## analyzed pedigree.

## Plotting the observed mean kinship and the distribution of mean kinship
## from the random sampling.
plotRes(gi)


##########################
##
##  Perform familial aggregation analysis using the kinship sum test
##
kcr <- kinshipSumTest(fad, trait=tcancer, traitName="cancer",
                      nsim=500)
kcr
head(result(kcr))


##########################
##
##  Perform familial aggregation analysis using the kinship group test,
##  stratifying by sex
##
kr <- kinshipGroupTest(fad, trait=tcancer, traitName="cancer",
                       nsim=500, strata=fad$sex)
kr
head(result(kr))



##########################
##
##  Estimate the time at risk given
##
## Define some birth dates and incidence dates and end date of study
bdates <- c("2012-04-17", "2014-05-29", "1999-12-31", "2002-10-10")
idates <- c(NA, NA, "2007-07-13", "2013-12-23")
edates <- rep("2015-09-15", 4)

## Estimate the time at risk. The time period is returned in days.
riskDays <- estimateTimeAtRisk(startDate=bdates, incidenceDate=idates, endDate=edates)
riskDays

##########################
##
##  Define the time spent in an age stratum given the indivduals'
##  age at incidence or end of study.
head(mbsub$endage)
## We "slice" the age in specified intervals/slices
stratAge <- sliceAge(mbsub$endage, slices=c(0, 40, 60, Inf))
head(stratAge)

## The first column lists the number of years spent in the first age
## stratum (0 < age <= 40) and the second in the second stratum
## (40 < age <= Inf)

## We could also stratify the disk days from above in per year strata.
sliceAge(riskDays/365, slices=c(0, 2.5, 5, 10, 20))


##########################
##
##  Simple example for factor2matrix: generate a matrix for factor $sex
head(factor2matrix(fad$sex))




}
\keyword{classes}

\name{FAIncidenceRateResults-class}
\Rdversion{1.1}
\docType{class}
\alias{$,FAIncidenceRateResults-method}
\alias{FAIncidenceRateResults-class}
\alias{FAIncidenceRateResults}
\alias{[,FAIncidenceRateResults,ANY-method}
\alias{[,FAIncidenceRateResults,ANY,ANY,ANY-method}
\alias{familialIncidenceRate,FAIncidenceRateResults-method}
\alias{plotPed,FAIncidenceRateResults-method}
\alias{plotRes,FAIncidenceRateResults-method}
\alias{result,FAIncidenceRateResults-method}
\alias{runSimulation,FAIncidenceRateResults-method}
\alias{timeAtRisk}
\alias{timeAtRisk<-}
\alias{timeAtRisk,FAIncidenceRateResults-method}
\alias{timeAtRisk<-,FAIncidenceRateResults-method}
\alias{trait<-,FAIncidenceRateResults-method}


\title{Familial Incidence Rate}

\description{
  The \code{FAIncidenceRateResults} object contains the results from a
  familial incidence rate calculation employing in addition Monte Carlo
  simulations to assess significance levels for the familial incidence
  rate of each individual.
  The familial incidence rate (FIR, also referred to as FR in Kerber
  (1995)) is an estimate for the risk per gene-time for each individuals
  for a certain disease given the disease experience in the cohort. The
  measure considers the kinship of each individual with any affected
  individual in the pedigree and the time at risk for each.

  Note that in contrast to e.g. \code{\link{FAKinSumResults}} a familial
  incidence rate and corresponding p-value are calculated and available
  for all individuals in the pedigree, not only for affected individuals.
}

\section{Objects from the Class}{
  \code{FAIncidenceRateResults} objects are created by the
  \code{\link{familialIncidenceRateTest}} method on a
  \code{\linkS4class{FAData}} object.
}
\usage{

\S4method{familialIncidenceRate}{FAIncidenceRateResults}(object, trait=NULL,
                                                         timeAtRisk=NULL,
                                                         ...)

\S4method{plotPed}{FAIncidenceRateResults}(object, id=NULL, family=NULL,
                                           filename=NULL, device="plot",
                                           only.phenotyped=FALSE, ...)

\S4method{plotRes}{FAIncidenceRateResults}(object, id=NULL, family=NULL,
                                           addLegend=TRUE, type="density", ...)

\S4method{result}{FAIncidenceRateResults}(object, method="BH")

\S4method{runSimulation}{FAIncidenceRateResults}(object, nsim=50000,
                                                 timeAtRisk=NULL,
                                                 strata=NULL, ...)

\S4method{timeAtRisk}{FAIncidenceRateResults}(object)

\S4method{timeAtRisk}{FAIncidenceRateResults}(object) <- value

\S4method{trait}{FAIncidenceRateResults}(object) <- value

}
\arguments{

  (in alphabetic order)

  \item{addLegend}{
    For \code{plotRes}: if a legend should be added to the plot.
  }

  \item{device}{
    For \code{plotPed}: see \code{\link{plotPed}} for more details.
  }

  \item{family}{
    For \code{plotPed}: the family for which the pedigree should be
    plotted.
    For \code{plotRes}: not supported.
  }

  \item{filename}{
    For \code{plotPed}: the file name to which the pedigree plot should
    be exported. See \code{\link{plotPed}} for more details.
  }

  \item{id}{
    For \code{plotPed} and \code{plotRes}: the id of the indiviual
    for which the pedigree or the simulation result should be plotted.
    Note: \code{id} can be a \code{numeric} or a
    \code{character}. Numeric ids will be internally converted to
    \code{character}.
  }

  \item{method}{
    The multiple hypothesis testing method. All methods supported by
    \code{\link{p.adjust}} are allowed.
  }

  \item{nsim}{
    Number of simulations.
  }

  \item{object}{
    The \code{FAIncidenceRateResults} object.
  }

  \item{only.phenotyped}{
    For \code{plotPed}: Wheter only phenotyped individuals,
    i.e. individuals with a non-NA value in column \code{affected} (the
    trait information). If \code{TRUE}, the function removes all
    non-phenotyped individuals, keeping only those that are required for
    the pedigree to be complete.
  }

  %% \item{rm.singletons}{
  %%   For \code{familialIncidenceRate}: not used, for
  %%   \code{familialIncidenceRateTest}: remove singletons from the data
  %%   set prior to the analysis. This does however not have an impact on
  %%   the results, as the FIR for singletons will anyway be \code{NA}
  %%   because they do not share kinship with any other individual in the
  %%   pedigree.
  %% }

  \item{strata}{
    For \code{runSimulation}: a numeric, character of factor
    characterizing each individual in the pedigree. The length of this
    vector and the ordering has to match the pedigree.
    This vector allows to perform stratified random sampling. See
    details on the \code{\link{PedigreeAnalysis}} help page or examples
    for more information.

  }

  \item{timeAtRisk}{
    For \code{runSimulation}: a numeric vector specifying the time at
    risk for each individual in the given trait. See also
    \code{\link{estimateTimeAtRisk}}, an utility function to estimate
    time at risk.
    For \code{familialIncidenceRate}: not used.

  }

  \item{trait}{
    For \code{familialIncidenceRate}: not used.
  }

  \item{type}{
    For \code{plotRes}: at present only \code{"density"} is supported.
  }

  \item{value}{
    For \code{trait<-}: can be a named numeric, character or factor
    vector. The names (at least some of them) have to match the ids in
    the pedigree of the object.
    For \code{timeAtRisk}: a numeric vector with the time at risk for
    each individual.
  }

  \item{...}{
    For \code{plotPed}: additional arguments to be submitted to the
    internal \code{buildPed} call and to \code{\link{plotPed}}.

    For \code{runSimulation}: additional arguments \code{prune} and
    \code{lowMem}. See below for details.
  }

}
\section{Extends}{
  Class \code{\linkS4class{FAData}} directly.
}
\section{Slots}{
  \describe{
    \item{nsim}{
      Number of simulations.
    }
    \item{sim}{
      The result of the simulation. This slot should not be accessed
      directly, use the \code{result} method to extract result
      information.
    }
    \item{timeAtRisk}{
      Numeric vector with the time at risk for each individual. Use the
      accessor method \code{timeAtRisk} or use \code{object$tar} to
      extract this data.
    }
  }
}
\section{Methods and Functions}{
  \describe{

    \item{object$name}{
      Access the familial incidence rate using \code{object$fir}, the
      (raw) p-value from the simulation using \code{object$pvalue} and the
      time at risk for each individual using \code{object$tar} or
      \code{object$timeAtRisk} with \code{object} being the
      \code{FAIncidenceRateResults} object.
    }

    \item{familialIncidenceRate}{
      Returns the familial incidence rate values calculated by a call to
      the \code{runSimulation} method or
      \code{\link{familialIncidenceRate}} method on a
      \code{\link{FAData}} object. In contrast to that latter method,
      which directly calculates the values, this method returns the
      values from a calculation stored inside the
      \code{FAIncidenceRateResults} object.

      The method returns a named numeric vector with the familial
      incidence rates for all individuals in the pedigree, the names
      being the ID of the individuals. Singletons as well as individuals
      that, after removing not phenotyped individuals or individuals
      without time at risk, do not share kinship with any other
      individual in the pedigree have a value of \code{NA}.
    }

    \item{plotPed}{
      Plots a pedigree for one of the affected individuals in the
      simulation results. The id of the selected affected individual
      (specified with argument \code{id}) is highlighted in red. The
      familial incidence rate value for each individual is drawn below
      the individual's id.
      See \code{\link{plotPed}} for more details.
    }

    \item{plotRes}{
      Plots the distribution of expected familial incidence rates
      calculated for the selected individuals from Monte Carlo
      simulations along with the actually observed familial incidence
      rate.
    }

    \item{result}{
      Returns the result from the simulation as a \code{data.frame} with
      columns:

      \code{"trait_name"}: the name of the trait.

      \code{"total_phenotyped"}: total number of phenotyped individuals in
      the trait.

      \code{"total_affected"}: total number of affected individuals in the
      trait.

      \code{"total_tested"}: the number of individuals in the pedigree
      considered for the simulation. This corresponds to all individuals
      with valid, non-NA, values in trait, timeAtRisk and eventually
      strata.

      \code{"id"}: the id of the individual.

      \code{"family"}: the family id.

      \code{"fir"}: the familial incidence rate. Note that this will be

      \code{NA} for all non-phenotyped individuals and singletons in the pedigree as well as for
      individuals that do not share kinship with at least one other
      phenotyped individual with valid time at risk (or valid value in
      parameter \code{strata}).

      \code{"pvalue"}: the p-value for the significance of the familial
      incidence rate assessed by Monte Carlo simulations.

      \code{"padj"}: the p-value adjusted for multiple hypothesis
      testing (with the method specified with argument \code{method}).

      The returned \code{data.frame} is sorted by column
      \code{"pvalue"}, its row names correspond to column
      \code{"id"}.
    }

    \item{runSimulation}{
      Performs the simulation analysis based on the pedigree and trait
      information stored in the object as well as the time at risk
      provided with argument \code{timeAtRisk}.
      Returns a \code{FAIncidenceRateResults} object with the results
      from the simulation.
    }

    \item{trait<-}{
      Set the trait information. This method will reset all simulation
      results saved in the \code{sim} slot.
    }

  }
}
\details{
  Monte Carlo simulation and empirical p-value estimation: the
  background distribution to calculate the p-value for a familial
  incidence rate (FIR) is determined by randomly sampling N affected
  individuals (N being the number of affected) and calculating the
  \emph{expected} FIR for all individuals in each simulation
  iteration. The p-value for an individual represents thus the number of
  times an expected FIR for that individual from the simulation was
  found to be larger than or equal to the observed FIR divided by the
  number of iterations.

  Calling the \code{runSimulation} method on a \code{FAIncidenceRateResults}
  object is the same as calling the
  \code{\link{familialIncidenceRateTest}} on a
  \code{\linkS4class{FAData}} object. In the first case the simulation
  is performed using the trait information data stored internally in the
  object, while in the latter case the trait information have to be
  submitted to the function call.

  By providing argument \code{strata}, the stratified random sampling is
  performed. See example below and the details section in
  \code{\link{PedigreeAnalysis}} for more details.

  The familial incidence rate can also be directly calculated, without
  simulation, using the \code{\link{familialIncidenceRate}} method of a
  \code{\link{FAData}} object.

  A call to the setter methods \code{trait<-} resets any simulation
  results present in the \code{sim} slot, thus, the object can be
  re-used to perform a simulation analysis using the new trait data.

  By default the Monte Carlo p-value estimation in the
  \code{runSimulation} method is quite memory demanding. For very large
  pedigrees the optional argument \code{lowMem=TRUE} might be passed to
  the method which results in faster and less memory demanding
  calculations. This will however disable the \code{plotRes} method on
  the resulting \code{FAIncidenceRateResults} as the distribution of
  familial incidence rates from the simulation runs is no longer
  reported.

  Note: the FIR for singletons and individuals that do not share kinship
  with at least one other phenotyped individual that has also a valid
  value in argument \code{timeAtRisk} (and eventually \code{strata})
  will be \code{NA}.
}
\note{
  Subsetting (using the \code{[} operator) is not supported.
}
\value{
  Refer to the method and function description above for detailed
  information on the returned result object.
}
\author{
  Johannes Rainer
}
\references{
  Kerber, R.A. (1995) Method for calculating risk associated with family
  history of a disease. \emph{Genet Epidemiol}, pp 291--301.
}
\seealso{
  \code{\linkS4class{FAData}},
  \code{\link{kinship}},
  \code{\link{trait}},
  \code{\link{probabilityTest}},
  \code{\link{kinshipGroupTest}},
  \code{\link{kinshipSumTest}},
  \code{\link{genealogicalIndexTest}},
  \code{\link{familialIncidenceRateTest}},
  \code{\link{fsirTest}},
  \code{\link{plotPed}},
  \code{\link{estimateTimeAtRisk}}
}
\examples{
##########################
##
##  Perform the simulation analysis
##
## Load the test data.
data(minnbreast)

## Subset to some families and generate a pedigree data.frame.
mbsub <- minnbreast[minnbreast$famid == 4 | minnbreast$famid == 5 |
                    minnbreast$famid == 6 | minnbreast$famid == 7 |
                    minnbreast$famid == 411, ]
PedDf <- mbsub[, c("famid", "id", "fatherid", "motherid", "sex")]
colnames(PedDf) <- c("family", "id", "father", "mother", "sex")

## Generate the FAData.
fad <- FAData(pedigree=PedDf)

## Specify the trait.
tcancer <- mbsub$cancer
names(tcancer) <- mbsub$id

## Spefify the "time at risk"; we are using column "endage"
tar <- mbsub$endage

## Perform the simulation test:
far <- familialIncidenceRateTest(fad, trait=tcancer, traitName="cancer",
                                 timeAtRisk=tar, nsim=1000)
head(result(far))

## We can easily extract the actual FIR values:
head(far$fir)
## Or
head(familialIncidenceRate(far))

## Access the p-value directly.
head(far$pvalue)

## Access the time at risk
head(timeAtRisk(far))
head(far$tar)
head(far$timeAtRisk)

## Plot the pedigree for a family with significant FIRs.
## The numbers below the IDs of the individuals represent the actual
## FIR values.
plotPed(far, family=result(far)$family[1])

## Plot also the result from the simulation run.
plotRes(far, id=result(far)$id[1])

}
\keyword{classes}

\name{FAStdIncidenceRateResults-class}
\Rdversion{1.1}
\docType{class}
\alias{$,FAStdIncidenceRateResults-method}
\alias{FAStdIncidenceRateResults-class}
\alias{FAStdIncidenceRateResults}
\alias{[,FAStdIncidenceRateResults,ANY-method}
\alias{[,FAStdIncidenceRateResults,ANY,ANY,ANY-method}
\alias{fsir,FAStdIncidenceRateResults-method}
\alias{lambda}
\alias{lambda,FAStdIncidenceRateResults-method}
\alias{plotPed,FAStdIncidenceRateResults-method}
\alias{plotRes,FAStdIncidenceRateResults-method}
\alias{result,FAStdIncidenceRateResults-method}
\alias{resultForId}
\alias{resultForId,FAStdIncidenceRateResults-method}
\alias{runSimulation,FAStdIncidenceRateResults-method}
\alias{timeInStrata}
\alias{timeInStrata,FAStdIncidenceRateResults-method}
\alias{trait<-,FAStdIncidenceRateResults-method}


\title{Familial Standardized Incidence Rate}

\description{
  The \code{FAStdIncidenceRateResults} object contains the results from a
  familial standardized incidence rate (FSIR) calculation employing in
  addition Monte Carlo simulations to assess significance levels for the
  individuals' FSIRs.

  The FSIR weights the disease status of relatives based on their degree
  of relatedness with the proband (Kerber, 1995). Formally, the FSIR is
  defined as the standardized incidence ratio (SIR) or standardized
  morality ratio in epidemiology, i.e. as the ratio between the observed
  and expected number of cases, only that both are in addition also
  weighted by the degree of relatedness (i.e. kinship value) between
  individuals in the pedigree.

}

\section{Objects from the Class}{
  \code{FAStdIncidenceRateResults} objects are created by the
  \code{\link{fsirTest}} method on a
  \code{\linkS4class{FAData}} object.
}
\usage{

\S4method{fsir}{FAStdIncidenceRateResults}(object, trait=NULL, lambda=NULL,
                                           timeInStrata=NULL,
                                           ...)

\S4method{lambda}{FAStdIncidenceRateResults}(object, ...)

\S4method{plotPed}{FAStdIncidenceRateResults}(object, id=NULL, family=NULL,
                                              filename=NULL, device="plot",
                                              only.phenotyped=FALSE, ...)

\S4method{plotRes}{FAStdIncidenceRateResults}(object, id=NULL, family=NULL,
                                              addLegend=TRUE, type="density", ...)

\S4method{result}{FAStdIncidenceRateResults}(object, method="BH")

\S4method{resultForId}{FAStdIncidenceRateResults}(object, id=NULL)

\S4method{runSimulation}{FAStdIncidenceRateResults}(object, nsim=50000,
                                                    lambda=NULL,
                                                    timeInStrata=NULL,
                                                    strata=NULL,
                                                    ...)

\S4method{timeInStrata}{FAStdIncidenceRateResults}(object)

\S4method{trait}{FAStdIncidenceRateResults}(object) <- value

}
\arguments{

  (in alphabetic order)

  \item{addLegend}{
    For \code{plotRes}: if a legend should be added to the plot.
  }

  \item{device}{
    For \code{plotPed}: see \code{\link{plotPed}} for more details.
  }

  \item{family}{
    For \code{plotPed}: the family for which the pedigree should be
    plotted.
    For \code{plotRes}: not supported.
  }

  \item{filename}{
    For \code{plotPed}: the file name to which the pedigree plot should
    be exported. See \code{\link{plotPed}} for more details.
  }

  \item{id}{
    For \code{plotPed} and \code{plotRes}: the id of the indiviual
    for which the pedigree or the simulation result should be plotted.
    Note: \code{id} can be a \code{numeric} or a
    \code{character}. Numeric ids will be internally converted to
    \code{character}.

    For \code{resultForId}: the ID of the individual from which the
    result should be returned.
  }

  \item{lambda}{
    Numeric vector with the incidence rates per stratum from the
    population. The length of this vector has to match the number of
    columns of argument \code{timeInStrata}.

    For \code{fsir}: not used.
  }

  \item{method}{
    The multiple hypothesis testing method. All methods supported by
    \code{\link{p.adjust}} are allowed.
  }

  \item{nsim}{
    Number of simulations.
  }

  \item{object}{
    The \code{FAStdIncidenceRateResults} object.
  }

  \item{only.phenotyped}{
    For \code{plotPed}: Wheter only phenotyped individuals,
    i.e. individuals with a non-NA value in column \code{affected} (the
    trait information). If \code{TRUE}, the function removes all
    non-phenotyped individuals, keeping only those that are required for
    the pedigree to be complete.
  }

  \item{strata}{
    For \code{runSimulation}: a numeric, character of factor
    characterizing each individual in the pedigree. The length of this
    vector and the ordering has to match the pedigree.
    This vector allows to perform stratified random sampling. See
    details on the \code{\link{PedigreeAnalysis}} help page or examples
    for more information.

  }

  \item{timeInStrata}{
    For \code{runSimulation}: a numeric matrix specifying the time at
    risk for each individual in each strata. Columns represent the
    strata, rows the individuals, each cell the time at risk for the
    individual in the respective strata. See example below. The
    \code{\link{factor2matrix}} could be useful in generating such a
    table.

    For \code{fsir}: not used.
  }

  \item{trait}{
    For \code{fsir}: not used.
  }

  \item{type}{
    For \code{plotRes}: at present only \code{"density"} is supported.
  }

  \item{value}{
    For \code{trait<-}: can be a named numeric, character or factor
    vector. The names (at least some of them) have to match the ids in
    the pedigree of the object.
  }

  \item{...}{
    For \code{plotPed}: additional arguments to be submitted to the
    internal \code{buildPed} call and to \code{\link{plotPed}}.

    For \code{runSimulation}: additional arguments \code{prune} and
    \code{lowMem}. See below for details.
  }

}
\section{Extends}{
  Class \code{\linkS4class{FAData}} directly.
}
\section{Slots}{
  \describe{
    \item{nsim}{
      Number of simulations.
    }
    \item{sim}{
      The result of the simulation. This slot should not be accessed
      directly, use the \code{result} method to extract result
      information.
    }
    \item{timeInStrata}{
      Numeric matrix specifying the time at risk in each strata
      (represented by the columns) of each individual (rows). Use the
      accessor method \code{timeInStrata} to
      extract this data.
    }
    \item{lambda}{
      Numeric vector with the incidence rates per stratum from the
      population.
    }
  }
}
\section{Methods and Functions}{
  \describe{

    \item{object$name}{
      Access various results and data stored in the object. \code{name}
      can be \code{fsir}, \code{pvalue}, \code{lambda} or
      \code{timeInStrata} to access the FSIR, the (raw) p-value from the
      simulation analysis, the lambda or the time in strata.
    }

    \item{fsir}{
      Returns the FSIR values calculated by a call to
      the \code{runSimulation} method or
      \code{\link{fsir}} method on a
      \code{\link{FAData}} object. In contrast to that latter method,
      which directly calculates the values, this method returns the
      values from a calculation stored inside the
      \code{FAStdIncidenceRateResults} object.

      The method returns a named numeric vector with the familial
      incidence rates, the names being the ID of the individuals.
      Singletons as well as individuals
      that, after removing not phenotyped individuals or individuals
      without time at risk, do not share kinship with any other
      individual in the pedigree have a value of \code{NA}.
    }

    \item{plotPed}{
      Plots a pedigree for one of the affected individuals in the
      simulation results. The id of the selected affected individual
      (specified with argument \code{id}) is highlighted in red. The
      FSIR value for each individual is drawn below
      the individual's id.
      See \code{\link{plotPed}} for more details.
    }

    \item{plotRes}{
      Plots the distribution of expected FSIR
      calculated for the selected individuals from Monte Carlo
      simulations along with the actually observed FSIR.
    }

    \item{result}{
      Returns the result from the simulation as a \code{data.frame} with
      columns:

      \code{"trait_name"}: the name of the trait.

      \code{"total_phenotyped"}: total number of phenotyped individuals in
      the trait.

      \code{"total_affected"}: total number of affected individuals in the
      trait.

      \code{"total_tested"}: the number of individuals in the pedigree
      considered for the simulation. This corresponds to all individuals
      with valid, non-NA, values in trait, timeAtRisk and eventually
      strata.

      \code{"id"}: the id of the individual.

      \code{"family"}: the family id.

      \code{"fsir"}: the familial standardized incidence rate. Note that this will be

      \code{NA} for all non-phenotyped individuals and singletons in the pedigree as well as for
      individuals that do not share kinship with at least one other
      phenotyped individual with valid time at risk (or valid value in
      parameter \code{strata}).

      \code{"pvalue"}: the p-value for the significance of the familial
      standardized incidence rate assessed by Monte Carlo simulations.

      \code{"padj"}: the p-value adjusted for multiple hypothesis
      testing (with the method specified with argument \code{method}).

      The returned \code{data.frame} is sorted by column
      \code{"pvalue"}, its row names correspond to column
      \code{"id"}.
    }

    \item{resultForId}{
      Extracts results information for a given individual. The method
      returns a \code{list} with elements \code{"id"}, \code{"fsir"},
      \code{"pvalue"}, \code{"timeInStrata"} and \code{"lambda"} with
      the ID of the individual, the FSIR and corresponding p-value
      estimated by the simulation, the row from the \code{timeInStrata}
      matrix of the individual and the \code{lambda} (incidence rates
      from the population).
    }

    \item{runSimulation}{
      Performs the simulation analysis based on the pedigree and trait
      information stored in the object as well as the time at risk
      provided with argument \code{timeAtRisk}.
      Returns a \code{FAStdIncidenceRateResults} object with the results
      from the simulation.
    }

    \item{trait<-}{
      Set the trait information. This method will reset all simulation
      results saved in the \code{sim} slot.
    }

  }
}
\details{
  Monte Carlo simulation and empirical p-value estimation: see details
  of \code{\link{FAIncidenceRateResults}} as the concept and calculation
  is essentially identical.

  Calling the \code{runSimulation} method on a \code{FAStdIncidenceRateResults}
  object is the same as calling the
  \code{\link{fsirTest}} on a
  \code{\linkS4class{FAData}} object. In the first case the simulation
  is performed using the trait information data stored internally in the
  object, while in the latter case the trait information have to be
  submitted to the function call.

  By providing argument \code{strata}, the stratified random sampling is
  performed. See example below and the details section in
  \code{\link{PedigreeAnalysis}} for more details.

  The FSIR can also be directly calculated, without
  simulation, using the \code{\link{fsir}} method of a
  \code{\link{FAData}} object.

  A call to the setter methods \code{trait<-} resets any simulation
  results present in the \code{sim} slot, thus, the object can be
  re-used to perform a simulation analysis using the new trait data.

  By default the Monte Carlo p-value estimation in the
  \code{runSimulation} method is quite memory demanding. For very large
  pedigrees the optional argument \code{lowMem=TRUE} might be passed to
  the method which results in faster and less memory demanding
  calculations. This will however disable the \code{plotRes} method on
  the resulting \code{FAStdIncidenceRateResults} as the distribution of
  FSIR from the simulation runs is no longer reported.

  Note: the FIR for singletons and individuals that do not share kinship
  with at least one other phenotyped individual that has also a valid
  value in argument \code{timeAtRisk} (and eventually \code{strata})
  will be \code{NA}.
}
\note{
  Subsetting (using the \code{[} operator) is not supported.
}
\value{
  Refer to the method and function description above for detailed
  information on the returned result object.
}
\author{
  Johannes Rainer
}
\references{
  Kerber, R.A. (1995) Method for calculating risk associated with family
  history of a disease. \emph{Genet Epidemiol}, pp 291--301.
}
\seealso{
  \code{\linkS4class{FAData}},
  \code{\link{kinship}},
  \code{\link{trait}},
  \code{\link{probabilityTest}},
  \code{\link{kinshipGroupTest}},
  \code{\link{kinshipSumTest}},
  \code{\link{genealogicalIndexTest}},
  \code{\link{familialIncidenceRateTest}},
  \code{\link{fsirTest}},
  \code{\link{plotPed}},
  \code{\link{estimateTimeAtRisk}}
}
\examples{
##########################
##
##  Perform the simulation analysis
##
## Load the test data.
data(minnbreast)

## Subset to some families and generate a pedigree data.frame.
mbsub <- minnbreast[minnbreast$famid == 4 | minnbreast$famid == 5 |
                    minnbreast$famid == 6 | minnbreast$famid == 7 |
                    minnbreast$famid == 411, ]
PedDf <- mbsub[, c("famid", "id", "fatherid", "motherid", "sex")]
colnames(PedDf) <- c("family", "id", "father", "mother", "sex")

## Generate the FAData.
fad <- FAData(pedigree=PedDf)

## Specify the trait.
tcancer <- mbsub$cancer
names(tcancer) <- mbsub$id

## Next we have to specify lambda and the timeInStrata matrix.
## For lambda we use information from Cancer Research UK:
## New breast cancer cases in females per year and 100000: 155.3
## New breast cancer cases in males per year and 100000:     1.1
## New prostate cancer cases in females per year and 100000: 0
## New prostate cancer cases in males per year and 100000: 134.3
lbda <- c(M=(1.1+134.4)/100000, F=155.3/100000)

## Next we need the time at risk of each individual in each strata.
## For strata we use male and females, for the time at risk we use
## column "endage":
stratMat <- factor2matrix(fad$sex)
## Next we have to multiply that with the endage, since each person
## spent that time "at risk" to get cancer.
stratMat <- stratMat * mbsub$endage

## Running the simulation
fsirs <- fsirTest(fad, trait=tcancer, traitName="cancer", lambda=lbda,
                  timeInStrata=stratMat, nsim=500)

## Showing some of the results
head(result(fsirs))

## Extract the FSIR
head(fsirs$fsir)

## We can also directly access the p-values from the simulation,
## these are however the raw, unadjusted p-values.
head(fsirs$pvalue)

## Get the time in strata for each individual
head(fsirs$timeInStrata)

## Plot the pedigree for a family with significant FSIRs.
## The numbers below the IDs of the individuals represent the actual
## FSIR values.
plotPed(fsirs, family=result(fsirs)$family[1])

## Plot also the result from the simulation run.
plotRes(fsirs, id=result(fsirs)$id[1])

## Extract the data and result for an individual
resultForId(fsirs, id="16424")

}
\keyword{classes}
